@@ -37,6 +37,7 @@
 
 #include "bhv_scan_field.h"
 #include "neck_scan_field.h"
+#include "neck_scan_players.h"
 
 #include <rcsc/player/player_agent.h>
 #include <rcsc/common/logger.h>
@@ -304,12 +305,15 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
                   ball_rel_angle_next.degree(),
                   next_view_width );
 
+    //
     // never look the ball
+    //
     if ( ball_rel_angle_next.abs()
          > ServerParam::i().maxNeckAngle() + next_view_width * 0.5 )
     {
         dlog.addText( Logger::ACTION,
                       __FILE__": (Neck_TurnToBall) never look. scan field" );
+        agent->debugClient().addMessage( "NeckBall:Scan" );
         Neck_ScanField().execute( agent );
         return true;
     }
@@ -335,132 +339,45 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
         angle_buf = std::max( 0.0, next_view_width * 0.5 - 20.0 ); // 2008-07-11
     }
 
-    if ( ( wm.ball().pos().x > 0.0
-           || wm.ball().pos().absY() > ServerParam::i().pitchHalfWidth() - 8.0
-           || ! opp
-           || opp->distFromBall() > 3.0
-           )
-         && wm.opponentsFromSelf().size() >= 11 )
+    //
+    // if possible,
+    // face to the angle that contans the largest number of players.
+    //
+    if ( wm.opponentsFromSelf().size() >= 11
+         && ( wm.ball().pos().x > 0.0
+              || wm.ball().pos().absY() > ServerParam::i().pitchHalfWidth() - 8.0
+              || ! opp
+              || opp->distFromBall() > 3.0 ) )
     {
-        //
-        // find the angle that contans the largest number of players.
-        //
+        double best_angle = Neck_ScanPlayers::INVALID_ANGLE;
 
-        double best_dir = -360.0;
-        int best_score = 0;
-        double best_angle_diff = 0.0;
-
-        const double dir_step = std::max( 5.0, ( angle_buf * 2.0 ) / 20.0 );
-        const double max_dir = ball_angle_next.degree() + angle_buf + dir_step * 0.5;
-
-        dlog.addText( Logger::ACTION,
-                      __FILE__": (Neck_TurnToBall) angle_buf=%.1f max_dir=%.2f dir_step=%.1f",
-                      angle_buf,
-                      max_dir, dir_step );
-        dlog.addText( Logger::ACTION,
-                      __FILE__": (Neck_TurnToBall) ball_angle=%.1f start_dir=%.1f",
-                      ball_angle_next.degree(),
-                      ball_angle_next.degree() - angle_buf );
-
-        const AbstractPlayerCont::const_iterator end = wm.allPlayers().end();
-        for ( double dir = ball_angle_next.degree() - angle_buf;
-              dir < max_dir;
-              dir += dir_step )
+        if ( ball_dist > ServerParam::i().visibleDistance() - 0.3 )
         {
-#if 1
-            AngleDeg neck_angle = AngleDeg( dir ) - my_body_next;
-            if ( neck_angle.degree() > ServerParam::i().maxNeckAngle()
-                 || neck_angle.degree() < ServerParam::i().minNeckAngle() )
-            {
-#ifdef DEBUG_NECK_TURN_TO_BALL
-                dlog.addText( Logger::ACTION,
-                              "__ dir=%.0f over the next neck range",
-                              dir );
-#endif
-                continue;
-            }
-#endif
-
-            AngleDeg left_angle = dir - next_view_width * 0.5;
-            AngleDeg right_angle = dir + next_view_width * 0.5;
-
-            int score = 0;
-            double angle_diff = 180.0;
-
-            for ( AbstractPlayerCont::const_iterator p = wm.allPlayers().begin();
-                  p != end;
-                  ++p )
-            {
-                if ( (*p)->isSelf() ) continue;
-                //if ( (*p)->isGhost() ) continue;
-                Vector2D pos = (*p)->pos() + (*p)->vel();
-                if ( pos.dist( my_next ) > 35.0 ) continue; // XXX magic number XXX
-
-                AngleDeg angle = ( pos - my_next ).th();
-                if ( angle.isRightOf( left_angle )
-                     && angle.isLeftOf( right_angle ) )
-                {
-                    if ( (*p)->goalie()
-                         && my_next.x > 33.0 )
-                    {
-                        score += (*p)->posCount() * 2 + 20;
-                    }
-                    else if ( pos.x > my_next.x - 5.0 )
-                    {
-                        score += (*p)->posCount() * 2 + 1;
-                    }
-                    else
-                    {
-                        score += (*p)->posCount();
-                    }
-
-                    double tmp_diff = std::min( ( angle - left_angle ).abs(),
-                                                ( angle - right_angle ).abs() );
-                    if ( tmp_diff < angle_diff )
-                    {
-                        angle_diff = tmp_diff;
-                    }
-                }
-            }
-#ifdef DEBUG_NECK_TURN_TO_BALL
+            double min_neck_angle = ball_rel_angle_next.degree() - angle_buf;
+            double max_neck_angle = ball_rel_angle_next.degree() + angle_buf;
             dlog.addText( Logger::ACTION,
-                          "__ dir=%.0f  score=%d  angle_diff=%.1f",
-                          dir,
-                          score,
-                          angle_diff );
-#endif
-            if ( score > best_score )
-            {
-                best_dir = dir;
-                best_score = score;
-                best_angle_diff = angle_diff;
-#ifdef DEBUG_NECK_TURN_TO_BALL
-                dlog.addText( Logger::ACTION,
-                              "__ update best dir (1)" );
-#endif
-            }
-            else if ( score == best_score )
-            {
-                if ( best_angle_diff < angle_diff )
-                {
-                    best_dir = dir;
-                    best_score = score;
-                    best_angle_diff = angle_diff;
-#ifdef DEBUG_NECK_TURN_TO_BALL
-                    dlog.addText( Logger::ACTION,
-                                  "__ update best dir (2)" );
-#endif
-                }
-            }
+                          __FILE__": (Neck_TurnToBall) angle_buf=%.1f min_neck=%.1f max_neck=%.1f",
+                          angle_buf,
+                          min_neck_angle, max_neck_angle );
+            best_angle = Neck_ScanPlayers::get_best_angle( agent,
+                                                           min_neck_angle,
+                                                           max_neck_angle );
+        }
+        else
+        {
+            dlog.addText( Logger::ACTION,
+                          __FILE__": (Neck_TurnToBall) visible distance" );
+            best_angle = Neck_ScanPlayers::get_best_angle( agent );
         }
 
-        if ( best_dir != -360.0 )
+        if ( best_angle != Neck_ScanPlayers::INVALID_ANGLE )
         {
-            AngleDeg best_angle = best_dir;
-            AngleDeg neck_moment = best_angle - my_body_next - wm.self().neck();
+            AngleDeg target_angle = best_angle;
+            AngleDeg neck_moment = target_angle - my_body_next - wm.self().neck();
             dlog.addText( Logger::ACTION,
-                          __FILE__": (Neck_TurnToBall)  best_dir=%.1f best_angle=%.1f  moment=%.1f",
-                          best_dir, best_angle.degree(), neck_moment.degree() );
+                          __FILE__": (Neck_TurnToBall) target_angle=%.1f moment=%.1f",
+                          target_angle.degree(), neck_moment.degree() );
+            agent->debugClient().addMessage( "NeckBall:ScanPl" );
             agent->doTurnNeck( neck_moment );
             return true;
         }
@@ -476,13 +393,45 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     AngleDeg left_rel_angle = ball_rel_angle_next - angle_buf;
     AngleDeg right_rel_angle = ball_rel_angle_next + angle_buf;
 
-    if ( left_rel_angle.isLeftOf( ServerParam::i().minNeckAngle() ) )
+    dlog.addText( Logger::ACTION,
+                  __FILE__": (Neck_TurnToBall) ball_rel=%.0f angle_buf=%.1f left_rel=%.0f right_rel=%.0f",
+                  ball_rel_angle_next.degree(),
+                  angle_buf,
+                  left_rel_angle.degree(), right_rel_angle.degree() );
+
+    if ( left_rel_angle.degree() < ServerParam::i().minNeckAngle() )
     {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": __ left_rel=%.0f < minNeck=%.0f",
+                      left_rel_angle.degree(),
+                      ServerParam::i().minNeckAngle() );
         left_rel_angle = ServerParam::i().minNeckAngle();
     }
 
-    if ( right_rel_angle.isRightOf( ServerParam::i().maxNeckAngle() ) )
+    if ( left_rel_angle.degree() > ServerParam::i().maxNeckAngle() )
     {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": __ left_rel=%.0f > maxNeck=%.0f",
+                      left_rel_angle.degree(),
+                      ServerParam::i().maxNeckAngle() );
+        left_rel_angle = ServerParam::i().maxNeckAngle();
+    }
+
+    if ( right_rel_angle.degree() < ServerParam::i().minNeckAngle() )
+    {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": __ right_rel%.0f < minNeck=%.0f",
+                      right_rel_angle.degree(),
+                      ServerParam::i().minNeckAngle() );
+        right_rel_angle = ServerParam::i().minNeckAngle();
+    }
+
+    if ( right_rel_angle.degree() > ServerParam::i().maxNeckAngle() )
+    {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": __ right_rel%.0f > maxNeck=%.0f",
+                      right_rel_angle.degree(),
+                      ServerParam::i().maxNeckAngle() );
         right_rel_angle = ServerParam::i().maxNeckAngle();
     }
 
@@ -506,10 +455,12 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
 
     if ( left_sum_count > right_sum_count )
     {
+        agent->debugClient().addMessage( "NeckBall:Left" );
         agent->doTurnNeck( left_rel_angle - wm.self().neck() );
     }
     else
     {
+        agent->debugClient().addMessage( "NeckBall:Right" );
         agent->doTurnNeck( right_rel_angle - wm.self().neck() );
     }
 
