@@ -90,7 +90,7 @@ Bhv_BeforeKickOff::execute( PlayerAgent * agent )
         }
         else
         {
-            kickoff_side = ( agent->world().isOurLeft()
+            kickoff_side = ( agent->world().ourSide() == LEFT
                              ? RIGHT
                              : LEFT );
         }
