@@ -107,7 +107,7 @@ Bhv_ScanField::findBall( PlayerAgent * agent )
         = wm.self().pos()
         + wm.self().vel();
     AngleDeg face_angle
-        = ( wm.ball().seenPos().valid()
+        = ( wm.ball().seenPos().isValid()
             ? ( wm.ball().seenPos() - mynext ).th()
             : ( -mynext ).th()  // face to field center
             );
