@@ -101,26 +101,36 @@ Bhv_Shoot2008::execute( PlayerAgent * agent )
     double one_step_speed = one_step_vel.r();
 
     dlog.addText( Logger::SHOOT,
-                  __FILE__": shoot to (%.2f, %.2f) speed=%.2f one_kick_max_speed=%.2f",
+                  __FILE__": shoot to (%.2f, %.2f) speed=%f one_kick_max_speed=%f",
                   target_point.x, target_point.y,
                   first_speed,
                   one_step_speed );
 
-    if ( one_step_speed > first_speed * 0.96 )
+    if ( one_step_speed > first_speed * 0.99 )
+    {
+        if ( Body_SmartKick( target_point,
+                             one_step_speed,
+                             one_step_speed * 0.99 - 0.0001,
+                             1 ).execute( agent ) )
+        {
+             agent->setNeckAction( new Neck_TurnToGoalieOrScan( -1 ) );
+             agent->debugClient().addMessage( "Force1Step" );
+             return true;
+        }
+    }
+
+    if ( Body_SmartKick( target_point,
+                         first_speed,
+                         first_speed * 0.99,
+                         3 ).execute( agent ) )
     {
-        Body_SmartKick( target_point,
-                        one_step_speed,
-                        one_step_speed - 0.0001,
-                        1 ).execute( agent );
         agent->setNeckAction( new Neck_TurnToGoalieOrScan( -1 ) );
-        agent->debugClient().addMessage( "Enforce1Step" );
         return true;
     }
 
-    Body_SmartKick( target_point, first_speed, first_speed * 0.96, 3 ).execute( agent );
-
-    agent->setNeckAction( new Neck_TurnToGoalieOrScan( -1 ) );
-    return true;
+    dlog.addText( Logger::SHOOT,
+                  __FILE__": failed" );
+    return false;
 }
 
 }
