@@ -43,7 +43,7 @@ namespace rcsc {
   \brief advanced shoot planning and behavior.
  */
 class Bhv_Shoot2008
-    : public BodyAction {
+    : public SoccerBehavior {
 private:
     //! cached shoot table
     static ShootTable2008 S_shoot_table;
