@@ -315,6 +315,12 @@ Body_Dribble2008::doTurnOnly( PlayerAgent * agent,
         return false;
     }
 
+    const ServerParam & SP = ServerParam::i();
+
+    const PlayerType & ptype = wm.self().playerType();
+    const double my_speed = wm.self().vel().r();
+    const double ball_speed = wm.ball().vel().r();
+
     //---------------------------------------------------------//
     // check next ball dist after turn
     Vector2D my_next = wm.self().pos() + wm.self().vel();
@@ -326,11 +332,10 @@ Body_Dribble2008::doTurnOnly( PlayerAgent * agent,
                   ball_next_dist );
 
     // not kickable at next cycle, if do turn at current cycle.
-    if ( ball_next_dist
-         > ( wm.self().kickableArea()
-             - wm.ball().vel().r() * ServerParam::i().ballRand()
-             - wm.self().vel().r() * ServerParam::i().playerRand()
-             - 0.15 ) )
+    if ( ball_next_dist > ( ptype.kickableArea()
+                            - ball_speed * SP.ballRand()
+                            - my_speed * SP.playerRand()
+                            - 0.15 ) )
     {
         dlog.addText( Logger::DRIBBLE,
                       __FILE__": doTurnOnly  not kickable at next. next_ball_dist=%f",
@@ -338,10 +343,10 @@ Body_Dribble2008::doTurnOnly( PlayerAgent * agent,
         return false;
     }
 
-
     //---------------------------------------------------------//
     // check required turn step.
-    if ( ! wm.self().canTurn( dir_diff ) )
+    //if ( ! wm.self().canTurn( dir_diff ) )
+    if ( dir_diff <= ptype.effectiveTurn( SP.maxMoment(), my_speed ) )
     {
         // it is necessary to turn more than one step.
         dlog.addText( Logger::DRIBBLE,
@@ -350,11 +355,10 @@ Body_Dribble2008::doTurnOnly( PlayerAgent * agent,
         Vector2D my_next2 = wm.self().inertiaPoint( 2 );
         Vector2D ball_next2 = wm.ball().inertiaPoint( 2 );
         double ball_dist_next2 = my_next2.dist( ball_next2 );
-        if ( ball_dist_next2
-             > ( wm.self().kickableArea()
-                 - wm.ball().vel().r() * ServerParam::i().ballRand()
-                 - wm.self().vel().r() * ServerParam::i().playerRand()
-                 - 0.15 ) )
+        if ( ball_dist_next2 > ( ptype.kickableArea()
+                                 - ball_speed * SP.ballRand()
+                                 - my_speed * SP.playerRand()
+                                 - 0.15 ) )
         {
             dlog.addText( Logger::DRIBBLE,
                           __FILE__": doTurnOnly  not kickable at 2 cycles later. next2_ball_dsit=%f",
@@ -532,7 +536,7 @@ Body_Dribble2008::doKickTurnsDash( PlayerAgent * agent,
           1 + n_turn, // kick + turns
           wm.self().playerType().playerDecay() );
 
-    const double control_dist = wm.self().kickableArea() * 0.7;
+    const double control_dist = wm.self().playerType().kickableArea() * 0.7;
 
     const PlayerObject * nearest_opp = wm.getOpponentNearestToBall( 5 );
     if ( nearest_opp
@@ -1060,7 +1064,7 @@ Body_Dribble2008::doKickDashes( PlayerAgent * agent,
                       __FILE__": doKickDashes() never reach. rotate." );
         agent->debugClient().addMessage( "DribKDFail" );
 
-        return Body_KickToRelative( wm.self().kickableArea() * 0.7,
+        return Body_KickToRelative( wm.self().playerType().kickableArea() * 0.7,
                                     keep_global_angle - wm.self().body(),
                                     false  // not need to stop
                                     ).execute( agent );
@@ -1096,7 +1100,7 @@ Body_Dribble2008::doKickDashes( PlayerAgent * agent,
                       __FILE__": doKickDashes() maybe collision. rotate. rel_angle=%.1f",
                       rotate_rel_angle.degree() );
         agent->debugClient().addMessage( "DribKDCol" );
-        return Body_KickToRelative( wm.self().kickableArea() * 0.7,
+        return Body_KickToRelative( wm.self().playerType().kickableArea() * 0.7,
                                     rotate_rel_angle,
                                     false  // not need to stop
                                     ).execute( agent );
@@ -1198,7 +1202,7 @@ Body_Dribble2008::doKickDashesWithBall( PlayerAgent * agent,
 
     const int DIST_DIVS = 10;
 
-    const double max_dist = wm.self().kickableArea() + 0.2;
+    const double max_dist = wm.self().playerType().kickableArea() + 0.2;
     double first_ball_dist = ( wm.self().playerType().playerSize()
                                + ServerParam::i().ballSize()
                                + 0.15 );
@@ -1218,7 +1222,7 @@ Body_Dribble2008::doKickDashesWithBall( PlayerAgent * agent,
             = ( arc_dist_step * 360.0 )
             / ( 2.0 * first_ball_dist * M_PI );
         const int ANGLE_DIVS
-            = ( first_ball_dist < wm.self().kickableArea() - 0.1
+            = ( first_ball_dist < wm.self().playerType().kickableArea() - 0.1
                 ? static_cast< int >( std::ceil( angle_range / angle_step ) ) + 1
                 : static_cast< int >( std::ceil( angle_range_forward / angle_step ) ) + 1 );
 
@@ -1307,7 +1311,7 @@ Body_Dribble2008::doKickDashesWithBall( PlayerAgent * agent,
             dlog.addCircle( Logger::DRIBBLE,
                             ball_pos, 0.05, "#0000FF" );
             dlog.addCircle( Logger::DRIBBLE,
-                            my_state[i], wm.self().kickableArea(), "#FF00FF" );
+                            my_state[i], wm.self().playerType().kickableArea(), "#FF00FF" );
             //agent->debugClient().addCircle( ball_pos, 0.05 );
             //agent->debugClient().addCircle( my_state[i], wm.self().kickableArea() );
         }
@@ -1359,9 +1363,7 @@ Body_Dribble2008::createSelfCache( PlayerAgent * agent,
     self_cache.clear();
     self_cache.reserve( turn_count + dash_count + 1 );
 
-    double my_stamina = wm.self().stamina();
-    double my_effort = wm.self().effort();
-    double my_recovery = wm.self().recovery();
+    StaminaModel stamina_model = wm.self().staminaModel();
 
     Vector2D my_pos = wm.self().pos();
     Vector2D my_vel = wm.self().vel();
@@ -1378,11 +1380,8 @@ Body_Dribble2008::createSelfCache( PlayerAgent * agent,
         self_cache.push_back( my_pos );
     }
 
-    wm.self().playerType().predictStaminaAfterWait( ServerParam::i(),
-                                                    1 + turn_count,
-                                                    &my_stamina,
-                                                    &my_effort,
-                                                    my_recovery );
+    stamina_model.simulateWaits( wm.self().playerType(), 1 + turn_count );
+
     AngleDeg accel_angle;
     if ( turn_count == 0 )
     {
@@ -1399,7 +1398,7 @@ Body_Dribble2008::createSelfCache( PlayerAgent * agent,
     {
         double available_stamina
             =  std::max( 0.0,
-                         my_stamina
+                         stamina_model.stamina()
                          - ServerParam::i().recoverDecThrValue()
                          - 300.0 );
         double consumed_stamina = ( dash_power > 0.0
@@ -1412,7 +1411,7 @@ Body_Dribble2008::createSelfCache( PlayerAgent * agent,
                               : consumed_stamina * -0.5 );
         double max_accel_mag = ( std::fabs( used_power )
                                  * wm.self().playerType().dashPowerRate()
-                                 * my_effort );
+                                 * stamina_model.effort() );
         double accel_mag = max_accel_mag;
         if ( wm.self().playerType().normalizeAccel( my_vel,
                                                     accel_angle,
@@ -1423,7 +1422,7 @@ Body_Dribble2008::createSelfCache( PlayerAgent * agent,
 
         Vector2D dash_accel
             = Vector2D::polar2vector( std::fabs( used_power )
-                                      * my_effort
+                                      * stamina_model.effort()
                                       * wm.self().playerType().dashPowerRate(),
                                       accel_angle );
         my_vel += dash_accel;
@@ -1433,14 +1432,14 @@ Body_Dribble2008::createSelfCache( PlayerAgent * agent,
 
         my_vel *= wm.self().playerType().playerDecay();
 
-        wm.self().playerType().predictStaminaAfterOneDash( ServerParam::i(),
-                                                           used_power,
-                                                           &my_stamina,
-                                                           &my_effort,
-                                                           &my_recovery );
+        stamina_model.simulateDash( wm.self().playerType(), used_power );
     }
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
 bool
 Body_Dribble2008::simulateKickDashes( const WorldModel & wm,
                                       const std::vector< Vector2D > & self_cache,
@@ -1463,7 +1462,7 @@ Body_Dribble2008::simulateKickDashes( const WorldModel & wm,
     const ServerParam & param = ServerParam::i();
     const double collide_dist = ( wm.self().playerType().playerSize()
                                   + param.ballSize() );
-    const double kickable_area = wm.self().kickableArea();
+    const double kickable_area = wm.self().playerType().kickableArea();
 
     const Vector2D first_ball_accel = first_ball_vel - wm.ball().vel();
     const double first_ball_accel_r = first_ball_accel.r();
@@ -1810,7 +1809,7 @@ Body_Dribble2008::doAvoidKick( PlayerAgent * agent,
     double dir_diff_abs = (target_angle - wm.self().body()).abs();
     double dir_margin_abs
         = std::max( 12.0,
-                    std::fabs( AngleDeg::atan2_deg( wm.self().kickableArea() * 0.8,
+                    std::fabs( AngleDeg::atan2_deg( wm.self().playerType().kickableArea() * 0.8,
                                                     ball_move_radius ) ) );
 
     double ball_first_speed;
@@ -2005,7 +2004,7 @@ Body_Dribble2008::canKickAfterDash( const PlayerAgent * agent,
                       ball_dist, noise_buf );
 
         if ( ( ( ball_next - my_pos ).th() - accel_angle ).abs() < 150.0
-             && ball_dist < wm.self().kickableArea() - noise_buf - 0.2
+             && ball_dist < wm.self().playerType().kickableArea() - noise_buf - 0.2
              && ( ball_dist - noise_buf > ( wm.self().playerType().playerSize()
                                             + ServerParam::i().ballSize() ) ) )
         {
@@ -2305,9 +2304,9 @@ Body_Dribble2008::getAvoidAngle( const PlayerAgent * agent,
                                       new_target_angle );
 
         if ( sub_target.absX()
-             > ServerParam::i().pitchHalfLength() - wm.self().kickableArea() - 0.2
+             > ServerParam::i().pitchHalfLength() - wm.self().playerType().kickableArea() - 0.2
              || sub_target.absY()
-             > ServerParam::i().pitchHalfWidth() - wm.self().kickableArea() - 0.2 )
+             > ServerParam::i().pitchHalfWidth() - wm.self().playerType().kickableArea() - 0.2 )
         {
             dlog.addText( Logger::DRIBBLE,
                           "avoid angle. out of pitch. angle=%.0f pos=(%.1f %.1f)",
