@@ -129,7 +129,7 @@ Body_HoldBall2008::avoidOpponent( PlayerAgent * agent )
 
     Vector2D point = searchKeepPoint( wm );
 
-    if ( ! point.valid() )
+    if ( ! point.isValid() )
     {
         dlog.addText( Logger::ACTION,
                       __FILE__": avoidOpponent() no candidate point" );
@@ -193,6 +193,13 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
 {
     const ServerParam & param = ServerParam::i();
 
+    const double max_pitch_x = ( param.keepawayMode()
+                                 ? param.keepawayLength() * 0.5 - 0.2
+                                 : param.pitchHalfLength() - 0.2 );
+    const double max_pitch_y = ( param.keepawayMode()
+                                 ? param.keepawayWidth() * 0.5 - 0.2
+                                 : param.pitchHalfWidth() - 0.2 );
+
     const int dir_divs = 20;
     const double dir_step = 360.0 / dir_divs;
 
@@ -241,8 +248,8 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
         // near side point
         {
         const Vector2D near_pos = my_next + unit_pos.setLengthVector( near_dist );
-        if ( near_pos.absX() < param.pitchHalfLength() - 0.2
-             && near_pos.absY() < param.pitchHalfWidth() - 0.2 )
+        if ( near_pos.absX() < max_pitch_x
+             && near_pos.absY() < max_pitch_y )
         {
             Vector2D ball_move = near_pos - wm.ball().pos();
             Vector2D kick_accel = ball_move - wm.ball().vel();
@@ -250,9 +257,7 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
             // can kick to the point by 1 step kick
             if ( kick_accel.r() < param.maxPower() * wm.self().kickRate() )
             {
-                double near_krate = wm.self().playerType().kickRate( param,
-                                                                     near_dist,
-                                                                     dir_diff );
+                double near_krate = wm.self().playerType().kickRate( near_dist, dir_diff );
                 // can stop the ball by 1 step kick
                 if ( ball_move.r() * param.ballDecay() < param.maxPower() * near_krate )
                 {
@@ -300,8 +305,8 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
         // middle point
         {
         const Vector2D mid_pos = my_next + unit_pos.setLengthVector( mid_dist );
-        if ( mid_pos.absX() < param.pitchHalfLength() - 0.2
-             && mid_pos.absY() < param.pitchHalfWidth() - 0.2 )
+        if ( mid_pos.absX() < max_pitch_x
+             && mid_pos.absY() < max_pitch_y )
         {
             Vector2D ball_move = mid_pos - wm.ball().pos();
             Vector2D kick_accel = ball_move - wm.ball().vel();
@@ -319,11 +324,9 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
                     * ( current_pos_rate + current_speed_rate );
                 // move noise is small
                 if ( ( my_noise + ball_noise + max_kick_rand ) * 0.95
-                     < wm.self().kickableArea() - mid_dist - 0.1 )
+                     < wm.self().playerType().kickableArea() - mid_dist - 0.1 )
                 {
-                    double mid_krate = wm.self().playerType().kickRate( param,
-                                                                        mid_dist,
-                                                                        dir_diff );
+                    double mid_krate = wm.self().playerType().kickRate( mid_dist, dir_diff );
                     // can stop the ball by 1 step kick
                     if ( move_dist * param.ballDecay()
                          < param.maxPower() * mid_krate )
@@ -387,8 +390,8 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
         // far side point
         {
         const Vector2D far_pos = my_next + unit_pos.setLengthVector( far_dist );
-        if ( far_pos.absX() < param.pitchHalfLength() - 0.2
-             && far_pos.absY() < param.pitchHalfWidth() - 0.2 )
+        if ( far_pos.absX() < max_pitch_x
+             && far_pos.absY() < max_pitch_y )
         {
             Vector2D ball_move = far_pos - wm.ball().pos();
             Vector2D kick_accel = ball_move - wm.ball().vel();
@@ -406,11 +409,9 @@ Body_HoldBall2008::createKeepPoints( const WorldModel & wm,
                     * ( current_pos_rate + current_speed_rate );
                 // move noise is small
                 if ( ( my_noise + ball_noise + max_kick_rand ) * 0.95
-                     < wm.self().kickableArea() - far_dist - 0.1 )
+                     < wm.self().playerType().kickableArea() - far_dist - 0.1 )
                 {
-                    double far_krate = wm.self().playerType().kickRate( param,
-                                                                        far_dist,
-                                                                        dir_diff );
+                    double far_krate = wm.self().playerType().kickRate( far_dist, dir_diff );
                     // can stop the ball by 1 step kick
                     if ( move_dist * param.ballDecay()
                          < param.maxPower() * far_krate )
@@ -735,8 +736,15 @@ Body_HoldBall2008::evaluateKeepPoint( const WorldModel & wm,
 bool
 Body_HoldBall2008::keepFront( PlayerAgent * agent )
 {
-    const WorldModel & wm = agent->world();
     const ServerParam & param = ServerParam::i();
+    const double max_pitch_x = ( param.keepawayMode()
+                                 ? param.keepawayLength() * 0.5 - 0.2
+                                 : param.pitchHalfLength() - 0.2 );
+    const double max_pitch_y = ( param.keepawayMode()
+                                 ? param.keepawayWidth() * 0.5 - 0.2
+                                 : param.pitchHalfWidth() - 0.2 );
+
+    const WorldModel & wm = agent->world();
     const double front_keep_dist
         = wm.self().playerType().playerSize()
         + param.ballSize() + 0.05;
@@ -746,8 +754,8 @@ Body_HoldBall2008::keepFront( PlayerAgent * agent )
         = my_next
         + Vector2D::polar2vector( front_keep_dist, wm.self().body() );
 
-    if ( front_pos.absX() > param.pitchHalfLength() - 0.2
-         || front_pos.absY() > param.pitchHalfWidth() - 0.2 )
+    if ( front_pos.absX() > max_pitch_x
+         || front_pos.absY() > max_pitch_y )
     {
         dlog.addText( Logger::ACTION,
                       __FILE__": keepFront() failed. out of pitch. point=(%.2f %.2f)",
@@ -797,7 +805,7 @@ Body_HoldBall2008::keepFront( PlayerAgent * agent )
 bool
 Body_HoldBall2008::keepReverse( PlayerAgent * agent )
 {
-    if ( ! M_kick_target_point.valid() )
+    if ( ! M_kick_target_point.isValid() )
     {
         return false;
     }
@@ -805,6 +813,13 @@ Body_HoldBall2008::keepReverse( PlayerAgent * agent )
     const WorldModel & wm = agent->world();
     const ServerParam & param = ServerParam::i();
 
+    const double max_pitch_x = ( param.keepawayMode()
+                                 ? param.keepawayLength() * 0.5 - 0.2
+                                 : param.pitchHalfLength() - 0.2 );
+    const double max_pitch_y = ( param.keepawayMode()
+                                 ? param.keepawayWidth() * 0.5 - 0.2
+                                 : param.pitchHalfWidth() - 0.2 );
+
     //const Vector2D my_inertia = wm.self().inertiaFinalPoint();
     const Vector2D my_inertia = wm.self().pos() + wm.self().vel();
 
@@ -840,8 +855,8 @@ Body_HoldBall2008::keepReverse( PlayerAgent * agent )
             = my_inertia
             + Vector2D::polar2vector( keep_dist, keep_angle );
 
-        if ( keep_pos.absX() > param.pitchHalfLength() - 0.2
-             || keep_pos.absY() > param.pitchHalfWidth() - 0.2 )
+        if ( keep_pos.absX() > max_pitch_x
+             || keep_pos.absY() > max_pitch_y )
         {
             continue;
         }
@@ -862,14 +877,12 @@ Body_HoldBall2008::keepReverse( PlayerAgent * agent )
             * ( kick_power / param.maxPower() )
             * ( current_pos_rate + current_speed_rate );
         if ( ( my_noise + ball_noise + max_kick_rand )
-             > wm.self().kickableArea() - keep_dist - 0.1 )
+             > wm.self().playerType().kickableArea() - keep_dist - 0.1 )
         {
             continue;
         }
 
-        double new_krate = wm.self().playerType().kickRate( param,
-                                                            keep_dist,
-                                                            dir_diff );
+        double new_krate = wm.self().playerType().kickRate( keep_dist, dir_diff );
         if ( move_dist * param.ballDecay() > new_krate * param.maxPower() )
         {
             continue;
@@ -906,14 +919,20 @@ Body_HoldBall2008::keepReverse( PlayerAgent * agent )
 bool
 Body_HoldBall2008::turnToPoint( PlayerAgent * agent )
 {
-    const WorldModel & wm = agent->world();
     const ServerParam & param = ServerParam::i();
+    const double max_pitch_x = ( param.keepawayMode()
+                                 ? param.keepawayLength() * 0.5 - 0.2
+                                 : param.pitchHalfLength() - 0.2 );
+    const double max_pitch_y = ( param.keepawayMode()
+                                 ? param.keepawayWidth() * 0.5 - 0.2
+                                 : param.pitchHalfWidth() - 0.2 );
 
+    const WorldModel & wm = agent->world();
     const Vector2D my_next = wm.self().pos() + wm.self().vel();
     const Vector2D ball_next = wm.ball().pos() + wm.ball().vel();
 
-    if ( ball_next.absX() > param.pitchHalfLength() - 0.2
-         || ball_next.absY() > param.pitchHalfWidth() - 0.2 )
+    if ( ball_next.absX() > max_pitch_x
+         || ball_next.absY() > max_pitch_y )
     {
         dlog.addText( Logger::ACTION,
                       __FILE__": turnToPoint() failed. out of pitch. ball_next=(%.2f %.2f)",
@@ -936,8 +955,12 @@ Body_HoldBall2008::turnToPoint( PlayerAgent * agent )
         return false;
     }
 
-    Vector2D face_point( ServerParam::i().pitchHalfLength() - 5.0,
-                         0.0 );
+    Vector2D face_point( 0.0, 0.0 );
+    if ( ! param.keepawayMode() )
+    {
+        face_point.x =  param.pitchHalfLength() - 5.0;
+    }
+
     if ( M_do_turn )
     {
         face_point = M_turn_target_point;
