@@ -1,7 +1,7 @@
 // -*-c++-*-
 
 /*!
-  \file body_intercept2008.cpp
+  \file body_intercept2009.cpp
   \brief ball chasing action including smart planning.
 */
 
@@ -111,7 +111,7 @@ Body_Intercept2009::execute( PlayerAgent * agent )
                       __FILE__": can get the ball only by inertia move. Turn!" );
 
         Vector2D face_point = M_face_point;
-        if ( ! face_point.valid() )
+        if ( ! face_point.isValid() )
         {
             face_point.assign( 50.5, wm.self().pos().y * 0.75 );
         }
@@ -156,7 +156,8 @@ Body_Intercept2009::execute( PlayerAgent * agent )
         return true;
     }
 
-    if ( M_save_recovery )
+    if ( M_save_recovery
+         && ! wm.self().staminaModel().capacityIsEmpty() )
     {
         double consumed_stamina = best_intercept.dashPower();
         if ( best_intercept.dashPower() < 0.0 ) consumed_stamina *= -2.0;
@@ -239,6 +240,12 @@ Body_Intercept2009::getBestIntercept( const WorldModel & wm,
 
     const Vector2D goal_pos( 65.0, 0.0 );
     const Vector2D our_goal_pos( -SP.pitchHalfLength(), 0.0 );
+    const double max_pitch_x = ( SP.keepawayMode()
+                                 ? SP.keepawayLength() * 0.5 - 1.0
+                                 : SP.pitchHalfLength() - 1.0 );
+    const double max_pitch_y = ( SP.keepawayMode()
+                                 ? SP.keepawayWidth() * 0.5 - 1.0
+                                 : SP.pitchHalfWidth() - 1.0 );
     const double penalty_x = SP.ourPenaltyAreaLineX();
     const double penalty_y = SP.penaltyAreaHalfWidth();
     const double speed_max = wm.self().playerType().realSpeedMax() * 0.9;
@@ -289,14 +296,15 @@ Body_Intercept2009::getBestIntercept( const WorldModel & wm,
                       cache[i].ballDist() );
 #endif
 
-        if ( ball_pos.absX() > SP.pitchHalfLength() - 1.0
-             || ball_pos.absY() > SP.pitchHalfWidth() - 1.0 )
+        if ( ball_pos.absX() > max_pitch_x
+             || ball_pos.absY() > max_pitch_y )
         {
             continue;
         }
 
 #ifdef USE_GOALIE_MODE
         if ( wm.self().goalie()
+             && wm.lastKickerSide() != wm.ourSide()
              && ball_pos.x < penalty_x - 1.0
              && ball_pos.absY() < penalty_y - 1.0
              && cycle < opp_min - 1 )
@@ -320,6 +328,7 @@ Body_Intercept2009::getBestIntercept( const WorldModel & wm,
         }
 
         if ( wm.self().goalie()
+             && wm.lastKickerSide() != wm.ourSide()
              && cycle < mate_min - 3
              && cycle < opp_min - 5
              && ( ball_pos.x > penalty_x - 1.0
@@ -844,7 +853,7 @@ Body_Intercept2009::doWaitTurn( PlayerAgent * agent,
          && info.turnCycle() == 1 )
     {
         Vector2D face_point = M_face_point;
-        if ( ! face_point.valid() )
+        if ( ! face_point.isValid() )
         {
             face_point.assign( 50.5, wm.self().pos().y * 0.9 );
         }
@@ -866,7 +875,7 @@ Body_Intercept2009::doWaitTurn( PlayerAgent * agent,
         }
     }
 
-    double dist_buf = wm.self().kickableArea() - 0.3 + extra_buf;
+    double dist_buf = wm.self().playerType().kickableArea() - 0.3 + extra_buf;
     dist_buf -= 0.1 * wm.ball().seenPosCount();
 
     dlog.addText( Logger::INTERCEPT,
@@ -891,7 +900,7 @@ Body_Intercept2009::doWaitTurn( PlayerAgent * agent,
         }
     }
 
-    if ( ! face_point.valid() )
+    if ( ! face_point.isValid() )
     {
         face_point.assign( 50.5, wm.self().pos().y * 0.9 );
     }
@@ -901,7 +910,7 @@ Body_Intercept2009::doWaitTurn( PlayerAgent * agent,
 
     Vector2D faced_rel = target_point - my_inertia;
     faced_rel.rotate( face_angle );
-    if ( faced_rel.absY() > wm.self().kickableArea() - ball_noise - 0.2 )
+    if ( faced_rel.absY() > wm.self().playerType().kickableArea() - ball_noise - 0.2 )
     {
         dlog.addText( Logger::INTERCEPT,
                       __FILE__": doWaitTurn. inertia_y_diff %.2f  ball_noise=%.2f",
@@ -954,7 +963,8 @@ Body_Intercept2009::doInertiaDash( PlayerAgent * agent,
     Vector2D ball_vel = wm.ball().vel() * std::pow( ServerParam::i().ballDecay(),
                                                     info.reachCycle() );
 
-    if ( ! wm.self().goalie()
+    if ( ( ! wm.self().goalie()
+           || wm.lastKickerSide() == wm.ourSide() )
          && wm.self().body().abs() < 50.0 )
     {
         double buf = 0.3;
@@ -1055,7 +1065,7 @@ Body_Intercept2009::doInertiaDash( PlayerAgent * agent,
 
         Vector2D my_inertia = wm.self().inertiaPoint( info.reachCycle() );
         Vector2D face_point = M_face_point;
-        if ( ! M_face_point.valid() )
+        if ( ! M_face_point.isValid() )
         {
             face_point.assign( 50.5, wm.self().pos().y * 0.75 );
         }
