@@ -55,7 +55,7 @@
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
-#define DEBUG
+//#define DEBUG
 
 namespace rcsc {
 
@@ -101,26 +101,6 @@ Body_Pass::execute( PlayerAgent * agent )
     agent->debugClient().addMessage( "pass" );
     agent->debugClient().setTarget( target_point );
 
-#if 0
-    Body_KickMultiStep( target_point,
-                        first_speed,
-                        false
-                        ).execute( agent ); // not enforce
-
-    dlog.addText( Logger::ACTION,
-                  "%s:%d: execute() register pass intention"
-                  ,__FILE__, __LINE__ );
-
-    if ( agent->world().gameMode().type() != GameMode::PlayOn )
-    {
-        agent->setIntention
-            ( new IntentionKick( target_point,
-                                 first_speed,
-                                 3, // max kick step
-                                 false, // not enforce
-                                 agent->world().time() ) );
-    }
-#else
     int kick_step = ( agent->world().gameMode().type() != GameMode::PlayOn
                       && agent->world().gameMode().type() != GameMode::GoalKick_
                       ? 1
@@ -148,7 +128,6 @@ Body_Pass::execute( PlayerAgent * agent )
             return false;
         }
     }
-#endif
 
     if ( agent->config().useCommunication()
          && receiver != Unum_Unknown )
@@ -363,7 +342,7 @@ Body_Pass::create_direct_pass( const WorldModel & world,
     if ( receiver->pos().x < world.self().pos().x + 5.0 )
     {
         if ( receiver->angleFromSelf().abs() < 40.0
-             || ( receiver->pos().x > world.defenseLineX() + 10.0
+             || ( receiver->pos().x > world.ourDefenseLineX() + 10.0
                   && receiver->pos().x > 0.0 )
              )
         {
@@ -539,19 +518,23 @@ Body_Pass::create_direct_pass( const WorldModel & world,
                                                          angle_new )
                                    )
                         );
+#ifdef DEBUG
         dlog.addText( Logger::PASS,
                       "Pass Success direct unum=%d pos=(%.1f %.1f). first_speed= %.1f",
                       receiver->unum(),
                       target_new.x, target_new.y,
                       first_speed );
+#endif
     }
     else
     {
+#ifdef DEBUG
         dlog.addText( Logger::PASS,
                       "Pass Failed direct unum=%d pos=(%.1f %.1f). first_speed= %.1f",
                       receiver->unum(),
                       target_new.x, target_new.y,
                       first_speed );
+#endif
     }
 }
 
@@ -731,7 +714,7 @@ Body_Pass::create_lead_pass( const WorldModel & world,
             // not safety area
             if ( target_point.x < -10.0 )
             {
-                if ( target_point.x < world.defenseLineX() + 10.0 )
+                if ( target_point.x < world.ourDefenseLineX() + 10.0 )
                 {
                     continue;
                 }
@@ -746,7 +729,7 @@ Body_Pass::create_lead_pass( const WorldModel & world,
                 {
                     // safety area
                 }
-                else if ( target_point.x > world.defenseLineX() + 20.0 )
+                else if ( target_point.x > world.ourDefenseLineX() + 20.0 )
                 {
                     // safety area
                 }
@@ -783,13 +766,16 @@ Body_Pass::create_lead_pass( const WorldModel & world,
                                            can_kick_by_one_step( world,
                                                                  first_speed,
                                                                  target_angle ) ) );
+#ifdef DEBUG
                 dlog.addText( Logger::PASS,
                               "Pass Success lead unum=%d pos=(%.1f %.1f) angle=%.1f first_speed=%.1f",
                               receiver->unum(),
                               target_point.x, target_point.y,
                               target_angle.degree(),
                               first_speed );
+#endif
             }
+#ifdef DEBUG
             else
             {
                 dlog.addText( Logger::PASS,
@@ -800,6 +786,7 @@ Body_Pass::create_lead_pass( const WorldModel & world,
                               first_speed );
 
             }
+#endif
         }
     }
 }
@@ -873,8 +860,8 @@ Body_Pass::create_through_pass(const WorldModel & world,
 #endif
         return;
     }
-    if ( world.defenseLineX() < 0.0
-         && receiver->pos().x < world.defenseLineX() - 15.0 )
+    if ( world.ourDefenseLineX() < 0.0
+         && receiver->pos().x < world.ourDefenseLineX() - 15.0 )
     {
 #ifdef DEBUG
         dlog.addText( Logger::PASS,
@@ -1015,6 +1002,8 @@ Body_Pass::create_through_pass(const WorldModel & world,
                                                                  target_angle )
                                            )
                                 );
+#ifdef DEBUG
+
                 dlog.addText( Logger::PASS,
                               "Pass Success through pass unum=%d pos=(%.1f %.1f) angle=%.1f first_speed=%.1f dash_step=%.1f ball_step=%.1f",
                               receiver->unum(),
@@ -1023,7 +1012,9 @@ Body_Pass::create_through_pass(const WorldModel & world,
                               first_speed,
                               dash_step,
                               ball_steps_to_target );
+#endif
             }
+#ifdef DEBUG
             else
             {
                 dlog.addText( Logger::PASS,
@@ -1035,6 +1026,7 @@ Body_Pass::create_through_pass(const WorldModel & world,
                               dash_step,
                               ball_steps_to_target );
             }
+#endif
         }
     }
 }
@@ -1375,6 +1367,7 @@ Body_Pass::evaluate_routes( const WorldModel & world )
             it->score_ *= 1.2;
         }
 
+#ifdef DEBUG
         dlog.addText( Logger::PASS,
                       "PASS Score %6.2f -- to%d(%.1f %.1f) recv_pos(%.1f %.1f) type %d "
                       " speed=%.2f",
@@ -1390,6 +1383,7 @@ Body_Pass::evaluate_routes( const WorldModel & world )
                       opp_dist_rate, x_diff_rate, pos_conf_rate,
                       dir_conf_rate, front_space_rate,
                       ( it->one_step_kick_ ? "one_step" : "" ) );
+#endif
     }
 
 }
