@@ -52,9 +52,9 @@
 #include <fstream>
 
 #define DEBUG_PROFILE
-//#define DEBUG
-//#define DEBUG_EVALUATE
-//#define DEBUG_THREE_STEP
+// #define DEBUG
+#define DEBUG_EVALUATE
+// #define DEBUG_THREE_STEP
 
 namespace rcsc {
 
@@ -270,7 +270,7 @@ KickTable::KickTable()
 bool
 KickTable::createTables()
 {
-    const PlayerType player_type( ServerParam::i() );
+    const PlayerType player_type; // default type
 
     if ( std::fabs( M_player_size - player_type.playerSize() ) < rcsc::EPS
          && std::fabs( M_kickable_margin - player_type.kickableMargin() ) < rcsc::EPS
@@ -561,8 +561,6 @@ KickTable::write( const std::string & file_path )
 void
 KickTable::createStateList( const PlayerType & player_type )
 {
-    const ServerParam & param = ServerParam::i();
-
     const double near_dist = calc_near_dist( player_type );
     const double mid_dist = calc_mid_dist( player_type );
     const double far_dist = calc_far_dist( player_type );
@@ -590,7 +588,7 @@ KickTable::createStateList( const PlayerType & player_type )
     {
         AngleDeg angle = -180.0 + ( near_angle_step * near );
         Vector2D pos = Vector2D::polar2vector( near_dist, angle );
-        double krate = player_type.kickRate( param, near_dist, angle.degree() );
+        double krate = player_type.kickRate( near_dist, angle.degree() );
         M_state_list.push_back( State( index, near_dist, pos, krate ) );
         ++index;
     }
@@ -599,7 +597,7 @@ KickTable::createStateList( const PlayerType & player_type )
     {
         AngleDeg angle = -180.0 + ( mid_angle_step * mid );
         Vector2D pos = Vector2D::polar2vector( mid_dist, angle );
-        double krate = player_type.kickRate( param, mid_dist, angle.degree() );
+        double krate = player_type.kickRate( mid_dist, angle.degree() );
         M_state_list.push_back( State( index, mid_dist, pos, krate ) );
         ++index;
     }
@@ -608,7 +606,7 @@ KickTable::createStateList( const PlayerType & player_type )
     {
         AngleDeg angle = -180.0 + ( far_angle_step * far );
         Vector2D pos = Vector2D::polar2vector( far_dist, angle );
-        double krate = player_type.kickRate( param, far_dist, angle.degree() );
+        double krate = player_type.kickRate( far_dist, angle.degree() );
         M_state_list.push_back( State( index, far_dist, pos, krate ) );
         ++index;
     }
@@ -710,10 +708,16 @@ KickTable::createStateCache( const WorldModel & world )
 #endif
 
     const ServerParam & param = ServerParam::i();
-    const Rect2D pitch( Vector2D( - param.pitchHalfLength(),
-                                  - param.pitchHalfWidth() ),
-                        Size2D( param.pitchLength(),
-                                param.pitchWidth() ) );
+    const Rect2D pitch
+        = param.keepawayMode()
+        ? Rect2D( Vector2D( - param.keepawayLength() * 0.5 + 0.2,
+                            - param.keepawayWidth() * 0.5 + 0.2 ),
+                  Size2D( param.keepawayLength() - 0.4,
+                          param.keepawayWidth() - 0.4 ) )
+        : Rect2D( Vector2D( - param.pitchHalfLength(),
+                            - param.pitchHalfWidth() ),
+                  Size2D( param.pitchLength(),
+                          param.pitchWidth() ) );
 
     const PlayerType & self_type = world.self().playerType();
     const double near_dist = calc_near_dist( self_type );
@@ -767,7 +771,7 @@ KickTable::createStateCache( const WorldModel & world )
         for ( int near = 0; near < STATE_DIVS_NEAR; ++near )
         {
             Vector2D pos = M_state_list[index].pos_;
-            double krate = self_type.kickRate( param, near_dist, pos.th().degree() );
+            double krate = self_type.kickRate( near_dist, pos.th().degree() );
 
             pos.rotate( world.self().body() );
             pos.setLength( near_dist );
@@ -792,7 +796,7 @@ KickTable::createStateCache( const WorldModel & world )
         for ( int mid = 0; mid < STATE_DIVS_MID; ++mid )
         {
             Vector2D pos = M_state_list[index].pos_;
-            double krate = self_type.kickRate( param, mid_dist, pos.th().degree() );
+            double krate = self_type.kickRate( mid_dist, pos.th().degree() );
 
             pos.rotate( world.self().body() );
             pos.setLength( mid_dist );
@@ -817,7 +821,7 @@ KickTable::createStateCache( const WorldModel & world )
         for ( int far = 0; far < STATE_DIVS_FAR; ++far )
         {
             Vector2D pos = M_state_list[index].pos_;
-            double krate = self_type.kickRate( param, far_dist, pos.th().degree() );
+            double krate = self_type.kickRate( far_dist, pos.th().degree() );
 
             pos.rotate( world.self().body() );
             pos.setLength( far_dist );
@@ -997,7 +1001,7 @@ KickTable::checkInterfereAt( const WorldModel & world,
 
         if ( ! (*o)->isGhost()
              && (*o)->posCount() <= 2
-             && opp_dist < control_area + 0.1 )
+             && opp_dist < control_area + 0.15 )
         {
             flag |= KICKABLE;
 #ifdef DEBUG
@@ -1156,7 +1160,7 @@ KickTable::checkInterfereAfterRelease( const WorldModel & world,
         if ( (*o)->distFromBall() > 10.0 ) break;
 
         Vector2D opp_pos = (*o)->inertiaPoint( cycle );
-        if ( ! opp_pos.valid() )
+        if ( ! opp_pos.isValid() )
         {
             opp_pos = (*o)->pos() + (*o)->vel();
         }
@@ -1188,8 +1192,8 @@ KickTable::checkInterfereAfterRelease( const WorldModel & world,
                                   && penalty_area.contains( ball_pos ) )
                                 ? ServerParam::i().catchableArea()
                                 : (*o)->playerTypePtr()->kickableArea() );
-        control_area += 0.15;
-        double control_area2 = control_area * control_area;
+        control_area += 0.1;
+        double control_area2 = std::pow( control_area, 2 );
 
         if ( ball_pos.dist2( opp_pos ) < control_area2 )
         {
