@@ -55,14 +55,13 @@ namespace rcsc {
 bool
 Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
 {
-    dlog.addText( Logger::ACTION,
-                  __FILE__": Neck_TurnToLowConfTeammate" );
-
     const WorldModel & wm = agent->world();
     const PlayerPtrCont & teammates = wm.teammatesFromSelf();
 
     if ( teammates.empty() )
     {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": no teammate" );
         return Neck_ScanField().execute( agent );
     }
 
@@ -152,7 +151,7 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
     const double neck_step = angle_range / 10.0;
 
     double best_dir = -360.0;
-    int best_score = 0;
+    double best_score = -1.0;
     double best_angle_diff = 0.0;
 
 
@@ -165,7 +164,7 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
             AngleDeg left = target_angle - next_view_half_width;
             AngleDeg right = target_angle + next_view_half_width;
 
-            int score = 0;
+            double score = 0.0;
             double angle_diff = 180.0;
 
             for ( AbstractPlayerCont::const_iterator p = wm.allPlayers().begin();
@@ -181,6 +180,19 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
                 if ( angle.isRightOf( left )
                      && angle.isLeftOf( right ) )
                 {
+#if 1
+
+                    int pos_count = (*p)->seenPosCount();
+                    if ( (*p)->isGhost() ) pos_count = std::min( 2, pos_count );
+
+                    double base_val = ( pos_count + 1 ) * 2.0;
+                    //double rate = std::exp( - std::pow( (*p)->distFromBall(), 2 )
+                    double rate = std::exp( - std::pow( (*p)->distFromSelf(), 2 )
+                                            / ( 2.0 * std::pow( 40.0, 2 ) ) ); // Magic Number: variance = 40.0
+                    score += base_val * rate;
+
+
+#else
                     if ( (*p)->goalie()
                          && next_self_pos.x > 33.0 )
                     {
@@ -194,6 +206,7 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
                     {
                         score += (*p)->posCount();
                     }
+#endif
 
                     double tmp_diff = std::min( ( angle - left ).abs(),
                                                 ( angle - right ).abs() );
@@ -205,7 +218,7 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
             }
 #ifdef DEBUG
             dlog.addText( Logger::ACTION,
-                          "__ dir=%.0f  score=%d  angle_diff=%.1f",
+                          "__ dir=%.0f  score=%.1f angle_diff=%.1f",
                           target_angle.degree(),
                           score,
                           angle_diff );
