@@ -47,6 +47,9 @@
 // #define DEBUG_PRINT
 // #define DEBUG_PRINT_LEVEL_2
 
+// #define DEBUG_PAINT_SUCCESS_COURSE
+// #define DEBUG_PAINT_FAILED_COURSE
+
 namespace rcsc {
 
 /*-------------------------------------------------------------------*/
@@ -67,14 +70,9 @@ ShootTable2008::search( const PlayerAgent * agent )
     }
 
     s_time = wm.time();
+    M_total_count = 0;
     M_shots.clear();
 
-    /////////////////////////////////////////////////////////////////////
-//     static const Rect2D
-//         shootable_area( Vector2D( ServerParam::i().theirPenaltyAreaLineX() - 5.0, // left
-//                                   -ServerParam::i().penaltyAreaHalfWidth() ), // top
-//                         Size2D( ServerParam::i().penaltyAreaLength() + 5.0, // length
-//                                 ServerParam::i().penaltyAreaWidth() ) ); // width
     static const Vector2D goal_c( ServerParam::i().pitchHalfLength(), 0.0 );
 
     if ( ! wm.self().isKickable() )
@@ -113,7 +111,7 @@ ShootTable2008::search( const PlayerAgent * agent )
 
 #ifdef DEBUG_PRINT
     dlog.addText( Logger::SHOOT,
-                  "Shoot search range=(%.1f %.1f)-(%.1f %.1f) dist_step=%.1f",
+                  __FILE__": ===== Shoot search range=(%.1f %.1f)-(%.1f %.1f) dist_step=%.1f =====",
                   goal_l.x, goal_l.y, goal_r.x, goal_r.y, dist_step );
 #endif
 
@@ -125,13 +123,21 @@ ShootTable2008::search( const PlayerAgent * agent )
           i < DIST_DIVS;
           ++i, shot_point.y += dist_step )
     {
+        ++M_total_count;
+#ifdef DEBUG_PRINT
+        dlog.addText( Logger::SHOOT,
+                      "%d: ===== shoot target(%.2f %.2f) ===== ",
+                      M_total_count,
+                      shot_point.x, shot_point.y );
+#endif
         calculateShotPoint( wm, shot_point, goalie );
     }
 
 #ifdef DEBUG_PROFILE
     dlog.addText( Logger::SHOOT,
-                  "Shoot found %d shots. search_shoot_elapsed=%.3f [ms]",
+                  __FILE__": PROFILE %d/%d. elapsed=%.3f [ms]",
                   (int)M_shots.size(),
+                  DIST_DIVS,
                   timer.elapsedReal() );
 #endif
 
@@ -159,6 +165,12 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
          && goalie_count < 30
          && wm.dirCount( shot_angle ) > 3 )
     {
+#ifdef DEBUG_PRINT
+        dlog.addText( Logger::SHOOT,
+                      "%d: __ xxx goalie_count=%d, low dir accuracy",
+                      M_total_count,
+                      goalie_count );
+#endif
         return;
     }
 
@@ -213,10 +225,10 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
             {
                 AngleDeg goalie_angle = ( goalie->pos() - wm.ball().pos() ).th();
                 double angle_diff = ( shot.angle_ - goalie_angle ).abs();
-//                 goalie_rate = 1.0 - std::exp( - std::pow( angle_diff * 0.1, 2 )
-//                                               / ( 2.0 * 90.0 * 0.1 ) );
                 goalie_rate = 1.0 - std::exp( - std::pow( angle_diff * 0.1, 2 )
-                                              / ( 2.0 * 40.0 * 0.1 ) );
+                                              // / ( 2.0 * 90.0 * 0.1 ) );
+                                              // / ( 2.0 * 40.0 * 0.1 ) ); // 2009-07
+                                              / ( 2.0 * 90.0 * 0.1 ) ); // 2009-12-13
                 shot.score_ = static_cast< int >( shot.score_ * goalie_rate );
 #ifdef DEBUG_PRINT
                 dlog.addText( Logger::SHOOT,
@@ -229,10 +241,11 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
 
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "<<< Shoot score=%d pos(%.1f %.1f)"
+                          "%d: ok shoot score=%d pos(%.1f %.1f)"
                           " angle=%.1f speed=%.1f"
                           " y_rate=%.2f g_rate=%.2f"
                           " one_step=%d GK_never_reach=%d",
+                          M_total_count,
                           shot.score_,
                           shot_point.x, shot_point.y,
                           shot_angle.degree(),
@@ -241,20 +254,41 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
                           ( one_step ? 1 : 0 ),
                           ( shot.goalie_never_reach_ ? 1 : 0 ) );
 #endif
-
+#ifdef DEBUG_PAINT_SUCCESS_COURSE
+            dlog.addRect( Logger::SHOOT,
+                          shot_point.x - 0.1, shot_point.y - 0.1,
+                          0.2, 0.2,
+                          "#00ff00" );
+            char num[8];
+            snprintf( num, 8, "%d", M_total_count );
+            dlog.addMessage( Logger::SHOOT,
+                             shot_point, num, "#ffffff" );
+#endif
             M_shots.push_back( shot );
         }
-#ifdef DEBUG_PRINT
         else
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          ">>> Shoot failed to(%.1f %.1f)"
+                          "%d: xxx shoot failed to(%.1f %.1f)"
                           " angle=%.1f speed=%.1f",
+                          M_total_count,
                           shot_point.x, shot_point.y,
                           shot_angle.degree(),
                           shot_first_speed );
-        }
 #endif
+#ifdef DEBUG_PAINT_FAILED_COURSE
+            dlog.addRect( Logger::SHOOT,
+                          shot_point.x - 0.1, shot_point.y - 0.1,
+                          0.2, 0.2,
+                          "#ff0000" );
+            char num[8];
+            snprintf( num, 8, "%d", M_total_count );
+            dlog.addMessage( Logger::SHOOT,
+                             shot_point, num, "#ffffff" );
+#endif
+        }
+
         shot_first_speed += 0.5;
     }
 }
@@ -287,8 +321,9 @@ ShootTable2008::canScore( const WorldModel & wm,
 
 #ifdef DEBUG_PRINT
     dlog.addText( Logger::SHOOT,
-                  "canScore() shot to (%.1f %.1f) vel=(%.2f %.2f)%.3f angle=%.1f"
+                  "%d: (canScore) target=(%.2f %.2f) vel=(%.2f %.2f) r=%.3f angle=%.1f"
                   " ball_reach_step=%.3f",
+                  M_total_count,
                   shot->point_.x, shot->point_.y,
                   shot->vel_.x, shot->vel_.y,
                   shot->speed_,
@@ -331,7 +366,8 @@ ShootTable2008::canScore( const WorldModel & wm,
 
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "__ opp %d(%.1f %.1f)",
+                          "%d: __ opp %d(%.1f %.1f)",
+                          M_total_count,
                           (*it)->unum(), (*it)->pos().x, (*it)->pos().y );
 #endif
             int cycle = predictOpponentReachStep( wm,
@@ -343,7 +379,8 @@ ShootTable2008::canScore( const WorldModel & wm,
                                                   ball_reach_step_i );
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "____ reach step=%d, ball_step=%d",
+                          "%d: ____ reach step=%d, ball_step=%d",
+                          M_total_count,
                           cycle,
                           ball_reach_step_i );
 #endif
@@ -352,7 +389,8 @@ ShootTable2008::canScore( const WorldModel & wm,
             {
 #ifdef DEBUG_PRINT
                 dlog.addText( Logger::SHOOT,
-                              "xx__ opp %d(%.1f %.1f) will get the ball",
+                              "%d: xxx opp %d(%.1f %.1f) will get the ball",
+                              M_total_count,
                               (*it)->unum(),
                               (*it)->pos().x, (*it)->pos().y );
 #endif
@@ -419,8 +457,9 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
         {
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "xx__ goalie can catch the ball. cycle=%d ball_pos(%.1f %.1f)"
+                          "%d: xxx goalie can catch the ball. cycle=%d ball_pos(%.1f %.1f)"
                           " ball_dist=%.3f",
+                          M_total_count,
                           cycle,
                           ball_pos.x, ball_pos.y,
                           ball_dist );
@@ -428,7 +467,8 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
             return true;
         }
 
-        if ( ball_dist < catchable_area + 1.75 ) // 2009-07-02: 1.0 -> 1.75
+        //if ( ball_dist < catchable_area + 1.75 ) // 2009-07-02: 1.0 -> 1.75
+        if ( ball_dist < catchable_area + 1.2 ) // 2009-12-13
         {
             shot->goalie_never_reach_ = false;
         }
@@ -452,8 +492,9 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
 
 #ifdef DEBUG_PRINT
         dlog.addText( Logger::SHOOT,
-                      "__ goalie. cycle=%d ball_pos(%.1f %.1f) g_body=%.0f"
+                      "%d: __ goalie. cycle=%d ball_pos(%.1f %.1f) g_body=%.0f"
                       " b_angle=%.1f angle_diff=%.1f turn_margin=%.1f",
+                      M_total_count,
                       cycle,
                       ball_pos.x, ball_pos.y,
                       goalie_body.degree(),
@@ -492,8 +533,9 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
             double d = goalie_pos.dist( ball_pos ) - seen_dist_noise;
 #ifdef DEBUG_PRINT_LEVEL_2
             dlog.addText( Logger::SHOOT,
-                          "__ goalie. cycle=%d turn=%d dash=%d ball(%.1f %.1f) angle=%.0f"
+                          "%d: __ goalie. cycle=%d turn=%d dash=%d ball(%.1f %.1f) angle=%.0f"
                           " goalie pos(%.1f %.1f) travel=%.1f dist=%.1f turn=%d dash=%d",
+                          M_total_count,
                           cycle, n_turn, i + 1,
                           ball_pos.x, ball_pos.y,
                           ball_angle.degree(),
@@ -516,8 +558,9 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
         {
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "xx__ goalie can reach. move_dist=%.3f ball_dist=%.3f"
+                          "%d: xx goalie can reach. move_dist=%.3f ball_dist=%.3f"
                           " (raw_dist=%.3f seen_noise=%.3f catch_area=%.3f",
+                          M_total_count,
                           goalie->pos().dist( goalie_pos ) * 1.05,
                           goalie->pos().dist( ball_pos ) - seen_dist_noise - catchable_area,
                           goalie->pos().dist( ball_pos ), seen_dist_noise, catchable_area );
@@ -594,7 +637,8 @@ ShootTable2008::predictOpponentReachStep( const WorldModel &,
             double opp_speed = opponent->vel().r();
 #ifdef DEBUG_PRINT_LEVEL_2
             dlog.addText( Logger::SHOOT,
-                          "____ speed=%.3f turn_marign=%.1f angle_diff=%.1f",
+                          "%d: (opponent) speed=%.3f turn_marign=%.1f angle_diff=%.1f",
+                          M_total_count,
                           opp_speed,
                           turn_margin, angle_diff );
 #endif
@@ -613,7 +657,8 @@ ShootTable2008::predictOpponentReachStep( const WorldModel &,
         {
 #ifdef DEBUG_PRINT_LEVEL_2
             dlog.addText( Logger::SHOOT,
-                          "xxxx reachable without dash. ball_dist=%.3f",
+                          "%d: xxx (opponent) reachable without dash. ball_dist=%.3f",
+                          M_total_count,
                           opp_to_ball_dist + control_area + opponent->distFromSelf() * 0.03 );
 #endif
             return cycle;
@@ -628,7 +673,8 @@ ShootTable2008::predictOpponentReachStep( const WorldModel &,
         {
 #ifdef DEBUG_PRINT_LEVEL_2
             dlog.addText( Logger::SHOOT,
-                          "xxxx reachable with dash. cycle=%d turn=%d dash=%d n_buf=%d",
+                          "%d: xxx (opponent) reachable with dash. cycle=%d turn=%d dash=%d n_buf=%d",
+                          M_total_count,
                           cycle,
                           n_turn,
                           n_step - n_turn + bound( 0, opponent->posCount() - 1, 2 ),
