@@ -106,6 +106,9 @@ public:
 
 private:
 
+    //! search count
+    int M_total_count;
+
     //! cached calculated shoot pathes
     ShotCont M_shots;
 
