@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,9 +35,10 @@
 
 #include "view_synch.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/player/see_state.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -48,10 +49,13 @@ namespace rcsc {
 bool
 View_Synch::execute( PlayerAgent * agent )
 {
-#if 0
-    agent->doChangeView( ViewWidth::SYNC, ViewQuality::HIGH );
-    return true;
-#endif
+    if ( SeeState::synch_see_mode() )
+    {
+        dlog.addText( Logger::ACTION,
+                      "%s:%d: change to narrow"
+                      ,__FILE__, __LINE__ );
+        return agent->doChangeView( ViewWidth::NARROW );
+    }
 
     if ( ! agent->seeState().isSynch() )
     {
