@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -154,9 +154,11 @@ template < std::size_t INPUT,
            typename FuncO = SigmoidFunc >
 class BPNetwork1 {
 public:
-    typedef double value_type;
+    typedef double value_type; //!< typedef of the value type
 
+    //! typedef of the input array type that uses template parameter.
     typedef boost::array< value_type, INPUT > input_array;
+    //! typedef of the output array type that uses template parameter.
     typedef boost::array< value_type, OUTPUT > output_array;
 
 private:
@@ -376,7 +378,7 @@ public:
           propagate( input, output );
           for ( std::size_t i = 0; i < OUTPUT; ++i )
           {
-              total_error += std::pow( teacher[i] - output[i], 2.0 );
+              total_error += std::pow( teacher[i] - output[i], 2 );
           }
           //std::cout << "  error = " << total_error << std::endl;
           return total_error;
