@@ -61,10 +61,13 @@ namespace rcsc {
 
 ///////////////////////////////////////////////////////////////////////
 /*!
-  \struct CoachAgentImpl
+  \struct Impl
   \brief coach agent implementation
 */
-struct CoachAgentImpl {
+struct CoachAgent::Impl {
+
+    //! reference to the agent instance
+    CoachAgent & agent_;
 
     //! flag to check if (think) message was received or not.
     bool think_received_;
@@ -90,13 +93,149 @@ struct CoachAgentImpl {
     /*!
       \brief initialize all members
     */
-    CoachAgentImpl()
-        : think_received_( false )
-        , server_cycle_stopped_( true )
-        , last_decision_time_( -1, 0 )
-        , current_time_( 0, 0 )
+    Impl( CoachAgent & agent )
+        : agent_( agent ),
+          think_received_( false ),
+          server_cycle_stopped_( true ),
+          last_decision_time_( -1, 0 ),
+          current_time_( 0, 0 )
       { }
 
+    /*!
+      \brief open offline client log file.
+    */
+    bool openOfflineLog();
+
+    /*!
+      \brief open debug log file.
+    */
+    bool openDebugLog();
+
+    /*!
+      \brief send init or reconnect command to server
+
+      init commad is sent in BasicClient's run() method
+      Do not call this method yourself!
+     */
+    void sendInitCommand();
+
+    /*!
+      \brief send disconnection command message to server
+      and set the server status to end.
+     */
+    void sendByeCommand();
+
+    /*!
+      \brief analyze init replay message
+      \param msg raw server message
+    */
+    void analyzeInit( const char * msg );
+
+    /*!
+      \brief analyze cycle info in server message
+      \param msg raw server message
+      \param by_see_global if message type is see_global, this value becomes true
+      \return parsing result status
+     */
+    bool analyzeCycle( const char * msg,
+                       const bool by_see_global );
+
+    /*!
+      \brief analyze see_global message
+      \param msg raw server message
+     */
+    void analyzeSeeGlobal( const char * msg );
+
+    /*!
+      \brief analyze hear message
+      \param msg raw server message
+     */
+    void analyzeHear( const char * msg );
+
+    /*!
+      \brief analyze referee message
+      \param msg raw server message
+     */
+    void analyzeHearReferee( const char * msg );
+
+    /*!
+      \brief analyze audio message from player
+      \param msg raw server message
+    */
+    void analyzeHearPlayer( const char * msg );
+
+    /*!
+      \brief analyze change_player_type message
+      \param msg raw server message
+    */
+    void analyzeChangePlayerType( const char * msg );
+
+    /*!
+      \brief analyze player_type parameter message
+      \param msg raw server message
+    */
+    void analyzePlayerType( const char * msg );
+
+    /*!
+      \brief analyze player_param parameter message
+      \param msg raw server message
+    */
+    void analyzePlayerParam( const char * msg );
+
+    /*!
+      \brief analyze server_param parameter message
+      \param msg raw server message
+    */
+    void analyzeServerParam( const char * msg );
+
+    /*!
+      \brief analyze clang version message
+      \param msg raw server message
+    */
+    void analyzeCLangVer( const char * msg );
+
+    /*!
+      \brief analyze score message
+      \param msg raw server message
+    */
+    void analyzeScore( const char * msg );
+
+    /*!
+      \brief analyze ok message
+      \param msg raw server message
+    */
+    void analyzeOK( const char * msg );
+
+    /*!
+      \brief analyze ok team_graphic_? message
+      \param msg raw server message
+    */
+    void analyzeOKTeamGraphic( const char * msg );
+
+    /*!
+      \brief analyze ok teamnames message
+      \param msg raw server message
+     */
+    void analyzeTeamNames( const char * msg );
+
+    /*!
+      \brief analyze error message
+      \param msg raw server message
+    */
+    void analyzeError( const char * msg );
+
+    /*!
+      \brief analyze warning message
+      \param msg raw server message
+    */
+    void analyzeWarning( const char * msg );
+
+    /*!
+      \brief analyze include message
+      \param msg raw server message
+    */
+    void analyzeInclude( const char * msg );
+
     /*!
       \brief update current time using analyzed time value
       \param new_time analyzed time value
@@ -119,8 +258,8 @@ struct CoachAgentImpl {
 
 */
 void
-CoachAgentImpl::updateCurrentTime( const long & new_time,
-                                   const bool by_see_global )
+CoachAgent::Impl::updateCurrentTime( const long & new_time,
+                                     const bool by_see_global )
 {
     if ( server_cycle_stopped_ )
     {
@@ -177,7 +316,7 @@ CoachAgentImpl::updateCurrentTime( const long & new_time,
 
 */
 void
-CoachAgentImpl::updateServerStatus()
+CoachAgent::Impl::updateServerStatus()
 {
     // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     //  called just after referee.parse()
@@ -203,7 +342,7 @@ CoachAgentImpl::updateServerStatus()
 */
 CoachAgent::CoachAgent()
     : SoccerAgent()
-    , M_impl( new CoachAgentImpl )
+    , M_impl( new Impl( *this ) )
 {
     boost::shared_ptr< AudioMemory > audio_memory( new AudioMemory );
 
@@ -280,7 +419,7 @@ CoachAgent::initImpl( CmdLineParser & cmd_parser )
     cmd_parser.parse( coach_param_map );
 
     if ( config().version() < 1.0
-         || 14.0 <= config().version() )
+         || 15.0 <= config().version() )
     {
         std::cerr << "Unsupported client version: " << config().version()
                   << std::endl;
@@ -295,15 +434,21 @@ CoachAgent::initImpl( CmdLineParser & cmd_parser )
         dlog.setLogFlag( &world().time(), Logger::ACTION, config().debugAction() );
         dlog.setLogFlag( &world().time(), Logger::INTERCEPT, config().debugIntercept() );
         dlog.setLogFlag( &world().time(), Logger::KICK, config().debugKick() );
+        dlog.setLogFlag( &world().time(), Logger::HOLD, config().debugHold() );
         dlog.setLogFlag( &world().time(), Logger::DRIBBLE, config().debugDribble() );
         dlog.setLogFlag( &world().time(), Logger::PASS, config().debugPass() );
         dlog.setLogFlag( &world().time(), Logger::CROSS, config().debugCross() );
         dlog.setLogFlag( &world().time(), Logger::SHOOT, config().debugShoot() );
         dlog.setLogFlag( &world().time(), Logger::CLEAR, config().debugClear() );
-        dlog.setLogFlag( &world().time(), Logger::TEAM, config().debugTeam() );
+        dlog.setLogFlag( &world().time(), Logger::BLOCK, config().debugBlock() );
+        dlog.setLogFlag( &world().time(), Logger::MARK, config().debugMark() );
+        dlog.setLogFlag( &world().time(), Logger::POSITIONING, config().debugPositioning() );
         dlog.setLogFlag( &world().time(), Logger::ROLE, config().debugRole() );
+        dlog.setLogFlag( &world().time(), Logger::PLAN, config().debugPlan() );
+        dlog.setLogFlag( &world().time(), Logger::TEAM, config().debugTeam() );
+        dlog.setLogFlag( &world().time(), Logger::COMMUNICATION, config().debugCommunication() );
         dlog.setLogFlag( &world().time(), Logger::ANALYZER, config().debugAnalyzer() );
-        dlog.setLogFlag( &world().time(), Logger::RECURSIVE, config().debugRecursive() );
+        dlog.setLogFlag( &world().time(), Logger::ACTION_CHAIN, config().debugActionChain() );
     }
 
     if ( config().offlineClientMode() )
@@ -335,7 +480,7 @@ CoachAgent::handleStart()
         return false;
     }
 
-    // just create a connection. init command is automaticcaly sent
+    // just create a connection. init command is automaticaly sent
     // by BasicClient's run() method.
     if ( ! M_client->connectTo( config().host().c_str(),
                                 config().port(),
@@ -344,17 +489,18 @@ CoachAgent::handleStart()
         std::cerr << config().teamName()
                   << " coach: ***ERROR*** Failed to connect." << std::endl;
         M_client->setServerAlive( false );
+        return false;
     }
 
     if ( config().offlineLogging() )
     {
-        if ( ! openOfflineLog() )
+        if ( ! M_impl->openOfflineLog() )
         {
             return false;
         }
     }
 
-    sendInitCommand();
+    M_impl->sendInitCommand();
     return true;
 }
 
@@ -370,7 +516,7 @@ CoachAgent::handleStartOffline()
         return false;
     }
 
-    if ( ! openOfflineLog() )
+    if ( ! M_impl->openOfflineLog() )
     {
         return false;
     }
@@ -558,7 +704,7 @@ CoachAgent::finalize()
 {
     if ( M_client->isServerAlive() )
     {
-        sendByeCommand();
+        M_impl->sendByeCommand();
     }
     std::cout << config().teamName() << " coach: finished."
               << std::endl;
@@ -569,9 +715,9 @@ CoachAgent::finalize()
 
  */
 bool
-CoachAgent::openOfflineLog()
+CoachAgent::Impl::openOfflineLog()
 {
-    std::string filepath = config().logDir();
+    std::string filepath = agent_.config().logDir();
 
     if ( ! filepath.empty() )
     {
@@ -581,16 +727,16 @@ CoachAgent::openOfflineLog()
         }
     }
 
-    filepath += config().teamName();
+    filepath += agent_.config().teamName();
     filepath += "-coach";
-    filepath += config().offlineLogExt();
+    filepath += agent_.config().offlineLogExt();
 
-    if ( ! M_client->openOfflineLog( filepath ) )
+    if ( ! agent_.M_client->openOfflineLog( filepath ) )
     {
         std::cerr << "Failed to open the offline client log file ["
                   << filepath
                   << "]" << std::endl;
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
         return false;
     }
 
@@ -602,9 +748,9 @@ CoachAgent::openOfflineLog()
 
  */
 bool
-CoachAgent::openDebugLog()
+CoachAgent::Impl::openDebugLog()
 {
-    std::string filepath = config().logDir();
+    std::string filepath = agent_.config().logDir();
 
     if ( ! filepath.empty() )
     {
@@ -614,18 +760,18 @@ CoachAgent::openDebugLog()
         }
     }
 
-    filepath += config().teamName();
+    filepath += agent_.config().teamName();
     filepath += "-coach";
-    filepath += config().debugLogExt();
+    filepath += agent_.config().debugLogExt();
 
     dlog.open( filepath );
 
     if ( ! dlog.isOpen() )
     {
-        std::cerr << config().teamName() << " coach: "
+        std::cerr << agent_.config().teamName() << " coach: "
                   << " Failed to open the debug log file [" << filepath << "]"
                   << std::endl;
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
         return false;
     }
 
@@ -668,17 +814,17 @@ CoachAgent::parse( const char * msg )
 {
     if ( ! std::strncmp( msg, "(see_global ", 12 ) )
     {
-        analyzeSeeGlobal( msg );
+        M_impl->analyzeSeeGlobal( msg );
     }
 #if 0
     else if ( ! std::strncmp( msg, "(see ", 5 ) )
     {
-        analyzeSeeGlobal( msg );
+        M_impl->analyzeSeeGlobal( msg );
     }
 #endif
     else if ( ! std::strncmp( msg, "(hear ", 6 ) )
     {
-        analyzeHear( msg );
+        M_impl->analyzeHear( msg );
     }
     else if ( ! std::strncmp( msg, "(think)", 7 ) )
     {
@@ -686,55 +832,55 @@ CoachAgent::parse( const char * msg )
     }
     else if ( ! std::strncmp( msg, "(change_player_type ", 20 ) )
     {
-        analyzeChangePlayerType( msg );
+        M_impl->analyzeChangePlayerType( msg );
     }
     else if ( ! std::strncmp( msg, "(clang ", 7 ) )
     {
-        analyzeCLangVer( msg );
+        M_impl->analyzeCLangVer( msg );
     }
     else if ( ! std::strncmp( msg, "(player_type ", 13 ) )  // hetero param
     {
-        analyzePlayerType( msg );
+        M_impl->analyzePlayerType( msg );
     }
     else if ( ! std::strncmp( msg, "(player_param ", 14 ) ) // tradeoff param
     {
-        analyzePlayerParam( msg );
+        M_impl->analyzePlayerParam( msg );
     }
     else if ( ! std::strncmp( msg, "(server_param ", 14 ) )
     {
-        analyzeServerParam( msg );
+        M_impl->analyzeServerParam( msg );
     }
     else if ( ! std::strncmp( msg, "(ok ", 4 ) )
     {
-        analyzeOK( msg );
+        M_impl->analyzeOK( msg );
     }
     else if ( ! std::strncmp( msg, "(error ", 7 ) )
     {
-        analyzeError( msg );
+        M_impl->analyzeError( msg );
     }
     else if ( ! std::strncmp( msg, "(warning ", 9 ) )
     {
-        analyzeWarning( msg );
+        M_impl->analyzeWarning( msg );
     }
     else if ( ! std::strncmp( msg, "(score ", 7 ) )
     {
-        analyzeScore( msg );
+        M_impl->analyzeScore( msg );
     }
     else if ( ! std::strncmp( msg, "(init ", 6 ) )
     {
-        analyzeInit( msg );
+        M_impl->analyzeInit( msg );
     }
     else if ( ! std::strncmp( msg, "(include ", 9 ) )
     {
-        analyzeInclude( msg );
+        M_impl->analyzeInclude( msg );
     }
     else
     {
         std::cerr << config().teamName()
                   << " coach: "
                   << world().time()
-                  << " received unsupported Message : ["
-                  << msg << "]" << std::endl;
+                  << " received unsupported Message : [" << msg << "]"
+                  << std::endl;
     }
 }
 
@@ -743,26 +889,28 @@ CoachAgent::parse( const char * msg )
 
 */
 void
-CoachAgent::analyzeInit( const char * msg )
+CoachAgent::Impl::analyzeInit( const char * msg )
 {
     // "(init l ok)" | "(init r ok)"
 
+    const CoachConfig & c = agent_.config();
+
     char side;
 
     if ( std::sscanf( msg, "(init %c ok)", &side ) != 1 )
     {
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
         return;
     }
 
     if ( side != 'l' && side != 'r' )
     {
-        std::cerr << config().teamName()
+        std::cerr << c.teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " received unexpected init message. " << msg
                   << std::endl;
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
         return;
     }
 
@@ -770,9 +918,9 @@ CoachAgent::analyzeInit( const char * msg )
     SideID side_id = ( side == 'l'
                        ? LEFT
                        : RIGHT );
-    M_worldmodel.init( side_id, config().version() );
+    agent_.M_worldmodel.init( side_id, c.version() );
 
-    if ( config().debug() )
+    if ( c.debug() )
     {
         openDebugLog();
     }
@@ -780,22 +928,22 @@ CoachAgent::analyzeInit( const char * msg )
     // send specific settings
     // if we intend to advise to the team,
     // set visual sensory
-    if ( config().useEye() )
+    if ( c.useEye() )
     {
-        doEye( true );
+        agent_.doEye( true );
     }
 
-    if ( config().hearSay() )
+    if ( c.hearSay() )
     {
-        M_impl->audio_.setTeamName( config().teamName() );
+        audio_.setTeamName( c.teamName() );
     }
 
     // set compression level
-    if ( 0 < config().compression()
-         && config().compression() <= 9 )
+    if ( 0 < c.compression()
+         && c.compression() <= 9 )
     {
-        CoachCompressionCommand com( config().compression() );
-        sendCommand( com );
+        CoachCompressionCommand com( c.compression() );
+        agent_.sendCommand( com );
     }
 }
 
@@ -804,7 +952,7 @@ CoachAgent::analyzeInit( const char * msg )
 
 */
 bool
-CoachAgent::analyzeCycle( const char * msg,
+CoachAgent::Impl::analyzeCycle( const char * msg,
                           const bool by_see_global )
 {
     char id[16];
@@ -812,16 +960,16 @@ CoachAgent::analyzeCycle( const char * msg,
     if ( std::sscanf( msg, "(%15s %ld ",
                       id, &cycle ) != 2 )
     {
-        std::cerr << config().teamName()
+        std::cerr << agent_.config().teamName()
                   << " coach:"
-                  << world().time()
+                  << agent_.world().time()
                   << " ***ERROR*** failed to parse time msg=["
                   << msg << "]"
                   << std::endl;
         return false;
     }
 
-    M_impl->updateCurrentTime( cycle, by_see_global );
+    updateCurrentTime( cycle, by_see_global );
     return true;
 }
 
@@ -830,7 +978,7 @@ CoachAgent::analyzeCycle( const char * msg,
 
 */
 void
-CoachAgent::analyzeSeeGlobal( const char * msg )
+CoachAgent::Impl::analyzeSeeGlobal( const char * msg )
 {
     if ( ! analyzeCycle( msg, true ) )
     {
@@ -838,15 +986,15 @@ CoachAgent::analyzeSeeGlobal( const char * msg )
     }
 
     // analyze message
-    M_impl->visual_.parse( msg,
-                           config().version(),
-                           M_impl->current_time_ );
+    visual_.parse( msg,
+                   agent_.config().version(),
+                   current_time_ );
 
     // update world model
-    if ( M_impl->visual_.time() == M_impl->current_time_ )
+    if ( visual_.time() == current_time_ )
     {
-        M_worldmodel.updateAfterSeeGlobal( M_impl->visual_,
-                                           M_impl->current_time_ );
+        agent_.M_worldmodel.updateAfterSeeGlobal( visual_,
+                                                  current_time_ );
     }
 }
 
@@ -855,7 +1003,7 @@ CoachAgent::analyzeSeeGlobal( const char * msg )
 
 */
 void
-CoachAgent::analyzeHear( const char * msg )
+CoachAgent::Impl::analyzeHear( const char * msg )
 {
     if ( ! analyzeCycle( msg, false ) )
     {
@@ -868,9 +1016,9 @@ CoachAgent::analyzeHear( const char * msg )
     if ( std::sscanf( msg, "(hear %ld %127s ",
                       &cycle, sender ) != 2 )
     {
-        std::cerr << config().teamName()
+        std::cerr << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " Error. failed to parse audio sender. ["
                   << msg
                   << std::endl;
@@ -895,38 +1043,92 @@ CoachAgent::analyzeHear( const char * msg )
 
 */
 void
-CoachAgent::analyzeHearReferee( const char * msg )
+CoachAgent::Impl::analyzeHearReferee( const char * msg )
 {
+    const CoachConfig & c = agent_.config();
+    const GlobalWorldModel & wm = agent_.world();
+
     long cycle;
     char mode[512];
 
     if ( std::sscanf( msg, "(hear %ld referee %511[^)]", &cycle, mode ) != 2 )
     {
-        std::cerr << config().teamName()
+        std::cerr << c.teamName()
                   << " coach: "
-                  << world().time()
+                  << wm.time()
                   << " ***ERROR*** Failed to scan playmode." << msg
                   << std::endl;
         return;
     }
 
-    if ( ! M_impl->game_mode_.update( mode, M_impl->current_time_ ) )
+    if ( ! game_mode_.update( mode, current_time_ ) )
     {
-        //std::cerr << config().teamName()
-        //          << " coach: Unknown playmode string." << mode
-        //          << std::endl;
+        if ( ! std::strncmp( mode, "yellow_card", std::strlen( "yellow_card" ) ) )
+        {
+            char side = '?';
+            int unum = Unum_Unknown;
+            if ( std::sscanf( mode, "yellow_card_%c_%d", &side, &unum ) != 2 )
+            {
+                std::cerr << c.teamName()
+                          << " coach: "
+                          << wm.time()
+                          << " could not parse the yellow card message [" << msg << ']'
+                          << std::endl;
+            }
+
+            agent_.M_worldmodel.setYellowCard( ( side == 'l'
+                                                 ? LEFT
+                                                 : side == 'r'
+                                                 ? RIGHT
+                                                 : NEUTRAL ),
+                                               unum );
+        }
+        else if ( ! std::strncmp( mode, "red_card", std::strlen( "red_card" ) ) )
+        {
+            char side = '?';
+            int unum = Unum_Unknown;
+            if ( std::sscanf( mode, "red_card_%c_%d", &side, &unum ) != 2 )
+            {
+                std::cerr << c.teamName()
+                          << " coach: "
+                          << wm.time()
+                          << " could not parse the red card message [" << msg << ']'
+                          << std::endl;
+            }
+
+            agent_.M_worldmodel.setRedCard( ( side == 'l'
+                                              ? LEFT
+                                              : side == 'r'
+                                              ? RIGHT
+                                              : NEUTRAL ),
+                                            unum );
+        }
+        else if ( ! std::strncmp( mode, "training", std::strlen( "training" ) ) )
+        {
+            // end keepaway (or some training) episode
+            agent_.M_worldmodel.setTrainingTime( current_time_ );
+            return;
+        }
+        else
+        {
+            std::cerr << c.teamName() << " coach: "
+                      << wm.time()
+                      << " Unknown playmode string." << mode
+                      << std::endl;
+        }
+
         return;
     }
 
-    M_impl->updateServerStatus();
+    updateServerStatus();
 
-    if ( M_impl->game_mode_.isGameEndMode() )
+    if ( game_mode_.isGameEndMode() )
     {
         sendByeCommand();
         return;
     }
 
-    M_worldmodel.updatePlayMode( M_impl->game_mode_, M_impl->current_time_ );
+    agent_.M_worldmodel.updateGameMode( game_mode_, current_time_ );
 }
 
 /*-------------------------------------------------------------------*/
@@ -934,14 +1136,14 @@ CoachAgent::analyzeHearReferee( const char * msg )
 
 */
 void
-CoachAgent::analyzeHearPlayer( const char * msg )
+CoachAgent::Impl::analyzeHearPlayer( const char * msg )
 {
     // (hear <time> (player "<teamname>" <unum>) "<message>")
     // (hear <time> (p "<teamname>" <unum>) "<message>")
 
-    if ( config().hearSay() )
+    if ( agent_.config().hearSay() )
     {
-        M_impl->audio_.parsePlayerMessage( msg, M_impl->current_time_ );
+        audio_.parsePlayerMessage( msg, current_time_ );
     }
 }
 
@@ -950,7 +1152,7 @@ CoachAgent::analyzeHearPlayer( const char * msg )
 
 */
 void
-CoachAgent::analyzeChangePlayerType( const char * msg )
+CoachAgent::Impl::analyzeChangePlayerType( const char * msg )
 {
     // teammate: "(change_player_type <unum> <type>)\n"
     //           "(ok change_player_type <unum> <type>)\n"
@@ -967,17 +1169,17 @@ CoachAgent::analyzeChangePlayerType( const char * msg )
                            &unum, &type ) == 2 )
     {
         // teammate
-        M_worldmodel.setPlayerType( world().ourSide(),
-                                    unum,
-                                    type );
+        agent_.M_worldmodel.setPlayerType( agent_.world().ourSide(),
+                                           unum,
+                                           type );
     }
     else if ( std::sscanf( msg, " ( change_player_type %d ) ",
                            &unum ) == 1 )
     {
         // opponent
-        M_worldmodel.setPlayerType( world().theirSide(),
-                                    unum,
-                                    Hetero_Unknown );
+        agent_.M_worldmodel.setPlayerType( agent_.world().theirSide(),
+                                           unum,
+                                           Hetero_Unknown );
     }
     else
     {
@@ -992,10 +1194,9 @@ CoachAgent::analyzeChangePlayerType( const char * msg )
 
 */
 void
-CoachAgent::analyzePlayerType( const char * msg )
+CoachAgent::Impl::analyzePlayerType( const char * msg )
 {
-    PlayerType player_type( ServerParam::i(),
-                            msg, config().version() );
+    PlayerType player_type( msg, agent_.config().version() );
     PlayerTypeSet::instance().insert( player_type );
 }
 
@@ -1004,9 +1205,9 @@ CoachAgent::analyzePlayerType( const char * msg )
 
 */
 void
-CoachAgent::analyzePlayerParam( const char * msg )
+CoachAgent::Impl::analyzePlayerParam( const char * msg )
 {
-    PlayerParam::instance().parse( msg, config().version() );
+    PlayerParam::instance().parse( msg, agent_.config().version() );
     //PlayerParam::i().print( std::cout );
 }
 
@@ -1015,19 +1216,19 @@ CoachAgent::analyzePlayerParam( const char * msg )
 
 */
 void
-CoachAgent::analyzeServerParam( const char * msg )
+CoachAgent::Impl::analyzeServerParam( const char * msg )
 {
-    ServerParam::instance().parse( msg, config().version() );
-    PlayerTypeSet::instance().resetDefaultType( ServerParam::i() );
+    ServerParam::instance().parse( msg, agent_.config().version() );
+    PlayerTypeSet::instance().resetDefaultType();
 
-    M_worldmodel.initFreeformCount();
+    agent_.M_worldmodel.initFreeformCount();
 
     if ( ! ServerParam::i().synchMode()
          && ServerParam::i().slowDownFactor() > 1 )
     {
-        long interval = ( config().intervalMSec()
+        long interval = ( agent_.config().intervalMSec()
                           * ServerParam::i().slowDownFactor() );
-        M_client->setIntervalMSec( interval );
+        agent_.M_client->setIntervalMSec( interval );
     }
 }
 
@@ -1036,7 +1237,7 @@ CoachAgent::analyzeServerParam( const char * msg )
 
 */
 void
-CoachAgent::analyzeCLangVer( const char * )
+CoachAgent::Impl::analyzeCLangVer( const char * )
 {
     //     std::cerr << config().teamName()
     //               << " coach: "
@@ -1049,7 +1250,7 @@ CoachAgent::analyzeCLangVer( const char * )
 
 */
 void
-CoachAgent::analyzeOK( const char * msg )
+CoachAgent::Impl::analyzeOK( const char * msg )
 {
     //std::cout << M_impl->current_time_
     //<< " Coach received ok " << std::endl;
@@ -1067,17 +1268,17 @@ CoachAgent::analyzeOK( const char * msg )
     else if ( ! std::strncmp( msg, "(ok look ",
                               std::strlen( "(ok look " ) ) )
     {
-        std::cout << config().teamName()
+        std::cout << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << "recv (ok look ..." << std::endl;
     }
     else if ( ! std::strncmp( msg, "(ok check_ball ",
                               std::strlen( "(ok check_ball " ) ) )
     {
-        std::cout << config().teamName()
+        std::cout << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " recv (ok check_ball ..." << std::endl;;
     }
     else if ( ! std::strncmp( msg, "(ok change_player_type ",
@@ -1091,34 +1292,34 @@ CoachAgent::analyzeOK( const char * msg )
         int level = 0;
         if ( std::sscanf( msg, "(ok compression %d)", &level ) == 1 )
         {
-            std::cout << config().teamName()
+            std::cout << agent_.config().teamName()
                       << " coach: "
-                      << world().time()
+                      << agent_.world().time()
                       << " set compression level " << level<< std::endl;
-            M_client->setCompressionLevel( level );
+            agent_.M_client->setCompressionLevel( level );
         }
     }
     else if ( ! std::strncmp( msg, "(ok eye ", std::strlen( "(ok eye " ) ) )
     {
-        std::cout << config().teamName()
+        std::cout << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " recv " << msg << std::endl;
     }
     else if ( ! std::strncmp( msg, "(ok team_names ",
                               std::strlen( "(ok team_names " ) ) )
     {
-        std::cout << config().teamName()
+        std::cout << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " recv " << msg << std::endl;
         analyzeTeamNames( msg );
     }
     else
     {
-        std::cout << config().teamName()
+        std::cout << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " recv " << msg << std::endl;
     }
 }
@@ -1128,7 +1329,7 @@ CoachAgent::analyzeOK( const char * msg )
 
 */
 void
-CoachAgent::analyzeOKTeamGraphic( const char * msg )
+CoachAgent::Impl::analyzeOKTeamGraphic( const char * msg )
 {
     // "(ok team_graphic <x> <y>)"
     // "(ok team_graphic <x> <y>)"
@@ -1141,15 +1342,15 @@ CoachAgent::analyzeOKTeamGraphic( const char * msg )
          || x < 0
          || y < 0 )
     {
-        std::cout << config().teamName()
+        std::cout << agent_.config().teamName()
                   << " coach: "
-                  << world().time()
+                  << agent_.world().time()
                   << " recv illegal message. " << msg
                   << std::endl;
         return;
     }
 
-    M_team_graphic_ok_set.insert( TeamGraphic::Index( x, y ) );
+    agent_.M_team_graphic_ok_set.insert( TeamGraphic::Index( x, y ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1157,7 +1358,7 @@ CoachAgent::analyzeOKTeamGraphic( const char * msg )
 
 */
 void
-CoachAgent::analyzeTeamNames( const char * msg )
+CoachAgent::Impl::analyzeTeamNames( const char * msg )
 {
     // "(ok team_names (team l <name>)[ (team r <name>)])"
 
@@ -1168,12 +1369,12 @@ CoachAgent::analyzeTeamNames( const char * msg )
                          left, right );
     if ( n == 2 )
     {
-        M_worldmodel.setTeamName( LEFT, left );
-        M_worldmodel.setTeamName( RIGHT, right );
+        agent_.M_worldmodel.setTeamName( LEFT, left );
+        agent_.M_worldmodel.setTeamName( RIGHT, right );
     }
     else if ( n == 1 )
     {
-        M_worldmodel.setTeamName( LEFT, left );
+        agent_.M_worldmodel.setTeamName( LEFT, left );
     }
 }
 
@@ -1183,11 +1384,11 @@ CoachAgent::analyzeTeamNames( const char * msg )
 
 */
 void
-CoachAgent::analyzeScore( const char * msg )
+CoachAgent::Impl::analyzeScore( const char * msg )
 {
-    std::cerr << config().teamName()
+    std::cerr << agent_.config().teamName()
               << " coach: "
-              << world().time()
+              << agent_.world().time()
               << " recv " << msg << std::endl;
 }
 
@@ -1196,11 +1397,11 @@ CoachAgent::analyzeScore( const char * msg )
 
 */
 void
-CoachAgent::analyzeError( const char * msg )
+CoachAgent::Impl::analyzeError( const char * msg )
 {
-    std::cerr << config().teamName()
+    std::cerr << agent_.config().teamName()
               << " coach: "
-              << world().time()
+              << agent_.world().time()
               << " recv " << msg << std::endl;
 }
 
@@ -1209,11 +1410,11 @@ CoachAgent::analyzeError( const char * msg )
 
 */
 void
-CoachAgent::analyzeWarning( const char * msg )
+CoachAgent::Impl::analyzeWarning( const char * msg )
 {
-    std::cerr << config().teamName()
+    std::cerr << agent_.config().teamName()
               << " coach: "
-              << world().time()
+              << agent_.world().time()
               << " recv " << msg
               << std::endl;
 }
@@ -1223,11 +1424,11 @@ CoachAgent::analyzeWarning( const char * msg )
 
 */
 void
-CoachAgent::analyzeInclude( const char * msg )
+CoachAgent::Impl::analyzeInclude( const char * msg )
 {
-    std::cerr << config().teamName()
+    std::cerr << agent_.config().teamName()
               << " coach: "
-              << world().time()
+              << agent_.world().time()
               << " recv " << msg << std::endl;
 }
 
@@ -1255,38 +1456,38 @@ CoachAgent::sendCommand( const CoachCommand & com )
 
 */
 void
-CoachAgent::sendInitCommand()
+CoachAgent::Impl::sendInitCommand()
 {
-    if ( ! M_client->isServerAlive() )
+    if ( ! agent_.M_client->isServerAlive() )
     {
-        std::cerr << config().teamName()
+        std::cerr << agent_.config().teamName()
                   << " coach: server is not alive" << std::endl;
         return;
     }
 
     // make command string
     bool success = true;
-    if ( config().useCoachName()
-         && ! config().coachName().empty() )
+    if ( agent_.config().useCoachName()
+         && ! agent_.config().coachName().empty() )
     {
-        CoachInitCommand com( config().teamName(),
-                              config().version(),
-                              config().coachName() );
-        success = sendCommand( com );
+        CoachInitCommand com( agent_.config().teamName(),
+                              agent_.config().version(),
+                              agent_.config().coachName() );
+        success = agent_.sendCommand( com );
     }
     else
     {
-        CoachInitCommand com( config().teamName(),
-                              config().version() );
-        success = sendCommand( com );
+        CoachInitCommand com( agent_.config().teamName(),
+                              agent_.config().version() );
+        success = agent_.sendCommand( com );
     }
 
     if ( ! success )
     {
-        std::cerr << config().teamName()
+        std::cerr << agent_.config().teamName()
                   << " coach: Failed to init coach...\nExit ..."
                   << std::endl;
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
     }
 }
 
@@ -1295,11 +1496,11 @@ CoachAgent::sendInitCommand()
 
 */
 void
-CoachAgent::sendByeCommand()
+CoachAgent::Impl::sendByeCommand()
 {
     CoachByeCommand com;
-    sendCommand( com );
-    M_client->setServerAlive( false );
+    agent_.sendCommand( com );
+    agent_.M_client->setServerAlive( false );
 }
 
 /*-------------------------------------------------------------------*/
