@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +35,8 @@
 
 #include "coach_command.h"
 
+#include <rcsc/common/player_param.h>
+
 namespace rcsc {
 
 /*-------------------------------------------------------------------*/
@@ -130,8 +132,8 @@ CoachEyeCommand::toStr( std::ostream & to ) const
 std::ostream &
 CoachChangePlayerTypeCommand::toStr( std::ostream & to ) const
 {
-    if ( M_type == Hetero_Unknown
-         || M_type == Hetero_MAX )
+    if ( M_type < 0
+         || PlayerParam::i().playerTypes() <= M_type )
     {
         std::cerr << "CoachChangePlayerTypeCommand: illegal type Id"
                   << M_type
@@ -147,6 +149,110 @@ CoachChangePlayerTypeCommand::toStr( std::ostream & to ) const
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+CoachChangePlayerTypesCommand::CoachChangePlayerTypesCommand( const int unum,
+                                                              const int type )
+{
+    add( unum, type );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+CoachChangePlayerTypesCommand::
+CoachChangePlayerTypesCommand( const std::vector< std::pair< int, int > > & types )
+{
+    M_types.reserve( types.size() );
+
+    for ( std::vector< std::pair< int, int > >::const_iterator it = types.begin();
+          it != types.end();
+          ++it )
+    {
+        add( it->first, it->second );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+CoachChangePlayerTypesCommand::add( const int unum,
+                                    const int type )
+{
+    if ( unum < 1
+         || 11 < unum )
+    {
+        std::cerr << "CoachChangePlayerTypesCommand::add() Illegal player number "
+                  << unum
+                  << std::endl;
+        return;
+    }
+
+    if ( type < 0
+         || PlayerParam::i().playerTypes() <= type )
+    {
+        std::cerr << "CoachChangePlayerTypesCommand::add() Illegal player type id "
+                  << type
+                  << std::endl;
+        return;
+    }
+
+    for ( std::vector< std::pair< int, int > >::iterator it = M_types.begin();
+          it != M_types.end();
+          ++it )
+    {
+        if ( it->first == unum )
+        {
+            std::cerr << "CoachChangePlayerTypesCommand::add() unum "
+                      << unum << " is already registered. overwritten."
+                      << std::endl;
+            it->second = type;
+            return;
+        }
+    }
+
+    M_types.push_back( std::pair< int, int >( unum, type ) );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+std::ostream &
+CoachChangePlayerTypesCommand::toStr( std::ostream & to ) const
+{
+    if ( M_types.empty() )
+    {
+        std::cerr << "CoachChangePlayerTypesCommand::toStr()  Empty data!"
+                  << std::endl;
+        return to;
+    }
+
+    to << "(change_player_types ";
+
+    for ( std::vector< std::pair< int, int > >::const_iterator it = M_types.begin();
+          it != M_types.end();
+          ++it )
+    {
+        if ( it->first < 1
+             || 11 < it->first
+             || it->second < 0
+             || PlayerParam::i().playerTypes() <= it->second )
+        {
+            continue;
+        }
+
+        to << '(' << it->first << ' ' << it->second << ')';
+    }
+
+    return to << ")";
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 std::ostream &
 CoachSayCommand::toStr( std::ostream & to ) const
