@@ -67,7 +67,7 @@ CoachConfig::setDefaultParam()
 {
     // basic setting
     M_team_name = "HELIOS_base";
-    M_version = 13;
+    M_version = 14;
 
     M_coach_name = "Coach_base";
     M_use_coach_name = false;
@@ -115,21 +115,28 @@ CoachConfig::setDefaultParam()
     //
 
     M_debug_log_ext = ".log";
+
     M_debug_system = false;
     M_debug_sensor = false;
     M_debug_world = false;
     M_debug_action = false;
     M_debug_intercept = false;
     M_debug_kick = false;
+    M_debug_hold = false;
     M_debug_dribble = false;
     M_debug_pass = false;
     M_debug_cross = false;
     M_debug_shoot = false;
     M_debug_clear = false;
-    M_debug_team = false;
+    M_debug_block = false;
+    M_debug_mark = false;
+    M_debug_positioning = false;
     M_debug_role = false;
+    M_debug_plan = false;
+    M_debug_team = false;
+    M_debug_communication = false;
     M_debug_analyzer = false;
-    M_debug_recursive = false;
+    M_debug_action_chain = false;
 }
 
 /*-------------------------------------------------------------------*/
@@ -176,21 +183,28 @@ CoachConfig::createParamMap( ParamMap & param_map )
         ( "offline_client_mode", "", BoolSwitch( &M_offline_client_mode ) )
 
         ( "debug_log_ext", "", &M_debug_log_ext )
+
         ( "debug_system", "", BoolSwitch( &M_debug_system ) )
         ( "debug_sensor", "", BoolSwitch( &M_debug_sensor ) )
         ( "debug_world", "", BoolSwitch( &M_debug_world ) )
         ( "debug_action", "", BoolSwitch( &M_debug_action ) )
         ( "debug_intercept", "", BoolSwitch( &M_debug_intercept ) )
         ( "debug_kick", "", BoolSwitch( &M_debug_kick ) )
+        ( "debug_hold", "", BoolSwitch( &M_debug_hold ) )
         ( "debug_dribble", "", BoolSwitch( &M_debug_dribble ) )
         ( "debug_pass", "", BoolSwitch( &M_debug_pass ) )
         ( "debug_cross", "", BoolSwitch( &M_debug_cross ) )
         ( "debug_shoot", "", BoolSwitch( &M_debug_shoot ) )
         ( "debug_clear", "", BoolSwitch( &M_debug_clear ) )
-        ( "debug_team", "", BoolSwitch( &M_debug_team ) )
+        ( "debug_block", "", BoolSwitch( &M_debug_block ) )
+        ( "debug_mark", "", BoolSwitch( &M_debug_mark ) )
+        ( "debug_positioning", "", BoolSwitch( &M_debug_positioning ) )
         ( "debug_role", "", BoolSwitch( &M_debug_role ) )
+        ( "debug_plan", "", BoolSwitch( &M_debug_plan ) )
+        ( "debug_team", "", BoolSwitch( &M_debug_team ) )
+        ( "debug_communication", "", BoolSwitch( &M_debug_communication ) )
         ( "debug_analyzer", "", BoolSwitch( &M_debug_analyzer ) )
-        ( "debug_recursive", "", BoolSwitch( &M_debug_recursive ) )
+        ( "debug_action_chain", "", BoolSwitch( &M_debug_action_chain ) )
         ;
 }
 
