@@ -131,6 +131,8 @@ private:
     bool M_kicked; //!< true if player performed the kick.
     int M_tackle_cycle; //!< if player is tackling, this value is incremented
 
+    int M_charged_cycle; //!< if player is charged, this value is incremented
+    Card M_card;//!< player's card status
 public:
 
     /*!
@@ -149,6 +151,8 @@ public:
         , M_pointto_angle( 0.0 )
         , M_kicked( false )
         , M_tackle_cycle( 0 )
+        , M_charged_cycle( 0 )
+        , M_card( NO_CARD )
       { }
 
     /*!
@@ -286,6 +290,24 @@ public:
           return ( M_tackle_cycle > 0 );
       }
 
+    /*!
+      \brief get player's charged status
+      \return the accumulated number of charged cycle
+     */
+    int chargedCycle() const
+      {
+          return M_charged_cycle;
+      }
+
+    /*!
+      \brief check if player is charged or not
+      \return true if player is charged
+     */
+    bool isCharged() const
+      {
+          return ( M_charged_cycle > 0 );
+      }
+
     //////////////////////////////////////////////////////////////
 
     /*!
@@ -377,6 +399,19 @@ public:
           M_tackle_cycle = 1;
       }
 
+    void setCharged()
+      {
+          M_charged_cycle = 1;
+      }
+
+    /*!
+      \brief set card status
+     */
+    void setCard( const Card card )
+      {
+          M_card = card;
+      }
+
     /*!
       \brief update with seen data
       \param p seen data
