@@ -102,11 +102,14 @@ GlobalVisualSensor::parseV7( const char * msg )
     /*
     sample version >= 7.0
     (see_global TIME ((g l) -52.5 0) ((g r) 52.5 0) ((b) <x> <y> <vx> <vy>)
-       ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ {t|k}])
+       ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ {t|k}][ {y|r}])
        ....)
         <-- arm is global
         <-- 't' means tackle
         <-- 'k' means kick
+        <-- 'f' means foul charged
+        <-- 'y' means yellow card
+        <-- 'r' means red card
     (ok look TIME ((g l) -52.5 0) ((g r) 52.5 0) ((b) <x> <y> <vx> <vy>)
        ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>) <-- no arm & tackle
        ....)
@@ -165,7 +168,7 @@ GlobalVisualSensor::parseV7( const char * msg )
     std::string team_name_left;
     std::string team_name_right;
 
-    // ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ t])
+    // ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ {k|t}][ {y|r}])
     while ( *msg != '\0' )
     {
         while ( *msg != '\0' && *msg != '(' ) ++msg;
@@ -261,21 +264,40 @@ GlobalVisualSensor::parseV7( const char * msg )
 
         while ( *msg != '\0' && *msg != ')' )
         {
-            while ( *msg != '\0' && *msg == ' ' ) ++msg;
-            if ( *msg == '\0' ) break;
+            while ( *msg == ' ' ) ++msg;
+            if ( *msg == ')' ) break;
 
-            if ( *msg == 'k' )
+            if ( *msg == 'k' ) // kick
             {
                 player.setKick( true );
             }
-            else if ( *msg == 't' )
+            else if ( *msg == 't' ) // tackle
             {
                 player.setTackle();
             }
-            else
+            else if ( *msg == 'f' ) // foul charged
+            {
+                player.setCharged();
+            }
+            else if ( *msg == 'y' ) // yellow card
+            {
+                player.setCard( YELLOW );
+            }
+            else if ( *msg == 'r' ) // red card
+            {
+                player.setCard( RED );
+            }
+            else // point_dir
             {
                 char *next;
-                player.setArm( std::strtod( msg, &next ) );
+                double point_dir = std::strtod( msg, &next );
+                if ( point_dir == HUGE_VAL
+                     || point_dir == -HUGE_VAL
+                     || msg == next )
+                {
+                    break;
+                }
+                player.setArm( point_dir );
                 msg = next;
             }
 
