@@ -53,6 +53,7 @@ GlobalWorldModel::GlobalWorldModel()
       M_time( -1, 0 ),
       M_see_time( -1, 0 ),
       M_our_side( NEUTRAL ),
+      M_training_time( -1, 0 ),
       M_player_type_analyzer( *this ),
       M_change_count_left( 0 ),
       M_change_count_right( 0 ),
@@ -213,14 +214,89 @@ GlobalWorldModel::setPlayerType( const SideID side,
 
 }
 
+
 /*-------------------------------------------------------------------*/
 /*!
 
 */
 void
-GlobalWorldModel::updatePlayMode( const GameMode & game_mode,
+GlobalWorldModel::setYellowCard( const SideID side,
+                                 const int unum )
+{
+    if ( unum < 1 || 11 < unum )
+    {
+        std::cerr << ourTeamName() << " coach:"
+                  << " ***ERROR*** (GlobalWorldModel::setYellowCard) "
+                  << " Illegal uniform number"
+                  << unum << std::endl;
+        return;
+    }
+
+    if ( side == LEFT )
+    {
+        M_yellow_card_left[unum - 1] = true;
+    }
+    else if ( side == RIGHT )
+    {
+        M_yellow_card_right[unum - 1] = true;
+    }
+    else
+    {
+        std::cerr << ourTeamName() << " coach:"
+                  << " ***ERROR*** (GlobalWorldModel::setYellowCard) "
+                  << " Illegal side" << std::endl;
+        return;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+GlobalWorldModel::setRedCard( const SideID side,
+                              const int unum )
+{
+    if ( unum < 1 || 11 < unum )
+    {
+        std::cerr << ourTeamName() << " coach:"
+                  << " ***ERROR*** (GlobalWorldModel::setRedCard) "
+                  << " Illegal uniform number"
+                  << unum << std::endl;
+        return;
+    }
+
+    if ( side == LEFT )
+    {
+        M_red_card_left[unum - 1] = true;
+    }
+    else if ( side == RIGHT )
+    {
+        M_red_card_right[unum - 1] = true;
+    }
+    else
+    {
+        std::cerr << ourTeamName() << " coach:"
+                  << " ***ERROR*** (GlobalWorldModel::setRedCard) "
+                  << " Illegal side" << std::endl;
+        return;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+GlobalWorldModel::updateGameMode( const GameMode & game_mode,
                                   const GameTime & current )
 {
+    if ( M_game_mode.type() != GameMode::PlayOn
+         && game_mode.type() == GameMode::PlayOn )
+    {
+        M_last_playon_start = current.cycle();
+    }
+
     M_time = current;
     M_game_mode = game_mode;
 
@@ -228,23 +304,31 @@ GlobalWorldModel::updatePlayMode( const GameMode & game_mode,
     {
         M_change_count_left = 0;
         M_change_count_right = 0;
+    }
 
+    //
+    // update freeform capacity
+    //
+    if ( game_mode.type() != GameMode::BeforeKickOff
+         && game_mode.type() != GameMode::TimeOver
+         && game_mode.type() != GameMode::AfterGoal_
+         && game_mode.type() != GameMode::OffSide_
+         && game_mode.type() != GameMode::BackPass_
+         && game_mode.type() != GameMode::FreeKickFault_
+         && game_mode.type() != GameMode::CatchFault_ )
+    {
         if ( current.cycle() > 0
-             && ( current.cycle() % ( ServerParam::i().nrNormalHalfs()
-                                      * ServerParam::i().halfTime() )
-                  == 0 )
-             )
+             && ServerParam::i().nrNormalHalfs() > 0
+             && current.cycle() % ( ServerParam::i().nrNormalHalfs()
+                                    * ServerParam::i().halfTime() ) == 0 )
         {
-            std::cerr << "coach: " << current
-                      << " add new freeform allowed"
+            std::cerr << ourTeamName() << " coach: "
+                      << current
+                      << " new freeform allowed"
                       << std::endl;
             M_freeform_allowed_count += ServerParam::i().coachSayCountMax();
         }
     }
-    else if ( game_mode.type() == GameMode::PlayOn )
-    {
-        M_last_playon_start = current.cycle();
-    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -439,6 +523,64 @@ GlobalWorldModel::heteroID( const SideID side,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+bool
+GlobalWorldModel::isYellowCarded( const SideID side,
+                                  const int unum ) const
+{
+    if ( side == NEUTRAL )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__
+                  << " ***ERROR*** Invalid side = " << side
+                  << std::endl;
+        return false;
+    }
+
+    if ( unum < 1 || 11 < unum )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__
+                  << " ***ERROR*** Invalid unum = " << unum
+                  << std::endl;
+        return false;
+    }
+
+    return ( side == LEFT
+             ? M_yellow_card_left[unum - 1]
+             : M_yellow_card_right[unum - 1] );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+GlobalWorldModel::isRedCarded( const SideID side,
+                               const int unum ) const
+{
+    if ( side == NEUTRAL )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__
+                  << " ***ERROR*** Invalid side = " << side
+                  << std::endl;
+        return false;
+    }
+
+    if ( unum < 1 || 11 < unum )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__
+                  << " ***ERROR*** Invalid unum = " << unum
+                  << std::endl;
+        return false;
+    }
+
+    return ( side == LEFT
+             ? M_red_card_left[unum - 1]
+             : M_red_card_right[unum - 1] );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 BallStatus
 GlobalWorldModel::getBallStatus() const
