@@ -129,6 +129,8 @@ PlayerTypeAnalyzer::update()
             M_opponent_data[i].invalid_flags_.resize( max_types, 0 );
         }
 
+#if 0
+        // heterogeneous goalie is available in v14 or later
         if ( ! PlayerParam::i().allowMultDefaultType()
              && PlayerParam::i().ptMax() == 1
              && max_types > 0 )
@@ -137,15 +139,19 @@ PlayerTypeAnalyzer::update()
             M_teammate_data[i].invalid_flags_[0] = 1;
             M_opponent_data[i].invalid_flags_[0] = 1;
         }
+#endif
     }
 
     if ( M_opponent_type_used_count.size() != max_types )
     {
         M_opponent_type_used_count.resize( max_types, 0 );
+#if 0
+        // heterogeneous goalie is available in v14 or later
         if ( max_types > 0 )
         {
             M_opponent_type_used_count[0] = 1; // goalie is always default type.
         }
+#endif
     }
 
     if ( M_updated_time.cycle() != M_world.time().cycle() - 1
@@ -298,12 +304,15 @@ PlayerTypeAnalyzer::analyze()
 
         Data & data = M_opponent_data[(*p)->unum() - 1];
 
-        // goalie is always the default type.
+#if 0
+        // heterogeneous goalie is available in v14 or later
+        // goalie is always the default type in v13 or previous.
         if ( (*p)->goalie() )
         {
             data.type_ = Hetero_Default;
             continue;
         }
+#endif
 
         // if player type is not changed, not need to analyze
         if ( data.type_ == Hetero_Default ) continue;
@@ -838,7 +847,7 @@ PlayerTypeAnalyzer::checkKick()
         {
             if ( ! M_teammate_data[i].turned_
                  && ! M_teammate_data[i].tackling_
-                 && M_teammate_data[i].pos_.valid() )
+                 && M_teammate_data[i].pos_.isValid() )
             {
                 if ( M_prev_ball.pos().dist2( M_teammate_data[i].pos_ )
                      < S_max_kickable_area2 )
@@ -850,7 +859,7 @@ PlayerTypeAnalyzer::checkKick()
 
             if ( ! M_opponent_data[i].turned_
                  && ! M_opponent_data[i].tackling_
-                 && M_opponent_data[i].pos_.valid() )
+                 && M_opponent_data[i].pos_.isValid() )
             {
                 if ( M_prev_ball.pos().dist2( M_opponent_data[i].pos_ )
                      < S_max_kickable_area2 )
@@ -969,7 +978,7 @@ PlayerTypeAnalyzer::checkPlayerDecay()
         {
             continue;
         }
-        if ( ! data.pos_.valid() ) continue;
+        if ( ! data.pos_.isValid() ) continue;
         if ( data.pos_.dist2( (*p)->pos() ) < 0.0001 ) continue;
 
         double rand_max = data.vel_.r() * ServerParam::i().playerRand();
@@ -1050,7 +1059,7 @@ PlayerTypeAnalyzer::checkPlayerSpeedMax()
         if ( data.maybe_referee_ ) continue;
         if ( data.tackling_ ) continue;
         if ( data.maybe_collide_ ) continue;
-        if ( ! data.pos_.valid() ) continue;
+        if ( ! data.pos_.isValid() ) continue;
 
         const Vector2D last_move( (*p)->pos().x - data.pos_.x,
                                   (*p)->pos().y - data.pos_.y );
