@@ -45,7 +45,22 @@ namespace rcsc {
 
 */
 AudioMemory::AudioMemory()
-    : M_ball_time( -1, 0 )
+    : M_time( -1, 0 ),
+      M_ball_time( -1, 0 ),
+      M_pass_time( -1, 0 ),
+      M_our_intercept_time( -1, 0 ),
+      M_opp_intercept_time( -1, 0 ),
+      M_goalie_time( -1, 0 ),
+      M_player_time( -1, 0 ),
+      M_offside_line_time( -1, 0 ),
+      M_defense_line_time( -1, 0 ),
+      M_wait_request_time( -1, 0 ),
+      M_pass_request_time( -1, 0 ),
+      M_run_request_time( -1, 0 ),
+      M_stamina_time( -1, 0 ),
+      M_recovery_time( -1, 0 ),
+      M_dribble_time( -1, 0 ),
+      M_free_message_time( -1, 0 )
 {
 
 }
@@ -208,6 +223,12 @@ AudioMemory::setPlayer( const int sender,
     M_player_time = current;
 
     M_time = current;
+
+    M_player_record.push_back( std::pair< GameTime, Player >( current, M_player.back() ) );
+    while ( M_player_record.size() > 30 )
+    {
+        M_player_record.pop_front();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -236,6 +257,13 @@ AudioMemory::setPlayer( const int sender,
     M_player_time = current;
 
     M_time = current;
+
+
+    M_player_record.push_back( std::pair< GameTime, Player >( current, M_player.back() ) );
+    while ( M_player_record.size() > 30 )
+    {
+        M_player_record.pop_front();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -369,7 +397,7 @@ AudioMemory::setStamina( const int sender,
                          const GameTime & current )
 {
     dlog.addText( Logger::WORLD,
-                  __FILE__": set heard stamina. sender=d rate=%.3f",
+                  __FILE__": set heard stamina. sender=%d rate=%.3f",
                   sender, rate );
 
     if ( M_stamina_time != current )
