@@ -38,6 +38,7 @@
 #include <iostream>
 #include <string>
 #include <vector>
+#include <deque>
 
 namespace rcsc {
 
@@ -345,6 +346,10 @@ public:
           { }
     };
 
+
+    //! alias of player record container type
+    typedef std::deque< std::pair< GameTime, Player > > PlayerRecord;
+
 protected:
 
     //! last updated time
@@ -395,6 +400,11 @@ protected:
     std::vector< FreeMessage > M_free_message; //!< heard info
     GameTime M_free_message_time; //!< info heard time
 
+
+    //! memory of heared players
+    PlayerRecord M_player_record;
+
+
 private:
     // not used
     AudioMemory( const AudioMemory & );
@@ -542,6 +552,12 @@ public:
           return M_player_time;
       }
 
+    /*!
+      \brief get player record container
+      \return PlayerRecord
+     */
+    const PlayerRecord & playerRecord() const { return M_player_record; }
+
     /*!
       \brief get heard offside line info
       \return offside line info container
