@@ -132,10 +132,13 @@ public:
     static const boost::int32_t CLEAR     = LEVEL_12; //!< log level definition alias
     static const boost::int32_t BLOCK     = LEVEL_13; //!< log level definition alias
     static const boost::int32_t MARK      = LEVEL_14; //!< log level definition alias
-    static const boost::int32_t ROLE      = LEVEL_15; //!< log level definition alias
-    static const boost::int32_t TEAM      = LEVEL_16; //!< log level definition alias
-    static const boost::int32_t ANALYZER  = LEVEL_17; //!< log level definition alias
-    static const boost::int32_t RECURSIVE = LEVEL_18; //!< log level definition alias
+    static const boost::int32_t POSITIONING = LEVEL_15; //!< log level definition alias
+    static const boost::int32_t ROLE      = LEVEL_16; //!< log level definition alias
+    static const boost::int32_t PLAN      = LEVEL_17; //!< log level definition alias
+    static const boost::int32_t TEAM      = LEVEL_18; //!< log level definition alias
+    static const boost::int32_t COMMUNICATION = LEVEL_19; //!< log level definition alias
+    static const boost::int32_t ANALYZER  = LEVEL_20; //!< log level definition alias
+    static const boost::int32_t ACTION_CHAIN = LEVEL_21; //!< log level definition alias
 
 private:
 
@@ -186,7 +189,7 @@ public:
     void open( const std::string & filepath );
 
     /*!
-      \brief check if file is opend
+      \brief check if file is opened
       \return true if file is opened
      */
     bool isOpen()
@@ -342,9 +345,6 @@ public:
       \param start_angle start angle
       \param span_angle span of the arc. direction is clockwise from start_angle
       \param color color name string
-      \param r green value
-      \param g green value
-      \param b blue value
      */
     void addArc( const boost::int32_t level,
                  const double & x,
@@ -362,9 +362,6 @@ public:
       \param start_angle start angle
       \param span_angle span of the arc. direction is clockwise from start_angle
       \param color color name string
-      \param r green value
-      \param g green value
-      \param b blue value
      */
     void addArc( const boost::int32_t level,
                  const Vector2D & center,
@@ -384,7 +381,7 @@ public:
       \param radius radius value of the circle
       \param start_angle start angle
       \param span_angle span of the arc. direction is clockwise from start_angle
-      \param r green value
+      \param r red value
       \param g green value
       \param b blue value
      */
@@ -403,7 +400,7 @@ public:
       \param radius radius value of the circle
       \param start_angle start angle
       \param span_angle span of the arc. direction is clockwise from start_angle
-      \param r green value
+      \param r red value
       \param g green value
       \param b blue value
      */
@@ -796,7 +793,7 @@ public:
       \param max_radius min radius value of the circle
       \param start_angle start angle
       \param span_angle span of the arc. direction is clockwise from start_angle
-      \param r green value
+      \param r red value
       \param g green value
       \param b blue value
       \param fill switch to select a sector is filled or not
@@ -819,7 +816,7 @@ public:
       \param max_radius min radius value of the circle
       \param start_angle start angle
       \param span_angle span of the arc. direction is clockwise from start_angle
-      \param r green value
+      \param r red value
       \param g green value
       \param b blue value
       \param fill switch to select a sector is filled or not
@@ -856,7 +853,7 @@ public:
       \brief add arc info to the buffer. message tag 'a'
       \param level log level value
       \param sector sector object
-      \param r green value
+      \param r red value
       \param g green value
       \param b blue value
       \param fill switch to select a sector is filled or not
