@@ -89,6 +89,14 @@ const double PlayerParam::DEFAULT_NEW_DASH_POWER_RATE_DELTA_MIN = 0.0;
 const double PlayerParam::DEFAULT_NEW_DASH_POWER_RATE_DELTA_MAX = 0.002;
 const double PlayerParam::DEFAULT_NEW_STAMINA_INC_MAX_DELTA_FACTOR = -10000.0;
 
+// v14
+const double PlayerParam::DEFAULT_KICK_POWER_RATE_DELTA_MIN = 0.0;
+const double PlayerParam::DEFAULT_KICK_POWER_RATE_DELTA_MAX = 0.0;
+const double PlayerParam::DEFAULT_FOUL_DETECT_PROBABILITY_DELTA_FACTOR = 0.0;
+
+const double PlayerParam::DEFAULT_CATCHABLE_AREA_L_STRETCH_MIN = 1.0;
+const double PlayerParam::DEFAULT_CATCHABLE_AREA_L_STRETCH_MAX = 1.3;
+
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -158,6 +166,13 @@ PlayerParam::setDefaultParam()
     M_new_stamina_inc_max_delta_factor = DEFAULT_NEW_STAMINA_INC_MAX_DELTA_FACTOR;
 
     M_random_seed = DEFAULT_RANDOM_SEED;
+
+    M_kick_power_rate_delta_min = DEFAULT_KICK_POWER_RATE_DELTA_MIN;
+    M_kick_power_rate_delta_max = DEFAULT_KICK_POWER_RATE_DELTA_MAX;
+    M_foul_detect_probability_delta_factor = DEFAULT_FOUL_DETECT_PROBABILITY_DELTA_FACTOR;
+
+    M_catchable_area_l_stretch_min = DEFAULT_CATCHABLE_AREA_L_STRETCH_MIN;
+    M_catchable_area_l_stretch_max = DEFAULT_CATCHABLE_AREA_L_STRETCH_MAX;
 }
 
 /*-------------------------------------------------------------------*/
@@ -191,7 +206,13 @@ PlayerParam::createMap()
         ( "random_seed", "", &M_random_seed )
         ( "new_dash_power_rate_delta_min", "", &M_new_dash_power_rate_delta_min )
         ( "new_dash_power_rate_delta_max", "", &M_new_dash_power_rate_delta_max )
-        ( "new_stamina_inc_max_delta_factor", "", &M_new_stamina_inc_max_delta_factor );
+        ( "new_stamina_inc_max_delta_factor", "", &M_new_stamina_inc_max_delta_factor )
+        ( "kick_power_rate_delta_min", "", &M_kick_power_rate_delta_min )
+        ( "kick_power_rate_delta_max", "", &M_kick_power_rate_delta_max )
+        ( "foul_detect_probability_delta_factor", "", &M_foul_detect_probability_delta_factor )
+        ( "catchable_area_l_stretch_min", "", &M_catchable_area_l_stretch_min )
+        ( "catchable_area_l_stretch_max", "", &M_catchable_area_l_stretch_max )
+        ;
 }
 
 /*-------------------------------------------------------------------*/
@@ -329,25 +350,14 @@ PlayerParam::convertFrom( const rcg::player_params_t & from )
     M_new_dash_power_rate_delta_max = rcg::nltohd( from.new_dash_power_rate_delta_max );
     M_new_stamina_inc_max_delta_factor = rcg::nltohd( from.new_stamina_inc_max_delta_factor );
 
-    /*
-      from.sparelong5;
-      from.sparelong6;
-      from.sparelong7;
-      from.sparelong8;
-      from.sparelong9;
-      from.sparelong10;
+    M_allow_mult_default_type = rcg::nstohb( from.allow_mult_default_type );
 
-      short spareshort1;
-      short spareshort2;
-      short spareshort3;
-      short spareshort4;
-      short spareshort5;
-      short spareshort6;
-      short spareshort7;
-      short spareshort8;
-      short spareshort9;
-      short spareshort10;
-    */
+    M_kick_power_rate_delta_min = rcg::nltohd( from.kick_power_rate_delta_min );
+    M_kick_power_rate_delta_max = rcg::nltohd( from.kick_power_rate_delta_max );
+    M_foul_detect_probability_delta_factor = rcg::nltohd( from.foul_detect_probability_delta_factor );
+
+    M_catchable_area_l_stretch_min = rcg::nltohd( from.catchable_area_l_stretch_min );
+    M_catchable_area_l_stretch_max = rcg::nltohd( from.catchable_area_l_stretch_max );
 }
 
 /*-------------------------------------------------------------------*/
@@ -390,24 +400,12 @@ PlayerParam::convertTo( rcg::player_params_t & to ) const
 
     to.allow_mult_default_type = rcg::hbtons( M_allow_mult_default_type );
 
-    /*
-      sparelong5;
-      sparelong6;
-      sparelong7;
-      sparelong8;
-      sparelong9;
-      sparelong10;
+    to.kick_power_rate_delta_min = rcg::hdtonl( M_kick_power_rate_delta_min );
+    to.kick_power_rate_delta_max = rcg::hdtonl( M_kick_power_rate_delta_max );
+    to.foul_detect_probability_delta_factor = rcg::hdtonl( M_foul_detect_probability_delta_factor );
 
-      short spareshort2;
-      short spareshort3;
-      short spareshort4;
-      short spareshort5;
-      short spareshort6;
-      short spareshort7;
-      short spareshort8;
-      short spareshort9;
-      short spareshort10;
-    */
+    to.catchable_area_l_stretch_min = rcg::hdtonl( M_catchable_area_l_stretch_min );
+    to.catchable_area_l_stretch_max = rcg::hdtonl( M_catchable_area_l_stretch_max );
 }
 
 /*-------------------------------------------------------------------*/
