@@ -86,6 +86,13 @@ private:
     static const double DEFAULT_NEW_DASH_POWER_RATE_DELTA_MAX; //!< default tradeoff parameter. dash power rate
     static const double DEFAULT_NEW_STAMINA_INC_MAX_DELTA_FACTOR; //!< default tradeoff parameter. stamina inc max
 
+    static const double DEFAULT_KICK_POWER_RATE_DELTA_MIN; //!< default tradeoff parameter. kick power rate
+    static const double DEFAULT_KICK_POWER_RATE_DELTA_MAX; //!< default tradeoff parameter. kick power rate
+    static const double DEFAULT_FOUL_DETECT_PROBABILITY_DELTA_FACTOR; //!< default tradeoff parameter. foul detect probability
+
+    static const double DEFAULT_CATCHABLE_AREA_L_STRETCH_MIN; //!< default tradeoff parameter. catch area length stretch
+    static const double DEFAULT_CATCHABLE_AREA_L_STRETCH_MAX; //!< default tradeoff parameter. catch area length stretch
+
     int M_player_types; //!< the number of player types
     int M_subs_max; //!< max number of player substitution oppotunity
     int M_pt_max; //!< max number of same player type at the same time
@@ -117,7 +124,15 @@ private:
 
     double M_new_dash_power_rate_delta_min; //!< tradeoff parameter for dash power rate
     double M_new_dash_power_rate_delta_max; //!< tradeoff parameter for dash power rate
-    double M_new_stamina_inc_max_delta_factor; //!< tradeoff parameter for
+    double M_new_stamina_inc_max_delta_factor; //!< tradeoff parameter for stamina inc max
+
+    // v14
+    double M_kick_power_rate_delta_min; //!< tradeoff parameter for kick power rate
+    double M_kick_power_rate_delta_max; //!< tradeoff parameter for kick power rate
+    double M_foul_detect_probability_delta_factor; //!< tradeoff parameter for foul detect probability
+
+    double M_catchable_area_l_stretch_min; //!< tradeoff parameter for catch area length & catch probability
+    double M_catchable_area_l_stretch_max; //!< tradeoff parameter for catch area length & catch probability
 
     /*!
       \brief default constructor.
@@ -429,6 +444,37 @@ public:
       {
           return M_new_stamina_inc_max_delta_factor;
       }
+
+    /*!
+      \brief get the player_param parameter
+      \return player_param parameter
+     */
+    const
+    double & kickPowerRateDeltaMin() const
+      {
+          return M_kick_power_rate_delta_min;
+      }
+
+    /*!
+      \brief get the player_param parameter
+      \return player_param parameter
+     */
+    const
+    double & kickPowerRateDeltaMax() const
+      {
+          return M_kick_power_rate_delta_max;
+      }
+
+    /*!
+      \brief get the player_param parameter
+      \return player_param parameter
+     */
+    const
+    double & foulDetectProbabilityDeltaFactor() const
+      {
+          return M_foul_detect_probability_delta_factor;
+      }
+
 };
 
 }
