@@ -8,12 +8,12 @@
 /*
  *Copyright:
 
- Copyright (C) Hidehisa AKIYAMA
+ Copyright (C) Hidehisa AKIYAMA, Hiroki SHIMORA
 
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,16 +34,20 @@
 #endif
 
 #include "player_type.h"
+
 #include "player_param.h"
 #include "server_param.h"
+#include "stamina_model.h"
 
 #include <rcsc/rcg/util.h>
 
 #include <sstream>
+#include <iostream>
 #include <algorithm>
 #include <string>
 #include <cstdio>
 #include <cstring>
+#include <cmath>
 
 namespace rcsc {
 
@@ -51,91 +55,44 @@ namespace rcsc {
 /*!
 
 */
-PlayerType::PlayerType( const ServerParam & sparam,
-                        const int id,
-                        const double & pspdmax,
-                        const double & sincmax,
-                        const double & pdecay,
-                        const double & imoment,
-                        const double & dprate,
-                        const double & psize,
-                        const double & kmargin,
-                        const double & krand,
-                        const double & estamina,
-                        const double & effmax,
-                        const double & effmin )
-    : M_id( id )
-    , M_player_speed_max( pspdmax )
-    , M_stamina_inc_max( sincmax )
-    , M_player_decay( pdecay )
-    , M_inertia_moment( imoment )
-    , M_dash_power_rate( dprate )
-    , M_player_size( psize )
-    , M_kickable_margin( kmargin )
-    , M_kick_rand( krand )
-    , M_extra_stamina( estamina )
-    , M_effort_max( effmax )
-    , M_effort_min( effmin )
-    , M_cycles_to_reach_max_speed( 6 )
-{
-    if ( id < Hetero_Unknown
-         || PlayerParam::i().playerTypes() <= id )
-    {
-        std::cerr << "PlayerType. Invalid player type id " << id
-                  << std::endl;
-    }
-
-    initAdditionalParams( sparam );
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-PlayerType::PlayerType( const ServerParam & sparam )
+PlayerType::PlayerType()
     : M_id( Hetero_Default )
-    , M_player_speed_max( sparam.defaultPlayerSpeedMax() )
-    , M_stamina_inc_max( sparam.defaultStaminaIncMax() )
-    , M_player_decay( sparam.defaultPlayerDecay() )
-    , M_inertia_moment( sparam.defaultInertiaMoment() )
-    , M_dash_power_rate( sparam.defaultDashPowerRate() )
-    , M_player_size( sparam.defaultPlayerSize() )
-    , M_kickable_margin( sparam.defaultKickableMargin() )
-    , M_kick_rand( sparam.defaultKickRand() )
-    , M_extra_stamina( sparam.defaultExtraStamina() )
-    , M_effort_max( sparam.defaultEffortMax() )
-    , M_effort_min( sparam.defaultEffortMin() )
 {
-    initAdditionalParams( sparam );
+    setDefault();
+    initAdditionalParams();
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
 */
-PlayerType::PlayerType( const ServerParam & sparam,
-                        const char * msg,
+PlayerType::PlayerType( const char * server_msg,
                         const double & version )
+    : M_id( Hetero_Unknown )
 {
+    setDefault();
+
     if ( version >= 8.0 )
     {
-        parseV8( msg );
+        parseV8( server_msg );
     }
     else
     {
-        parseV7( msg );
+        parseV7( server_msg );
     }
 
-    initAdditionalParams( sparam );
+    initAdditionalParams();
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
 */
-PlayerType::PlayerType( const ServerParam & sparam,
-                        const rcg::player_type_t & from )
+PlayerType::PlayerType( const rcg::player_type_t & from )
+    : M_id( Hetero_Unknown )
 {
+    setDefault();
+
     M_id = rcg::nstohi( from.id );
     M_player_speed_max = rcg::nltohd( from.player_speed_max );
     M_stamina_inc_max = rcg::nltohd( from.stamina_inc_max );
@@ -149,20 +106,11 @@ PlayerType::PlayerType( const ServerParam & sparam,
     M_effort_max = rcg::nltohd( from.effort_max );
     M_effort_min = rcg::nltohd( from.effort_min );
 
-    /*
-      long sparelong1;
-      long sparelong2;
-      long sparelong3;
-      long sparelong4;
-      long sparelong5;
-      long sparelong6;
-      long sparelong7;
-      long sparelong8;
-      long sparelong9;
-      long sparelong10;
-    */
+    if ( from.kick_power_rate != 0 ) M_kick_power_rate = rcg::nltohd( from.kick_power_rate );
+    if ( from.foul_detect_probability != 0 ) M_foul_detect_probability = rcg::nltohd( from.foul_detect_probability );
+    if ( from.catchable_area_l_stretch != 0 ) M_catchable_area_l_stretch = rcg::nltohd( from.catchable_area_l_stretch );
 
-    initAdditionalParams( sparam );
+    initAdditionalParams();
 }
 
 /*-------------------------------------------------------------------*/
@@ -185,18 +133,9 @@ PlayerType::convertTo( rcg::player_type_t & to ) const
     to.effort_max = rcg::hdtonl( M_effort_max );
     to.effort_min = rcg::hdtonl( M_effort_min );
 
-    /*
-      long sparelong1;
-      long sparelong2;
-      long sparelong3;
-      long sparelong4;
-      long sparelong5;
-      long sparelong6;
-      long sparelong7;
-      long sparelong8;
-      long sparelong9;
-      long sparelong10;
-    */
+    to.kick_power_rate = rcg::hdtonl( M_kick_power_rate );
+    to.foul_detect_probability = rcg::hdtonl( M_foul_detect_probability );
+    to.catchable_area_l_stretch = rcg::hdtonl( M_catchable_area_l_stretch );
 }
 
 /*-------------------------------------------------------------------*/
@@ -221,6 +160,9 @@ PlayerType::toStr() const
        << "(extra_stamina " << M_extra_stamina << ')'
        << "(effort_max " << M_effort_max << ')'
        << "(effort_min " << M_effort_min << ')'
+       << "(kick_power_rate " << M_kick_power_rate << ')'
+       << "(foul_detect_probability " << M_foul_detect_probability << ')'
+       << "(catchable_area_l_stretch " << M_catchable_area_l_stretch << ')'
        << ')';
 
     return os.str();
@@ -229,6 +171,31 @@ PlayerType::toStr() const
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+PlayerType::setDefault()
+{
+    const ServerParam & SP = ServerParam::instance();
+
+    M_player_speed_max = SP.defaultPlayerSpeedMax();
+    M_stamina_inc_max = SP.defaultStaminaIncMax();
+    M_player_decay = SP.defaultPlayerDecay();
+    M_inertia_moment = SP.defaultInertiaMoment();
+    M_dash_power_rate = SP.defaultDashPowerRate();
+    M_player_size = SP.defaultPlayerSize();
+    M_kickable_margin = SP.defaultKickableMargin();
+    M_kick_rand = SP.defaultKickRand();
+    M_extra_stamina = SP.defaultExtraStamina();
+    M_effort_max = SP.defaultEffortMax();
+    M_effort_min = SP.defaultEffortMin();
+    M_kick_power_rate = SP.kickPowerRate();
+    M_foul_detect_probability = SP.foulDetectProbability();
+    M_catchable_area_l_stretch = 1.0;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 PlayerType::parseV8( const char * msg )
@@ -323,6 +290,18 @@ PlayerType::parseV8( const char * msg )
         {
             M_effort_min = val;
         }
+        else if ( ! std::strcmp( name, "kick_power_rate" ) )
+        {
+            M_kick_power_rate = val;
+        }
+        else if ( ! std::strcmp( name, "foul_detect_probability" ) )
+        {
+            M_foul_detect_probability = val;
+        }
+        else if ( ! std::strcmp( name, "catchable_area_l_stretch" ) )
+        {
+            M_catchable_area_l_stretch = val;
+        }
         else
         {
             std::cerr << "player_type: param name error " << msg << std::endl;
@@ -388,16 +367,29 @@ PlayerType::parseV7( const char * msg )
 
 */
 void
-PlayerType::initAdditionalParams( const ServerParam & sparam )
+PlayerType::initAdditionalParams()
 {
-    M_kickable_area = playerSize() + kickableMargin() + sparam.ballSize();
+    const ServerParam & SP = ServerParam::i();
+
+    M_kickable_area = playerSize() + kickableMargin() + SP.ballSize();
+
+    /////////////////////////////////////////////////////////////////////
+    const double catch_stretch_length_x = ( catchAreaLengthStretch() - 1.0 ) * SP.catchAreaLength();
+    const double catch_length_min_x = SP.catchAreaLength() - catch_stretch_length_x;
+    const double catch_length_max_x = SP.catchAreaLength() + catch_stretch_length_x;
+
+    const double catch_half_width2 = std::pow( SP.catchAreaWidth() / 2.0, 2 );
+
+    M_reliable_catchable_dist = std::sqrt( std::pow( catch_length_min_x, 2 )
+                                           + catch_half_width2 );
+    M_max_catchable_dist = std::sqrt( std::pow( catch_length_max_x, 2 )
+                                      + catch_half_width2 );
 
     ///////////////////////////////////////////////////////////////////
-
-    double accel = sparam.maxDashPower() * dashPowerRate() * effortMax();
+    double accel = SP.maxDashPower() * dashPowerRate() * effortMax();
 
     // see also soccer_math.h
-    M_real_speed_max = accel / (1.0 - playerDecay()); // sum inf geom series
+    M_real_speed_max = accel / ( 1.0 - playerDecay() ); // sum inf geom series
     if ( M_real_speed_max > playerSpeedMax() )
     {
         M_real_speed_max = playerSpeedMax();
@@ -409,10 +401,9 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
 
     /////////////////////////////////////////////////////////////////////
     double speed = 0.0;
-    double dash_power = sparam.maxDashPower();
-    double stamina = sparam.staminaMax();
-    double effort = this->effortMax();
-    double recover = sparam.recoverInit();
+    double dash_power = SP.maxDashPower();
+    StaminaModel stamina_model;
+    stamina_model.init( *this );
 
     double reach_dist = 0.0;
 
@@ -426,8 +417,8 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
         if ( speed + accel > playerSpeedMax() )
         {
             accel = playerSpeedMax() - speed;
-            dash_power = std::min( sparam.maxDashPower(),
-                                   accel / ( dashPowerRate() * effort ) );
+            dash_power = std::min( SP.maxDashPower(),
+                                   accel / ( dashPowerRate() * stamina_model.effort() ) );
         }
 
         speed += accel;
@@ -444,32 +435,120 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
 
         speed *= playerDecay();
 
-        predictStaminaAfterOneDash( sparam,
-                                    dash_power,
-                                    &stamina, &effort, &recover );
+        stamina_model.simulateDash( *this, dash_power );
 
-        //M_stamina_table.push_back( sparam.staminaMax() - stamina );
-
-        if ( stamina == 0.0 )
+        if ( stamina_model.stamina() <= 0.0 )
         {
             break;
         }
     }
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+double
+PlayerType::reliableCatchableDist() const
+{
+    if ( ServerParam::i().catchProbability() < 1.0 )
+    {
+        return 0.0;
+    }
+    else
+    {
+        return M_reliable_catchable_dist;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+double
+PlayerType::reliableCatchableDist( const double prob ) const
+{
+    if ( prob > 1.0 )
+    {
+        std::cerr << "internal error at PlayerType::reliableCatchableDist(): "
+                  << "probability " << prob << " too big" << std::endl;
+
+        return reliableCatchableDist();
+    }
+    else if ( prob < 0.0 )
+    {
+        std::cerr << "internal error at PlayerType::reliableCatchableDist(): "
+                  << "probability " << prob << " too small" << std::endl;
+
+        return maxCatchableDist();
+    }
+
+    const ServerParam & SP = ServerParam::i();
+
+    const double target_prob = prob / SP.catchProbability();
+    if ( target_prob > 1.0 )
+    {
+        return 0.0;
+    }
+
+    const double catch_stretch_length_x = ( catchAreaLengthStretch() - 1.0 ) * SP.catchAreaLength();
+    const double catch_length_min_x = SP.catchAreaLength() - catch_stretch_length_x;
+
+    const double dist_x = catch_length_min_x + ( catch_stretch_length_x * 2.0 * ( 1.0 - target_prob ) );
+
+    const double diagonal_length = std::sqrt( std::pow( dist_x, 2 )
+                                              + std::pow( SP.catchAreaWidth() / 2.0, 2 ) );
+    return diagonal_length;
+}
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+double
+PlayerType::getCatchProbability( const double dist ) const
+{
+    const ServerParam & SP = ServerParam::i();
+
+    if ( dist < M_reliable_catchable_dist )
+    {
+        if ( dist < 0.0 )
+        {
+            std::cerr << "internal error at PlayerType::getCatchProbability(dist): negative distance "
+                      << dist << std::endl;
+        }
+
+        return SP.catchProbability();
+    }
+    else if ( dist > M_max_catchable_dist )
+    {
+        return 0.0;
+    }
+
+    const double catch_stretch_length_x = ( catchAreaLengthStretch() - 1.0 ) * SP.catchAreaLength();
+    const double catch_length_min_x = SP.catchAreaLength() - catch_stretch_length_x;
+
+    const double dist_x = std::sqrt( std::pow( dist, 2 )
+                                     - std::pow( SP.catchAreaWidth() / 2.0, 2 ) );
+
+    const double fail_prob = ( dist_x - catch_length_min_x ) / ( catch_stretch_length_x * 2.0 );
+
+    return ( 1.0 - fail_prob ) * SP.catchProbability();
+}
+
+
+
 /*-------------------------------------------------------------------*/
 /*!
 
 */
 int
-PlayerType::getMaxDashCyclesSavingStamina( const ServerParam & sparam,
-                                           const double & dash_power,
-                                           const double & current_stamina,
-                                           const double & current_recovery ) const
+PlayerType::getMaxDashCyclesSavingRecovery( const double & dash_power,
+                                            const double & current_stamina,
+                                            const double & current_recovery ) const
 {
     double available_stamina
         = current_stamina
-        - sparam.recoverDecThrValue()
+        - ServerParam::i().recoverDecThrValue()
         - 1.0;
     double used_stamina = ( dash_power > 0.0 ? dash_power : dash_power * -2.0 );
 
@@ -560,6 +639,11 @@ PlayerType::cyclesToReachMaxSpeed( const double & dash_power ) const
 int
 PlayerType::cyclesToReachDistance( const double & dash_dist ) const
 {
+    if ( dash_dist <= 0.001 )
+    {
+        return 0;
+    }
+
     std::vector< double >::const_iterator
         it = std::lower_bound( M_dash_distance_table.begin(),
                                M_dash_distance_table.end(),
@@ -585,19 +669,14 @@ PlayerType::cyclesToReachDistance( const double & dash_dist ) const
 
 */
 double
-PlayerType::getDashPowerToKeepMaxSpeed(  const ServerParam & sparam,
-                                         const double & effort ) const
+PlayerType::getDashPowerToKeepMaxSpeed( const double & effort ) const
 {
     // required accel in 1 step to keep max speed
     double required_power = playerSpeedMax() * ( 1.0 - playerDecay() );
     // required dash power to keep max speed
     required_power /= ( effort * dashPowerRate() );
 
-    if ( required_power > sparam.maxDashPower() )
-    {
-        required_power = sparam.maxDashPower();
-    }
-    return required_power;
+    return std::min( required_power, ServerParam::i().maxDashPower() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -605,15 +684,14 @@ PlayerType::getDashPowerToKeepMaxSpeed(  const ServerParam & sparam,
 
 */
 double
-PlayerType::kickRate( const ServerParam & sparam,
-                      const double & ball_dist,
+PlayerType::kickRate( const double & ball_dist,
                       const double & dir_diff ) const
 {
     return (
-            sparam.kickPowerRate()
+            kickPowerRate() //ServerParam::i().kickPowerRate()
             * ( 1.0
                 - 0.25 * std::fabs( dir_diff ) / 180.0
-                - ( 0.25 * ( ball_dist - sparam.ballSize() - playerSize() )
+                - ( 0.25 * ( ball_dist - ServerParam::i().ballSize() - playerSize() )
                     / kickableMargin() ) )
             );
 }
@@ -638,8 +716,8 @@ PlayerType::normalizeAccel( const Vector2D & velocity,
                             const AngleDeg & accel_angle,
                             double * accel_mag ) const
 {
-    // if ( *accel_mag > sparam.playerAccelMax() )
-    //     *accel_mag = sparam.playerAccelMax();
+    // if ( *accel_mag > SP.playerAccelMax() )
+    //     *accel_mag = SP.playerAccelMax();
 
     Vector2D dash_move = velocity;
     dash_move += Vector2D::polar2vector( *accel_mag, accel_angle ); // add dash accel
@@ -666,8 +744,8 @@ PlayerType::normalizeAccel( const Vector2D & velocity,
                             Vector2D * accel ) const
 {
     // double tmp = accel_mag->r();
-    // if ( tmp > sparam.playerAccelMax() )
-    //     *accel *= sparam.playerAccelMax() / tmp;
+    // if ( tmp > SP.playerAccelMax() )
+    //     *accel *= SP.playerAccelMax() / tmp;
 
     if ( ( velocity + (*accel) ).r2() > playerSpeedMax2() + 0.0001 )
     {
@@ -685,137 +763,6 @@ PlayerType::normalizeAccel( const Vector2D & velocity,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
-void
-PlayerType::predictStaminaAfterWait( const ServerParam & sparam,
-                                     const int n_wait,
-                                     double * stamina,
-                                     double * effort,
-                                     const double & recovery ) const
-{
-    for ( int i = 0; i < n_wait; ++i )
-    {
-        // recovery check is skipped,
-        // because recovery is never increased.
-
-        // check effort
-        if ( *stamina <= sparam.effortDecThrValue() )
-        {
-            if ( *effort > effortMin() )
-            {
-                *effort -= sparam.effortDec();
-                if ( *effort < effortMin() )
-                {
-                    *effort = effortMin();
-                }
-            }
-        }
-        else if ( *stamina > sparam.effortIncThrValue() )
-        {
-            if ( *effort < effortMax() )
-            {
-                *effort += sparam.effortInc();
-                if ( *effort > effortMax() )
-                {
-                    *effort = effortMax();
-                }
-            }
-        }
-
-        // recover stamina
-        *stamina += staminaIncMax() * recovery;
-        if ( *stamina > sparam.staminaMax() )
-        {
-            *stamina = sparam.staminaMax();
-        }
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-PlayerType::predictStaminaAfterOneDash( const ServerParam & sparam,
-                                        const double & dash_power,
-                                        double * stamina,
-                                        double * effort,
-                                        double * recovery ) const
-{
-    // decay stamina
-    *stamina -= ( dash_power > 0.0 ? dash_power : (dash_power * -2.0) );
-    if ( *stamina < 0.0 )
-    {
-        *stamina = 0.0;
-    }
-
-    // check recovery decay
-    if ( *stamina <= sparam.recoverDecThrValue() )
-    {
-        if ( *recovery > sparam.recoverMin() )
-        {
-            *recovery -= sparam.recoverDec();
-            *recovery = std::max( *recovery, sparam.recoverMin() );
-        }
-    }
-
-    // check effort decay
-    if ( *stamina <= sparam.effortDecThrValue() )
-    {
-        if ( *effort > effortMin() )
-        {
-            *effort -= sparam.effortDec();
-            *effort = std::max( *effort, effortMin() );
-        }
-    }
-    else if ( *stamina >= sparam.effortIncThrValue() )
-    {
-        if ( *effort < effortMax() )
-        {
-            *effort += sparam.effortInc();
-            *effort = std::min( *effort, effortMax() );
-        }
-    }
-
-    // increase stamina
-    *stamina += staminaIncMax() * (*recovery);
-    *stamina = std::min( *stamina, sparam.staminaMax() );
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-PlayerType::predictStaminaAfterDashes( const ServerParam & sparam,
-                                       const double & dash_power,
-                                       const int n_dash,
-                                       double * stamina,
-                                       double * effort,
-                                       double * recovery ) const
-{
-    int n_safety_dash =
-        getMaxDashCyclesSavingStamina( sparam, dash_power, *stamina, *recovery );
-    if ( n_safety_dash > n_dash )
-    {
-        n_safety_dash = n_dash;
-    }
-
-    double one_cycle_consumpation
-        = ( dash_power > 0.0 ? dash_power : dash_power * -2.0 )
-        - ( staminaIncMax() * (*recovery) );
-    *stamina -= one_cycle_consumpation * n_safety_dash;
-
-    const int n_exhaust_dash = n_dash - n_safety_dash;
-    for ( int i = 0; i < n_exhaust_dash; ++i )
-    {
-        predictStaminaAfterOneDash( sparam, dash_power, stamina, effort, recovery );
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
 */
 std::ostream &
 PlayerType::print( std::ostream & os ) const
@@ -856,21 +803,8 @@ PlayerTypeSet::instance()
 
 */
 PlayerTypeSet::PlayerTypeSet()
-    : M_dummy_type( ServerParam::i(),
-                    Hetero_Unknown,
-                    1.2, // player speed max
-                    45.0, // stamina inc max
-                    0.4, // decay
-                    5.0, // inertia moment
-                    0.006, // dash power rate
-                    0.3, // size
-                    0.7, // kickable margin
-                    0.0, // kick rand
-                    0.0, // extra stamina
-                    1.0, // effort max
-                    0.6 ) // effort min
 {
-    resetDefaultType( ServerParam::i() );
+    resetDefaultType();
 }
 
 /*-------------------------------------------------------------------*/
@@ -887,9 +821,9 @@ PlayerTypeSet::~PlayerTypeSet()
 
 */
 void
-PlayerTypeSet::resetDefaultType( const ServerParam & sparam )
+PlayerTypeSet::resetDefaultType()
 {
-    PlayerType default_type( sparam );
+    PlayerType default_type;
     insert( default_type );
     M_dummy_type = default_type;
 }
@@ -904,7 +838,7 @@ PlayerTypeSet::insert( const PlayerType & param )
     if ( static_cast< int >( M_player_type_map.size() )
          >= PlayerParam::i().playerTypes() )
     {
-        std::cerr << "player types over flow" << std::endl;
+        std::cerr << "# of player type over flow" << std::endl;
         return;
     }
 
@@ -926,59 +860,15 @@ PlayerTypeSet::insert( const PlayerType & param )
 void
 PlayerTypeSet::createDummyType()
 {
-#if 0
-    double player_speed_max = 0.0;
-    double stamina_inc_max = 0.0;
-    double player_decay = 1.0;
-    double inertia_moment = 10.0;
-    double dash_power_rate = 0.0;
-    double player_size = 1.0;
-    double kickable_margin = 0.0;
-    double kick_rand = 1.0;
-    double extra_stamina = 0.0;
-    double effort_max = 0.0;
-    double effort_min = 1.0;
     for ( PlayerTypeMap::iterator it = M_player_type_map.begin();
           it != M_player_type_map.end();
           ++it )
     {
-        player_speed_max = std::max( player_speed_max, it->second.playerSpeedMax() );
-        stamina_inc_max = std::max( stamina_inc_max, it->second.staminaIncMax() );
-        player_decay = std::min( player_decay, it->second.playerDecay() );
-        inertia_moment = std::min( inertia_moment, it->second.inertiaMoment() );
-        dash_power_rate = std::max( dash_power_rate, it->second.dashPowerRate() );
-        player_size = std::min( player_size, it->second.playerSize() );
-        kickable_margin = std::max( kickable_margin, it->second.kickableMargin() );
-        kick_rand = std::min( kick_rand, it->second.kickRand() );
-        extra_stamina = std::max( extra_stamina, it->second.extraStamina() );
-        effort_max = std::max( effort_max, it->second.effortMax() );
-        effort_min = std::min( effort_min, it->second.effortMin() );
-    }
-
-    M_dummy_type = PlayerType( ServerParam::i(),
-                               Hetero_Unknown,
-                               player_speed_max,
-                               stamina_inc_max,
-                               player_decay,
-                               inertia_moment,
-                               dash_power_rate,
-                               player_size,
-                               kickable_margin,
-                               kick_rand,
-                               extra_stamina,
-                               effort_max,
-                               effort_min );
-#endif
-    for ( PlayerTypeMap::iterator it = M_player_type_map.begin();
-          it != M_player_type_map.end();
-          ++it )
-    {
-        if ( it->second.playerSpeedMax() > M_dummy_type.playerSpeedMax() )
+        if ( it->second.realSpeedMax() > M_dummy_type.realSpeedMax() )
         {
             M_dummy_type = it->second;
         }
-        else if ( std::fabs( it->second.playerSpeedMax()
-                             - M_dummy_type.playerSpeedMax() ) < 0.01
+        else if ( std::fabs( it->second.realSpeedMax() - M_dummy_type.realSpeedMax() ) < 0.01
                   && it->second.cyclesToReachMaxSpeed() < M_dummy_type.cyclesToReachMaxSpeed() )
         {
             M_dummy_type = it->second;
