@@ -8,7 +8,7 @@
 /*
  *Copyright:
 
- Copyright (C) Hidehisa AKIYAMA
+ Copyright (C) Hidehisa AKIYAMA, Hiroki SHIMORA
 
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -43,30 +43,36 @@
 
 namespace rcsc {
 
-class ServerParam;
-
 /*!
   \class PlayerType
   \brief heterogeneous player parametor class
  */
 class PlayerType {
 private:
-    int M_id;
-    double M_player_speed_max;
-    double M_stamina_inc_max;
-    double M_player_decay;
-    double M_inertia_moment;
-    double M_dash_power_rate;
-    double M_player_size;
-    double M_kickable_margin;
-    double M_kick_rand;
-    double M_extra_stamina;
-    double M_effort_max;
-    double M_effort_min;
+    int M_id; //!< player type id
+    double M_player_speed_max; //!< maximum speed
+    double M_stamina_inc_max; //!< stamina inc max
+    double M_player_decay; //!< player decay
+    double M_inertia_moment; //!< inertia moment
+    double M_dash_power_rate; //!< dash power rate
+    double M_player_size; //!< player radius
+    double M_kickable_margin; //!< kickable area margin
+    double M_kick_rand; //!< kick random
+    double M_extra_stamina; //!< extra stamina value
+    double M_effort_max; //!< maximum(initial) effort
+    double M_effort_min; //!< minimum effort
+    // v14
+    double M_kick_power_rate; //!< kick power rate
+    double M_foul_detect_probability; //!< foul detect probability
+    double M_catchable_area_l_stretch; //!< catch area length stretch factor
 
+    //
     // additional parameters
+    //
 
-    double M_kickable_area;
+    double M_kickable_area; //!< cached kickable area
+    double M_reliable_catchable_dist; //!< cached reliable catchable dist
+    double M_max_catchable_dist; //!< cached maximum catchable dist
 
     // if player's dprate & effort is not enough,
     // player never reach player_speed_max
@@ -86,68 +92,24 @@ private:
 
 public:
     /*!
-      \brief default constructo
-
-      just set Hetero_Unknown to Id
+      \brief default constructor. create default type parameter using ServerParam
      */
-    PlayerType()
-        : M_id( Hetero_Unknown )
-      { }
-
-    /*!
-      \brief constructor with all parameters
-      \param sparam server parameter
-      \param id hetero Id
-      \param pspdmax player_speed_max parameter
-      \param sincmax stamina_inc_max parameter
-      \param pdecay player_decay parameter
-      \param imoment inertia_moment parameter
-      \param dprate dash_power_rate parameter
-      \param psize player_size parameter
-      \param kmargin kickable_margin parameter
-      \param krand kick_rand parameter
-      \param estamina extra_stamina parameter
-      \param effmax effort_max parameter
-      \param effmin effort_min parameter
-    */
-    PlayerType( const ServerParam & sparam,
-                const int id,
-                const double & pspdmax,
-                const double & sincmax,
-                const double & pdecay,
-                const double & imoment,
-                const double & dprate,
-                const double & psize,
-                const double & kmargin,
-                const double & krand,
-                const double & estamina,
-                const double & effmax,
-                const double & effmin );
-
-    /*!
-      \brief construct default type parameter using ServerParam
-      \param sparam const reference to the ServerParam
-    */
-    explicit
-    PlayerType( const ServerParam & sparam );
+    PlayerType();
 
     /*!
       \brief construct with message from rcssserver
-      \param sparam const reference to the ServerParam
-      \param msg raw message from rcssserver
+      \param server_msg raw message from rcssserver
       \param version client version that determins message protocol
     */
-    PlayerType( const ServerParam & sparam,
-                const char * msg,
+    PlayerType( const char * server_msg,
                 const double & version );
 
     /*!
       \brief construct with monitor protocol
-      \param sparam const reference to the ServerParam
       \param from monitor protocol data
      */
-    PlayerType( const ServerParam & sparam,
-                const rcg::player_type_t & from );
+    explicit
+    PlayerType( const rcg::player_type_t & from );
 
     /*!
       \brief conver to the monitor protocol format
@@ -163,6 +125,11 @@ public:
 
 private:
 
+    /*!
+      \brief set default values by ServerParam.
+     */
+    void setDefault();
+
     /*!
       \brief analyze version 8 protocol server message
       \param msg raw message string from rcssserver
@@ -176,10 +143,9 @@ private:
     void parseV7( const char * msg );
 
     /*!
-      \brief set additional parameters value
-      \param sparam const reference to the ServerParam
+      \brief set additional parameters
      */
-    void initAdditionalParams( const ServerParam & sparam );
+    void initAdditionalParams();
 
 public:
 
@@ -302,6 +268,36 @@ public:
           return M_effort_min;
       }
 
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
+    const
+    double & kickPowerRate() const
+      {
+          return M_kick_power_rate;
+      }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
+    const
+    double & foulDetectProbability() const
+      {
+          return M_foul_detect_probability;
+      }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
+    const
+    double & catchAreaLengthStretch() const
+      {
+          return M_catchable_area_l_stretch;
+      }
+
     ////////////////////////////////////////////////
     // additional parameters
 
@@ -315,6 +311,37 @@ public:
           return M_kickable_area;
       }
 
+    /*!
+      \brief get reliable catchable distance
+      \return threshold distance not to fail
+     */
+    double reliableCatchableDist() const;
+
+    /*!
+      \brief get reliable catchable distance
+      \param prob threshold of catch probability
+      \return threshold distance that catch probability is greater equal to prob.
+      if catch_probalibity server parameter is not 1, this method returns 0.0
+     */
+    double reliableCatchableDist( const double prob ) const;
+
+    /*!
+      \brief get catch probability
+      \param dist distance of player and ball
+      \return probability of successful catching
+     */
+    double getCatchProbability( const double dist ) const;
+
+    /*!
+      \brief get the maximum catchable distance
+      \return maximum catchable distance
+     */
+    const
+    double & maxCatchableDist() const
+      {
+          return M_max_catchable_dist;
+      }
+
     /*!
       \brief get the reachable speed max
       \return reachable speed max
@@ -358,16 +385,14 @@ public:
     ////////////////////////////////////////////////
     /*!
       \brief calculate enable cycles to keep to dash using max power
-      \param sparam server parameter
       \param dash_power used dash power
       \param current_stamina current agent's stamina
       \param current_recovery current agent's recovery
       \return max cycles to keep same dash power
     */
-    int getMaxDashCyclesSavingStamina( const ServerParam & sparam,
-                                       const double & dash_power,
-                                       const double & current_stamina,
-                                       const double & current_recovery ) const;
+    int getMaxDashCyclesSavingRecovery( const double & dash_power,
+                                        const double & current_stamina,
+                                        const double & current_recovery ) const;
 
     /*
       \brief estimate the number of available dashes with max power
@@ -427,21 +452,18 @@ public:
     ////////////////////////////////////////////////
     /*!
       \brief estimate dash power to keep max speed
-      \param sparam server parameter
       \param effort current agent's effort
       \return conserved dash power
     */
-    double getDashPowerToKeepMaxSpeed( const ServerParam & sparam,
-                                       const double & effort ) const;
+    double getDashPowerToKeepMaxSpeed( const double & effort ) const;
 
     /*!
       \brief estimate dash power to keep max speed with max effort value
-      \param sparam server parameter
       \return conserved dash power
     */
-    double getDashPowerToKeepMaxSpeed( const ServerParam & sparam ) const
+    double getDashPowerToKeepMaxSpeed() const
       {
-          return getDashPowerToKeepMaxSpeed( sparam, effortMax() );
+          return getDashPowerToKeepMaxSpeed( effortMax() );
       }
 
     /*!
@@ -459,25 +481,22 @@ public:
 
     /*!
       \brief estimate one cycle stamina comsumption to keep mas speed
-      \param sparam server parameter
       \return get the comsumed stamina value
      */
-    double getOneStepStaminaComsumption( const ServerParam & sparam ) const
+    double getOneStepStaminaComsumption() const
       {
-          return getDashPowerToKeepMaxSpeed( sparam, effortMax() ) - staminaIncMax();
+          return getDashPowerToKeepMaxSpeed( effortMax() ) - staminaIncMax();
       }
 
 
     ////////////////////////////////////////////////
     /*!
       \brief calculate kick rate
-      \param sparam server parameter
       \param ball_dist ball distance from agent
       \param dir_diff ball angle difference from agent body angle
       \return kick rate value
     */
-    double kickRate( const ServerParam & sparam,
-                     const double & ball_dist,
+    double kickRate( const double & ball_dist,
                      const double & dir_diff ) const;
 
     /*!
@@ -596,50 +615,6 @@ public:
     */
     bool normalizeAccel( const Vector2D & velocity,
                          Vector2D * accel ) const;
-    ////////////////////////////////////////////////
-    /*!
-      \brief predict agent's stamina and effort
-      \param sparam server parameter
-      \param n_wait number of wait cycles
-      \param stamina pointer to stamina variable
-      \param effort pointer to effort variable
-      \param recovery current agent's recovery
-    */
-    void predictStaminaAfterWait( const ServerParam & sparam,
-                                  const int n_wait,
-                                  double * stamina,
-                                  double * effort,
-                                  const double & recovery ) const;
-
-    /*!
-      \brief predict agent's stamina related values after one dash
-      \param sparam server parameter
-      \param dash_power used dash power
-      \param stamina pointer to stamina variable
-      \param effort pointer to effort variable
-      \param recovery pointer to recovery variable
-    */
-    void predictStaminaAfterOneDash( const ServerParam & sparam,
-                                     const double & dash_power,
-                                     double * stamina,
-                                     double * effort,
-                                     double * recovery ) const;
-
-    /*!
-      \brief predict stamina related values after nr dashes
-      \param sparam server parameter
-      \param dash_power used dash power
-      \param n_dash number of dash cycles
-      \param stamina pointer to stamina variable
-      \param effort pointer to effort variable
-      \param recovery pointer to recovery variable
-     */
-    void predictStaminaAfterDashes( const ServerParam & sparam,
-                                    const double & dash_power,
-                                    const int n_dash,
-                                    double * stamina,
-                                    double * effort,
-                                    double * recovery ) const;
 
     /*!
       \brief output parameters to stream
@@ -692,8 +667,7 @@ public:
       \return const reference to the player type set instance
      */
     inline
-    static
-    const
+    static const
     PlayerTypeSet & i()
       {
           return instance();
@@ -701,9 +675,8 @@ public:
 
     /*!
       \brief regenerate default player type parameter using server param
-      \param sparam const reference to the server parameter instance
      */
-    void resetDefaultType( const ServerParam & sparam );
+    void resetDefaultType();
 
     /*!
       \brief add new player type parameter
@@ -732,16 +705,6 @@ public:
           return M_player_type_map;
       }
 
-    /*!
-      \brief get dummy type parameter
-      \return const reference to the dummy player type paramaeter
-     */
-    const
-    PlayerType & dummyType() const
-      {
-          return M_dummy_type;
-      }
-
     /*!
       \brief get player type parameter that Id is id
       \param id wanted player type Id
