@@ -46,7 +46,6 @@
 
 namespace rcsc {
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -77,7 +76,7 @@ BallMessageParser::parse( const int sender ,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "***ERROR*** BallMessageParser::parse()"
                   << " Illegal ball message [" << msg << "]"
@@ -112,7 +111,7 @@ BallMessageParser::parse( const int sender ,
     M_memory->setBall( sender, ball_pos, ball_vel, current );
 
     return slength();
-};
+}
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -143,7 +142,7 @@ PassMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "PassMessageParser::parse()"
                   << " Illegal pass pass message ["
@@ -235,7 +234,7 @@ InterceptMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "InterceptMessageParser::parse()"
                   << " Illegal message = [" << msg << "]"
@@ -314,7 +313,7 @@ GoalieMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "GoalieMessageParser::parse()."
                   << " Illegal message [" << msg
@@ -361,6 +360,121 @@ GoalieMessageParser::parse( const int sender,
 }
 
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+GoalieAndPlayerMessageParser::GoalieAndPlayerMessageParser( boost::shared_ptr< AudioMemory > memory )
+    : M_memory( memory )
+{
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+GoalieAndPlayerMessageParser::parse( const int sender,
+                                   const double & ,
+                                   const char * msg,
+                                   const GameTime & current )
+{
+    // format:
+    //    "g<pos_body:4,unm_pos:3>"
+    // the length of message == 8
+
+    // ( 22 * 105/0.63 * 68/0.63 ) * ( 16.0/0.1 * 40.0/0.1 * 360 )
+    // -> (22 * 168 * 109) * (160 * 400 * 360) = 9281986560000
+
+    // 74^6 = 164206490176
+    //        9281986560000
+    // 74^7 = 12151280273024
+
+    // ( 22 * 105/0.55 * 68/0.55 ) * ( 16.0/0.1 * 40.0/0.1 * 360 )
+    // -> (22 * 192 * 125) * (160 * 400 * 360) = 12165120000000
+
+    // ==========
+    // ( 22 * 105/0.555 * 68/0.555 ) * ( 16.0/0.1 * 40.0/0.1 * 360 )
+    // -> (22 * 191 * 124) * (160 * 400 * 360) = 12004945920000
+    // ==========
+
+    // ( 22 * 105/0.56 * 68/0.55 ) * ( 16.0/0.1 * 40.0/0.1 * 360 )
+    // -> (22 * 189 * 123) * (160 * 400 * 360) = 11783439360000
+
+
+    if ( *msg != sheader() )
+    {
+        return 0;
+    }
+
+    if ( (int)std::strlen( msg ) < slength() )
+    {
+        std::cerr << "Goalie1PlayerMessageParser::parse()."
+                  << " Illegal message [" << msg
+                  << "] len = " << std::strlen( msg )
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "Goalie1PlayerMessageParser: Failed to decode Goalie Info [%s]",
+                      msg );
+        return -1;
+    }
+    ++msg;
+
+    boost::int64_t ival = 0;
+    if ( ! AudioCodec::i().decodeStrToInt64( std::string( msg, slength() - 1 ),
+                                             &ival ) )
+    {
+        std::cerr << "Goalie1PlayerMessageParser::parse()"
+                  << " Failed to parse [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "Goalie1PlayerMessageParser: Failed to decode Goalie Info [%s]",
+                      msg );
+        return -1;
+    }
+
+    Vector2D goalie_pos;
+    AngleDeg goalie_body;
+    int player_number = Unum_Unknown;
+    Vector2D player_pos;
+
+    // 124 > 68/0.555 + 1
+    player_pos.y = ( ival % 124 ) * 0.555 - 34.0;
+    ival /= 124;
+
+    // 191 > 105/0.555 + 1
+    player_pos.x = ( ival % 191 ) * 0.555 - 52.5;
+    ival /= 191;
+
+    // 22
+    player_number = ( ival % 22 ) + 1;
+    ival /= 22;
+
+
+    goalie_body = static_cast< double >( ival % 360 - 180 );
+    ival /= 360;
+
+    goalie_pos.y = ( ival % 400 ) * 0.1 - 20.0;
+    ival /= 400;
+
+    goalie_pos.x = ( ival % 160 ) * 0.1 + ( 53.0 - 16.0 );
+    // ival /= 160;
+
+
+    dlog.addText( Logger::SENSOR,
+                  "GoalieAndPlayerMessageParser: success! goalie pos=(%.2f %.2f) body=%.1f",
+                  goalie_pos.x, goalie_pos.y, goalie_body.degree() );
+    dlog.addText( Logger::SENSOR,
+                  "____ player number=%d pos=(%.2f %.2f)",
+                  player_number, player_pos.x, player_pos.y );
+
+    M_memory->setOpponentGoalie( sender, goalie_pos, goalie_body, current );
+    M_memory->setPlayer( sender, player_number, player_pos, current );
+
+    return slength();
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -390,7 +504,7 @@ OffsideLineMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "OffsideLineMessageParser::parse()"
                   << " Illegal message [" << msg << "]"
@@ -456,7 +570,7 @@ DefenseLineMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "DefenseLineMessageParser::parse()"
                   << " Illegal message [" << msg << "]"
@@ -551,7 +665,7 @@ PassRequestMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "PassRequestMessageParser::parse()"
                   << " Illegal pass request message ["
@@ -614,7 +728,7 @@ StaminaMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "StaminaMessageParser::parse()"
                   << " Illegal message [" << msg << "]"
@@ -678,7 +792,7 @@ RecoveryMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "RecoveryMessageParser::parse()"
                   << " Illegal message [" << msg << "]"
@@ -744,7 +858,7 @@ DribbleMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "DribbleMessageParser::parse()"
                   << " Illegal message ["
@@ -822,7 +936,7 @@ BallGoalieMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "BallGoalieMessageParser::parse()"
                   << " Illegal message ["
@@ -921,12 +1035,14 @@ OnePlayerMessageParser::parse( const int sender,
     // ( 22 * 105/0.63 * 68/0.63 ) = 395767.195767196 < 74^3(=405224)
     //  -> 22 * 168 * 109 = 402864
 
+    //               74^3 = 405224
+
     if ( *msg != sheader() )
     {
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "OnePlayerMessageParser::parse()"
                   << " Illegal message ["
@@ -1004,13 +1120,16 @@ TwoPlayerMessageParser::parse( const int sender,
 
     // ( 22 * 105/0.63 * 68/0.63 ) = 395767.195767196 < 74^3(=405224)
     //  -> 22 * 168 * 109 = 402864
+    //     (22 * 168 * 109)^2 = 162299402496
+
+    //                   74^6 = 164206490176
 
     if ( *msg != sheader() )
     {
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "TwoPlayerMessageParser::parse()"
                   << " Illegal message ["
@@ -1110,13 +1229,16 @@ ThreePlayerMessageParser::parse( const int sender,
 
     // ( 22 * 105/0.63 * 68/0.63 ) = 395767.195767196 < 74^3(=405224)
     //  -> 22 * 168 * 109 = 402864
+    //     (22 * 168 * 109)^3 = 65384586487148544
+
+    //                   74^9 = 66540410775079424
 
     if ( *msg != sheader() )
     {
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "ThreePlayerMessageParser::parse()"
                   << " Illegal message ["
@@ -1238,7 +1360,7 @@ SelfMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "SelfMessageParser::parse()"
                   << " Illegal message ["
@@ -1334,7 +1456,7 @@ TeammateMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "TeammateMessageParser::parse()"
                   << " Illegal message ["
@@ -1428,7 +1550,7 @@ OpponentMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "OpponentMessageParser::parse()"
                   << " Illegal message ["
@@ -1517,7 +1639,7 @@ BallPlayerMessageParser::parse( const int sender,
         return 0;
     }
 
-    if ( std::strlen( msg ) < slength() )
+    if ( (int)std::strlen( msg ) < slength() )
     {
         std::cerr << "OnePlayerMessageParser::parse()"
                   << " Illegal message ["
