@@ -35,6 +35,7 @@
 #include <rcsc/types.h>
 
 #include <boost/shared_ptr.hpp>
+
 #include <string>
 
 namespace rcsc {
@@ -48,6 +49,11 @@ class GameTime;
   \brief abstract player's say message parser
  */
 class SayMessageParser {
+public:
+
+    //! pointer type alias
+    typedef boost::shared_ptr< SayMessageParser > Ptr;
+
 private:
 
     // not used
@@ -126,29 +132,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'b';
-      }
+    char sheader() { return 'b'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 6;
-      }
+    int slength() { return 6; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -197,29 +194,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'p';
-      }
+    char sheader() { return 'p'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 10;
-      }
+    int slength() { return 10; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -268,29 +256,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'i';
-      }
+    char sheader() { return 'i'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 3;
-      }
+    int slength() { return 3; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -312,7 +291,7 @@ public:
 /*-------------------------------------------------------------------*/
 /*!
   \class GoalieMessageParser
-  \brief goalie info message parser
+  \brief opponent goalie info message parser
 
   format:
   "g<pos_body:4>"
@@ -339,29 +318,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'g';
-      }
+    char sheader() { return 'g'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 5;
-      }
+    int slength() { return 5; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -380,6 +350,70 @@ public:
 
 };
 
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class GoalieAndPlayerMessageParser
+  \brief opponent goalie & field player message parser
+
+  format:
+  "g<pos_body:4,unum_pos:3>"
+  the length of message == 8
+ */
+class GoalieAndPlayerMessageParser
+    : public SayMessageParser {
+private:
+
+    //! pointer to the audio memory
+    boost::shared_ptr< AudioMemory > M_memory;
+
+public:
+
+    /*!
+      \brief construct with audio memory
+      \param memory pointer to the memory
+     */
+    explicit
+    GoalieAndPlayerMessageParser( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    static
+    char sheader() { return 'e'; }
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    char header() const { return sheader(); }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    int slength() { return 8; }
+
+    /*!
+      \brief virtual method which analyzes audio messages.
+      \param sender sender's uniform number
+      \param dir sender's direction
+      \param msg raw audio message
+      \param current current game time
+      \retval bytes read if success
+      \retval 0 message ID is not match. other parser should be tried.
+      \retval -1 failed to parse
+    */
+    int parse( const int sender,
+               const double & dir,
+               const char * msg,
+               const GameTime & current );
+
+};
+
+
 /*-------------------------------------------------------------------*/
 /*!
   \class OffsideLineMessageParser
@@ -410,29 +444,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'o';
-      }
+    char sheader() { return 'o'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 2;
-      }
+    int slength() { return 2; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -481,29 +506,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'd';
-      }
+    char sheader() { return 'd'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 2;
-      }
+    int slength() { return 2; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -552,29 +568,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'w';
-      }
+    char sheader() { return 'w'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 1;
-      }
+    int slength() { return 1; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -623,29 +630,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'h';
-      }
+    char sheader() { return 'h'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 4;
-      }
+    int slength() { return 4; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -694,29 +692,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 's';
-      }
+    char sheader() { return 's'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 2;
-      }
+    int slength() { return 2; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -765,29 +754,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'r';
-      }
+    char sheader() { return 'r'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 2;
-      }
+    int slength() { return 2; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -836,29 +816,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'D';
-      }
+    char sheader() { return 'D'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 4;
-      }
+    int slength() { return 4; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -907,29 +878,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'G';
-      }
+    char sheader() { return 'G'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 10;
-      }
+    int slength() { return 10; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -978,29 +940,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'P';
-      }
+    char sheader() { return 'P'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 4;
-      }
+    int slength() { return 4; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -1049,29 +1002,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'Q';
-      }
+    char sheader() { return 'Q'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 7;
-      }
+    int slength() { return 7; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -1120,29 +1064,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'R';
-      }
+    char sheader() { return 'R'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 10;
-      }
+    int slength() { return 10; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -1191,29 +1126,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'S';
-      }
+    char sheader() { return 'S'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 5;
-      }
+    int slength() { return 5; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -1262,29 +1188,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'T';
-      }
+    char sheader() { return 'T'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 5;
-      }
+    int slength() { return 5; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -1333,29 +1250,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'O';
-      }
+    char sheader() { return 'O'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 5;
-      }
+    int slength() { return 5; }
 
     /*!
       \brief virtual method which analyzes audio messages.
@@ -1404,29 +1312,20 @@ public:
       \return header character.
      */
     static
-    char sheader()
-      {
-          return 'B';
-      }
+    char sheader() { return 'B'; }
 
     /*!
       \brief get the header character.
       \return header character.
      */
-    char header() const
-      {
-          return sheader();
-      }
+    char header() const { return sheader(); }
 
     /*!
       \brief get the length of this message.
       \return the length of encoded message
     */
     static
-    std::size_t slength()
-      {
-          return 10;
-      }
+    int slength() { return 10; }
 
     /*!
       \brief virtual method which analyzes audio messages.
