@@ -325,6 +325,12 @@ const double ServerParam::BACK_DASH_RATE = 0.5;
 const double ServerParam::MAX_DASH_POWER = 100.0;
 const double ServerParam::MIN_DASH_POWER = -100.0;
 
+// 14.0.0
+const double ServerParam::TACKLE_RAND_FACTOR = 2.0;
+const double ServerParam::FOUL_DETECT_PROBABILITY = 0.5;
+const double ServerParam::FOUL_EXPONENT = 10.0;
+const int ServerParam::FOUL_CYCLES = 5;
+
 /*
   It is necessary to check out whether server is NEW_QSTEP mode
   bool defined_qstep_l = false;
@@ -622,6 +628,14 @@ ServerParam::setDefaultParam()
     M_max_dash_power = MAX_DASH_POWER;
     M_min_dash_power = MIN_DASH_POWER;
 
+    // 14.0.0
+    M_tackle_rand_factor = TACKLE_RAND_FACTOR;
+    M_foul_detect_probability = FOUL_DETECT_PROBABILITY;
+    M_foul_exponent = FOUL_EXPONENT;
+    M_foul_cycles = FOUL_CYCLES;
+    M_random_seed = -1;
+    M_golden_goal = true;
+
     setAdditionalParam();
 }
 
@@ -897,6 +911,7 @@ ServerParam::createMap()
         ( "player_speed_max_min", "", &M_player_speed_max_min )
         ( "extra_stamina", "", &M_extra_stamina )
         ( "synch_see_offset", "", &M_synch_see_offset )
+        ( "max_monitors", "", &M_max_monitors )
         // 12.1.3
         ( "extra_half_time", "",  &M_extra_half_time )
         // 13.0.0
@@ -908,8 +923,14 @@ ServerParam::createMap()
         ( "back_dash_rate", "", &M_back_dash_rate )
         ( "max_dash_power", "", &M_max_dash_power )
         ( "min_dash_power", "", &M_min_dash_power )
-
-        ( "max_monitors", "", &M_max_monitors )
+        // 14.0.0
+        ( "tackle_rand_factor", "", &M_tackle_rand_factor )
+        ( "foul_detect_probability", "", &M_foul_detect_probability )
+        ( "foul_exponent", "", &M_foul_exponent )
+        ( "foul_cycles", "", &M_foul_cycles )
+        ( "random_seed", "", &M_random_seed )
+        ( "golden_goal", "", &M_golden_goal )
+        // test
         ( "min_catch_probability", "", &M_min_catch_probability )
         ( "reliable_catch_area_l", "", &M_reliable_catch_area_l )
         ;
