@@ -33,6 +33,8 @@
 #define RCSC_PARAM_SERVER_PARAM_H
 
 #include <rcsc/geom/vector_2d.h>
+#include <rcsc/geom/rect_2d.h>
+#include <rcsc/math_util.h>
 
 #include <boost/scoped_ptr.hpp>
 
@@ -330,6 +332,11 @@ public:
     static const double BACK_DASH_RATE;
     static const double MAX_DASH_POWER;
     static const double MIN_DASH_POWER;
+    // 14.0.0
+    static const double TACKLE_RAND_FACTOR;
+    static const double FOUL_DETECT_PROBABILITY;
+    static const double FOUL_EXPONENT;
+    static const int FOUL_CYCLES;
 private:
 
     //////////////////////////////////////////////////////
@@ -604,6 +611,15 @@ private:
     double M_max_dash_power;
     double M_min_dash_power;
 
+    // 14.0.0
+    double M_tackle_rand_factor;
+    double M_foul_detect_probability;
+    double M_foul_exponent;
+    int M_foul_cycles;
+
+    int M_random_seed;
+    bool M_golden_goal;
+
     // 999
     int M_max_monitors;
 
@@ -1751,6 +1767,37 @@ public:
           return M_min_dash_power;
       }
 
+    // v14
+    const
+    double & tackleRandFactor() const
+      {
+          return M_tackle_rand_factor;
+      }
+    const
+    double & foulDetectProbability() const
+      {
+          return M_foul_detect_probability;
+      }
+    const
+    double & foulExponent() const
+      {
+          return M_foul_exponent;
+      }
+    int foulCycles() const
+      {
+          return M_foul_cycles;
+      }
+
+    int randomSeed() const
+      {
+          return M_random_seed;
+      }
+
+    bool goldenGoal() const
+      {
+          return M_golden_goal;
+      }
+
 
     // automatically defined values
 
@@ -1809,6 +1856,39 @@ public:
           return pitchHalfLength() - penaltyAreaLength();
       }
 
+
+    Rect2D ourPenaltyArea() const
+      {
+          return Rect2D::from_corners( -pitchHalfLength(),
+                                       -penaltyAreaHalfWidth(),
+                                       -pitchHalfLength() + penaltyAreaLength(),
+                                       +penaltyAreaHalfWidth() );
+      }
+
+    Rect2D theirPenaltyArea() const
+      {
+          return Rect2D::from_corners( pitchHalfLength() - penaltyAreaLength(),
+                                       -penaltyAreaHalfWidth(),
+                                       +pitchHalfLength(),
+                                       +penaltyAreaHalfWidth() );
+      }
+
+    Rect2D ourGoalArea() const
+      {
+          return Rect2D::from_corners( -pitchHalfLength(),
+                                       -goalAreaHalfWidth(),
+                                       -pitchHalfLength() + goalAreaLength(),
+                                       +goalAreaHalfWidth() );
+      }
+
+    Rect2D theirGoalArea() const
+      {
+          return Rect2D::from_corners( +pitchHalfLength() - goalAreaLength(),
+                                       -goalAreaHalfWidth(),
+                                       +pitchHalfLength(),
+                                       +goalAreaHalfWidth() );
+      }
+
     const
     double & defaultRealSpeedMax() const
       {
@@ -1934,6 +2014,35 @@ public:
           return neck_angle;
       }
 
+
+    /*!
+      \brief calculate ball moves step for the input first speed & move distance
+      \param first_ball_speed ball first speed. have to be a positive value.
+      \param ball_move_dist total bal move distance. have to be a positive value.
+      \return numerically calculated result(by ceiled integer)
+     */
+    int ballMoveStep( const double & first_ball_speed,
+                      const double & ball_move_dist ) const
+      {
+          return static_cast< int >( std::ceil( calc_length_geom_series( first_ball_speed,
+                                                                         ball_move_dist,
+                                                                         ballDecay() ) )
+                                     + 1.0e-10 );
+      }
+
+    /*!
+      \brief calculate the first ball speed for the input move distance & steps
+      \param ball_move_dist total ball move distance, have to be a positive value.
+      \param total_step desired ball move steps. have to be greater than 1.
+      \return numerically calculated result. the value may be greater than ballSpeedMax()
+     */
+    double firstBallSpeed( const double & ball_move_dist,
+                           const int total_step ) const
+      {
+          return calc_first_term_geom_series( ball_move_dist,
+                                              ballDecay(),
+                                              total_step );
+      }
 };
 
 }
