@@ -289,7 +289,7 @@ FormationCDT::interpolate( const int unum,
         Segment2D segment_12( vertex_1, vertex_2 );
         Vector2D intersection_12 = segment_12.intersection( line_0 );
 
-        if ( ! intersection_12.valid() )
+        if ( ! intersection_12.isValid() )
         {
             if ( focus_point.dist2( vertex_0 ) < 1.0e-5 )
             {
