@@ -376,7 +376,7 @@ FormationDT::interpolate( const int unum,
 
         Vector2D intersection_12 = segment_12.intersection( line_0 );
 
-        if ( ! intersection_12.valid() )
+        if ( ! intersection_12.isValid() )
         {
             if ( focus_point.dist2( tri->vertex( 0 )->pos() ) < 1.0e-5 )
             {
