@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_rbf.h
-	\brief formation data classes using RBF network Header File.
+  \file formation_rbf.h
+  \brief formation data classes using RBF network Header File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,20 +49,20 @@ class FormationRBF
     : public Formation {
 public:
 
+    static const std::string NAME; //!< type name
+
     /*!
       \brief formation parameter using RBF network.
       one Param instance realizes just one player's position.
     */
     class Param {
     public:
-        static const double PITCH_LENGTH;
-        static const double PITCH_WIDTH;
+        static const double PITCH_LENGTH; //!< field length
+        static const double PITCH_WIDTH; //!< field width
 
     private:
-
-        std::string M_role_name;
-
-        RBFNetwork M_net;
+        std::string M_role_name; //!< role name string
+        RBFNetwork M_net; //!< RBFnetwork instance
 
     public:
 
@@ -103,7 +103,7 @@ public:
         /*!
           \brief get strategic position
           \param ball_pos focus point, usually ball position.
-          \param type side type, if this is mirror, the refered param is used.
+          \param type side type, if this is symmetry, the refered param is used.
           \return coordinate value
         */
         Vector2D getPosition( const Vector2D & ball_pos,
@@ -166,7 +166,7 @@ public:
 
 private:
 
-    //! key: unum. but size is not always 11 if mirror player exists.
+    //! key: unum. but size is not always 11 if symmetric player exists.
     std::map< int, boost::shared_ptr< Param > > M_param_map;
 
 public:
@@ -184,7 +184,8 @@ public:
     static
     std::string name()
       {
-          return std::string( "RBF" );
+          //return std::string( "RBF" );
+          return NAME;
       }
 
     /*!
@@ -192,13 +193,20 @@ public:
       \return pointer to the new object
     */
     static
-    Formation * create()
+    Formation::Ptr create()
       {
-          return ( new FormationRBF );
+          return Formation::Ptr( new FormationRBF() );
       }
 
     //--------------------------------------------------------------
 
+    /*!
+      \brief create default formation. assign role and initial positions.
+      \return snapshow variable for the initial stat(ball pos=(0,0)).
+    */
+    virtual
+    void createDefaultData();
+
     /*!
       \brief get the name of this formation
       \return name string
@@ -209,13 +217,6 @@ public:
           return FormationRBF::name();
       }
 
-    /*!
-      \brief create default formation. assign role and initial positions.
-      \return snapshow variable for the initial stat(ball pos=(0,0)).
-    */
-    virtual
-    Snapshot createDefaultParam();
-
 protected:
 
     /*!
@@ -252,12 +253,13 @@ public:
     /*!
       \brief get position for the current focus point
       \param unum player number
-      \param forcus_point current focus point, usually ball position.
+      \param focus_point current focus point, usually ball position.
     */
     virtual
     Vector2D getPosition( const int unum,
                           const Vector2D & focus_point ) const;
-    /*
+
+    /*!
       \brief get all positions for the current focus point
       \param focus_point current focus point, usually ball position
       \param positions contaner to store the result
@@ -268,25 +270,27 @@ public:
 
     /*!
       \brief update formation paramter using training data set
-      \param train_data training data container
     */
     virtual
-    void train( const std::list< Snapshot > & train_data );
+    void train();
+
+protected:
 
     /*!
-      \brief restore data from the input stream.
+      \brief restore conf data from the input stream.
       \param is reference to the input stream.
-      \return parsing result
+      \return pasing result.
     */
-    bool read( std::istream & is );
+    virtual
+    bool readConf( std::istream & is );
 
     /*!
       \brief put data to the output stream.
       \param os reference to the output stream
       \return reference to the output stream
     */
-    std::ostream & print( std::ostream & os ) const;
-
+    virtual
+    std::ostream & printConf( std::ostream & os ) const;
 
 private:
 
