@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,9 +32,9 @@
 #ifndef RCSC_GEOM_ANGLEDEG_H
 #define RCSC_GEOM_ANGLEDEG_H
 
+#include <functional>
 #include <iostream>
 #include <cmath>
-#include <functional>
 
 namespace rcsc {
 
@@ -59,6 +59,10 @@ public:
     //! epsilon value
     static const double EPSILON;
 
+    //! pi valur
+    static const double PI;
+    //! 2*pi valur
+    static const double TWO_PI;
     //! constant variable to convert DEGREE to RADIAN.
     static const double DEG2RAD;
     //! constant variable to convert RADIAN to DEGREE.
@@ -101,14 +105,21 @@ private:
     const
     AngleDeg & normalize()
       {
-          while ( M_degree < -180.0 )
+          if ( M_degree < -360.0 || 360.0 < M_degree )
+          {
+              M_degree = std::fmod( M_degree, 360.0 );
+          }
+
+          if ( M_degree < -180.0 )
           {
               M_degree += 360.0;
           }
-          while ( M_degree > 180.0 )
+
+          if ( M_degree > 180.0 )
           {
               M_degree -= 360.0;
           }
+
           return *this;
       }
 
@@ -351,14 +362,21 @@ public:
     static
     double normalize_angle( double dir )
       {
-          while ( dir < -180.0 )
+          if ( dir < -360.0 || 360.0 < dir )
+          {
+              dir = std::fmod( dir, 360.0 );
+          }
+
+          if ( dir < -180.0 )
           {
               dir += 360.0;
           }
-          while ( dir > 180.0 )
+
+          if ( dir > 180.0 )
           {
               dir -= 360.0;
           }
+
           return dir;
       }
 
@@ -737,8 +755,8 @@ operator==( const rcsc::AngleDeg & lhs,
  */
 inline
 bool
-operator==( const rcsc::AngleDeg& lhs,
-            const double& rhs )
+operator==( const rcsc::AngleDeg & lhs,
+            const double & rhs )
 {
     return std::fabs( lhs.degree() - rhs ) < rcsc::AngleDeg::EPSILON;
 }
