@@ -520,7 +520,7 @@ DelaunayTriangulation::updateContainedVertex( const Vertex * new_vertex,
         new_tri[i] = createTriangle( old_edges[i],
                                      new_edges[ edge_pairs[i].first ],
                                      new_edges[ edge_pairs[i].second ] );
-        if ( ! new_tri[i]->circumcenter().valid() )
+        if ( ! new_tri[i]->circumcenter().isValid() )
         {
             std::cerr << __FILE__ << ':' << __LINE__
                       << " updateContainedVertex() detect illegal vertex\n"
@@ -659,7 +659,7 @@ DelaunayTriangulation::updateOnlineVertex( const Vertex * new_vertex,
             new_tri_in_tri[i] = createTriangle( old_edge_in_tri[i],
                                                 new_edge[i],
                                                 new_edge_in_tri );
-            if ( ! new_tri_in_tri[i]->circumcenter().valid() )
+            if ( ! new_tri_in_tri[i]->circumcenter().isValid() )
             {
                 std::cerr << __FILE__ << ':' << __LINE__
                           << " updateOnlineVertex() detect illegal vertex normal."
@@ -733,7 +733,7 @@ DelaunayTriangulation::updateOnlineVertex( const Vertex * new_vertex,
             new_tri_in_adjacent[i] = createTriangle( old_edge_in_adjacent[i],
                                                      new_edge[i],
                                                      new_edge_in_adjacent );
-            if ( ! new_tri_in_adjacent[i]->circumcenter().valid() )
+            if ( ! new_tri_in_adjacent[i]->circumcenter().isValid() )
             {
                 std::cerr << __FILE__ << ':' << __LINE__
                           << " updateOnlineVertex() detect illegal vertex adjacent"
@@ -896,7 +896,7 @@ DelaunayTriangulation::legalizeEdge( TrianglePtr new_tri,
         flipped_tri[i] = createTriangle( new_edge,
                                          edge_in_new_tri[i],
                                          edge_in_adjacent[i] );
-        if ( ! flipped_tri[i]->circumcenter().valid() )
+        if ( ! flipped_tri[i]->circumcenter().isValid() )
         {
             std::cerr << __FILE__ << ':' << __LINE__
                       << " legalizeEdge() detect illegal vertex \n"
