@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -511,6 +511,10 @@ public:
 
     ////////////////////////////////////////////////////////////////
 
+    typedef std::vector< Vertex > VertexCont; //!< vertex container type
+    typedef std::map< int, EdgePtr > EdgeCont; //!< edge pointer container type
+    typedef std::map< int, TrianglePtr > TriangleCont; //!< triangle pointer container type
+
 private:
 
     //! counter to set Id to edges
@@ -525,13 +529,13 @@ private:
     EdgePtr M_initial_edge[3];
 
     //! instance of vertices. these are refered by edge and triangle.
-    std::vector< Vertex > M_vertices;
+    VertexCont M_vertices;
 
     //! edge instance holder. key: id
-    std::map< int, EdgePtr > M_edge_map;
+    EdgeCont M_edges;
 
     //! triangle instance holder. key: id
-    std::map< int, TrianglePtr > M_triangle_map;
+    TriangleCont M_triangles;
 
     // not used
     DelaunayTriangulation & operator=( const DelaunayTriangulation & );
@@ -562,36 +566,6 @@ public:
      */
     ~DelaunayTriangulation();
 
-    /*!
-      \brief get vertices
-      \return const reference to the vertices container
-     */
-    const
-    std::vector< Vertex > & vertices() const
-      {
-          return M_vertices;
-      }
-
-    /*!
-      \brief get edge set
-      \return const referenct to the map container. key=id, value=raw pointer
-     */
-    const
-    std::map< int, EdgePtr > & edgeMap() const
-      {
-          return M_edge_map;
-      }
-
-    /*!
-      \brief get triangle set
-      \return const referenct to the map container. key=id, value=raw pointer
-     */
-    const
-    std::map< int, TrianglePtr > & triangleMap() const
-      {
-          return M_triangle_map;
-      }
-
     /*!
       \brief initialize with target field rectangle data.
       All data are cleared.
@@ -608,6 +582,36 @@ public:
      */
     void clear();
 
+    /*!
+      \brief get vertices
+      \return const reference to the vertices container
+     */
+    const
+    VertexCont & vertices() const
+      {
+          return M_vertices;
+      }
+
+    /*!
+      \brief get edge set
+      \return const referenct to the map container. key=id, value=raw pointer
+     */
+    const
+    EdgeCont & edges() const
+      {
+          return M_edges;
+      }
+
+    /*!
+      \brief get triangle set
+      \return const referenct to the map container. key=id, value=raw pointer
+     */
+    const
+    TriangleCont & triangles() const
+      {
+          return M_triangles;
+      }
+
     /*!
       \brief add new vertex
       \param x coordinate x
@@ -632,6 +636,12 @@ public:
           return addVertex( p.x, p.y );
       }
 
+    /*!
+      \brief set vertices.
+      \param v container of vertices.
+     */
+    void addVertices( const std::vector< Vector2D > & v );
+
     /*!
       \brief get the const pointer to vertex specified by Id number.
       \param id wanted vertex Id number.
@@ -722,11 +732,11 @@ private:
      */
     void removeEdge( int id )
       {
-          std::map< int, EdgePtr >::iterator it = M_edge_map.find( id );
-          if ( it != M_edge_map.end() )
+          EdgeCont::iterator it = M_edges.find( id );
+          if ( it != M_edges.end() )
           {
               delete it->second;
-              M_edge_map.erase( it );
+              M_edges.erase( it );
           }
       }
 
@@ -748,8 +758,8 @@ private:
      */
     void removeTriangle( int id )
       {
-          std::map< int, TrianglePtr >::iterator it = M_triangle_map.find( id );
-          if ( it != M_triangle_map.end() )
+          TriangleCont::iterator it = M_triangles.find( id );
+          if ( it != M_triangles.end() )
           {
               //std::cout << "remove triangle " << id
               //          << it->second->vertex( 0 )->pos()
@@ -757,7 +767,7 @@ private:
               //          << it->second->vertex( 2 )->pos()
               //          << std::endl;
               delete it->second;
-              M_triangle_map.erase( it );
+              M_triangles.erase( it );
           }
       }
 
@@ -783,7 +793,7 @@ private:
                         const Vertex * v1 )
       {
           EdgePtr ptr = new Edge( M_edge_count++, v0, v1 );
-          M_edge_map.insert( std::pair< int, EdgePtr >( ptr->id(), ptr ) );
+          M_edges.insert( EdgeCont::value_type( ptr->id(), ptr ) );
           return ptr;
       }
 
@@ -800,7 +810,7 @@ private:
       {
           // triangle is set to edges in the constructor of Triangle
           TrianglePtr ptr = new Triangle( M_tri_count++, e0, e1, e2 );
-          M_triangle_map.insert( std::pair< int, TrianglePtr >( ptr->id(), ptr ) );
+          M_triangles.insert( TriangleCont::value_type( ptr->id(), ptr ) );
           return ptr;
       }
 
