@@ -42,6 +42,110 @@
 
 namespace rcsc {
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+const
+Rect2D &
+Rect2D::setTopLeft( const double & x,
+                    const double & y )
+{
+    double new_left = std::min( right(), x );
+    double new_right = std::max( right(), x );
+    double new_top = std::min( bottom(), y );
+    double new_bottom = std::max( bottom(), y );
+
+    return assign( new_left,
+                   new_top,
+                   new_right - new_left,
+                   new_bottom - new_top );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+const
+Rect2D &
+Rect2D::setBottomRight( const double & x,
+                        const double & y )
+{
+    double new_left = std::min( left(), x );
+    double new_right = std::max( left(), x );
+    double new_top = std::min( top(), y );
+    double new_bottom = std::max( top(), y );
+
+    return assign( new_left,
+                   new_top,
+                   new_right - new_left,
+                   new_bottom - new_top );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+const
+Rect2D &
+Rect2D::setLeft( const double & x )
+{
+    double new_left = std::min( right(), x );
+    double new_right = std::max( right(), x );
+
+    M_top_left.x = new_left;
+    M_size.setLength( new_right - new_left );
+    return *this;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+const
+Rect2D &
+Rect2D::setRight( const double & x )
+{
+    double new_left = std::min( left(), x );
+    double new_right = std::max( left(), x );
+
+    M_top_left.x = new_left;
+    M_size.setLength( new_right - new_left );
+    return *this;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+const
+Rect2D &
+Rect2D::setTop( const double & y )
+{
+    double new_top = std::min( bottom(), y );
+    double new_bottom = std::max( bottom(), y );
+
+    M_top_left.y = new_top;
+    M_size.setWidth( new_bottom - new_top );
+    return *this;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+const
+Rect2D &
+Rect2D::setBottom( const double & y )
+{
+    double new_top = std::min( top(), y );
+    double new_bottom = std::max( top(), y );
+
+    M_top_left.y = new_top;
+    M_size.setWidth( new_bottom - new_top );
+    return *this;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -60,28 +164,28 @@ Rect2D::intersection( const Line2D & line,
     const double bottom_y = bottom();
 
     if ( n_sol < 2
-         && ( tsol[n_sol] = leftEdge().intersection( line ) ).valid()
+         && ( tsol[n_sol] = leftEdge().intersection( line ) ).isValid()
          && top_y <= tsol[n_sol].y && tsol[n_sol].y <= bottom_y )
     {
         ++n_sol;
     }
 
     if ( n_sol < 2
-         && ( tsol[n_sol] = rightEdge().intersection( line ) ).valid()
+         && ( tsol[n_sol] = rightEdge().intersection( line ) ).isValid()
          && top_y <= tsol[n_sol].y && tsol[n_sol].y <= bottom_y )
     {
         ++n_sol;
     }
 
     if ( n_sol < 2
-         && ( tsol[n_sol] = topEdge().intersection( line ) ).valid()
+         && ( tsol[n_sol] = topEdge().intersection( line ) ).isValid()
          && left_x <= tsol[n_sol].x && tsol[n_sol].x <= right_x )
     {
         ++n_sol;
     }
 
     if ( n_sol < 2
-         && ( tsol[n_sol] = bottomEdge().intersection( line ) ).valid()
+         && ( tsol[n_sol] = bottomEdge().intersection( line ) ).isValid()
          && left_x <= tsol[n_sol].x && tsol[n_sol].x <= right_x )
     {
         ++n_sol;
@@ -197,8 +301,8 @@ const
 Rect2D &
 Rect2D::operator&=( const Rect2D & other )
 {
-    if ( ! this->valid()
-         || ! other.valid() )
+    if ( ! this->isValid()
+         || ! other.isValid() )
     {
         M_top_left.assign( 0.0, 0.0 );
         M_size.assign( 0.0, 0.0 );
@@ -230,8 +334,8 @@ const
 Rect2D &
 Rect2D::operator|=( const Rect2D & other )
 {
-    if ( ! this->valid()
-         && ! other.valid() )
+    if ( ! this->isValid()
+         && ! other.isValid() )
     {
         M_top_left.assign( 0.0, 0.0 );
         M_size.assign( 0.0, 0.0 );
