@@ -254,45 +254,276 @@ public:
       }
 
     /*!
-      \brief set a new top left corner point
-      \param x new x coordinate
-      \param y new y coordinate
+      \brief move the rectangle.
+      the center point is set to the given position.
+      the size is unchanged.
+      \param point new center coordinates
       \return const referenct to itself
      */
     const
-    Rect2D & setTopLeft( const double & x,
-                         const double & y )
+    Rect2D & moveCenter( const Vector2D & point )
       {
-          M_top_left.assign( x, y );
+          M_top_left.assign( point.x - M_size.length() * 0.5,
+                             point.y - M_size.width() * 0.5 );
           return *this;
       }
 
     /*!
-      \brief set a new top left corner point
-      \param point new coordinate
+      \brief move the rectangle.
+      the top-left coner is set to the given position.
+      the size is unchanged.
+      \param point new top-left corner
       \return const referenct to itself
      */
     const
-    Rect2D & setTopLeft( const Vector2D & point )
+    Rect2D & moveTopLeft( const Vector2D & point )
       {
           M_top_left = point;
           return *this;
       }
 
     /*!
-      \brief set a new center point. only top left corner is moved.
-      size is not changed.
-      \param point new center coordinate
+      \brief move the rectangle.
+      the bottom-right coner is set to the given position.
+      the size is unchanged.
+      \param point new bottom-right conrer
       \return const referenct to itself
      */
     const
-    Rect2D & setCenter( const Vector2D & point )
+    Rect2D & moveBottomRight( const Vector2D & point )
       {
-          M_top_left.assign( point.x - M_size.length() * 0.5,
-                             point.y - M_size.width() * 0.5 );
+          M_top_left.assign( point.x - M_size.length(),
+                             point.y - M_size.width() );
           return *this;
       }
 
+    /*!
+      \brief move the rectangle.
+      the left line is set to the given position.
+      the size is unchanged.
+      \param x new left value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveLeft( const double & x )
+      {
+          M_top_left.x = x;
+          return *this;
+      }
+
+    /*!
+      \brief alias of moveLeft.
+      \param x new left value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveMinX( const double & x )
+      {
+          return moveLeft( x );
+      }
+
+    /*!
+      \brief move the rectangle.
+      the right line is set to the given value.
+      the size is unchanged.
+      \param x new right value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveRight( const double & x )
+      {
+          M_top_left.x = x - M_size.length();
+          return *this;
+      }
+
+    /*!
+      \brief alias of moveRight.
+      \param x new right value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveMaxX( const double & x )
+      {
+          return moveRight( x );
+      }
+
+    /*!
+      \brief move the rectangle.
+      the top line is set to the given value.
+      the size is unchanged.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveTop( const double & y )
+      {
+          M_top_left.y = y;
+          return *this;
+      }
+
+    /*!
+      \brief alias of moveTop.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveMinY( const double & y )
+      {
+          return moveTop( y );
+      }
+
+    /*!
+      \brief move the rectangle.
+      the top line is set to the given value.
+      the size is unchanged.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveBottom( const double & y )
+      {
+          M_top_left.y = y - M_size.width();
+          return *this;
+      }
+
+    /*!
+      \brief alias of moveTop.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & moveMaxY( const double & y )
+      {
+          return moveBottom( y );
+      }
+
+
+    /*!
+      \brief set the top-left corner of the rectangle.
+      the size may be changed, but the bottom-right corner will never be changed.
+      \param x new x coordinate
+      \param y new y coordinate
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setTopLeft( const double & x,
+                         const double & y );
+
+    /*!
+      \brief set the top-left corner of the rectangle.
+      the size may be changed, but the bottom-right corner will never be changed.
+      \param point new coordinate
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setTopLeft( const Vector2D & point )
+      {
+          return setTopLeft( point.x, point.y );
+      }
+
+    /*!
+      \brief set the bottom-right corner of the rectangle.
+      the size may be changed, but the top-left corner will never be changed.
+      \param x new x coordinate
+      \param y new y coordinate
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setBottomRight( const double & x,
+                             const double & y );
+
+    /*!
+      \brief set the bottom-right corner of the rectangle.
+      the size may be changed, but the top-left corner will never be changed.
+      \param point new coordinate
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setBottomRight( const Vector2D & point )
+      {
+          return setBottomRight( point.x, point.y );
+      }
+
+    /*!
+      \brief set the left of rectangle.
+      the size may be changed, but the right will never be changed.
+      \param x new left value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setLeft( const double & x );
+
+    /*!
+      \brief alias of setLeft.
+      \param x new left value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setMinX( const double & x )
+      {
+          return setLeft( x );
+      }
+
+    /*!
+      \brief set the right of rectangle.
+      the size may be changed, but the left will never be changed.
+      \param x new right value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setRight( const double & x );
+
+    /*!
+      \brief alias of setRight.
+      \param x new right value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setMaxX( const double & x )
+      {
+          return setRight( x );
+      }
+
+    /*!
+      \brief set the top of rectangle.
+      the size may be changed, but the bottom will never be changed.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setTop( const double & y );
+
+    /*!
+      \brief alias of setTop.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setMinY( const double & y )
+      {
+          return setTop( y );
+      }
+
+    /*!
+      \brief set the bottom of rectangle.
+      the size may be changed, but the top will never be changed.
+      \param y new bottom value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setBottom( const double & y );
+
+    /*!
+      \brief alias of setBottom.
+      \param y new top value
+      \return const referenct to itself
+     */
+    const
+    Rect2D & setMaxY( const double & y )
+      {
+          return setBottom( y );
+      }
+
     /*!
       \brief set a new x-range
       \param length new range
@@ -344,18 +575,8 @@ public:
       }
 
     /*!
-      \brief check if this rectangle is empty or not.  equivalent to isValid()
-      \return true if the size of this rectangle is 0.
-     */
-    bool valid() const
-      {
-          return M_size.length() > 0.0
-              && M_size.width() > 0.0;
-      }
-
-    /*!
-      \brief check if this rectangle is empty or not. equivalent to valid()
-      \return true if the size of this rectangle is 0.
+      \brief check if this rectangle is valid or not.
+      \return true if the area of this rectangle is not 0.
      */
     bool isValid() const
       {
