@@ -94,7 +94,7 @@ Segment2D::intersection( const Segment2D & other,
 {
     Vector2D sol = this->line().intersection( other.line() );
 
-    if ( ! sol.valid()
+    if ( ! sol.isValid()
          || ! this->contains( sol )
          || ! other.contains( sol ) )
     {
@@ -152,7 +152,7 @@ Segment2D::intersection( const Line2D & other ) const
 
     Vector2D tmp_sol = my_line.intersection( other );
 
-    if ( ! tmp_sol.valid()
+    if ( ! tmp_sol.isValid()
          || ! this->contains( tmp_sol ) )
     {
         return Vector2D::INVALIDATED;
@@ -323,8 +323,8 @@ Segment2D::dist( const Vector2D & p ) const
         return std::fabs( Triangle2D::double_signed_area( origin(), terminal(), p ) / len );
     }
 
-    return std::min( origin().dist( p ),
-                     terminal().dist( p ) );
+    return std::sqrt( std::min( origin().dist2( p ),
+                                terminal().dist2( p ) ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -352,8 +352,8 @@ Segment2D::dist( const Segment2D & seg ) const
 double
 Segment2D::farthestDist( const Vector2D & p ) const
 {
-    return std::max( origin().dist( p ),
-                     terminal().dist( p ) );
+    return std::sqrt( std::max( origin().dist2( p ),
+                                terminal().dist2( p ) ) );
 }
 
 /*-------------------------------------------------------------------*/
