@@ -1,7 +1,7 @@
 // -*-c++-*-
 
 /*!
-  \file polygon_2d_test.cpp
+  \file test_polygon_2d.cpp
   \brief test code for rcsc::Polygon2D
 */
 
@@ -551,7 +551,7 @@ Polygon2DTest::testXYCenter()
     const rcsc::Polygon2D r( rect );
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL( +100.0 - r.area(), 0.0, EPS );
-    CPPUNIT_ASSERT_DOUBLES_EQUAL( +200.0 - r.signedArea2(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL( +200.0 - r.doubleSignedArea(), 0.0, EPS );
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL( rcsc::Vector2D( 15.0, 15.0 ).dist( r.xyCenter() ), 0.0, EPS );
 }
@@ -564,7 +564,7 @@ void
 Polygon2DTest::testSignedArea2()
 {
     //
-    // counter clockwise/clockwise, signedArea2
+    // counter clockwise/clockwise, doubleSignedArea
     //
     std::vector< rcsc::Vector2D > points;
     const rcsc::Polygon2D empty(points);
@@ -581,11 +581,11 @@ Polygon2DTest::testSignedArea2()
     points.push_back( rcsc::Vector2D( 10, 20 ) );
     const rcsc::Polygon2D rectangle(points);
 
-    CPPUNIT_ASSERT_DOUBLES_EQUAL(    0.0 - empty    .signedArea2(), 0.0, EPS );
-    CPPUNIT_ASSERT_DOUBLES_EQUAL(    0.0 - point    .signedArea2(), 0.0, EPS );
-    CPPUNIT_ASSERT_DOUBLES_EQUAL(    0.0 - line     .signedArea2(), 0.0, EPS );
-    CPPUNIT_ASSERT_DOUBLES_EQUAL( +100.0 - triangle .signedArea2(), 0.0, EPS );
-    CPPUNIT_ASSERT_DOUBLES_EQUAL( +200.0 - rectangle.signedArea2(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(    0.0 - empty    .doubleSignedArea(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(    0.0 - point    .doubleSignedArea(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(    0.0 - line     .doubleSignedArea(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL( +100.0 - triangle .doubleSignedArea(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL( +200.0 - rectangle.doubleSignedArea(), 0.0, EPS );
 
 
     CPPUNIT_ASSERT_EQUAL( false, empty.isCounterclockwise() );
@@ -613,8 +613,8 @@ Polygon2DTest::testSignedArea2()
     r_points.push_back( rcsc::Vector2D( 10, 10 ) );
     const rcsc::Polygon2D r_rectangle(r_points);
 
-    CPPUNIT_ASSERT_DOUBLES_EQUAL( -100.0 - r_triangle .signedArea2(), 0.0, EPS );
-    CPPUNIT_ASSERT_DOUBLES_EQUAL( -200.0 - r_rectangle.signedArea2(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL( -100.0 - r_triangle .doubleSignedArea(), 0.0, EPS );
+    CPPUNIT_ASSERT_DOUBLES_EQUAL( -200.0 - r_rectangle.doubleSignedArea(), 0.0, EPS );
 
     CPPUNIT_ASSERT_EQUAL( false, r_triangle.isCounterclockwise() );
     CPPUNIT_ASSERT_EQUAL( true , r_triangle.isClockwise() );
