@@ -1,7 +1,7 @@
 // -*-c++-*-
 
 /*!
-  \file segment_2d_test.cpp
+  \file test_segment_2d.cpp
   \brief test code for rcsc::Segment2D
 */
 
