@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,6 +67,138 @@ Triangle2D::contains( const Vector2D & point ) const
     return false;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+int
+Triangle2D::intersection( const Line2D & line,
+                          Vector2D * sol1,
+                          Vector2D * sol2 ) const
+{
+    int n_sol = 0;
+    Vector2D tsol[2];
+
+    if ( n_sol < 2
+         && ( tsol[n_sol] = Segment2D( a(), b() ).intersection( line ) ).isValid() )
+    {
+        ++n_sol;
+    }
+
+    if ( n_sol < 2
+         && ( tsol[n_sol] = Segment2D( b(), c() ).intersection( line ) ).isValid() )
+    {
+        ++n_sol;
+    }
+
+    if ( n_sol < 2
+         && ( tsol[n_sol] = Segment2D( c(), a() ).intersection( line ) ).isValid() )
+    {
+        ++n_sol;
+    }
+
+    if ( n_sol == 2
+         && std::fabs( tsol[0].x - tsol[1].x ) < 1.0e-5
+         && std::fabs( tsol[0].y - tsol[1].y ) < 1.0e-5 )
+    {
+        n_sol = 1;
+    }
+
+    if ( n_sol > 0
+         && sol1 )
+    {
+        *sol1 = tsol[0];
+    }
+
+    if ( n_sol > 1
+         && sol2 )
+    {
+        *sol2 = tsol[1];
+    }
+
+    return n_sol;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+int
+Triangle2D::intersection( const Ray2D & ray,
+                          Vector2D * sol1,
+                          Vector2D * sol2 ) const
+{
+    Vector2D tsol1, tsol2;
+    int n_sol = intersection( ray.line(), &tsol1, &tsol2 );
+
+    if ( n_sol > 1
+         && ! ray.inRightDir( tsol2, 1.0 ) )
+    {
+        --n_sol;
+    }
+
+    if ( n_sol > 0
+         && ! ray.inRightDir( tsol1, 1.0 ) )
+    {
+        tsol1 = tsol2;
+        --n_sol;
+    }
+
+    if ( n_sol > 0
+         && sol1 )
+    {
+        *sol1 = tsol1;
+    }
+
+    if ( n_sol > 1
+         && sol2 )
+    {
+        *sol2 = tsol2;
+    }
+
+    return n_sol;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+int
+Triangle2D::intersection( const Segment2D & segment,
+                          Vector2D * sol1,
+                          Vector2D * sol2 ) const
+{
+    Vector2D tsol1, tsol2;
+    int n_sol = intersection( segment.line(), &tsol1, &tsol2 );
+
+    if ( n_sol > 1
+         && ! segment.contains( tsol2 ) )
+    {
+        --n_sol;
+    }
+
+    if ( n_sol > 0
+         && ! segment.contains( tsol1 ) )
+    {
+        tsol1 = tsol2;
+        --n_sol;
+    }
+
+    if ( n_sol > 0
+         && sol1 )
+    {
+        *sol1 = tsol1;
+    }
+
+    if ( n_sol > 1
+         && sol2 )
+    {
+        *sol2 = tsol2;
+    }
+
+    return n_sol;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -177,136 +309,4 @@ Triangle2D::orthocenter( const Vector2D & a,
     return perpend_a.intersection( perpend_b );
 }
 
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-int
-Triangle2D::intersection( const Line2D & line,
-                          Vector2D * sol1,
-                          Vector2D * sol2 ) const
-{
-    int n_sol = 0;
-    Vector2D tsol[2];
-
-    if ( n_sol < 2
-         && ( tsol[n_sol] = Segment2D( a(), b() ).intersection( line ) ).valid() )
-    {
-        ++n_sol;
-    }
-
-    if ( n_sol < 2
-         && ( tsol[n_sol] = Segment2D( b(), c() ).intersection( line ) ).valid() )
-    {
-        ++n_sol;
-    }
-
-    if ( n_sol < 2
-         && ( tsol[n_sol] = Segment2D( c(), a() ).intersection( line ) ).valid() )
-    {
-        ++n_sol;
-    }
-
-    if ( n_sol == 2
-         && std::fabs( tsol[0].x - tsol[1].x ) < 1.0e-5
-         && std::fabs( tsol[0].y - tsol[1].y ) < 1.0e-5 )
-    {
-        n_sol = 1;
-    }
-
-    if ( n_sol > 0
-         && sol1 )
-    {
-        *sol1 = tsol[0];
-    }
-
-    if ( n_sol > 1
-         && sol2 )
-    {
-        *sol2 = tsol[1];
-    }
-
-    return n_sol;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-int
-Triangle2D::intersection( const Ray2D & ray,
-                          Vector2D * sol1,
-                          Vector2D * sol2 ) const
-{
-    Vector2D tsol1, tsol2;
-    int n_sol = intersection( ray.line(), &tsol1, &tsol2 );
-
-    if ( n_sol > 1
-         && ! ray.inRightDir( tsol2, 1.0 ) )
-    {
-        --n_sol;
-    }
-
-    if ( n_sol > 0
-         && ! ray.inRightDir( tsol1, 1.0 ) )
-    {
-        tsol1 = tsol2;
-        --n_sol;
-    }
-
-    if ( n_sol > 0
-         && sol1 )
-    {
-        *sol1 = tsol1;
-    }
-
-    if ( n_sol > 1
-         && sol2 )
-    {
-        *sol2 = tsol2;
-    }
-
-    return n_sol;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-int
-Triangle2D::intersection( const Segment2D & segment,
-                          Vector2D * sol1,
-                          Vector2D * sol2 ) const
-{
-    Vector2D tsol1, tsol2;
-    int n_sol = intersection( segment.line(), &tsol1, &tsol2 );
-
-    if ( n_sol > 1
-         && ! segment.contains( tsol2 ) )
-    {
-        --n_sol;
-    }
-
-    if ( n_sol > 0
-         && ! segment.contains( tsol1 ) )
-    {
-        tsol1 = tsol2;
-        --n_sol;
-    }
-
-    if ( n_sol > 0
-         && sol1 )
-    {
-        *sol1 = tsol1;
-    }
-
-    if ( n_sol > 1
-         && sol2 )
-    {
-        *sol2 = tsol2;
-    }
-
-    return n_sol;
-}
-
 }
