@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -76,8 +76,8 @@ public:
     */
     Triangle2D( const Segment2D & seg,
                 const Vector2D & v )
-        : M_a( seg.a() )
-        , M_b( seg.b() )
+        : M_a( seg.origin() )
+        , M_b( seg.terminal() )
         , M_c( v )
       { }
 
@@ -99,6 +99,20 @@ public:
           return *this;
       }
 
+    /*!
+      \brief check if this triangle is valid or not.
+      \return true if triangle is valid.
+    */
+    bool isValid() const
+      {
+          return M_a.isValid()
+              && M_b.isValid()
+              && M_c.isValid()
+              && M_a != M_b
+              && M_b != M_c
+              && M_a != M_a;
+      }
+
     /*!
       \brief assign new segment and vertex point
       \param seg segment consist of triangle, first and second vertex points
@@ -109,8 +123,8 @@ public:
     Triangle2D & assign( const Segment2D & seg,
                          const Vector2D & v )
       {
-          M_a = seg.a();
-          M_b = seg.b();
+          M_a = seg.origin();
+          M_b = seg.terminal();
           M_c = v;
           return *this;
       }
@@ -158,28 +172,30 @@ public:
       }
 
     /*!
-      \brief get area with sign
-      \return value of area with sign.
+      \brief get a signed area. this method is equivalent to signed_area().
+      \return signed area value
       If points a, b, c are placed counterclockwise order, returns positive number.
       If points a, b, c are placed clockwise order, returns negative number.
       If points a, b, c are placed on a line, returns 0.
      */
     double signedArea() const
       {
-          return signedArea2() / 2.0;
+          //return doubleSignedArea() / 2.0;
+          return signed_area( a(), b(), c() );
       }
 
     /*!
-      \brief get twice of signed area
-      \return twice of signed area
+      \brief get a double of signed area value. this method is equivalent to double_signed_area().
+      \return double of signed area value
       If points a, b, c are placed counterclockwise order, returns positive number.
       If points a, b, c are placed clockwise order, returns negative number.
       If points a, b, c are placed on a line, returns 0.
      */
-    double signedArea2() const
+    double doubleSignedArea() const
       {
-          return(   (a().x - c().x) * (b().y - c().y)
-                  + (b().x - c().x) * (c().y - a().y) );
+          //return ( ( a().x - c().x ) * ( b().y - c().y )
+          //         + ( b().x - c().x ) * ( c().y - a().y ) );
+          return double_signed_area( a(), b(), c() );
       }
 
     /*!
@@ -230,8 +246,74 @@ public:
     //return coordinates of excenter
     //Vector2D excenter() const;
 
+    /*!
+      \brief calculate intersection point with line.
+      \param line considerd line.
+      \param sol1 pointer to the 1st solution variable
+      \param sol2 pointer to the 2nd solution variable
+      \return number of intersection
+    */
+    int intersection( const Line2D & line,
+                      Vector2D * sol1,
+                      Vector2D * sol2 ) const;
 
-    // static utility
+    /*!
+      \brief calculate intersection point with ray.
+      \param ray considerd ray line.
+      \param sol1 pointer to the 1st solution variable
+      \param sol2 pointer to the 2nd solution variable
+      \return number of intersection
+    */
+    int intersection( const Ray2D & ray,
+                      Vector2D * sol1,
+                      Vector2D * sol2 ) const;
+
+    /*!
+      \brief calculate intersection point with line segment.
+      \param segment considerd line segment.
+      \param sol1 pointer to the 1st solution variable
+      \param sol2 pointer to the 2nd solution variable
+      \return number of intersection
+    */
+    int intersection( const Segment2D & segment,
+                      Vector2D * sol1,
+                      Vector2D * sol2 ) const;
+
+
+    //
+    // static methods
+    //
+
+    /*!
+      \brief get a signed area value
+      \return value of area with sign.
+      If points a, b, c are placed counterclockwise order, returns positive number.
+      If points a, b, c are placed clockwise order, returns negative number.
+      If points a, b, c are placed on a line, returns 0.
+     */
+    static
+    double double_signed_area( const Vector2D & a,
+                               const Vector2D & b,
+                               const Vector2D & c )
+      {
+          return ( ( a.x - c.x ) * ( b.y - c.y )
+                   + ( b.x - c.x ) * ( c.y - a.y ) );
+      }
+
+    /*!
+      \brief get a double of signed area
+      \return double of signed area
+      If points a, b, c are placed counterclockwise order, returns positive number.
+      If points a, b, c are placed clockwise order, returns negative number.
+      If points a, b, c are placed on a line, returns 0.
+     */
+    static
+    double signed_area( const Vector2D & a,
+                        const Vector2D & b,
+                        const Vector2D & c )
+      {
+          return double_signed_area( a, b, c ) * 0.5;
+      }
 
     /*!
       \brief get the center of gravity(JUU-SIN)
@@ -247,11 +329,11 @@ public:
                        const Vector2D & b,
                        const Vector2D & c )
       {
-//           Vector2D g( a );
-//           g += b;
-//           g += c;
-//           g /= 3.0;
-//           return g;
+          //Vector2D g( a );
+          //g += b;
+          //g += c;
+          //g /= 3.0;
+          //return g;
           return Vector2D( a ).add( b ).add( c ) /= 3.0;
       }
 
@@ -293,39 +375,6 @@ public:
                           const Vector2D & b,
                           const Vector2D & c );
 
-    /*!
-      \brief calculate intersection point with line.
-      \param line considerd line.
-      \param sol1 pointer to the 1st solution variable
-      \param sol2 pointer to the 2nd solution variable
-      \return number of intersection
-    */
-    int intersection( const Line2D & line,
-                      Vector2D * sol1,
-                      Vector2D * sol2 ) const;
-
-    /*!
-      \brief calculate intersection point with ray.
-      \param ray considerd ray line.
-      \param sol1 pointer to the 1st solution variable
-      \param sol2 pointer to the 2nd solution variable
-      \return number of intersection
-    */
-    int intersection( const Ray2D & ray,
-                      Vector2D * sol1,
-                      Vector2D * sol2 ) const;
-
-    /*!
-      \brief calculate intersection point with line segment.
-      \param segment considerd line segment.
-      \param sol1 pointer to the 1st solution variable
-      \param sol2 pointer to the 2nd solution variable
-      \return number of intersection
-    */
-    int intersection( const Segment2D & segment,
-                      Vector2D * sol1,
-                      Vector2D * sol2 ) const;
-
 };
 
 }
