@@ -84,16 +84,7 @@ public:
       { }
 
     /*!
-      \brief check if this vector has validate values. equivalent to isValid().
-      \return true if component values are validate.
-    */
-    bool valid() const
-      {
-          return ( ( x != ERROR_VALUE ) && ( y != ERROR_VALUE ) );
-      }
-
-    /*!
-      \brief check if this vector has validate values. equivalent to valid().
+      \brief check if this vector is valid or not.
       \return true if component values are validate.
     */
     bool isValid() const
@@ -101,15 +92,6 @@ public:
           return ( ( x != ERROR_VALUE ) && ( y != ERROR_VALUE ) );
       }
 
-    //     /*!
-    //       \brief type conversion operator. alias of valid().
-    //       \return true if value is validate.
-    //      */
-    //     operator bool() const
-    //       {
-    //           return valid();
-    //       }
-
     /*!
       \brief assign XY value directly.
       \param xx assigned x value
@@ -356,7 +338,7 @@ public:
     Vector2D & operator/=( const double & scalar )
       {
           //if ( scalar != 0 )
-          if ( scalar > EPSILON )
+          if ( std::fabs( scalar ) > EPSILON )
           {
               x /= scalar;
               y /= scalar;
@@ -739,7 +721,7 @@ public:
 
     /*!
       \class XYCmp
-      \brief comparison predicate for XY value.
+      \brief comparison predicate for XY value (X -> Y order).
      */
     class XYCmp
         : public std::binary_function< Vector2D, Vector2D, bool > {
@@ -764,6 +746,48 @@ public:
           }
 
     };
+
+    /*!
+      \class YXCmp
+      \brief comparison predicatio for XY value (Y -> X order)
+     */
+    class YXCmp
+        : public std::binary_function< Vector2D, Vector2D, bool > {
+    public:
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
+        result_type operator()( const first_argument_type & lhs,
+                                const second_argument_type & rhs ) const
+          {
+              return ( lhs.y < rhs.y
+                       || ( lhs.y == rhs.y && lhs.x < rhs.x ) );
+          }
+    };
+
+    /*!
+      \class Equal
+      \brief check if two vectors are completely same or not.
+     */
+    class Equal
+        : public std::binary_function< Vector2D, Vector2D, bool > {
+    public:
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
+        result_type operator()( const first_argument_type & lhs,
+                                const second_argument_type & rhs ) const
+          {
+              return lhs.equals( rhs );
+          }
+    };
+
     //////////////////////////////////////////////
     // functor for region
 
