@@ -120,7 +120,7 @@ public:
     /*!
       \brief destructor.
 
-      close opend file
+      close opened file
       delete dynamically allocated internal buffer
     */
     virtual
@@ -128,7 +128,7 @@ public:
 
     /*!
       \brief check if file is open.
-      \return returns true if file is opend, else false.
+      \return returns true if file is opened, else false.
      */
     bool is_open();
 
