@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -107,8 +107,6 @@ MonitorDropBallCommand::MonitorDropBallCommand( const double & x,
 std::ostream &
 MonitorDropBallCommand::toStr( std::ostream & to ) const
 {
-    using namespace std;
-
     to << "(dispfoul "
        << static_cast< int >( rint( M_x * rcg::SHOWINFO_SCALE ) ) << " "
        << static_cast< int >( rint( M_y * rcg::SHOWINFO_SCALE ) ) << " "
@@ -137,8 +135,6 @@ MonitorFreeKickCommand::MonitorFreeKickCommand( const double & x,
 std::ostream &
 MonitorFreeKickCommand::toStr( std::ostream & to ) const
 {
-    using namespace std;
-
     to << "(dispfoul "
        << static_cast< int >( rint( M_x * rcg::SHOWINFO_SCALE ) ) << " "
        << static_cast< int >( rint( M_y * rcg::SHOWINFO_SCALE ) ) << " "
@@ -165,6 +161,7 @@ MonitorMovePlayerCommand::MonitorMovePlayerCommand( const SideID side,
     {
         std::cerr << "MonitorMovePlayerCommand illegal uniform number "
                   << M_unum << std::endl;
+        M_unum = Unum_Unknown;
     }
 
     if ( M_side != LEFT
@@ -172,7 +169,7 @@ MonitorMovePlayerCommand::MonitorMovePlayerCommand( const SideID side,
     {
         std::cerr << "MonitorMovePlayerCommand illegal side type "
                   << M_side << std::endl;
-        M_unum = 0;
+        M_unum = Unum_Unknown;
     }
 
 }
@@ -180,24 +177,12 @@ MonitorMovePlayerCommand::MonitorMovePlayerCommand( const SideID side,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
-bool
-MonitorMovePlayerCommand::check() const
-{
-
-    return true;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
 */
 std::ostream &
 MonitorMovePlayerCommand::toStr( std::ostream & to ) const
 {
-    using namespace std;
-
-    if ( 1 <= M_unum && M_unum <= 11 )
+    if ( M_side != NEUTRAL
+         && 1 <= M_unum && M_unum <= 11 )
     {
         to << "(dispplayer "
            << M_side << " "
@@ -222,7 +207,7 @@ MonitorDiscardPlayerCommand::MonitorDiscardPlayerCommand( const SideID side,
     {
         std::cerr << "MonitorDiscardPlayerCommand: illegal uniform number "
                   << M_unum << std::endl;
-        M_unum = 1;
+        M_unum = Unum_Unknown;
     }
 
     if ( side != LEFT
@@ -230,7 +215,7 @@ MonitorDiscardPlayerCommand::MonitorDiscardPlayerCommand( const SideID side,
     {
         std::cerr << "MonitorDiscardPlayerCommand: illegal side type "
                   << M_side << std::endl;
-        M_unum = 0;
+        M_unum = Unum_Unknown;
     }
 }
 
@@ -241,7 +226,8 @@ MonitorDiscardPlayerCommand::MonitorDiscardPlayerCommand( const SideID side,
 std::ostream &
 MonitorDiscardPlayerCommand::toStr( std::ostream & to ) const
 {
-    if ( 1 <= M_unum && M_unum <= 11 )
+    if ( M_side != NEUTRAL
+         && 1 <= M_unum && M_unum <= 11 )
     {
         to << "(dispdiscard "
            << M_side << " "
@@ -251,6 +237,64 @@ MonitorDiscardPlayerCommand::toStr( std::ostream & to ) const
     return to;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+MonitorCardCommand::MonitorCardCommand( const SideID side,
+                                        const int unum,
+                                        const Card card )
+    : M_side( side )
+    , M_unum( unum )
+    , M_card( card )
+{
+    if ( unum < 1 || 11 < unum )
+    {
+        std::cerr << "MonitorCardCommand: illegal uniform number "
+                  << M_unum << std::endl;
+        M_unum = Unum_Unknown;
+    }
+
+    if ( side != LEFT
+         && side != RIGHT )
+    {
+        std::cerr << "MonitorCardCommand: illegal side type "
+                  << M_side << std::endl;
+        M_unum = Unum_Unknown;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+std::ostream &
+MonitorCardCommand::toStr( std::ostream & to ) const
+{
+    if ( M_side != NEUTRAL
+         && 1 <= M_unum && M_unum <= 11
+         && M_card != NO_CARD )
+    {
+        to << "(dispcard "
+           << M_side << ' '
+           << M_unum;
+
+        if ( M_card == YELLOW )
+        {
+            to << " yellow";
+        }
+        else if ( M_card == RED )
+        {
+            to << " red";
+        }
+
+        to << ')';
+    }
+
+    return to;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
