@@ -58,6 +58,8 @@ public:
         PLAYER, // move player
         DISCARD, // discard player
 
+        CARD, // yellow/red card
+
         COMPRESSION,
 
         ILLEGAL
@@ -241,7 +243,9 @@ class MonitorDropBallCommand
     : public MonitorCommand {
 private:
     double M_x; //!< drop point x
-    double M_y; //!< drop point
+    double M_y; //!< drop point y
+
+    MonitorDropBallCommand(); // not used
 public:
     /*!
       \brief construct with real coordinate
@@ -292,6 +296,8 @@ private:
     double M_x; //!< free kick point y
     double M_y; //!< free kick point y
     SideID M_side; //!< free kick awarded side. NEUTRAL means drop ball.
+
+    MonitorFreeKickCommand(); // not used
 public:
     /*!
       \brief construct with real coordinate
@@ -345,12 +351,7 @@ private:
     double M_y; //!< move target point y
     double M_angle; //!< player's body angle after move. *degree*
 
-    /*!
-      \brief check if player info has valid id
-      \return false if side of unum is invalid
-     */
-    bool check() const;
-
+    MonitorMovePlayerCommand(); // not used
 public:
     /*!
       \brief construct with all needed info
@@ -395,13 +396,15 @@ public:
 //////////////////////////////////////////////////////////////////////
 /*!
   \class MonitorDiscardPlayerCommand
-  \brief command to award red card to player
+  \brief command to disscard a player from the server
  */
 class MonitorDiscardPlayerCommand
     : public MonitorCommand {
 private:
     SideID M_side; //!< target player's side id
     int M_unum; //!< target player's uniform number
+
+    MonitorDiscardPlayerCommand(); // not used
 public:
     /*!
       \brief construct with all info
@@ -437,6 +440,55 @@ public:
       }
 };
 
+//////////////////////////////////////////////////////////////////////
+/*!
+  \class MonitorCardCommand
+  \brief command to give yellow/red card to a player
+ */
+class MonitorCardCommand
+    : public MonitorCommand {
+private:
+    SideID M_side; //!< target player's side id
+    int M_unum; //!< target player's uniform number
+    Card M_card; //!< card type
+
+    MonitorCardCommand(); // not used
+public:
+    /*!
+      \brief construct with all info
+      \param side target player's side id
+      \param unum target player's uniform number
+     */
+    MonitorCardCommand( const SideID side,
+                        const int unum,
+                        const Card card );
+
+    /*!
+      \brief get command type id
+      \return command type
+     */
+    Type type() const
+      {
+          return CARD;
+      }
+
+    /*!
+      \brief put command string to the output stream
+      \param to referenct to the toutput stream
+      \return referenct to the toutput stream
+     */
+    std::ostream & toStr( std::ostream & to ) const;
+
+    /*!
+      \brief get command name string
+      \return command name string
+     */
+    std::string name() const
+      {
+          return std::string( "dispcard" );
+      }
+};
+
 //////////////////////////////////////////////////////////////////////
 /*!
   \class MonitorCompressionCommand
@@ -448,6 +500,7 @@ private:
     //! gzip compression level
     int M_level;
 
+    MonitorCompressionCommand(); // not used
 public:
     /*!
       \brief construct with compression level
