@@ -208,14 +208,14 @@ BasicSocket::setAddr( const char * hostname,
     M_dest->addr_.sin_family = AF_INET;
     M_dest->addr_.sin_port = htons( port );
 
-    freeaddrinfo( res );
+    ::freeaddrinfo( res );
     return true;
 
 #endif
 
 #ifdef HAVE_GETHOSTBYNAME
 #ifdef HAVE_INET_ADDR
-    M_dest->addr_.sin_addr.s_addr = inet_addr( hostname );
+    M_dest->addr_.sin_addr.s_addr = ::inet_addr( hostname );
     if ( M_dest->addr_.sin_addr.s_addr == 0xffffffff )
 #endif
     {
