@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,9 +47,11 @@ namespace rcsc {
 
 */
 ConfFileParser::ConfFileParser( const std::string & file_path,
-                                const std::string & delim )
+                                const std::string & delim,
+                                const std::string & name_space )
     : M_file_path( file_path )
     , M_delimiters( delim )
+    , M_namespace( name_space )
 {
 
 }
@@ -87,6 +89,30 @@ ConfFileParser::parse( ParamMap & param_map )
             continue;
         }
 
+        if ( ! M_namespace.empty() )
+        {
+            char ns[256];
+            int n_read = 0;
+            if ( std::sscanf( line_buf.c_str(), " %255s[^ :] :: %n ",
+                              ns, &n_read ) != 1
+                 || n_read == 0 )
+            {
+                // illegal namespace format
+                //std:: cerr << "***WARNING*** ConfFileParser "
+                //           << " Illegal namespace format ["
+                //           << line_buf << "]" << std::endl;
+                continue;
+            }
+
+            if ( M_namespace != ns )
+            {
+                // namespace does not match.
+                continue;
+            }
+
+            line_buf.erase( 0, n_read );
+        }
+
         // search delimiter charecters
         std::string::size_type delim_pos = line_buf.find_first_of( M_delimiters );
         if ( delim_pos != std::string::npos )
@@ -105,7 +131,7 @@ ConfFileParser::parse( ParamMap & param_map )
         }
 
         // get parameter entry from map
-        ParamPtr param_ptr = param_map.getParamPtr( name_str );
+        ParamPtr param_ptr = param_map.findLongName( name_str );
 
         if ( ! param_ptr )
         {
