@@ -41,12 +41,25 @@ class ParamMap;
   \brief abstract parameter paraser class
  */
 class ParamParser {
+private:
+
+    // non copyable
+    ParamParser( const ParamParser & );
+    ParamParser & operator=( const ParamParser & );
 
 protected:
+
+    /*!
+      \brief protected access
+     */
     ParamParser()
       { }
 
 public:
+
+    /*!
+      \brief virtual destructor
+     */
     virtual
     ~ParamParser()
       { }
