@@ -74,6 +74,7 @@ ActionEffector::ActionEffector( const PlayerAgent & agent )
     , M_catch_time( 0, 0 )
     , M_tackle_power( 0.0 )
     , M_tackle_dir( 0.0 )
+    , M_tackle_foul( false )
     , M_turn_neck_moment( 0.0 )
     , M_say_message( "" )
     , M_pointto_pos( 0.0, 0.0 )
@@ -305,6 +306,7 @@ ActionEffector::checkCommandCount( const BodySensor & sense )
         M_last_body_command_type = PlayerCommand::ILLEGAL;
         M_tackle_power = 0.0;
         M_tackle_dir = 0.0;
+        M_tackle_foul = false;
         M_command_counter[PlayerCommand::TACKLE] = sense.tackleCount();
     }
 
@@ -414,7 +416,7 @@ ActionEffector::makeCommand( std::ostream & to )
     }
     else
     {
-        if ( ! M_agent.world().self().isFreezed() )
+        if ( ! M_agent.world().self().isFrozen() )
         {
             dlog.addText( Logger::SYSTEM,
                            __FILE__": WARNING. no body command." );
@@ -1038,13 +1040,18 @@ ActionEffector::setCatch()
                                ServerParam::i().catchAreaLength() );
 
     // relative angle
-    AngleDeg catch_angle = M_agent.world().ball().angleFromSelf() - M_agent.world().self().body();
-    dlog.addText( Logger::ACTION,
-                   __FILE__" (setCatch) ball_rel_angle= %.1f  diagonal_angle= %.1f",
-                  catch_angle.degree(), diagonal_angle );
-
+    AngleDeg ball_rel_angle = M_agent.world().ball().angleFromSelf() - M_agent.world().self().body();
     // add diagonal angle
-    catch_angle += diagonal_angle;
+    AngleDeg catch_angle = ball_rel_angle + diagonal_angle;
+
+    dlog.addText( Logger::ACTION,
+                   __FILE__" (setCatch) ball_dir=%.1f diagonal_angle=%.1f -> catch_angle=%.1f(gloabl=%.1f)",
+                  ball_rel_angle.degree(),
+                  diagonal_angle,
+                  catch_angle.degree(),
+                  ( catch_angle + M_agent.world().self().body() ).degree() );
+
+
 
     //////////////////////////////////////////////////
     // create command object
@@ -1061,60 +1068,62 @@ ActionEffector::setCatch()
 
 */
 void
-ActionEffector::setTackle( const double & power_or_dir )
+ActionEffector::setTackle( const double & power_or_dir,
+                           const bool foul )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__" (setTackle) register tackle. power_or_dir=%.1f",
-                  power_or_dir );
+                   __FILE__" (setTackle) register tackle. power_or_dir=%.1f foul=%s",
+                  power_or_dir,
+                  foul ? "on" : "off" );
 
-    double command_arg = power_or_dir;
+    double actual_power_or_dir = power_or_dir;
 
     if ( M_agent.config().version() >= 12.0 )
     {
-        if ( command_arg < -180.0 || 180.0 < command_arg )
+        if ( actual_power_or_dir < -180.0 || 180.0 < actual_power_or_dir )
         {
             std::cerr << M_agent.world().teamName() << ' '
                       << M_agent.world().self().unum() << ": "
                       << M_agent.world().time()
-                      << "tackle dir over the range. dir=" << command_arg
+                      << "tackle dir over the range. dir=" << actual_power_or_dir
                       << std::endl;
             dlog.addText( Logger::ACTION,
                            __FILE__": (setTackle) dir over. %f",
-                          command_arg );
+                          actual_power_or_dir );
         }
-        command_arg = AngleDeg::normalize_angle( power_or_dir );
+        actual_power_or_dir = AngleDeg::normalize_angle( power_or_dir );
     }
     else
     {
-        if ( command_arg > ServerParam::i().maxTacklePower() + 0.01 )
+        if ( actual_power_or_dir > ServerParam::i().maxTacklePower() + 0.01 )
         {
             std::cerr << M_agent.world().teamName() << ' '
                       << M_agent.world().self().unum() << ": "
                       << M_agent.world().time()
-                      << "tackle power overflow. com=" << command_arg
+                      << "tackle power overflow. com=" << actual_power_or_dir
                       << " > sparam=" << ServerParam::i().maxTacklePower()
                       << std::endl;
             dlog.addText( Logger::ACTION,
                            __FILE__": (setTackle) power over. %f",
-                          command_arg );
-            command_arg = ServerParam::i().maxTacklePower();
+                          actual_power_or_dir );
+            actual_power_or_dir = ServerParam::i().maxTacklePower();
         }
 
-        if ( command_arg < - ServerParam::i().maxBackTacklePower() - 0.01 )
+        if ( actual_power_or_dir < - ServerParam::i().maxBackTacklePower() - 0.01 )
         {
             std::cerr << M_agent.world().teamName() << ' '
                       << M_agent.world().self().unum() << ": "
                       << M_agent.world().time()
-                      << "tackle power underflow " << command_arg
+                      << "tackle power underflow " << actual_power_or_dir
                       << std::endl;
             dlog.addText( Logger::ACTION,
                            __FILE__": (setTackle) power underflow. %f",
-                          command_arg );
-            command_arg = ServerParam::i().minPower();
+                          actual_power_or_dir );
+            actual_power_or_dir = ServerParam::i().minPower();
         }
     }
 
-    command_arg = rint( command_arg * 1000.0 ) * 0.001;
+    actual_power_or_dir = rint( actual_power_or_dir * 1000.0 ) * 0.001;
 
     //////////////////////////////////////////////////
     // create command object
@@ -1123,21 +1132,23 @@ ActionEffector::setTackle( const double & power_or_dir )
         delete M_command_body;
         M_command_body = static_cast< PlayerBodyCommand * >( 0 );
     }
-    M_command_body = new PlayerTackleCommand( command_arg );
+    M_command_body = new PlayerTackleCommand( actual_power_or_dir, foul );
 
     // set estimated command effect
-    M_tackle_power = command_arg;
+    M_tackle_power = actual_power_or_dir;
     if ( M_agent.config().version() >= 12.0 )
     {
         M_tackle_power = ServerParam::i().maxTacklePower();
-        M_tackle_dir = command_arg;
+        M_tackle_dir = actual_power_or_dir;
     }
     else
     {
-        M_tackle_dir = ( command_arg > 0.0
+        M_tackle_dir = ( actual_power_or_dir > 0.0
                          ? M_agent.world().self().body().degree()
                          : ( M_agent.world().self().body() + 180.0 ).degree() );
     }
+
+    M_tackle_foul = foul;
 }
 
 /*-------------------------------------------------------------------*/
@@ -1460,7 +1471,15 @@ ActionEffector::makeSayCommand()
           it != end;
           ++it )
     {
-        (*it)->toStr( M_say_message );
+        if ( ! (*it)->toStr( M_say_message ) )
+        {
+            std::cerr << M_agent.world().self().unum() << " : "
+                      << M_agent.world().time() << " Error say message builder. type=["
+                      << (*it)->header() << ']'
+                      << std::endl;
+            dlog.addText( Logger::ACTION,
+                          __FILE__" (makeSayCommand) error occured." );
+        }
     }
 
     if ( M_say_message.empty() )
@@ -1471,7 +1490,7 @@ ActionEffector::makeSayCommand()
     M_command_say = new PlayerSayCommand( M_say_message,
                                           M_agent.config().version() );
 
-    dlog.addText( rcsc::Logger::ACTION,
+    dlog.addText( Logger::ACTION,
                   __FILE__" (makeSayCommand) say message [%s]",
                   M_say_message.c_str() );
 }
@@ -1555,7 +1574,8 @@ ActionEffector::queuedNextBallKickable() const
     Vector2D my_next = queuedNextSelfPos();
     Vector2D ball_next = queuedNextBallPos();
 
-    return my_next.dist( ball_next ) < M_agent.world().self().kickableArea() - 0.06;
+    return my_next.dist( ball_next )
+        < M_agent.world().self().playerType().kickableArea() - 0.06;
 }
 
 /*-------------------------------------------------------------------*/
