@@ -108,8 +108,9 @@ private:
     GameTime M_catch_time; //!< last catch time
 
     // tackle effect
-    double M_tackle_power;  //!< last tackle power
-    double M_tackle_dir;    //!< last tackle dir == body dir
+    double M_tackle_power; //!< last tackle power
+    double M_tackle_dir;   //!< last tackle dir == body dir
+    bool M_tackle_foul;    //!< last tackle foul switch
 
     // turn_neck effect
     double M_turn_neck_moment; //!< last turn_neck moment
@@ -297,10 +298,12 @@ public:
     /*!
       \brief create tackle command and its effect with tackle parameter
       \param power_or_dir tackle power or directin for command argument
+      \param foul intentional foul switch
 
       power is normalized by server parameter
     */
-    void setTackle( const double & power_or_dir );
+    void setTackle( const double & power_or_dir,
+                    const bool foul );
 
     ///////////////////////////////////////////////////////////////
     // register support command
@@ -477,10 +480,21 @@ public:
       \param dir variable pointer to store the tackle direction
     */
     void getTackleInfo( double * power,
-                        double * dir ) const
+                        double * dir,
+                        bool * foul ) const
       {
           if ( power ) *power = M_tackle_power;
           if ( dir ) *dir = M_tackle_dir;
+          if ( foul ) *foul = M_tackle_foul;
+      }
+
+    /*!
+      \brief get the last tackle foul switch.
+      \return the value of the last tackle foul switch.
+     */
+    bool tackleFoul() const
+      {
+          return M_tackle_foul;
       }
 
     //////////////////////////////////////////
