@@ -45,7 +45,7 @@
 
 #include <iostream>
 
-// #define DEBUG_PRINT
+#define DEBUG_PRINT
 
 namespace rcsc {
 
@@ -376,6 +376,14 @@ BallObject::updateByCollision( const Vector2D & pos,
                                const Vector2D & vel,
                                const int vel_count )
 {
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateByCollision)"
+                  " pos(%.2f %.2f)count=%d rpos=(%2.f %.2f)count=%d vel=(%.2f %.2f)count=%d",
+                  pos.x, pos.y, pos_count,
+                  rpos.x, rpos.y, rpos_count,
+                  vel.x, vel.y, vel_count );
+#endif
     M_state.pos_ = pos;
     M_state.pos_count_ = pos_count;
     M_state.rpos_ = rpos;
@@ -392,6 +400,13 @@ void
 BallObject::updateOnlyRelativePos( const Vector2D & rpos,
                                    const Vector2D & rpos_err )
 {
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateOnlyRelativePos)"
+                  " rpos=(%2.f %.2f) error=(%f %f)",
+                  rpos.x, rpos.y,
+                  rpos_err.x, rpos_err.y );
+#endif
     M_state.rpos_ = rpos;
     M_state.rpos_error_ = rpos_err;
     M_state.rpos_count_ = 0;
@@ -408,6 +423,14 @@ BallObject::updateOnlyVel( const Vector2D & vel,
                            const Vector2D & vel_err,
                            const int vel_count )
 {
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateOnlyVel)"
+                  " vel=(%2.f %.2f) error=(%f %f) count=%d",
+                  vel.x, vel.y,
+                  vel_err.x, vel_err.y,
+                  vel_count );
+#endif
     M_state.vel_ = vel;
     M_state.vel_error_ = vel_err;
     M_state.vel_count_ = vel_count;
@@ -423,6 +446,11 @@ BallObject::updateOnlyVel( const Vector2D & vel,
 void
 BallObject::setOpponentControlEffect()
 {
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (setOpponentControlEffect)"
+                  " velocity is set to Zero." );
+#endif
     M_state.vel_error_ += vel();
     M_state.vel_count_ += 1;
 
@@ -440,15 +468,21 @@ BallObject::updatePos( const Vector2D & pos,
                        const Vector2D & rpos,
                        const Vector2D & rpos_err )
 {
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updatePos)"
+                  " pos(%.2f %.2f) count=%d",
+                  pos.x, pos.y, pos_count );
+#endif
+
     M_state.pos_ = pos;
     M_state.pos_error_ = pos_err;
     M_state.pos_count_ = pos_count;
-    M_state.rpos_ = rpos;
-    M_state.rpos_error_ = rpos_err;
-    M_state.rpos_count_ = 0;
     M_state.seen_pos_ = pos;
     M_state.seen_pos_count_ = 0;
-    M_state.seen_rpos_ = rpos;
+
+    updateOnlyRelativePos( rpos, rpos_err );
+
     M_state.lost_count_ = 0;
     M_state.ghost_count_ = 0;
 }
@@ -467,6 +501,10 @@ BallObject::updateAll( const Vector2D & pos,
                        const Vector2D & vel_err,
                        const int vel_count )
 {
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateAll)" );
+#endif
     updatePos( pos, pos_err, pos_count, rpos, rpos_err );
     updateOnlyVel( vel, vel_err, vel_count );
 }
@@ -479,9 +517,31 @@ void
 BallObject::updateByHear( const ActionEffector & act,
                           const double & sender_to_ball_dist,
                           const Vector2D & heard_pos,
-                          const Vector2D & heard_vel,
-                          const GameTime & )
+                          const Vector2D & heard_vel )
 {
+    double heard_speed = 0.0;
+    if ( heard_vel.isValid() )
+    {
+        heard_speed = heard_vel.r();
+#ifdef DEBUG_PRINT
+        dlog.addText( Logger::WORLD,
+                      __FILE__" (updateByHear)"
+                      " heard_pos=(%.2f, %.2f)"
+                      " heard_vel=(%.2f, %.2f)",
+                      heard_pos.x, heard_pos.y,
+                      heard_vel.x, heard_vel.y );
+#endif
+    }
+#ifdef DEBUG_PRINT
+    else
+    {
+        dlog.addText( Logger::WORLD,
+                      __FILE__" (updateByHear)"
+                      " heard_pos=(%.2f, %.2f) no vel",
+                      heard_pos.x, heard_pos.y );
+    }
+#endif
+
     M_state.heard_pos_ = heard_pos;
     M_state.heard_pos_count_ = 0;
     M_state.heard_vel_ = heard_vel;
@@ -491,11 +551,7 @@ BallObject::updateByHear( const ActionEffector & act,
     {
 #ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__" (updateByHear) last command is kick."
-                      " heard_pos=(%.2f, %.2f)"
-                      " heard_vel=(%.2f, %.2f)",
-                      heard_pos.x, heard_pos.y,
-                      heard_vel.x, heard_vel.y );
+                      __FILE__" (updateByHear) last command is kick." );
 #endif
         return;
     }
@@ -506,15 +562,16 @@ BallObject::updateByHear( const ActionEffector & act,
         {
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__" (updateByHear) ghost detected. heard_pos=(%.2f, %.2f)"
-                          " heard_vel=(%.2f, %.2f)",
-                          heard_pos.x, heard_pos.y,
-                          heard_vel.x, heard_vel.y );
+                          __FILE__" (updateByHear) ghost detected." );
 #endif
             M_state.pos_ = heard_pos;
             M_state.pos_count_ = 1;
-            M_state.vel_ = heard_vel;
-            M_state.vel_count_ = 1;
+            if ( heard_vel.isValid()
+                 && heard_speed > 0.001 )
+            {
+                M_state.vel_ = heard_vel;
+                M_state.vel_count_ = 1;
+            }
             return;
         }
     }
@@ -527,20 +584,20 @@ BallObject::updateByHear( const ActionEffector & act,
     {
 #ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__" (updateByHear) big difference from last internal state."
-                      " heard_pos=(%.2f, %.2f)"
-                      " heard_vel=(%.2f, %.2f)",
-                      heard_pos.x, heard_pos.y,
-                      heard_vel.x, heard_vel.y );
+                      __FILE__" (updateByHear) big difference from last internal state." );
 #endif
         M_state.pos_ = heard_pos;
         M_state.pos_count_ = 1;
-        M_state.vel_ = heard_vel;
-        M_state.vel_count_ = 1;
+        if ( heard_vel.isValid()
+             && heard_speed > 0.001 )
+        {
+            M_state.vel_ = heard_vel;
+            M_state.vel_count_ = 1;
+        }
         return;
     }
 
-    const double prev_dist = ( M_rpos_prev.valid()
+    const double prev_dist = ( M_rpos_prev.isValid()
                                ? M_rpos_prev.r()
                                : 1000.0 );
 
@@ -549,49 +606,61 @@ BallObject::updateByHear( const ActionEffector & act,
     {
 #ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__" (updateByHear) sent from the player near to the ball."
-                      " heard_pos=(%.2f, %.2f)"
-                      " heard_vel=(%.2f, %.2f)",
-                      heard_pos.x, heard_pos.y,
-                      heard_vel.x, heard_vel.y );
+                      __FILE__" (updateByHear) sent from the player near to the ball." );
 #endif
         M_state.pos_ = heard_pos;
         M_state.pos_count_ = 1;
-        M_state.vel_ = heard_vel;
-        M_state.vel_count_ = 1;
+        if ( heard_vel.isValid()
+             && heard_speed > 0.001 )
+        {
+            M_state.vel_ = heard_vel;
+            M_state.vel_count_ = 1;
+        }
     }
-    else if ( //sender_to_ball_dist < prev_dist
-             sender_to_ball_dist < 20.0
+    else if ( sender_to_ball_dist < 20.0
               && posCount() >= 2 )
     {
 #ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__" (updateByHear) low accuracy."
-                      " heard_pos=(%.2f, %.2f)"
-                      " heard_vel=(%.2f, %.2f)",
-                      heard_pos.x, heard_pos.y,
-                      heard_vel.x, heard_vel.y );
+                      __FILE__" (updateByHear) low accuracy." );
 #endif
         M_state.pos_ = heard_pos;
         M_state.pos_count_ = 1;
-        M_state.vel_ = heard_vel;
-        M_state.vel_count_ = 1;
+        if ( heard_vel.isValid()
+             && heard_speed > 0.001 )
+        {
+            M_state.vel_ = heard_vel;
+            M_state.vel_count_ = 1;
+        }
     }
-    else if ( ( prev_dist > 1.4 && velCount() >= 3 )
-              || velCount() >= 4 )
+    else if ( heard_vel.isValid()
+              && heard_speed > 0.001
+              && ( velCount() >= 4
+                   || ( velCount() >= 3
+                        && prev_dist > 1.4
+                        && sender_to_ball_dist < 10.0 )
+                   || ( velCount() >= 2
+                        && prev_dist > 1.4
+                        && sender_to_ball_dist < 10.0 )
+                   )
+              )
     {
 #ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
                       __FILE__" (updateByHear) low vel accuracy."
-                      " sender_ball_dist=%.3f prev_my_dist=%.3f"
-                      " heard_vel=(%.2f, %.2f)",
-                      sender_to_ball_dist, prev_dist,
-                      heard_vel.x, heard_vel.y );
+                      " velCount=%d"
+                      " sender_ball_dist=%.3f prev_my_dist=%.3f",
+                      velCount(),
+                      sender_to_ball_dist, prev_dist );
 #endif
         M_state.pos_ = heard_pos;
         M_state.pos_count_ = 1;
-        M_state.vel_ = heard_vel;
-        M_state.vel_count_ = 1;
+        if ( heard_vel.isValid()
+             && heard_speed > 0.001 )
+        {
+            M_state.vel_ = heard_vel;
+            M_state.vel_count_ = 1;
+        }
     }
 }
 
@@ -613,8 +682,8 @@ BallObject::updateSelfRelated( const SelfObject & self )
     else
     {
         // update rpos
-        if ( M_rpos_prev.valid()
-             && self.lastMove().valid() )
+        if ( M_rpos_prev.isValid()
+             && self.lastMove().isValid() )
         {
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
@@ -634,7 +703,7 @@ BallObject::updateSelfRelated( const SelfObject & self )
         // update dist & angle
 
         // at least, rpos is valid
-        if ( rpos().valid()
+        if ( rpos().isValid()
              && posCount() > rposCount() )
         {
 #ifdef DEBUG_PRINT
@@ -684,6 +753,22 @@ BallObject::updateRecord()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+const
+BallObject::State *
+BallObject::getState( const size_t history_index ) const
+{
+    if ( history_index >= M_state_record.size() )
+    {
+        return static_cast< State * >( 0 );
+    }
+
+    return &M_state_record[ history_index ];
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 Vector2D
 BallObject::inertiaTravel( const int cycle ) const
