@@ -76,6 +76,8 @@ BodySensor::BodySensor()
     , M_ball_collided( false )
     , M_player_collided( false )
     , M_post_collided( false )
+    , M_charged_expires( 0 )
+    , M_card( NO_CARD )
 {
 
 }
@@ -125,14 +127,22 @@ BodySensor::parse1( const char * msg,
     //  (focus (target none) (count 0)) (tackle (expires 0) (count 0))
     //  (collision {none|[(ball)][player][post]}))
 
+    // ver. 14
+    // (sense_body 66 (view_mode high normal) (stamina 3503.4 1 124000) (speed 0.06 -79)
+    //  (head_angle 89) (kick 4) (dash 20) (turn 24) (say 0) (turn_neck 28) (catch 0)
+    //  (move 1) (change_view 16) (arm (movable 0) (expires 0) (target 0 0) (count 0))
+    //  (focus (target none) (count 0)) (tackle (expires 0) (count 0))
+    //  (collision {none|[(ball)][player][post]}))
+    //  (foul (charged 0) (card {none|yellow|red}))
+
     //char ss[8];
 
     M_time = current;
 
     ++msg; // skip first paren
-    while ( *msg != '(' ) ++msg; // skip "sense_body <time> "
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip "sense_body <time> "
 
-    while ( *msg != ' ' ) ++msg; // skip "(view_mode"
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(view_mode"
     ++msg; // skip space
     // parse view quality
     switch ( *msg ) {
@@ -147,7 +157,7 @@ BodySensor::parse1( const char * msg,
         break;
     }
 
-    while ( *msg != ' ' ) ++msg; // skip view_quality string
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip view_quality string
     ++msg; // skip space
 
     // parse view width
@@ -169,8 +179,8 @@ BodySensor::parse1( const char * msg,
     char *next;
 
     // read stamina values
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(stamina"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(stamina"
     M_stamina = std::strtod( msg, &next ); msg = next;
     M_effort = std::strtod( msg, &next ); msg = next;
     if ( version >= 13.0 )
@@ -182,8 +192,8 @@ BodySensor::parse1( const char * msg,
     }
 
     // read speed values
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(speed"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(speed"
     M_speed_mag = std::strtod( msg, &next );
     msg = next; // this value is quantized by 0.01
     if ( version >= 6.0 )
@@ -202,23 +212,23 @@ BodySensor::parse1( const char * msg,
         msg = next;
     }
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(kick"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(kick"
     M_kick_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(dash"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(dash"
     M_dash_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(turn"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(turn"
     M_turn_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(say"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(say"
     M_say_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
@@ -227,8 +237,8 @@ BodySensor::parse1( const char * msg,
         return;
     }
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(turn_neck"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(turn_neck"
     M_turn_neck_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
@@ -237,13 +247,13 @@ BodySensor::parse1( const char * msg,
         return;
     }
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(catch"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(catch"
     M_catch_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(move"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(move"
     M_move_count  = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
@@ -259,37 +269,37 @@ BodySensor::parse1( const char * msg,
 
     // `(arm (movable <MOVABLE>) (expires <EXPIRES>)
     //   (target <DIST> <DIR>) (count <COUNT>))'
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(arm"
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(movable"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(arm"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(movable"
     M_arm_movable = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(expires"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(expires"
     M_arm_expires = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(target"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(target"
     ++msg;
     M_pointto_dist = std::strtod( msg, &next );
     msg = next;
     M_pointto_dir = std::strtod( msg, &next );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(count"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(count"
     M_pointto_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
     // `(focus (target <SIDE> [<UNUM>]) (count <COUNT>)'
     // <SIDE> := "none" | "l" | "r"
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(focus"
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(target"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(focus"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(target"
     ++msg; // skip space
     if ( *msg == 'n' ) // "none"
     {
@@ -315,21 +325,21 @@ BodySensor::parse1( const char * msg,
         std::cerr << "sense_body: focus ?? [" << msg << std::endl;
     }
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(count"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(count"
     M_attentionto_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
     // `(tackle (expires <EXPIRES>) (count <COUNT>))'
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(tackle"
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(expires"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(tackle"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(expires"
     M_tackle_expires = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
-    while ( *msg != '(' ) ++msg;
-    while ( *msg != ' ' ) ++msg; // skip "(count"
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(count"
     M_tackle_count = static_cast< int >( std::strtol( msg, &next, 10 ) );
     msg = next;
 
@@ -340,7 +350,21 @@ BodySensor::parse1( const char * msg,
 
     while ( *msg != '\0' && *msg != '(' ) ++msg;
 
-    parseCollision( msg );
+    //
+    // (collision {none|[(ball)][(player)][(post)]})
+    //
+    parseCollision( msg, &next );
+    msg = next;
+
+    if ( version < 14.0 )
+    {
+        return;
+    }
+
+    //
+    // (foul (charged 0) (card {none|yellow|red}))
+    //
+    parseFoul( msg, &next );
 
 }
 
@@ -414,7 +438,7 @@ BodySensor::parse2( const char * msg,
 
         if ( version >= 12.0 )
         {
-            parseCollision( msg + n_read );
+            parseCollision( msg + n_read, NULL );
         }
     }
     else if ( version >= 7.0
@@ -521,8 +545,9 @@ BodySensor::parse2( const char * msg,
 /*!
 
 */
-void
-BodySensor::parseCollision( const char * msg )
+bool
+BodySensor::parseCollision( const char * msg,
+                            char ** next )
 {
     // (collision {none|[(ball)][(player)][(post)]})
 
@@ -530,7 +555,7 @@ BodySensor::parseCollision( const char * msg )
     {
         std::cerr << M_time << " sense_body. illegal collision tag ["
                   << msg << "]" << std::endl;
-        return;
+        return false;
     }
 
     M_none_collided = false;
@@ -546,7 +571,13 @@ BodySensor::parseCollision( const char * msg )
     if ( ! std::strncmp( msg, "none", 4 ) )
     {
         M_none_collided = true;
-        return;
+
+        if ( next )
+        {
+            while ( *msg != '\0' && *msg != '(' ) ++msg;
+            *next = const_cast< char * >( msg );
+        }
+        return true;
     }
 
     int n_read = 0;
@@ -578,6 +609,87 @@ BodySensor::parseCollision( const char * msg )
                       << name << "]" << std::endl;
         }
     }
+
+    if ( next )
+    {
+        while ( *msg != '\0' && *msg != '(' ) ++msg;
+        *next = const_cast< char * >( msg );
+    }
+
+    return true;
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+BodySensor::parseFoul( const char * msg,
+                       char ** next )
+{
+    // (foul (charged 0) (card {none|yellow|red}))
+
+    int n_read = 0;
+
+    while ( *msg != '\0' && *msg != '(' ) ++msg;
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(foul"
+
+    int cycles = 0;
+
+    if ( std::sscanf( msg, " ( charged %d ) %n ",
+                      &cycles, &n_read ) != 1
+         || n_read == 0 )
+    {
+        std::cerr << M_time << " sense_body. illegal foul charge expires ["
+                  << msg << "]" << std::endl;
+        return false;
+    }
+    msg += n_read;
+    n_read = 0;
+
+    M_charged_expires = cycles;
+
+    char card[8];
+
+    if ( std::sscanf( msg, " ( card %7[^)] ) %n",
+                      card, &n_read ) != 1
+         || n_read == 0 )
+    {
+        std::cerr << M_time << " sense_body. illegal card information ["
+                  << msg << "]" << std::endl;
+        return false;
+    }
+    msg += n_read;
+
+    if ( ! std::strcmp( card, "none" ) )
+    {
+        M_card = NO_CARD;
+    }
+    else if ( ! std::strcmp( card, "yellow" ) )
+    {
+        M_card = YELLOW;
+    }
+    else if ( ! std::strcmp( card, "red" ) )
+    {
+        M_card = RED;
+    }
+    else
+    {
+        std::cerr << M_time << " sense_body. unknown card type ["
+                  << card << "]" << std::endl;
+        M_card = NO_CARD;
+    }
+
+    while ( *msg == ' ' ) ++msg;
+    if ( *msg == ')' ) ++msg;
+
+    if ( next )
+    {
+        *next = const_cast< char * >( msg );
+    }
+
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
