@@ -100,7 +100,7 @@ public:
       the number of cycles the current tackle will last for.
       if 0, player is not tackling.
     */
-    int M_tackle_expires;
+    int M_tackle_expires; //!< tackle expire cycles
     int M_tackle_count; //!< sensed command count1
 
     bool M_none_collided; //!< true if 'none' collides
@@ -108,6 +108,9 @@ public:
     bool M_player_collided; //!< player collision info
     bool M_post_collided; //!< post collision info
 
+    int M_charged_expires; //!< foul charged expire cycle
+    Card M_card; //!< yellow/red card
+
 public:
     /*!
       \brief init member variables
@@ -149,10 +152,22 @@ public:
 private:
 
     /*!
-      \brief analyzer collision information contained by sense_body message.
+      \brief analyze collision information contained by sense_body message.
       \param msg server message started with (collision
+      \param pointer pointer to the next character after parsing
+      \return parsing result
      */
-    void parseCollision( const char * msg );
+    bool parseCollision( const char * msg,
+                         char ** next );
+
+    /*!
+      \brief analyze card information
+      \param msg server message started with (card
+      \param pointer pointer to the next character after parsing
+      \return parsing result
+     */
+    bool parseFoul( const char * msg,
+                    char ** next );
 
 public:
 
@@ -446,6 +461,24 @@ public:
           return M_post_collided;
       }
 
+    /*!
+      \brief get expire cycle of foul charge
+      \return expire cycle of foul charge
+     */
+    int chargedExpires() const
+      {
+          return M_charged_expires;
+      }
+
+    /*!
+      \brief get the yellow/red card status
+      \return card type
+     */
+    Card card() const
+      {
+          return M_card;
+      }
+
     /*!
       \brief put data to output stream
       \param os reference to the output stream
