@@ -115,6 +115,17 @@ public:
           return true;
       }
 
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const
+      {
+          os << "[Free:" << M_message << ']';
+          return os;
+      }
+
 };
 
 }
