@@ -35,6 +35,8 @@
 #include <rcsc/common/audio_message.h>
 #include <rcsc/types.h>
 
+#include <boost/shared_ptr.hpp>
+
 #include <string>
 
 namespace rcsc {
@@ -47,6 +49,11 @@ class WorldModel;
   \brief coach's freeform message parser
  */
 class FreeformParser {
+public:
+
+    //! pointer type alias
+    typedef boost::shared_ptr< FreeformParser > Ptr;
+
 private:
 
     // not used
