@@ -170,81 +170,96 @@ FullstateSensor::parseV8( const char * msg )
 {
     /*
       fullstate v8+ format:
+
       (fullstate <time>
-      (pmode {goalie_catch_ball_{l|r}|<play mode>})
-      (vmode {high|low} {narrow|normal|high})
-      //(stamina <stamina> <effort> <recovery>)
-      (count <kicks> <dashes> <turns> <catches> <moves>
-      <turn_necks> <change_views> <says>)
-      (arm (movable <MOVABLE>) (expires <EXPIRES>)
-      (target <DIST> <DIR>) (count <COUNT>))
-      (score <team_points> <enemy_points>)
-      ((b) <pos.x> <pos.y> <vel.x> <vel.y>)
-      <players>)
-
-      players : {<player>|<player> <players>}
-
-      player : ((p {l|r} <unum> {g|<player_type_id>}) <pos.x> <pos.y>
-      <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
-      (<stamina> <effort> <recovery>[ <capacity>]))
+        (pmode {goalie_catch_ball_{l|r}|<play mode>})
+        (vmode {high|low} {narrow|normal|high})
+        //(stamina <stamina> <effort> <recovery>)
+        (count <kicks> <dashes> <turns> <catches> <moves>
+               <turn_necks> <change_views> <says>)
+        (arm (movable <MOVABLE>) (expires <EXPIRES>)
+        (target <DIST> <DIR>) (count <COUNT>))
+        (score <team_points> <enemy_points>)
+        ((b) <pos.x> <pos.y> <vel.x> <vel.y>)
+        <players>)
 
       // after rcssserver-8.03, self stamina info has been omit.
       // and added "arm" info.
+
+      players : {<player>|<player> <players>}
+
+      player : (v8-13)
+       ((p {l|r} <unum> {g|<player_type_id>})
+        <pos.x> <pos.y> <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
+        (<stamina> <effort> <recovery>[ <capacity>]))
+
+      player : (v14)
+        ((p {l|r} <unum> [g] <player_type_id>)
+         <pos.x> <pos.y> <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
+         (<stamina> <effort> <recovery>[ <capacity>])
+         [t|k] [y|r])
+
       */
 
-    char *next;
+    char * next;
 
-    while ( *msg != ' ' ) ++msg; // skip "(fullstate"
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(fullstate"
     // play mode
-    while ( *msg != '(' ) ++msg; // skip to "(pmode"
-    while ( *msg != ')' ) ++msg; // ignore playmode info
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to "(pmode"
+    while ( *msg != '\0' && *msg != ')' ) ++msg; // ignore playmode info
 
     // view mode
-    while ( *msg != '(' ) ++msg; // skip to (vmode
-    while ( *msg != ')' ) ++msg;// get_view_mode(msg); // ignore view mode info
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to (vmode
+    while ( *msg != '\0' && *msg != ')' ) ++msg;// get_view_mode(msg); // ignore view mode info
 
     // stamina info or count info
-    while ( *msg != '(' ) ++msg; // skip to next token
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to next token
     if ( ! std::strncmp( msg, "(stamina", 8 ) )
     {
-        while (*msg != ')') ++msg; // ignore stamina info
+        while (  *msg != '\0' && *msg != ')' ) ++msg; // ignore stamina info
     }
     // count info
     // (count <kicks> <dashes> <turns> <catches> <moves> <turn_necks> <change_views> <says>)
-    while ( *msg != '(' ) ++msg; // skip to "(count"
-    while ( *msg != ')' ) ++msg; // ignore count info
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to "(count"
+    while ( *msg != '\0' && *msg != ')' ) ++msg; // ignore count info
 
     // arm info
     // (arm (movable <MOVABLE>) (expires <EXP>) (target <DIST> <DIR>) (count <CNT>))
-    while ( *msg != '(' ) ++msg; // skip to "(arm..."
-    while ( *msg != ')' ) ++msg; // skip to movable end
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to "(arm..."
+    while ( *msg != '\0' && *msg != ')' ) ++msg; // skip to movable end
     ++msg;
-    while ( *msg != ')' ) ++msg; // skip to expires end
+    while ( *msg != '\0' && *msg != ')' ) ++msg; // skip to expires end
     ++msg;
-    while ( *msg != ')' ) ++msg; // skip to target end
+    while ( *msg != '\0' && *msg != ')' ) ++msg; // skip to target end
     ++msg;
-    while ( *msg != ')' ) ++msg; // skip to count end
+    while ( *msg != '\0' && *msg != ')' ) ++msg; // skip to count end
 
     // score info
     // (score <team_points> <enemy_points>)
-    while ( *msg != '(' ) ++msg; // skip to (score
-    while ( *msg != ' ' ) ++msg; // skip to " LSCORE..."
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to (score
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip to " LSCORE..."
     M_left_score = static_cast< int >( std::strtol( msg, &next, 10) ); msg = next;
     M_right_score = static_cast< int >( std::strtol( msg, &next, 10 ) ); msg = next;
 
     // ball info
     // ((b) <pos.x> <pos.y> <vel.x> <vel.y>)
-    while ( *msg != '(' ) ++msg; // skip to (ball
-    while ( *msg != ' ' ) ++msg; // skip "(ball"
+    while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to (ball
+    while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip "(ball"
 
     M_ball.pos_.x = std::strtod( msg, &next ); msg = next;
     M_ball.pos_.y = std::strtod( msg, &next ); msg = next;
     M_ball.vel_.x = std::strtod( msg, &next ); msg = next;
     M_ball.vel_.y = std::strtod( msg, &next ); msg = next;
 
-    //((p {l|r} <unum> {g|<player_type_id>}) <pos.x> <pos.y>
-    // <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
-    // (<stamina> <effort> <recovery>[ <capacity>]))
+    //((p {l|r} <unum>{g|<player_type_id>}) <pos.x> <pos.y>
+    //   <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
+    //   (<stamina> <effort> <recovery>[ <capacity>])
+    //   [t|k] [y|r])
+    //
+    //((p {l|r} <unum> [g] <player_type_id>}) <pos.x> <pos.y>
+    //   <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
+    //   (<stamina> <effort> <recovery>[ <capacity>])
+    //   [t|k|f] [y|r])
     while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to "(p"
     while ( *msg != '\0' )
     {
@@ -265,22 +280,24 @@ FullstateSensor::parseV8( const char * msg )
         msg += 2; // skip "l " or "r "
         player.unum_ = static_cast< int >( std::strtol( msg, &next, 10 ) );
         msg = next;
-        if ( *msg == ' ' )
+
+        while ( *msg == ' ' ) ++msg;
+
+        if ( *msg == 'g' )
         {
-            if ( *( msg + 1 ) == 'g' )
-            {
-                player.goalie_ = true;
-                player.type_ = Hetero_Default;
-            }
-            else
-            {
-                player.type_ = static_cast< int >( std::strtol( msg, &next, 10 ) );
-                msg = next;
-            }
-            while ( *msg != ')' ) ++msg; // skip " g" or " ID"
+            player.goalie_ = true;
+            player.type_ = Hetero_Default;
+            ++msg;
+            while ( *msg == ' ' ) ++msg;
         }
 
-        while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip to x pos
+        if ( std::isdigit( *msg ) )
+        {
+            player.type_ = static_cast< int >( std::strtol( msg, &next, 10 ) );
+            msg = next;
+        }
+
+        while ( *msg == ' ' || *msg == ')' ) ++msg; // skip to x pos
 
         player.pos_.x = std::strtod( msg, &next ); msg = next;
         player.pos_.y = std::strtod( msg, &next ); msg = next;
@@ -305,15 +322,35 @@ FullstateSensor::parseV8( const char * msg )
             player.stamina_capacity_ = std::strtod( msg, &next ); msg = next;
         }
 
+        while ( *msg == ')' ) ++msg;
         while ( *msg == ' ' ) ++msg;
 
-        if ( *msg == 'k' )
+        if ( *msg == 'k' ) // kick
         {
             player.kicked_ = true;
+            ++msg;
+            while ( *msg == ' ' ) ++msg;
         }
-        else if ( *msg == 't' )
+        else if ( *msg == 't' ) // tackle
         {
             player.tackle_ = true;
+            ++msg;
+            while ( *msg == ' ' ) ++msg;
+        }
+        else if ( *msg == 'f' )
+        {
+            player.charged_ = true;
+            ++msg;
+            while ( *msg == ' ' ) ++msg;
+        }
+
+        if ( *msg == 'y' ) // yellow card
+        {
+            player.card_ = YELLOW;
+        }
+        else if ( *msg == 'r' ) // red card
+        {
+            player.card_ = RED;
         }
 
         if ( LEFT == player.side_ )
