@@ -58,6 +58,7 @@ const std::size_t InterceptTable::MAX_CYCLE = 30;
 */
 InterceptTable::InterceptTable( const WorldModel & world )
     : M_world( world )
+    , M_update_time( 0, 0 )
 {
     M_ball_pos_cache.reserve( MAX_CYCLE + 2 );
     //M_ball_vel_cache.reserve( MAX_CYCLE + 2 );
@@ -108,13 +109,11 @@ InterceptTable::clear()
 void
 InterceptTable::update()
 {
-    static GameTime s_update_time( 0, 0 );
-
-    if ( M_world.time() == s_update_time )
+    if ( M_world.time() == M_update_time )
     {
         return;
     }
-    s_update_time = M_world.time();
+    M_update_time = M_world.time();
 
     dlog.addText( Logger::INTERCEPT,
                   __FILE__" (update)" );
@@ -123,11 +122,8 @@ InterceptTable::update()
     this->clear();
 
     // playmode check
-    // if ball is always stopped in the current playmode,
-    // not needed to predict ball chaser.
-    if ( M_world.gameMode().type() != GameMode::PlayOn
-         && M_world.gameMode().type() != GameMode::GoalKick_
-         && M_world.gameMode().type() != GameMode::PenaltyTaken_ )
+    if ( M_world.gameMode().type() == GameMode::TimeOver
+         || M_world.gameMode().type() == GameMode::BeforeKickOff )
     {
         return;
     }
@@ -334,9 +330,14 @@ InterceptTable::hearOpponent( const int unum,
 void
 InterceptTable::createBallCache()
 {
-    const double pitch_x_max = ServerParam::i().pitchHalfLength() + 5.0;
-    const double pitch_y_max = ServerParam::i().pitchHalfWidth() + 5.0;
-    const double bdecay = ServerParam::i().ballDecay();
+    const ServerParam & SP = ServerParam::i();
+    const double pitch_x_max = ( SP.keepawayMode()
+                                 ? SP.keepawayLength() * 0.5
+                                 : SP.pitchHalfLength() + 5.0 );
+    const double pitch_y_max = ( SP.keepawayMode()
+                                 ? SP.keepawayWidth() * 0.5
+                                 : SP.pitchHalfWidth() + 5.0 );
+    const double bdecay = SP.ballDecay();
 
     Vector2D bpos = M_world.ball().pos();
     Vector2D bvel = M_world.ball().vel();
@@ -403,6 +404,8 @@ InterceptTable::predictSelf()
                   << M_world.time()
                   << " Interecet Self cache is empty!"
                   << std::endl;
+        dlog.addText( Logger::INTERCEPT,
+                      "Intercept Self. Self cache is empty!" );
         // if self cache is empty,
         // reach point should be the inertia final point of the ball
         return;
