@@ -33,6 +33,7 @@
 #define RCSC_PLAYER_INTERCEPT_TABLE_H
 
 #include <rcsc/geom/vector_2d.h>
+#include <rcsc/game_time.h>
 #include <vector>
 
 namespace rcsc {
@@ -271,6 +272,9 @@ private:
     //! const reference to the WorldModel instance
     const WorldModel & M_world;
 
+    //! last updated time
+    GameTime M_update_time;
+
     //! cache of predicted future ball positions
     std::vector< Vector2D > M_ball_pos_cache;
     //std::vector< Vector2D > M_ball_vel_cache;
