@@ -61,12 +61,12 @@ const double ObjectTable::SERVER_EPS = 1.0e-10;
 /*!
 
  */
-ObjectTable::ObjectTable( const SideID ourside )
+ObjectTable::ObjectTable()
 {
     M_static_table.reserve( 400 );
     M_movable_table.reserve( 70 );
 
-    createLandmarkMap( ourside );
+    createLandmarkMap();
 
     createTable();
 }
@@ -76,7 +76,7 @@ ObjectTable::ObjectTable( const SideID ourside )
 
 */
 void
-ObjectTable::createLandmarkMap( const SideID ourside )
+ObjectTable::createLandmarkMap()
 {
     ///////////////////////////////////////////////////////////////////////
     const double pitch_half_w   = ServerParam::i().pitchHalfWidth();
@@ -86,78 +86,76 @@ ObjectTable::createLandmarkMap( const SideID ourside )
     const double goal_half_w    = ServerParam::i().goalHalfWidth();
     ///////////////////////////////////////////////////////////////////////
 
-    const double rotate = ( ourside == LEFT ) ? 1.0 : -1.0;
+    M_landmark_map[Goal_L]    = Vector2D( -pitch_half_l,   0.0 );
+    M_landmark_map[Goal_R]    = Vector2D( +pitch_half_l,   0.0 );
 
-    M_landmark_map[Goal_L]    = Vector2D(-pitch_half_l,   0.0) * rotate;
-    M_landmark_map[Goal_R]    = Vector2D( pitch_half_l,   0.0) * rotate;
+    M_landmark_map[Flag_C]    = Vector2D(           0.0,           0.0 );
+    M_landmark_map[Flag_CT]   = Vector2D(           0.0, -pitch_half_w );
+    M_landmark_map[Flag_CB]   = Vector2D(           0.0, +pitch_half_w );
+    M_landmark_map[Flag_LT]   = Vector2D( -pitch_half_l, -pitch_half_w );
+    M_landmark_map[Flag_LB]   = Vector2D( -pitch_half_l, +pitch_half_w );
+    M_landmark_map[Flag_RT]   = Vector2D( +pitch_half_l, -pitch_half_w );
+    M_landmark_map[Flag_RB]   = Vector2D( +pitch_half_l, +pitch_half_w );
 
-    M_landmark_map[Flag_C]    = Vector2D(          0.0,           0.0) * rotate;
-    M_landmark_map[Flag_CT]   = Vector2D(          0.0, -pitch_half_w) * rotate;
-    M_landmark_map[Flag_CB]   = Vector2D(          0.0,  pitch_half_w) * rotate;
-    M_landmark_map[Flag_LT]   = Vector2D(-pitch_half_l, -pitch_half_w) * rotate;
-    M_landmark_map[Flag_LB]   = Vector2D(-pitch_half_l,  pitch_half_w) * rotate;
-    M_landmark_map[Flag_RT]   = Vector2D( pitch_half_l, -pitch_half_w) * rotate;
-    M_landmark_map[Flag_RB]   = Vector2D( pitch_half_l,  pitch_half_w) * rotate;
+    M_landmark_map[Flag_PLT]  = Vector2D( -(pitch_half_l - penalty_l), -penalty_half_w );
+    M_landmark_map[Flag_PLC]  = Vector2D( -(pitch_half_l - penalty_l),             0.0 );
+    M_landmark_map[Flag_PLB]  = Vector2D( -(pitch_half_l - penalty_l), +penalty_half_w );
+    M_landmark_map[Flag_PRT]  = Vector2D( +(pitch_half_l - penalty_l), -penalty_half_w );
+    M_landmark_map[Flag_PRC]  = Vector2D( +(pitch_half_l - penalty_l),             0.0 );
+    M_landmark_map[Flag_PRB]  = Vector2D( +(pitch_half_l - penalty_l), +penalty_half_w );
 
-    M_landmark_map[Flag_PLT]  = Vector2D(-(pitch_half_l - penalty_l), -penalty_half_w) * rotate;
-    M_landmark_map[Flag_PLC]  = Vector2D(-(pitch_half_l - penalty_l),             0.0) * rotate;
-    M_landmark_map[Flag_PLB]  = Vector2D(-(pitch_half_l - penalty_l),  penalty_half_w) * rotate;
-    M_landmark_map[Flag_PRT]  = Vector2D( (pitch_half_l - penalty_l), -penalty_half_w) * rotate;
-    M_landmark_map[Flag_PRC]  = Vector2D( (pitch_half_l - penalty_l),             0.0) * rotate;
-    M_landmark_map[Flag_PRB]  = Vector2D( (pitch_half_l - penalty_l),  penalty_half_w) * rotate;
+    M_landmark_map[Flag_GLT]  = Vector2D( -pitch_half_l, -goal_half_w );
+    M_landmark_map[Flag_GLB]  = Vector2D( -pitch_half_l, +goal_half_w );
+    M_landmark_map[Flag_GRT]  = Vector2D( +pitch_half_l, -goal_half_w );
+    M_landmark_map[Flag_GRB]  = Vector2D( +pitch_half_l, +goal_half_w );
 
-    M_landmark_map[Flag_GLT]  = Vector2D(-pitch_half_l, -goal_half_w) * rotate;
-    M_landmark_map[Flag_GLB]  = Vector2D(-pitch_half_l,  goal_half_w) * rotate;
-    M_landmark_map[Flag_GRT]  = Vector2D( pitch_half_l, -goal_half_w) * rotate;
-    M_landmark_map[Flag_GRB]  = Vector2D( pitch_half_l,  goal_half_w) * rotate;
+    M_landmark_map[Flag_TL50] = Vector2D( -50.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TL40] = Vector2D( -40.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TL30] = Vector2D( -30.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TL20] = Vector2D( -20.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TL10] = Vector2D( -10.0, -pitch_half_w - 5.0 );
 
-    M_landmark_map[Flag_TL50] = Vector2D(-50.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TL40] = Vector2D(-40.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TL30] = Vector2D(-30.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TL20] = Vector2D(-20.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TL10] = Vector2D(-10.0, -pitch_half_w - 5.0) * rotate;
+    M_landmark_map[Flag_T0]   = Vector2D(  0.0, -pitch_half_w - 5.0 );
 
-    M_landmark_map[Flag_T0]   = Vector2D(  0.0, -pitch_half_w - 5.0) * rotate;
+    M_landmark_map[Flag_TR10] = Vector2D( +10.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TR20] = Vector2D( +20.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TR30] = Vector2D( +30.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TR40] = Vector2D( +40.0, -pitch_half_w - 5.0 );
+    M_landmark_map[Flag_TR50] = Vector2D( +50.0, -pitch_half_w - 5.0 );
 
-    M_landmark_map[Flag_TR10] = Vector2D( 10.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TR20] = Vector2D( 20.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TR30] = Vector2D( 30.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TR40] = Vector2D( 40.0, -pitch_half_w - 5.0) * rotate;
-    M_landmark_map[Flag_TR50] = Vector2D( 50.0, -pitch_half_w - 5.0) * rotate;
+    M_landmark_map[Flag_BL50] = Vector2D( -50.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BL40] = Vector2D( -40.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BL30] = Vector2D( -30.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BL20] = Vector2D( -20.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BL10] = Vector2D( -10.0,  pitch_half_w + 5.0 );
 
-    M_landmark_map[Flag_BL50] = Vector2D(-50.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BL40] = Vector2D(-40.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BL30] = Vector2D(-30.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BL20] = Vector2D(-20.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BL10] = Vector2D(-10.0,  pitch_half_w + 5.0) * rotate;
+    M_landmark_map[Flag_B0]   = Vector2D(   0.0,  pitch_half_w + 5.0);
 
-    M_landmark_map[Flag_B0]   = Vector2D(  0.0,  pitch_half_w + 5.0) * rotate;
+    M_landmark_map[Flag_BR10] = Vector2D( +10.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BR20] = Vector2D( +20.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BR30] = Vector2D( +30.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BR40] = Vector2D( +40.0,  pitch_half_w + 5.0 );
+    M_landmark_map[Flag_BR50] = Vector2D( +50.0,  pitch_half_w + 5.0 );
 
-    M_landmark_map[Flag_BR10] = Vector2D( 10.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BR20] = Vector2D( 20.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BR30] = Vector2D( 30.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BR40] = Vector2D( 40.0,  pitch_half_w + 5.0) * rotate;
-    M_landmark_map[Flag_BR50] = Vector2D( 50.0,  pitch_half_w + 5.0) * rotate;
+    M_landmark_map[Flag_LT30] = Vector2D( -pitch_half_l - 5.0, -30.0 );
+    M_landmark_map[Flag_LT20] = Vector2D( -pitch_half_l - 5.0, -20.0 );
+    M_landmark_map[Flag_LT10] = Vector2D( -pitch_half_l - 5.0, -10.0 );
 
-    M_landmark_map[Flag_LT30] = Vector2D(-pitch_half_l - 5.0, -30.0) * rotate;
-    M_landmark_map[Flag_LT20] = Vector2D(-pitch_half_l - 5.0, -20.0) * rotate;
-    M_landmark_map[Flag_LT10] = Vector2D(-pitch_half_l - 5.0, -10.0) * rotate;
+    M_landmark_map[Flag_L0]   = Vector2D( -pitch_half_l - 5.0,   0.0 );
 
-    M_landmark_map[Flag_L0]   = Vector2D(-pitch_half_l - 5.0,   0.0) * rotate;
+    M_landmark_map[Flag_LB10] = Vector2D( -pitch_half_l - 5.0,  10.0 );
+    M_landmark_map[Flag_LB20] = Vector2D( -pitch_half_l - 5.0,  20.0 );
+    M_landmark_map[Flag_LB30] = Vector2D( -pitch_half_l - 5.0,  30.0 );
 
-    M_landmark_map[Flag_LB10] = Vector2D(-pitch_half_l - 5.0,  10.0) * rotate;
-    M_landmark_map[Flag_LB20] = Vector2D(-pitch_half_l - 5.0,  20.0) * rotate;
-    M_landmark_map[Flag_LB30] = Vector2D(-pitch_half_l - 5.0,  30.0) * rotate;
+    M_landmark_map[Flag_RT30] = Vector2D( +pitch_half_l + 5.0, -30.0 );
+    M_landmark_map[Flag_RT20] = Vector2D( +pitch_half_l + 5.0, -20.0 );
+    M_landmark_map[Flag_RT10] = Vector2D( +pitch_half_l + 5.0, -10.0 );
 
-    M_landmark_map[Flag_RT30] = Vector2D( pitch_half_l + 5.0, -30.0) * rotate;
-    M_landmark_map[Flag_RT20] = Vector2D( pitch_half_l + 5.0, -20.0) * rotate;
-    M_landmark_map[Flag_RT10] = Vector2D( pitch_half_l + 5.0, -10.0) * rotate;
+    M_landmark_map[Flag_R0]   = Vector2D( +pitch_half_l + 5.0,   0.0 );
 
-    M_landmark_map[Flag_R0]   = Vector2D( pitch_half_l + 5.0,   0.0) * rotate;
-
-    M_landmark_map[Flag_RB10] = Vector2D( pitch_half_l + 5.0,  10.0) * rotate;
-    M_landmark_map[Flag_RB20] = Vector2D( pitch_half_l + 5.0,  20.0) * rotate;
-    M_landmark_map[Flag_RB30] = Vector2D( pitch_half_l + 5.0,  30.0) * rotate;
+    M_landmark_map[Flag_RB10] = Vector2D( +pitch_half_l + 5.0,  10.0 );
+    M_landmark_map[Flag_RB20] = Vector2D( +pitch_half_l + 5.0,  20.0 );
+    M_landmark_map[Flag_RB30] = Vector2D( +pitch_half_l + 5.0,  30.0 );
 }
 
 /*-------------------------------------------------------------------*/
