@@ -179,7 +179,14 @@ PlayerCatchCommand::toStr( std::ostream & to ) const
 std::ostream &
 PlayerTackleCommand::toStr( std::ostream & to ) const
 {
-    return to << "(tackle " << M_power_or_dir << ")";
+    to << "(tackle " << M_power_or_dir;
+    if ( M_foul )
+    {
+        to << " on";
+    }
+    to << ')';
+
+    return to;
 }
 
 /*-------------------------------------------------------------------*/
