@@ -323,76 +323,6 @@ public:
     virtual
     std::string name() const = 0;
 
-
-    /*!
-      \brief get move command parameter
-      \return move position
-     */
-    virtual
-    Vector2D movePos() const
-      {
-          return Vector2D::INVALIDATED;
-      }
-
-    /*!
-      \brief get dash command parameter
-      \return dash power
-     */
-    virtual
-    double dashPower() const
-      {
-          return 0.0;
-      }
-
-    /*!
-      \brief get turn command parameter
-      \return turn moment
-     */
-    virtual
-    double turnMoment() const
-      {
-          return 0.0;
-      }
-
-    /*!
-      \brief get kick command parameter
-      \return kick power
-     */
-    virtual
-    double kickPower() const
-      {
-          return 0.0;
-      }
-
-    /*!
-      \brief get kick command parameter
-      \return kick direction
-     */
-    virtual
-    double kickDir() const
-      {
-          return 0.0;
-      }
-
-    /*!
-      \brief get catch command parameter
-      \return catch direction
-     */
-    virtual
-    double catchDir() const
-      {
-          return 0.0;
-      }
-
-    /*!
-      \brief get tackle command parameter
-      \return tackle power
-     */
-    virtual
-    double tacklePower() const
-      {
-          return 0.0;
-      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -732,12 +662,14 @@ public:
   <pre>
   Format:
   <- (tackle <power_or_dir>)
+  <- (tackle <power_or_dir> <foul>)
   </pre>
 */
 class PlayerTackleCommand
     : public PlayerBodyCommand {
 private:
     double M_power_or_dir; //!< tackle power or direction
+    bool M_foul; //!< intentional foul switch
 public:
     /*!
       \brief construct with tackle power
@@ -745,7 +677,14 @@ public:
     */
     explicit
     PlayerTackleCommand( const double & power_or_dir )
-        : M_power_or_dir( power_or_dir )
+        : M_power_or_dir( power_or_dir ),
+          M_foul( false )
+      { }
+
+    PlayerTackleCommand( const double & power_or_dir,
+                         const bool foul )
+        : M_power_or_dir( power_or_dir ),
+          M_foul( foul )
       { }
 
     /*!
@@ -777,11 +716,15 @@ public:
       \brief get tackle command parameter
       \return tackle power
      */
-    virtual
     double tacklePowerOrDir() const
       {
           return M_power_or_dir;
       }
+
+    bool tackleFoul() const
+      {
+          return M_foul;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
