@@ -70,7 +70,7 @@ PlayerConfig::setDefaultParam()
 {
     // basic setting
     M_team_name = "HELIOS_base";
-    M_version = 13;
+    M_version = 14;
     M_reconnect_number = 0;
     M_goalie = false;
 
@@ -159,10 +159,13 @@ PlayerConfig::setDefaultParam()
     M_debug_clear = false;
     M_debug_block = false;
     M_debug_mark = false;
+    M_debug_positioning = false;
     M_debug_role = false;
+    M_debug_plan = false;
     M_debug_team = false;
+    M_debug_communication = false;
     M_debug_analyzer = false;
-    M_debug_recursive = false;
+    M_debug_action_chain = false;
 }
 
 /*-------------------------------------------------------------------*/
@@ -230,6 +233,7 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "offline_client_number", "", &M_offline_client_number )
 
         ( "debug_log_ext", "", &M_debug_log_ext )
+
         ( "debug_system", "", BoolSwitch( &M_debug_system ) )
         ( "debug_sensor", "", BoolSwitch( &M_debug_sensor ) )
         ( "debug_world", "", BoolSwitch( &M_debug_world ) )
@@ -244,10 +248,13 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "debug_clear", "", BoolSwitch( &M_debug_clear ) )
         ( "debug_block", "", BoolSwitch( &M_debug_block ) )
         ( "debug_mark", "", BoolSwitch( &M_debug_mark ) )
+        ( "debug_positioning", "", BoolSwitch( &M_debug_positioning ) )
         ( "debug_role", "", BoolSwitch( &M_debug_role ) )
+        ( "debug_plan", "", BoolSwitch( &M_debug_plan ) )
         ( "debug_team", "", BoolSwitch( &M_debug_team ) )
+        ( "debug_communication", "", BoolSwitch( &M_debug_communication ) )
         ( "debug_analyzer", "", BoolSwitch( &M_debug_analyzer ) )
-        ( "debug_recursive", "", BoolSwitch( &M_debug_recursive ) )
+        ( "debug_action_chain", "", BoolSwitch( &M_debug_action_chain ) )
         ;
 }
 
