@@ -149,10 +149,13 @@ private:
     bool M_debug_clear; //!< debug level flag
     bool M_debug_block; //!< debug level flag
     bool M_debug_mark; //!< debug level flag
+    bool M_debug_positioning; //!< debug level flag
     bool M_debug_role; //!< debug level flag
+    bool M_debug_plan; //!< debug level flag
     bool M_debug_team; //!< debug level flag
+    bool M_debug_communication; //!< debug level flag
     bool M_debug_analyzer; //!< debug level flag
-    bool M_debug_recursive; //!< debug level flag
+    bool M_debug_action_chain; //!< debug level flag
 
 public:
 
@@ -188,175 +191,115 @@ public:
       \brief get the team name string
       \return team name string
      */
-    const
-    std::string & teamName() const
-      {
-          return M_team_name;
-      }
+    const std::string & teamName() const { return M_team_name; }
 
     /*!
       \brief get the client version
       \return client version
      */
-    const
-    double & version() const
-      {
-          return M_version;
-      }
+    const double & version() const { return M_version; }
 
     /*!
       \brief get the uniform numver for a reconnect command
       \return the uniform numver for a reconnect command
      */
-    int reconnectNumber() const
-      {
-          return M_reconnect_number;
-      }
+    int reconnectNumber() const { return M_reconnect_number; }
 
     /*!
       \brief get the goalie flag
       \return goalie flag
      */
-    bool goalie() const
-      {
-          return M_goalie;
-      }
+    bool goalie() const { return M_goalie; }
 
     /*!
       \brief get the interrupt interval for decision making
       \return interrupt interval in milli-seconds
      */
-    int intervalMSec() const
-      {
-          return M_interval_msec;
-      }
+    int intervalMSec() const { return M_interval_msec; }
 
     /*!
       \brief get the maximum time to wait server response
       \return wait time in seconds.
      */
-    int serverWaitSeconds() const
-      {
-          return M_server_wait_seconds;
-      }
+    int serverWaitSeconds() const { return M_server_wait_seconds; }
 
     /*!
       \brief get the maximum time to wait see message for synch view mode
       \return wait time in milli-seconds
      */
-    int waitTimeThrSynchView() const
-      {
-          return M_wait_time_thr_synch_view;
-      }
+    int waitTimeThrSynchView() const { return M_wait_time_thr_synch_view; }
 
     /*!
       \brief get the maximum time to wait see message for asynch view mode
       \return wait time in milli-seconds
      */
-    int waitTimeThrNoSynchView() const
-      {
-          return M_wait_time_thr_nosynch_view;
-      }
+    int waitTimeThrNoSynchView() const { return M_wait_time_thr_nosynch_view; }
 
     /*!
       \brief get the threshold time to change to normal view width for old timer synch view mode
       \return the threshold time to change to normal view width
      */
-    int normalViewTimeThr() const
-      {
-          return M_normal_view_time_thr;
-      }
+    int normalViewTimeThr() const { return M_normal_view_time_thr; }
 
     /*!
       \brief get the server host name string
       \return server host name string
      */
-    const
-    std::string & host() const
-      {
-          return M_rcssserver_host;
-      }
+    const std::string & host() const { return M_rcssserver_host; }
 
     /*!
       \brief get the server port number
       \return server port number
      */
-    int port() const
-      {
-          return M_rcssserver_port;
-      }
+    int port() const { return M_rcssserver_port; }
 
     /*!
       \brief get the server message compression level
       \return server message compression level
      */
-    int compression() const
-      {
-          return M_compression;
-      }
+    int compression() const { return M_compression; }
 
     /*!
       \brief get the minimum clang version to be sent
       \return minimum clang version to be sent
      */
-    int clangMin() const
-      {
-          return M_clang_min;
-      }
+    int clangMin() const { return M_clang_min; }
 
     /*!
       \brief get the maximum clang version to be sent
       \return maximum clang version to be sent
      */
-    int clangMax() const
-      {
-          return M_clang_max;
-      }
+    int clangMax() const { return M_clang_max; }
 
     /*!
       \brief get the communication mode flag
       \return communication mode flag
      */
-    bool useCommunication() const
-      {
-          return M_use_communication;
-      }
+    bool useCommunication() const { return M_use_communication; }
 
     /*!
       \brief get the auditory sensor mode flag for opponent
       \return auditory sensor mode flag for opponent
      */
-    bool hearOpponentAudio() const
-      {
-          return M_hear_opponent_audio;
-      }
+    bool hearOpponentAudio() const { return M_hear_opponent_audio; }
 
     /*!
       \brief get the fullstate mode flag that determines whether the world model is updated by fullstate information
       \return fullstate mode flag
      */
-    bool useFullstate() const
-      {
-          return M_use_fullstate;
-      }
+    bool useFullstate() const { return M_use_fullstate; }
 
     /*!
       \brief get the switch variable thate determines whether a fullstate world model for debugging is used or not.
       \return debugging fullstate world model switch
      */
-    bool debugFullstate() const
-      {
-          return M_debug_fullstate;
-      }
+    bool debugFullstate() const { return M_debug_fullstate; }
 
     /*!
       \brief get the synch_see mode flag
       \return synch_see mode flag
      */
-    bool synchSee() const
-      {
-          return M_synch_see;
-      }
+    bool synchSee() const { return M_synch_see; }
 
     // confidence value
 
@@ -364,110 +307,73 @@ public:
       \brief get the self position accuracy count threshold
       \return self position accuracy count threshold
      */
-    int selfPosCountThr() const
-      {
-          return M_self_pos_count_thr;
-      }
+    int selfPosCountThr() const { return M_self_pos_count_thr; }
 
     /*!
       \brief get the self velocity accuracy count threshold
       \return self velocity accuracy count threshold
      */
-    int selfVelCountThr() const
-      {
-          return M_self_vel_count_thr;
-      }
+    int selfVelCountThr() const { return M_self_vel_count_thr; }
 
     /*!
       \brief get the self face angle accuracy count threshold
       \return self face angle accuracy count threshold
      */
-    int selfFaceCountThr() const
-      {
-          return M_self_face_count_thr;
-      }
+    int selfFaceCountThr() const { return M_self_face_count_thr; }
 
     /*!
       \brief get the ball position accuracy count threshold
       \return ball position accuracy count threshold
      */
-    int ballPosCountThr() const
-      {
-          return M_ball_pos_count_thr;
-      }
+    int ballPosCountThr() const { return M_ball_pos_count_thr; }
 
     /*!
       \brief get the ball relative position accuracy count threshold
       \return ball relative position accuracy count threshold
      */
-    int ballRPosCountThr() const
-      {
-          return M_ball_rpos_count_thr;
-      }
+    int ballRPosCountThr() const { return M_ball_rpos_count_thr; }
 
     /*!
       \brief get the ball velocity accuracy count threshold
       \return ball velocity accuracy count threshold
      */
-    int ballVelCountThr() const
-      {
-          return M_ball_vel_count_thr;
-      }
+    int ballVelCountThr() const { return M_ball_vel_count_thr; }
 
     /*!
       \brief get the other player position accuracy count threshold
       \return other player position accuracy count threshold
      */
-    int playerPosCountThr() const
-      {
-          return M_player_pos_count_thr;
-      }
+    int playerPosCountThr() const { return M_player_pos_count_thr; }
 
     /*!
       \brief get the other player position accuracy count threshold
       \return other player velocity accuracy count threshold
      */
-    int playerVelCountThr() const
-      {
-          return M_player_vel_count_thr;
-      }
+    int playerVelCountThr() const { return M_player_vel_count_thr; }
 
     /*!
       \brief get the other player face angle accuracy count threshold
       \return other player face angle accuracy count threshold
      */
-    int playerFaceCountThr() const
-      {
-          return M_player_face_count_thr;
-      }
+    int playerFaceCountThr() const { return M_player_face_count_thr; }
 
     /*!
       \brief get the player number (not a uniform number)
       \return player number
      */
-    int playerNumber() const
-      {
-          return M_player_number;
-      }
+    int playerNumber() const { return M_player_number; }
 
     /*!
       \brief set a player number (not a uniform number)
       \param num player number
      */
-    void setPlayerNumber( const int num )
-      {
-          M_player_number = num;
-      }
+    void setPlayerNumber( const int num ) { M_player_number = num; }
 
     /*!
       \brief get the configulation directory path
       \return configulation directory path
      */
-    const
-    std::string & configDir() const
-      {
-          return M_config_dir;
-      }
+    const std::string & configDir() const { return M_config_dir; }
 
     //
     // debug
@@ -477,20 +383,13 @@ public:
       \brief get the debug flag
       \return debug flag
      */
-    bool debug() const
-      {
-          return M_debug;
-      }
+    bool debug() const { return M_debug; }
 
     /*!
       \brief get the debug log directory path string.
       \return the debug log directory string.
      */
-    const
-    std::string & logDir() const
-      {
-          return M_log_dir;
-      }
+    const std::string & logDir() const { return M_log_dir; }
 
     //
     // debug server settings
@@ -500,38 +399,25 @@ public:
       \brief get the debug flag whether debug information is sent to a debug server
       \return debug flag whether debug information is sent to a debug server
      */
-    bool debugServerConnect() const
-      {
-          return M_debug_server_connect;
-      }
+    bool debugServerConnect() const { return M_debug_server_connect; }
 
     /*!
       \brief get the debug flag whether debug information is written to files
       \return debug flag whether debug information is written to files
      */
-    bool debugServerLogging() const
-      {
-          return M_debug_server_logging;
-      }
+    bool debugServerLogging() const { return M_debug_server_logging; }
 
     /*!
       \brief get the debug server host name string
       \return debug server host name string
      */
-    const
-    std::string & debugServerHost() const
-      {
-          return M_debug_server_host;
-      }
+    const std::string & debugServerHost() const { return M_debug_server_host; }
 
     /*!
       \brief get the debug server port number
       \return debug server port number
      */
-    int debugServerPort() const
-      {
-          return M_debug_server_port;
-      }
+    int debugServerPort() const { return M_debug_server_port; }
 
     //
     // offline client
@@ -541,30 +427,20 @@ public:
       \brief get the switch for offline logging. this value is ignored if client runs in offline clinet mode.
       \return switch value for offline logging.
      */
-    bool offlineLogging() const
-      {
-          return M_offline_logging;
-      }
+    bool offlineLogging() const { return M_offline_logging; }
 
     /*!
       \brief get the offline client log file extention string.
       \return the offline client log file extention string.
      */
-    const
-    std::string & offlineLogExt() const
-      {
-          return M_offline_log_ext;
-      }
+    const std::string & offlineLogExt() const { return M_offline_log_ext; }
 
     /*!
       \brief get the uniform number for offline client mode.
       1-11 means offline mode, other values mean online mode.
       \return the uniform number for offline client mode.
      */
-    int offlineClientNumber() const
-      {
-          return M_offline_client_number;
-      }
+    int offlineClientNumber() const { return M_offline_client_number; }
 
     //
     // debug logging
@@ -574,173 +450,135 @@ public:
       \brief get the debug log file extention string.
       \return the debug log file extention string.
      */
-    const
-    std::string & debugLogExt() const
-      {
-          return M_debug_log_ext;
-      }
+    const std::string & debugLogExt() const { return M_debug_log_ext; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugSystem() const
-      {
-          return M_debug_system;
-      }
+    bool debugSystem() const { return M_debug_system; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugSensor() const
-      {
-          return M_debug_sensor;
-      }
+    bool debugSensor() const { return M_debug_sensor; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugWorld() const
-      {
-          return M_debug_world;
-      }
+    bool debugWorld() const { return M_debug_world; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugAction() const
-      {
-          return M_debug_action;
-      }
+    bool debugAction() const { return M_debug_action; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugIntercept() const
-      {
-          return M_debug_intercept;
-      }
+    bool debugIntercept() const { return M_debug_intercept; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugKick() const
-      {
-          return M_debug_kick;
-      }
+    bool debugKick() const { return M_debug_kick; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugHold() const
-      {
-          return M_debug_hold;
-      }
+    bool debugHold() const { return M_debug_hold; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugDribble() const
-      {
-          return M_debug_dribble;
-      }
+    bool debugDribble() const { return M_debug_dribble; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugPass() const
-      {
-          return M_debug_pass;
-      }
+    bool debugPass() const { return M_debug_pass; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugCross() const
-      {
-          return M_debug_cross;
-      }
+    bool debugCross() const { return M_debug_cross; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugShoot() const
-      {
-          return M_debug_shoot;
-      }
+    bool debugShoot() const { return M_debug_shoot; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugClear() const
-      {
-          return M_debug_clear;
-      }
+    bool debugClear() const { return M_debug_clear; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugBlock() const
-      {
-          return M_debug_block;
-      }
+    bool debugBlock() const { return M_debug_block; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugMark() const
-      {
-          return M_debug_mark;
-      }
+    bool debugMark() const { return M_debug_mark; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugRole() const
-      {
-          return M_debug_role;
-      }
+    bool debugPositioning() const { return M_debug_positioning; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugTeam() const
-      {
-          return M_debug_team;
-      }
+    bool debugRole() const { return M_debug_role; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugAnalyzer() const
-      {
-          return M_debug_analyzer;
-      }
+    bool debugPlan() const { return M_debug_plan; }
 
     /*!
       \brief get the debug flag
       \return debug flag
      */
-    bool debugRecursive() const
-      {
-          return M_debug_recursive;
-      }
+    bool debugTeam() const { return M_debug_team; }
+
+
+    /*!
+      \brief get the debug flag
+      \return debug flag
+     */
+    bool debugCommunication() const { return M_debug_communication; }
+
+
+    /*!
+      \brief get the debug flag
+      \return debug flag
+     */
+    bool debugAnalyzer() const { return M_debug_analyzer; }
+
+    /*!
+      \brief get the debug flag
+      \return debug flag
+     */
+    bool debugActionChain() const { return M_debug_action_chain; }
 
 };
 
