@@ -60,7 +60,6 @@ PlayerObject::PlayerObject()
     , M_rpos_count( 1000 )
     , M_pointto_angle( 0.0 )
     , M_pointto_count( 1000 )
-    , M_kicked( false )
     , M_tackle_count( 1000 )
 {
 
@@ -78,7 +77,6 @@ PlayerObject::PlayerObject( const SideID side,
     , M_rpos( p.rpos_ )
     , M_pointto_angle( 0.0 )
     , M_pointto_count( 1000 )
-    , M_kicked( false )
     , M_tackle_count( 1000 )
     , M_heard_stamina( -1.0 )
 {
@@ -253,7 +251,7 @@ PlayerObject::updateBySee( const SideID side,
                                    : ServerParam::i().defaultPlayerSpeedMax() );
         const double decay = ( M_player_type
                                ? M_player_type->playerDecay()
-                               : ServerParam::i().defaultPlayerSpeedMax() );
+                               : ServerParam::i().defaultPlayerDecay() );
 
         M_vel = last_seen_move / static_cast< double >( last_seen_pos_count );
         double tmp = M_vel.r();
@@ -295,15 +293,15 @@ PlayerObject::updateBySee( const SideID side,
         M_body_count = M_face_count = 0;
     }
     else if ( last_seen_pos_count <= 2
-              && last_seen_move.r2() > std::pow( 0.2, 2 ) ) // TODO: magic number
+              && last_seen_move.r2() > std::pow( 0.2, 2 ) ) // Magic Number
     {
         M_body = last_seen_move.th();
-        M_body_count = last_seen_pos_count;
+        M_body_count = std::max( 0, last_seen_pos_count - 1 );
         M_face = 0.0;
         M_face_count = 1000;
     }
     else if ( velValid()
-              && vel().r2() > std::pow( 0.2, 2 ) )
+              && vel().r2() > std::pow( 0.2, 2 ) ) // Magic Number
     {
         M_body = vel().th();
         M_body_count = velCount();
@@ -412,10 +410,11 @@ PlayerObject::updateByHear( const SideID heard_side,
         M_side = heard_side;
     }
 
-    if ( heard_unum != Unum_Unknown )
+    if ( heard_unum != Unum_Unknown
+         && unumCount() > 0 )
     {
         M_unum = heard_unum;
-        M_unum_count = 1;
+        //M_unum_count = 1;
     }
 
     if ( goalie )
