@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,7 +59,6 @@ private:
     //! validation count threshold value for M_body and M_face
     static int S_face_count_thr;
 
-    double M_dist_from_self; //!< distance from self
     AngleDeg M_angle_from_self; //!< angle from self global
 
     int M_ghost_count; //!< count since this object was recognized as a ghost object.
@@ -70,8 +69,6 @@ private:
     AngleDeg M_pointto_angle; //!< global pointing angle
     int M_pointto_count; //!< pointing angle accuracy
 
-    bool M_kicked; //!< kicking state
-
     int M_tackle_count; //!< tackle info accuracy
 
     double M_heard_stamina; //!< heard stamina
@@ -109,16 +106,6 @@ public:
 
     // ------------------------------------------
 
-    /*!
-      \brief get distance from self
-      \return distance value from self
-    */
-    const
-    double & distFromSelf() const
-      {
-          return M_dist_from_self;
-      }
-
     /*!
       \brief get global angle from self position
       \return angle value from self position
@@ -176,15 +163,6 @@ public:
           return M_pointto_count;
       }
 
-    /*!
-      \brief get kicking state information
-      \return true if player performed the kick.
-     */
-    bool kicked() const
-      {
-          return M_kicked;
-      }
-
     /*!
       \brief get tackling status accuracy
       \return count from last observation
@@ -335,7 +313,7 @@ public:
                        const int heard_unum,
                        const bool goalie,
                        const Vector2D & heard_pos,
-                       const AngleDeg & heard_body,
+                       const double & heard_body,
                        const double & heard_stamina );
 
     /*!
@@ -417,7 +395,9 @@ public:
       \brief predicate functor to compare player's accuracy. pointer version
     */
     class PtrCountCmp
-        : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
+        : public std::binary_function< const PlayerObject *,
+                                       const PlayerObject *,
+                                       bool > {
     public:
         /*!
           \brief operation function
@@ -425,8 +405,8 @@ public:
           \param rhs right hand side variable
           \return compared result
          */
-        result_type operator()( const first_argument_type lhs,
-                                const second_argument_type rhs ) const
+        result_type operator()( first_argument_type lhs,
+                                second_argument_type rhs ) const
           {
               if ( lhs->goalie() ) return true;
               if ( rhs->goalie() ) return false;
@@ -440,7 +420,9 @@ public:
       \brief predicate functor to compare player's distance from self
     */
     class PtrSelfDistCmp
-        : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
+        : public std::binary_function< const PlayerObject *,
+                                       const PlayerObject *,
+                                       bool > {
     public:
         /*!
           \brief operation function
@@ -448,8 +430,8 @@ public:
           \param rhs right hand side variable
           \return compared result
          */
-        result_type operator()( const first_argument_type lhs,
-                                const second_argument_type rhs ) const
+        result_type operator()( first_argument_type lhs,
+                                second_argument_type rhs ) const
           {
               return lhs->distFromSelf() < rhs->distFromSelf();
           }
@@ -460,7 +442,9 @@ public:
       \brief predicate functor to compare player's distance from ball
     */
     class PtrBallDistCmp
-        : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
+        : public std::binary_function< const PlayerObject *,
+                                       const PlayerObject *,
+                                       bool > {
     public:
         /*!
           \brief operation function
@@ -468,8 +452,8 @@ public:
           \param rhs right hand side variable
           \return compared result
          */
-        result_type operator()( const first_argument_type lhs,
-                                const second_argument_type rhs ) const
+        result_type operator()( first_argument_type lhs,
+                                second_argument_type rhs ) const
           {
               return lhs->distFromBall() < rhs->distFromBall();
           }
