@@ -1401,7 +1401,7 @@ public:
           }
           else
           {
-              return p.pos().x < bound( M_world.ball().pos().x, M_world.defenseLineX(), 0.0 );
+              return p.pos().x < bound( M_world.ball().pos().x, M_world.ourDefenseLineX(), 0.0 );
           }
       }
 
@@ -1496,7 +1496,7 @@ public:
 
     /*!
       \brief construct with the geometry object
-      \param geometry object for checking
+      \param geom geometry object for checking
     */
     ContainsPlayerPredicate( const T & geom )
         : M_geom( geom )
