@@ -51,8 +51,7 @@ namespace rcsc {
 bool
 BallMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "BallMessage. over the message size : buf = %d, this = %d",
@@ -66,7 +65,7 @@ BallMessage::toStr( std::string & to ) const
     if ( ! AudioCodec::i().encodePosVelToStr5( M_ball_pos,
                                                M_ball_vel,
                                                msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** BallMessage. "
@@ -113,8 +112,7 @@ BallMessage::printDebug( std::ostream & os ) const
 bool
 PassMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "PassMessage. over the message size : buf = %d, this = %d",
@@ -155,7 +153,7 @@ PassMessage::toStr( std::string & to ) const
         return false;
     }
 
-    if ( msg.length() != slength() - 1 )
+    if ( (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** PassMessage. length"
@@ -205,8 +203,7 @@ PassMessage::printDebug( std::ostream & os ) const
 bool
 GoalieMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "GoalieMessage. over the message size : buf = %d, this = %d",
@@ -250,7 +247,7 @@ GoalieMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** GoalieMessage. "
@@ -293,6 +290,113 @@ GoalieMessage::printDebug( std::ostream & os ) const
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+bool
+GoalieAndPlayerMessage::toStr( std::string & to ) const
+{
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "GoalieAndPlayerMessage. over the message size : buf = %d, this = %d",
+                      to.length(), slength() );
+        return false;
+    }
+
+    if ( M_goalie_pos.x < 53.0 - 16.0
+         || 52.9 < M_goalie_pos.x
+         || M_goalie_pos.absY() > 19.9 )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "GoalieAndPlayerMessage. over the position range : (%.1f %.1f)",
+                      M_goalie_pos.x, M_goalie_pos.y );
+        return false;
+    }
+
+    if ( M_player_number < 1 || 23 < M_player_number )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "GoalieAndPlayerMessage. illegal player number %d",
+                      M_player_number );
+    }
+
+    boost::int64_t ival = 0;
+
+    double goalie_x = bound( 53.0 - 16.0, M_goalie_pos.x, 52.9 ) - ( 53.0 - 16.0 );
+    double goalie_y = bound( -19.9, M_goalie_pos.y, 19.9 ) + 20.0;
+    double goalie_body = M_goalie_body.degree() + 180.0;
+
+    // ival *= 160
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( goalie_x / 0.1 ), 159.0 ) );
+
+    ival *= 400;
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( goalie_y / 0.1 ), 399.0 ) );
+
+    ival *= 360;
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( goalie_body ), 359.0 ) );
+
+    double player_x = bound( -52.49, M_player_pos.x, 52.49 ) + 52.5;
+    double player_y = bound( -33.99, M_player_pos.y, 33.99 ) + 34.0;
+
+    ival *= 22;
+    ival += M_player_number - 1;
+
+    ival *= 191;
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( player_x / 0.555 ), 190.0 ) );
+
+    ival *= 124;
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( player_y / 0.555 ), 123.0 ) );
+
+
+    std::string msg;
+    msg.reserve( slength() - 1 );
+
+    if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
+         || (int)msg.length() != slength() - 1 )
+    {
+        std::cerr << __FILE__ << ":" << __LINE__
+                  << " ***ERROR*** GoalieAndPlayerMessage. "
+                  << std::endl;
+
+        dlog.addText( Logger::SENSOR,
+                      "GoalieAndPlayerMessage. error! goalie unum=%d (%.2f %.2f) body=%.1f"
+                      " player=%d (%.2f %.2f)",
+                      M_goalie_unum, M_goalie_pos.x, M_goalie_pos.y,
+                      M_goalie_body.degree(),
+                      M_player_number, M_player_pos.x, M_player_pos.y );
+        return false;
+    }
+
+    dlog.addText( Logger::SENSOR,
+                  "GoalieAndPlayerMessage. success! goalie=%d (%.2f %.2f) x=%f y=%f"
+                  " player num=%d (%.2f %.2f) -> [%s]",
+                  M_goalie_unum, M_goalie_pos.x, M_goalie_pos.y,
+                  goalie_x, goalie_y,
+                  M_player_number, player_x, player_y,
+                  msg.c_str() );
+
+    to += header();
+    to += msg;
+
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+std::ostream &
+GoalieAndPlayerMessage::printDebug( std::ostream & os ) const
+{
+    os << "[Goalie:"
+       << ( M_player_number > 11 ? "O_" : "T_" )
+       << ( M_player_number > 11 ? M_player_number - 11 : M_player_number )
+       << ']';
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 OffsideLineMessage::toStr( std::string & to ) const
@@ -302,8 +406,7 @@ OffsideLineMessage::toStr( std::string & to ) const
         return false;
     }
 
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "OffsideLineMessage. over the message size : buf = %d, this = %d",
@@ -361,8 +464,7 @@ DefenseLineMessage::toStr( std::string & to ) const
         return false;
     }
 
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "DefenseLineMessage. over the message size : buf = %d, this = %d",
@@ -414,8 +516,7 @@ DefenseLineMessage::printDebug( std::ostream & os ) const
 bool
 WaitRequestMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "WaitRequestMessage. over the message size : buf = %d, this = %d",
@@ -449,8 +550,7 @@ WaitRequestMessage::printDebug( std::ostream & os ) const
 bool
 InterceptMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "InterceptMessage. over the message size : buf = %d, this = %d",
@@ -511,8 +611,7 @@ InterceptMessage::printDebug( std::ostream & os ) const
 bool
 PassRequestMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "PassRequestMessage. over the message size : buf = %d, this = %d",
@@ -524,7 +623,7 @@ PassRequestMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodePosToStr3( M_target_point, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** PassRequestMessage. "
@@ -567,8 +666,7 @@ PassRequestMessage::printDebug( std::ostream & os ) const
 bool
 StaminaMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "StaminaMessage. over the message size : buf = %d, this = %d",
@@ -597,7 +695,7 @@ StaminaMessage::toStr( std::string & to ) const
     to += header();
     to += ch;
 
-    return false;
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -619,8 +717,7 @@ StaminaMessage::printDebug( std::ostream & os ) const
 bool
 RecoveryMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "RecoveryMessage. over the message size : buf = %d, this = %d",
@@ -674,8 +771,7 @@ RecoveryMessage::printDebug( std::ostream & os ) const
 bool
 DribbleMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "DribbleMessage. over the message size : buf = %d, this = %d",
@@ -706,7 +802,7 @@ DribbleMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** DribbleMessage. target=" << M_target_point
@@ -752,8 +848,7 @@ DribbleMessage::printDebug( std::ostream & os ) const
 bool
 BallGoalieMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "BallGoalieMessage. over the message size : buf = %d, this = %d",
@@ -765,11 +860,11 @@ BallGoalieMessage::toStr( std::string & to ) const
          || M_goalie_pos.absY() > 20.0 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " ***ERROR*** GoalieMessage. over the position range. "
+                  << " ***ERROR*** BallGoalieMessage. over the position range. "
                   << M_goalie_pos
                   << std::endl;
         dlog.addText( Logger::SENSOR,
-                      "GoalieMessage. over the position range : (%.1f %.1f)",
+                      "BallGoalieMessage. over the position range : (%.1f %.1f)",
                       M_goalie_pos.x, M_goalie_pos.y );
         return false;
     }
@@ -792,17 +887,21 @@ BallGoalieMessage::toStr( std::string & to ) const
     ival *= 680;
     ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / 0.1 ), 679.0 ) );
 
-    //dval = min_max( -2.6, M_ball_vel.x, 2.6 ) + 2.6;
-    //ival *= 52;
-    dval = min_max( -max_speed, M_ball_vel.x, max_speed ) + max_speed;
-    ival *= 63;
-    ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / prec ), 62.0 ) );
+    if ( M_ball_vel.isValid() )
+    {
+        dval = min_max( -max_speed, M_ball_vel.x, max_speed ) + max_speed;
+        ival *= 63;
+        ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / prec ), 62.0 ) );
 
-    //dval = min_max( -2.6, M_ball_vel.y, 2.6 ) + 2.6;
-    //ival *= 52;
-    dval = min_max( -max_speed, M_ball_vel.y, max_speed ) + max_speed;
-    ival *= 63;
-    ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / prec ), 62.0 ) );
+        dval = min_max( -max_speed, M_ball_vel.y, max_speed ) + max_speed;
+        ival *= 63;
+        ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / prec ), 62.0 ) );
+    }
+    else
+    {
+        ival *= 63;
+        ival *= 63;
+    }
 
     dval = min_max( 52.5 - 16.0, M_goalie_pos.x, 52.5 ) - ( 52.5 - 16.0 );
     ival *= 160;
@@ -820,7 +919,7 @@ BallGoalieMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** BallGoalieMessage. "
@@ -875,8 +974,7 @@ BallGoalieMessage::printDebug( std::ostream & os ) const
 bool
 OnePlayerMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "OnePlayerMessage. over the message size : buf = %d, this = %d",
@@ -897,23 +995,23 @@ OnePlayerMessage::toStr( std::string & to ) const
     }
 
     boost::int64_t ival = 0;
-    double dval = 0.0;
+    double player_x = min_max( -52.49, M_player_pos.x, 52.49 ) + 52.5;
+    double player_y = min_max( -33.99, M_player_pos.y, 33.99 ) + 34.0;
 
+    // ival *= 22;
     ival += M_unum - 1;
 
-    dval = min_max( -52.49, M_player_pos.x, 52.49 ) + 52.5;
     ival *= 168;
-    ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / 0.63 ), 167.0 ) );
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( player_x / 0.63 ), 167.0 ) );
 
-    dval = min_max( -33.99, M_player_pos.y, 33.99 ) + 34.0;
     ival *= 109;
-    ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / 0.63 ), 108.0 ) );
+    ival += static_cast< boost::int64_t >( bound( 0.0, rint( player_y / 0.63 ), 108.0 ) );
 
     std::string msg;
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** OnePlayerMessage. "
@@ -979,8 +1077,7 @@ TwoPlayerMessage::TwoPlayerMessage( const int player0_unum,
 bool
 TwoPlayerMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "TwoPlayerMessage. over the message size : buf = %d, this = %d",
@@ -1021,7 +1118,7 @@ TwoPlayerMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** TwoPlayerMessage. "
@@ -1106,8 +1203,7 @@ ThreePlayerMessage::ThreePlayerMessage( const int player0_unum,
 bool
 ThreePlayerMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "ThreePlayerMessage. over the message size : buf = %d, this = %d",
@@ -1148,7 +1244,7 @@ ThreePlayerMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** ThreePlayerMessage. "
@@ -1217,8 +1313,7 @@ ThreePlayerMessage::printDebug( std::ostream & os ) const
 bool
 SelfMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "SelfMessage. over the message size : buf = %d, this = %d",
@@ -1253,7 +1348,7 @@ SelfMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** SelfMessage. "
@@ -1304,8 +1399,7 @@ SelfMessage::printDebug( std::ostream & os ) const
 bool
 TeammateMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "TeammateMessage. over the message size : buf = %d, this = %d",
@@ -1346,7 +1440,7 @@ TeammateMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** TeammateMessage. "
@@ -1397,8 +1491,7 @@ TeammateMessage::printDebug( std::ostream & os ) const
 bool
 OpponentMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "OpponentMessage. over the message size : buf = %d, this = %d",
@@ -1439,7 +1532,7 @@ OpponentMessage::toStr( std::string & to ) const
     msg.reserve( slength() - 1 );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, slength() - 1, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** OpponentMessage. "
@@ -1490,8 +1583,7 @@ OpponentMessage::printDebug( std::ostream & os ) const
 bool
 BallPlayerMessage::toStr( std::string & to ) const
 {
-    if ( static_cast< int >( to.length() + slength() )
-         > ServerParam::i().playerSayMsgSize() )
+    if ( (int)to.length() + slength() > ServerParam::i().playerSayMsgSize() )
     {
         dlog.addText( Logger::SENSOR,
                       "BallPlayerMessage. over the message size : buf = %d, this = %d",
@@ -1559,7 +1651,7 @@ BallPlayerMessage::toStr( std::string & to ) const
     ival += static_cast< boost::int64_t >( bound( 0.0, rint( dval / 2.0 ), 179.0 ) );
 
     if ( ! AudioCodec::i().encodeInt64ToStr( ival, 4, msg )
-         || msg.length() != slength() - 1 )
+         || (int)msg.length() != slength() - 1 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** BallPlayerMessage. "
