@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,13 +29,14 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCSC_PLAYER_PLAYER_MESSAGE_BUILDER_H
-#define RCSC_PLAYER_PLAYER_MESSAGE_BUILDER_H
+#ifndef RCSC_PLAYER_SAY_MESSAGE_BUILDER_H
+#define RCSC_PLAYER_SAY_MESSAGE_BUILDER_H
 
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/common/say_message_parser.h>
 
 #include <string>
+#include <iostream>
 
 namespace rcsc {
 
@@ -80,7 +81,7 @@ public:
       \return the length of encoded message
     */
     virtual
-    std::size_t length() const = 0;
+    int length() const = 0;
 
     /*!
       \brief append the audio message to be sent
@@ -90,6 +91,14 @@ public:
     virtual
     bool toStr( std::string & to ) const = 0;
 
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    virtual
+    std::ostream & printDebug( std::ostream & os ) const = 0;
+
 };
 
 
@@ -136,7 +145,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return BallMessageParser::slength();
       }
@@ -145,7 +154,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -157,6 +166,13 @@ public:
     */
     bool toStr( std::string & to ) const;
 
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -211,7 +227,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return PassMessageParser::slength();
       }
@@ -220,7 +236,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -232,6 +248,13 @@ public:
     */
     bool toStr( std::string & to ) const;
 
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -281,7 +304,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return InterceptMessageParser::slength();
       }
@@ -290,7 +313,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -301,6 +324,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -350,7 +381,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return GoalieMessageParser::slength();
       }
@@ -359,7 +390,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -370,6 +401,100 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class GoalieAndPlayerMessage
+  \brief goalie & 1 field player message encoder
+
+  format:
+  "e<pos_body:4><num_pos:3>"
+  the length of message == 8
+*/
+class GoalieAndPlayerMessage
+    : public SayMessage {
+private:
+
+    int M_goalie_unum; //!< goalie's uniform number
+    Vector2D M_goalie_pos; //!< goalie's position
+    AngleDeg M_goalie_body; //!< goalie's body angle
+
+    int M_player_number; //!< teammate unum [1-11]. or opponent unum [12-23]
+    Vector2D M_player_pos;
+
+public:
+
+    /*!
+      \brief construct with raw information
+      \param goalie_unum goalie's uniform number
+      \param goalie_pos goalie's global position
+      \param goalie_body goalie's body direction
+      \param unum player's number
+      \param player_pos player'ss global position
+    */
+    GoalieAndPlayerMessage( const int goalie_unum,
+                            const Vector2D & goalie_pos,
+                            const AngleDeg & goalie_body,
+                            const int player_number,
+                            const Vector2D & player_pos )
+        : M_goalie_unum( goalie_unum ),
+          M_goalie_pos( goalie_pos ),
+          M_goalie_body( goalie_body ),
+          M_player_number( player_number ),
+          M_player_pos( player_pos )
+      { }
+
+    /*!
+      \brief get the header character of this message
+      \return header character of this message
+     */
+    char header() const
+      {
+          return GoalieAndPlayerMessageParser::sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    int slength()
+      {
+          return GoalieAndPlayerMessageParser::slength();
+      }
+
+    /*!
+      \brief get the length of this message
+      \return the length of encoded message
+    */
+    int length() const
+      {
+          return slength();
+      }
+
+    /*!
+      \brief append this info to the audio message
+      \param to reference to the message string instance
+      \return result status of encoding
+    */
+    bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -412,7 +537,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return OffsideLineMessageParser::slength();
       }
@@ -421,7 +546,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -432,6 +557,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -474,7 +607,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return DefenseLineMessageParser::slength();
       }
@@ -483,7 +616,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -494,6 +627,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -531,7 +672,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return WaitRequestMessageParser::slength();
       }
@@ -540,7 +681,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -551,6 +692,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -593,7 +742,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return PassRequestMessageParser::slength();
       }
@@ -602,7 +751,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -613,6 +762,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -655,7 +812,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return StaminaMessageParser::slength();
       }
@@ -664,7 +821,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -675,6 +832,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -717,7 +882,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return RecoveryMessageParser::slength();
       }
@@ -726,7 +891,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -737,6 +902,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -782,7 +955,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return DribbleMessageParser::slength();
       }
@@ -791,7 +964,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -802,6 +975,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -855,7 +1036,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return BallGoalieMessageParser::slength();
       }
@@ -864,7 +1045,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -875,6 +1056,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -883,8 +1072,8 @@ public:
   \brief one player info message encoder
 
   format:
-  "P<unum_pos_body:4>"
-  the length of message == 5
+  "P<unum_pos:3>"
+  the length of message == 4
 */
 class OnePlayerMessage
     : public SayMessage {
@@ -892,22 +1081,18 @@ private:
 
     int M_unum; //!< player's unum [1-22]. if opponent, unum > 11
     Vector2D M_player_pos; //!< player's position
-    AngleDeg M_player_body; //!< player's body angle
 
 public:
 
     /*!
       \brief construct with raw information
       \param unum player's unum
-      \param player_pos goalie's global position
-      \param player_body goalie's body angle
+      \param player_pos player's global position
     */
     OnePlayerMessage( const int unum,
-                      const Vector2D & player_pos,
-                      const AngleDeg & player_body )
+                      const Vector2D & player_pos )
         : M_unum( unum )
         , M_player_pos( player_pos )
-        , M_player_body( player_body )
       { }
 
     /*!
@@ -924,7 +1109,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return OnePlayerMessageParser::slength();
       }
@@ -933,7 +1118,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -944,8 +1129,169 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
+/*-------------------------------------------------------------------*/
+/*!
+  \class TwoPlayerMessage
+  \brief 2 players info message encoder
+
+  format:
+  "Q<unum_pos:3,unum_pos:3>"
+  the length of message == 7
+*/
+class TwoPlayerMessage
+    : public SayMessage {
+private:
+
+    int M_player_unum[2]; //!< player's unum [1-22]. if opponent, unum > 11
+    Vector2D M_player_pos[2]; //!< player's position
+
+    // not usd
+    TwoPlayerMessage();
+public:
+
+    /*!
+      \brief construct with raw information
+      \param player0_unum player's unum, if opponent, += 11
+      \param player0_pos goalie's global position
+      \param player1_unum player's unum, if opponent, += 11
+      \param player1_pos goalie's global position
+    */
+    TwoPlayerMessage( const int player0_unum,
+                      const Vector2D & player0_pos,
+                      const int player1_unum,
+                      const Vector2D & player1_pos );
+
+    /*!
+      \brief get the header character of this message
+      \return header character of this message
+     */
+    char header() const
+      {
+          return TwoPlayerMessageParser::sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    int slength()
+      {
+          return TwoPlayerMessageParser::slength();
+      }
+
+    /*!
+      \brief get the length of this message
+      \return the length of encoded message
+    */
+    int length() const
+      {
+          return slength();
+      }
+
+    /*!
+      \brief append this info to the audio message
+      \param to reference to the message string instance
+      \return result status of encoding
+    */
+    bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class ThreePlayerMessage
+  \brief 3 players info message encoder
+
+  format:
+  "Q<unum_pos:3,unum_pos:3,unum_pos:3>"
+  the length of message == 10
+*/
+class ThreePlayerMessage
+    : public SayMessage {
+private:
+
+    int M_player_unum[3]; //!< player's unum [1-22]. if opponent, unum > 11
+    Vector2D M_player_pos[3]; //!< player's position
+
+public:
+
+    /*!
+      \brief construct with raw information
+      \param player0_unum player's unum, if opponent, += 11
+      \param player0_pos goalie's global position
+      \param player1_unum player's unum, if opponent, += 11
+      \param player1_pos goalie's global position
+      \param player2_unum player's unum, if opponent, += 11
+      \param player2_pos goalie's global position
+    */
+    ThreePlayerMessage( const int player0_unum,
+                        const Vector2D & player0_pos,
+                        const int player1_unum,
+                        const Vector2D & player1_pos,
+                        const int player2_unum,
+                        const Vector2D & player2_pos );
+
+    /*!
+      \brief get the header character of this message
+      \return header character of this message
+     */
+    char header() const
+      {
+          return ThreePlayerMessageParser::sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    int slength()
+      {
+          return ThreePlayerMessageParser::slength();
+      }
+
+    /*!
+      \brief get the length of this message
+      \return the length of encoded message
+    */
+    int length() const
+      {
+          return slength();
+      }
+
+    /*!
+      \brief append this info to the audio message
+      \param to reference to the message string instance
+      \return result status of encoding
+    */
+    bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
+};
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -994,7 +1340,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return SelfMessageParser::slength();
       }
@@ -1003,7 +1349,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -1014,6 +1360,168 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class TeammateMessage
+  \brief teammate info message encoder
+
+  format:
+  "T<unum_pos_body:4>"
+  the length of message == 5
+*/
+class TeammateMessage
+    : public SayMessage {
+private:
+
+    int M_unum; //!< player's unum [1-11]
+    Vector2D M_player_pos; //!< player's position
+    AngleDeg M_player_body; //!< player's body angle
+
+public:
+
+    /*!
+      \brief construct with raw information
+      \param unum player's unum
+      \param player_pos goalie's global position
+      \param player_body goalie's body angle
+    */
+    TeammateMessage( const int unum,
+                     const Vector2D & player_pos,
+                     const AngleDeg & player_body )
+        : M_unum( unum )
+        , M_player_pos( player_pos )
+        , M_player_body( player_body )
+      { }
+
+    /*!
+      \brief get the header character of this message
+      \return header character of this message
+     */
+    char header() const
+      {
+          return TeammateMessageParser::sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    int slength()
+      {
+          return TeammateMessageParser::slength();
+      }
+
+    /*!
+      \brief get the length of this message
+      \return the length of encoded message
+    */
+    int length() const
+      {
+          return slength();
+      }
+
+    /*!
+      \brief append this info to the audio message
+      \param to reference to the message string instance
+      \return result status of encoding
+    */
+    bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class OpponentMessage
+  \brief opponent info message encoder
+
+  format:
+  "O<unum_pos_body:4>"
+  the length of message == 5
+*/
+class OpponentMessage
+    : public SayMessage {
+private:
+
+    int M_unum; //!< player's unum [1-11]
+    Vector2D M_player_pos; //!< player's position
+    AngleDeg M_player_body; //!< player's body angle
+
+public:
+
+    /*!
+      \brief construct with raw information
+      \param unum player's unum
+      \param player_pos goalie's global position
+      \param player_body goalie's body angle
+    */
+    OpponentMessage( const int unum,
+                     const Vector2D & player_pos,
+                     const AngleDeg & player_body )
+        : M_unum( unum )
+        , M_player_pos( player_pos )
+        , M_player_body( player_body )
+      { }
+
+    /*!
+      \brief get the header character of this message
+      \return header character of this message
+     */
+    char header() const
+      {
+          return OpponentMessageParser::sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    int slength()
+      {
+          return OpponentMessageParser::slength();
+      }
+
+    /*!
+      \brief get the length of this message
+      \return the length of encoded message
+    */
+    int length() const
+      {
+          return slength();
+      }
+
+    /*!
+      \brief append this info to the audio message
+      \param to reference to the message string instance
+      \return result status of encoding
+    */
+    bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -1022,7 +1530,7 @@ public:
   \brief ball & player info message encoder
 
   format:
-  "1<bpos_bvel_unum_ppos_pbody:9>"
+  "B<bpos_bvel_unum_ppos_pbody:9>"
   the length of message == 10
 */
 class BallPlayerMessage
@@ -1072,7 +1580,7 @@ public:
       \return the length of encoded message
     */
     static
-    std::size_t slength()
+    int slength()
       {
           return BallPlayerMessageParser::slength();
       }
@@ -1081,7 +1589,7 @@ public:
       \brief get the length of this message
       \return the length of encoded message
     */
-    std::size_t length() const
+    int length() const
       {
           return slength();
       }
@@ -1092,6 +1600,14 @@ public:
       \return result status of encoding
     */
     bool toStr( std::string & to ) const;
+
+    /*!
+      \brief append the debug message
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDebug( std::ostream & os ) const;
+
 };
 
 }
