@@ -38,6 +38,8 @@
 #include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 
+// #define DEBUG_PRINT
+
 namespace rcsc {
 
 bool SeeState::S_synch_see_mode = false;
@@ -267,9 +269,11 @@ SeeState::setNewCycle( const GameTime & new_time )
     M_see_count_history[0] = M_current_see_count;
     M_current_see_count = 0;
 
+#ifdef DEBUG_PRINT
     dlog.addText( Logger::SYSTEM,
                   __FILE__" (updateBySee) set new cycle. cycle till next see = %d",
                   M_cycles_till_next_see );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -279,9 +283,11 @@ SeeState::setNewCycle( const GameTime & new_time )
 void
 SeeState::setLastSeeTiming( const Timing last_timing )
 {
+#ifdef DEBUG_PRINT
     dlog.addText( Logger::SYSTEM,
                   __FILE__" (setLastSeeTiming) %d",
                   last_timing );
+#endif
     M_last_timing = last_timing;
 }
 
@@ -410,8 +416,10 @@ SeeState::setViewMode( const ViewWidth & new_width,
 {
     if ( M_last_see_time != M_current_time )
     {
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::SYSTEM,
                       __FILE__" (setViewMode) no current cycle see arrival" );
+#endif
         return;
     }
 
@@ -424,20 +432,26 @@ SeeState::setViewMode( const ViewWidth & new_width,
         case ViewWidth::WIDE:
             M_cycles_till_next_see = 3;
             M_synch_type = SYNCH_WIDE;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) synch wide: cycle = 3" );
+#endif
             break;
         case ViewWidth::NORMAL:
             M_cycles_till_next_see = 2;
             M_synch_type = SYNCH_NORMAL;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) synch normal: cycle = 2" );
+#endif
             break;
         case ViewWidth::NARROW:
             M_cycles_till_next_see = 1;
             M_synch_type = SYNCH_NARROW;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) synch narrow: cycle = 1" );
+#endif
             break;
         default:
             break;
@@ -453,16 +467,20 @@ SeeState::setViewMode( const ViewWidth & new_width,
         case ViewWidth::WIDE:
             M_cycles_till_next_see = 3;
             M_synch_type = SYNCH_WIDE;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) 00:wide: cycle = %d",
                           M_cycles_till_next_see );
+#endif
             break;
         case ViewWidth::NORMAL:
             M_cycles_till_next_see = 1;
             M_synch_type = SYNCH_EVERY;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) 00:normal: cycle = %d",
                           M_cycles_till_next_see );
+#endif
             break;
         case ViewWidth::NARROW:
             std::cerr << M_current_time
@@ -484,23 +502,29 @@ SeeState::setViewMode( const ViewWidth & new_width,
         case ViewWidth::WIDE:
             M_cycles_till_next_see = 3;
             M_synch_type = SYNCH_WIDE;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) 50:wide: cycle = %d",
                           M_cycles_till_next_see );
+#endif
             break;
         case ViewWidth::NORMAL:
             M_cycles_till_next_see = 2;
             M_synch_type = SYNCH_NORMAL;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) 50:normal: cycle = %d",
                           M_cycles_till_next_see );
+#endif
             break;
         case ViewWidth::NARROW:
             M_cycles_till_next_see = 1;
             M_synch_type = SYNCH_EVERY;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SYSTEM,
                           __FILE__" (setViewMode) 50:narrow: cycle = %d",
                           M_cycles_till_next_see );
+#endif
             break;
         default:
             break;
