@@ -62,32 +62,35 @@ int SelfObject::S_face_count_thr = 5;
 
 */
 SelfObject::SelfObject()
-    : AbstractPlayerObject()
-    , M_time( -1, 0 )
-    , M_sense_body_time( -1, 0 )
-    , M_pos_error( 0.0, 0.0 )
-    , M_pos_prev( Vector2D::INVALIDATED )
-    , M_vel_error( 0.0, 0.0 )
-    , M_neck( 0.0 )
-    , M_face_error( 0.0 )
-    , M_view_width( ViewWidth::NORMAL )
-    , M_view_quality( ViewQuality::HIGH )
-    , M_last_catch_time( 0, 0 )
-    , M_tackle_expires( 0 )
-    , M_arm_movable( 0 )
-    , M_pointto_pos( Vector2D::INVALIDATED )
-    , M_last_pointto_time( 0, 0 )
-    , M_attentionto_side( NEUTRAL )
-    , M_attentionto_unum( 0 )
-    , M_last_move( 0.0, 0.0 )
-    , M_collision_estimated( false )
-    , M_collides_with_none( false )
-    , M_collides_with_ball( false )
-    , M_collides_with_player( false )
-    , M_collides_with_post( false )
-    , M_kickable( false )
-    , M_kick_rate( 0.0 )
-    , M_tackle_probability( 0.0 )
+    : AbstractPlayerObject(),
+      M_time( -1, 0 ),
+      M_sense_body_time( -1, 0 ),
+      M_pos_error( 0.0, 0.0 ),
+      M_pos_prev( Vector2D::INVALIDATED ),
+      M_vel_error( 0.0, 0.0 ),
+      M_neck( 0.0 ),
+      M_face_error( 0.0 ),
+      M_view_width( ViewWidth::NORMAL ),
+      M_view_quality( ViewQuality::HIGH ),
+      M_last_catch_time( 0, 0 ),
+      M_tackle_expires( 0 ),
+      M_charged_expires( 0 ),
+      M_arm_movable( 0 ),
+      M_pointto_pos( Vector2D::INVALIDATED ),
+      M_last_pointto_time( 0, 0 ),
+      M_attentionto_side( NEUTRAL ),
+      M_attentionto_unum( 0 ),
+      M_last_move( 0.0, 0.0 ),
+      M_collision_estimated( false ),
+      M_collides_with_none( false ),
+      M_collides_with_ball( false ),
+      M_collides_with_player( false ),
+      M_collides_with_post( false ),
+      M_kickable( false ),
+      M_kick_rate( 0.0 ),
+      M_catch_probability( 0.0 ),
+      M_tackle_probability( 0.0 ),
+      M_foul_probability( 0.0 )
 {
     M_unum_count = 0;
     M_type = Hetero_Default;
@@ -174,32 +177,6 @@ SelfObject::faceValid() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
-double
-SelfObject::kickableArea() const
-{
-    return ( this->playerType().kickableMargin()
-             + this->playerType().playerSize()
-             + ServerParam::i().ballSize() );
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-double
-SelfObject::catchableArea() const
-{
-    //double t = atan2(0.5, 2.0); // atan2(w * 0.5, l);
-    //double len = 2.0 / cos(t);
-
-    return std::sqrt( std::pow( ServerParam::i().catchAreaWidth() * 0.5, 2 )
-                      + std::pow( ServerParam::i().catchAreaLength(), 2 ) );
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
 */
 void
 SelfObject::update( const ActionEffector & act,
@@ -212,6 +189,7 @@ SelfObject::update( const ActionEffector & act,
 
     M_time = current;
 
+    M_kicked = false;
     M_pos_prev = M_pos;
 
     Vector2D accel( 0.0, 0.0 );
@@ -229,7 +207,11 @@ SelfObject::update( const ActionEffector & act,
         act.getTurnInfo( &turn_moment, &turn_err );
         break;
     case PlayerCommand::TACKLE:
-        M_tackle_expires = ServerParam::i().tackleCycles();
+        if ( ! act.tackleFoul() )
+        {
+            M_tackle_expires = ServerParam::i().tackleCycles();
+        }
+        M_kicked = true;
         break;
     case PlayerCommand::MOVE:
         M_pos = act.getMovePos();
@@ -240,6 +222,7 @@ SelfObject::update( const ActionEffector & act,
         M_last_catch_time = act.lastActionTime();
         break;
     case PlayerCommand::KICK:
+        M_kicked = true;
         break;
     default:
         //std::cerr << current << " self update : no command!!"
@@ -271,7 +254,7 @@ SelfObject::update( const ActionEffector & act,
     }
 
     // stamina
-    M_stamina.update( playerType(), dash_power );
+    M_stamina.simulateDash( playerType(), dash_power );
 
 #ifdef DEBUG_PRINT
     dlog.addText( Logger::WORLD,
@@ -279,7 +262,7 @@ SelfObject::update( const ActionEffector & act,
                   M_stamina.stamina(),
                   M_stamina.effort(),
                   M_stamina.recovery(),
-                  M_stamina.staminaCapacity() );
+                  M_stamina.capacity() );
 #endif
 
     /////////////////////////////////////////////////////////////////
@@ -328,7 +311,6 @@ SelfObject::update( const ActionEffector & act,
         const double velrnd = ServerParam::i().playerRand() * M_vel.r();
 
         M_pos_error.add( velrnd, velrnd ); // add new vel rand
-
         M_vel_error.add( velrnd, velrnd ); // add new vel rand
 
 #ifdef DEBUG_PRINT
@@ -361,6 +343,8 @@ SelfObject::update( const ActionEffector & act,
     M_tackle_expires = std::max( 0, M_tackle_expires - 1 );
     M_arm_movable = std::max( 0, M_arm_movable - 1 );
 
+    M_charged_expires = std::max( 0, M_charged_expires - 1 );
+
     /////////////////////////////////////////////////////////////////
     // last move vector
     M_last_move = M_vel / this->playerType().playerDecay();
@@ -475,6 +459,18 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
     // internal update
     update( act, current );
 
+    M_kicked = ( act.lastBodyCommandType() == PlayerCommand::KICK
+                 || act.lastBodyCommandType() == PlayerCommand::TACKLE );
+
+    if ( act.lastBodyCommandType() == PlayerCommand::CATCH )
+    {
+        M_last_catch_time = current;
+    }
+    else if ( M_last_catch_time == current )
+    {
+        M_last_catch_time.assign( 0, 0 );
+    }
+
     // ------------------------------------------------
     // use sense_body
 
@@ -485,7 +481,10 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
 
     ////////////////////////////////////////////////////
     // stamina
-    M_stamina.updateAfterSense( sense, current );
+    M_stamina.updateBySenseBody( sense.stamina(),
+                                 sense.effort(),
+                                 sense.staminaCapacity(),
+                                 current );
 
     ////////////////////////////////////////////////////
     // neck relative
@@ -651,6 +650,12 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
         if ( ! collisionEstimated() )
         {
             M_last_move = M_vel / this->playerType().playerDecay();
+#ifdef DEBUG_PRINT
+            dlog.addText( Logger::WORLD,
+                          __FILE__" (Self::updateAfterSense)"
+                          " set last_move=(%lf %lf)",
+                          M_last_move.x, M_last_move.y );
+#endif
         }
         else
         {
@@ -680,30 +685,41 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
     M_attentionto_unum = sense.attentiontoUnum();
 
     // tackle
+#ifdef DEBUG_PRINT
     if ( M_tackle_expires > 0 )
     {
-#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
                       __FILE__" (updateAfterSense)"
                       " internal_tackle_exp=%d, sense_body=%d",
                       M_tackle_expires, sense.tackleExpires() );
-#endif
     }
 
     if ( M_tackle_expires != sense.tackleExpires() )
     {
         std::cerr << current
-                  << " (updateAfterSense) internal tackle expire has invalie value."
+                  << " (updateAfterSense) internal tackle expire has an illegal value."
                   << std::endl;
-#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
                       __FILE__" (updateAfterSense)"
                       " internal_tackle_exp is not match to sense_body" );
-#endif
     }
+#endif
 
     M_tackle_expires = sense.tackleExpires();
     M_arm_movable = sense.armMovable();
+
+    M_charged_expires = sense.chargedExpires();
+
+    M_card = sense.card();
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateAfterSense) card=%s",
+                  sense.card() == YELLOW
+                  ? "yellow"
+                  : sense.card() == RED
+                  ? "red"
+                  : "none" );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -717,6 +733,8 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
 {
     update( act, current );
 
+    M_last_move = my_state.pos_ - M_seen_pos;
+
     M_pos = my_state.pos_;
     M_pos_error.assign( 0.0, 0.0 );
     M_pos_count = 0;
@@ -737,7 +755,10 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
     M_face_error = 0.0;
     M_face_count = 0;
 
-    M_stamina.updateAfterFullstate( my_state );
+    M_stamina.updateByFullstate( my_state.stamina_,
+                                 my_state.effort_,
+                                 my_state.recovery_,
+                                 my_state.stamina_capacity_ );
 }
 
 
@@ -804,10 +825,16 @@ SelfObject::updatePosBySee( const Vector2D & pos,
 
         if ( M_seen_pos_count == 1
              && ( collisionEstimated()
-                  || M_last_move.valid() ) )
+                  || ! M_last_move.isValid() ) )
         {
             M_last_move = new_pos - M_seen_pos;
             M_last_moves[0] = M_last_move;
+#ifdef DEBUG_PRINT
+            dlog.addText( Logger::WORLD,
+                          __FILE__" (Self::updatePosBySee)"
+                          " set last_move=(%lf %lf)",
+                          M_last_move.x, M_last_move.y );
+#endif
         }
     }
     else
@@ -930,8 +957,8 @@ SelfObject::updateVelDirAfterSee( const BodySensor & sense,
 
 #ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__" (updateVelDirAfterSee)"
-                          " last_move=(%.4f %.4f)",
+                          __FILE__" (Self::updateVelDirAfterSee)"
+                          " set last_move=(%lf %lf)",
                           M_last_move.x, M_last_move.y );
 #endif
         }
@@ -947,7 +974,9 @@ SelfObject::updateBallInfo( const BallObject & ball )
 {
     M_kickable = false;
     M_kick_rate = 0.0;
+    M_catch_probability = 0.0;
     M_tackle_probability = 0.0;
+    M_foul_probability = 0.0;
 
     if ( ! ball.posValid() && ball.rposCount() != 0 )
     {
@@ -960,30 +989,33 @@ SelfObject::updateBallInfo( const BallObject & ball )
     //------------------------------------------------------
     // check kickable
 
-    if ( ball.distFromSelf() <= this->kickableArea() )
+    if ( ball.distFromSelf() <= playerType().kickableArea() )
     {
         double buf = 0.055;
         if ( ball.seenPosCount() >= 2 ) buf = 0.255;
         if ( ball.seenPosCount() >= 1 ) buf = 0.155;
 
-        if ( ball.distFromSelf() <= this->kickableArea() - buf )
+        if ( ball.distFromSelf() <= playerType().kickableArea() - buf )
         {
             M_kickable = true;
         }
 
         M_kick_rate
             = kick_rate( ball.distFromSelf(),
-                         ( ball.angleFromSelf() - this->body() ).degree(),
-                         ServerParam::i().kickPowerRate(),
+                         ( ball.angleFromSelf() - body() ).degree(),
+                         playerType().kickPowerRate(), //ServerParam::i().kickPowerRate(),
                          ServerParam::i().ballSize(),
-                         this->playerType().playerSize(),
-                         this->playerType().kickableMargin() );
+                         playerType().playerSize(),
+                         playerType().kickableMargin() );
     }
 
+    const Vector2D player2ball = ( ball.pos() - pos() ).rotatedVector( - body() );
+    //------------------------------------------------------
+    // check catch
+    M_catch_probability = playerType().getCatchProbability( player2ball.r() );
+
     //------------------------------------------------------
     // check tackle
-    Vector2D player2ball = ball.pos() - this->pos();
-    player2ball.rotate( - this->body() );
 
     double tackle_dist;
     if ( player2ball.x > 0.0 )
@@ -995,29 +1027,47 @@ SelfObject::updateBallInfo( const BallObject & ball )
         tackle_dist = ServerParam::i().tackleBackDist();
     }
 
-    double fail_prob = 1.0;
+    double tackle_fail_prob = 1.0;
+    double foul_fail_prob = 1.0;
     if ( tackle_dist > 1.0e-5 )
     {
-        fail_prob = ( std::pow( player2ball.absX() / tackle_dist,
-                                ServerParam::i().tackleExponent() )
-                      + std::pow( player2ball.absY() / ServerParam::i().tackleWidth(),
-                                  ServerParam::i().tackleExponent() ) );
+        tackle_fail_prob = ( std::pow( player2ball.absX() / tackle_dist,
+                                       ServerParam::i().tackleExponent() )
+                             + std::pow( player2ball.absY() / ServerParam::i().tackleWidth(),
+                                         ServerParam::i().tackleExponent() ) );
+        foul_fail_prob = ( std::pow( player2ball.absX() / tackle_dist,
+                                     ServerParam::i().foulExponent() )
+                           + std::pow( player2ball.absY() / ServerParam::i().tackleWidth(),
+                                       ServerParam::i().foulExponent() ) );
     }
 
-    if ( fail_prob < 1.0 )
+    if ( tackle_fail_prob < 1.0 )
     {
         // success_prob = 1.0 - fail_prob
         // r = random(); <- [0,1]
         // if (r <= success_prob) SUCCESS;
         // else FAILURE;
-        M_tackle_probability = 1.0 - fail_prob;
+        M_tackle_probability = 1.0 - tackle_fail_prob;
     }
 
-#ifdef DEBUG_PRINT
+    if ( foul_fail_prob < 1.0 )
+    {
+        M_foul_probability = 1.0 - foul_fail_prob;
+    }
+
+    //#ifdef DEBUG_PRINT
     dlog.addText( Logger::WORLD,
-                  __FILE__" (updateBallInfo) ball_dist=%.3f my_kickable=%.3f tackle_prob=%.3f",
-                  ball.distFromSelf(), kickableArea(), M_tackle_probability );
-#endif
+                  __FILE__" (updateBallInfo) kickable=%.3f ball_dist=%.3f rpos=(%.3f, %3f)",
+                  playerType().kickableArea(),
+                  ball.distFromSelf(),
+                  player2ball.x, player2ball.y );
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateBallInfo) kick_power_rate=%f kick_rate=%f",
+                  playerType().kickPowerRate(), M_kick_rate );
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (updateBallInfo) tackle_prob=%.3f foul_prob=%.3f",
+                  M_tackle_probability, M_foul_probability );
+    //#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -1032,84 +1082,41 @@ SelfObject::updateKickableState( const BallObject & ball,
 {
     if ( ! M_kickable
          && ball.seenPosCount() == 0
-         && ( self_reach_cycle >= 10 // XXX magic number XXX
-              || ( opponent_reach_cycle < self_reach_cycle - 7
-                   && opponent_reach_cycle < teammate_reach_cycle - 7 ) )
-         && ball.distFromSelf() < this->kickableArea() - 0.001 )
+         && ball.distFromSelf() < playerType().kickableArea() - 0.001 )
     {
-        M_kickable = true;
 
+        if ( self_reach_cycle >= 10 // XXX magic number XXX
+             || ( opponent_reach_cycle < self_reach_cycle - 7
+                  && opponent_reach_cycle < teammate_reach_cycle - 7 ) )
+        {
+            M_kickable = true;
 #ifdef DEBUG_PRINT
-        dlog.addText( Logger::WORLD,
-                      __FILE__" (updateKickableState) set to kickable" );
+            dlog.addText( Logger::WORLD,
+                          __FILE__" (updateKickableState) force set kickable(1)" );
 #endif
-    }
-}
+            return;
+        }
 
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-double
-SelfObject::getSafetyDashPower( const double & dash_power ) const
-{
-    double normalized_power = ServerParam::i().normalizeDashPower( dash_power );
-
-    // required stamina value
-    double required_stamina = ( normalized_power > 0.0
-                                ? normalized_power
-                                : normalized_power * -2.0 );
-
-    if ( required_stamina < 0.0 )
-    {
-        std::cerr << "required stamina should be positive now "
-                  << " input dash power = " << dash_power
-                  << " normalized power=" << normalized_power
-                  << std::endl;
+        int min_cycle = std::min( self_reach_cycle,
+                                  std::min( teammate_reach_cycle,
+                                            opponent_reach_cycle ) );
+        Vector2D ball_pos = ball.inertiaPoint( min_cycle );
+        if ( ball_pos.absX() > ServerParam::i().pitchHalfLength()
+             || ball_pos.absY() > ServerParam::i().pitchHalfWidth() )
+        {
+            M_kickable = true;
 #ifdef DEBUG_PRINT
-        dlog.addText( Logger::ACTION,
-                      __FILE__" (getSafetyDashPower) input_dash_power=%.2f normalized_power=%.1f required_stamina=%.1f",
-                      dash_power,
-                      normalized_power,
-                      required_stamina );
+            dlog.addText( Logger::WORLD,
+                          __FILE__" (updateKickableState) force set kickable(2)" );
 #endif
+            return;
+        }
     }
 
-    // check rest stamina
-
-    const double safety_stamina
-        = stamina() - ServerParam::i().recoverDecThrValue() - 1.0;
-
-    double available_stamina = std::max( 0.0, safety_stamina );
-
-    double result_power = std::min( required_stamina, available_stamina );
-    // backward dash case
-    if ( normalized_power < 0.0 )
+    if ( opponent_reach_cycle > 0 )
     {
-        result_power *= -0.5;
+        M_foul_probability = 0.0;
     }
-
-    if ( std::fabs( result_power ) > std::fabs( normalized_power ) )
-    {
-        return normalized_power;
-    }
-
-    //dlog.addText( Logger::ACTION,
-    //              __FILE__" (getSafetyDashPower) cur_stamina=%.1f, required=%.1f, result=%.2f",
-    //              stamina(), required_stamina, result_power );
-    return result_power;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-bool
-SelfObject::canTurn( const double & moment ) const
-{
-    return std::fabs( moment )
-        <= playerType().effectiveTurn( ServerParam::i().maxMoment(),
-                                       vel().r() );
 }
 
 }
