@@ -34,9 +34,9 @@
 
 #include <rcsc/player/abstract_player_object.h>
 #include <rcsc/player/view_mode.h>
-#include <rcsc/player/stamina_model.h>
 #include <rcsc/player/fullstate_sensor.h>
 
+#include <rcsc/common/stamina_model.h>
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_time.h>
 #include <rcsc/types.h>
@@ -81,8 +81,8 @@ private:
     // some action effect
     GameTime M_last_catch_time; //!< last goalie catch time
 
-    //! tackle counter. if positive, agent cannot perfome any body action.
-    int M_tackle_expires;
+    int M_tackle_expires; //!< tackle expires. if positive, agent cannot perfome any body action.
+    int M_charged_expires; //!< foul charged expires. if positive, agent cannot perfome any body action.
 
     int M_arm_movable; //!< cycles till arm movable
     Vector2D M_pointto_rpos; //!< estimated pointing position relative to the neck
@@ -106,7 +106,9 @@ private:
     // status values for action
     bool M_kickable; //!< ball kickable flag
     double M_kick_rate; //!< current kick power rate
+    double M_catch_probability; //!< estimated catch success probability
     double M_tackle_probability; //!< estimated tackle success probability
+    double M_foul_probability; //!< estimated tackle success probability
 
     //! not used
     SelfObject( const SelfObject & self );
@@ -292,13 +294,19 @@ public:
           return M_tackle_expires;
       }
 
+    int chargedExpires() const
+      {
+          return M_charged_expires;
+      }
+
     /*!
       \brief check if body is freezed by tackle effect
       \return true if body is freezed.
     */
-    bool isFreezed() const
+    bool isFrozen() const
       {
-          return M_tackle_expires > 0;
+          return M_tackle_expires > 0
+              || M_charged_expires > 0;
       }
 
     /*!
@@ -307,7 +315,7 @@ public:
      */
     bool isTackling() const
       {
-          return isFreezed();
+          return M_tackle_expires > 0;
       }
 
     /*!
@@ -377,6 +385,16 @@ public:
           return M_stamina.stamina();
       }
 
+    /*!
+      \brief get current stamina capacity value
+      \return raw stamina capacity value from stamina object
+    */
+    const
+    double & staminaCapacity() const
+      {
+          return M_stamina.capacity();
+      }
+
     /*!
       \brief get current effort value
       \return effort value from stamina object
@@ -502,27 +520,24 @@ public:
           return effort() * playerType().dashPowerRate();
       }
 
+    /*!
+      \brief get estimated catch success probability
+      \return probability [0,1]
+    */
+    const double & catchProbability() const { return M_catch_probability; }
+
     /*!
       \brief get estimated tackle success probability
       \return probability [0,1]
     */
-    const
-    double & tackleProbability() const
-      {
-          return M_tackle_probability;
-      }
+    const double & tackleProbability() const { return M_tackle_probability; }
 
     /*!
-      \brief get current kickable radius
-      \return kickable radius value
+      \brief get estimated foul success probability
+      \return probability [0,1]
     */
-    double kickableArea() const;
+    const double & foulProbability() const { return M_foul_probability; }
 
-    /*!
-      \brief get current catchable radius calculated by diagonal length
-      \return strict catchable radius value
-    */
-    double catchableArea() const;
 
     ///////////////////////////////////////////////////////////////
 
@@ -681,14 +696,10 @@ public:
       \param dash_power evaluated dash power
       \return filtered dash power
     */
-    double getSafetyDashPower( const double & dash_power ) const;
-
-    /*!
-      \brief check if agent can turn the specified moment
-      \param moment required turn moment value
-      \return true if agent can turn by 1 step
-     */
-    bool canTurn( const double & moment ) const;
+    double getSafetyDashPower( const double & dash_power ) const
+      {
+          return staminaModel().getSafetyDashPower( playerType(), dash_power );
+      }
 
 };
 
