@@ -36,6 +36,7 @@
 #include <rcsc/player/ball_object.h>
 #include <rcsc/player/player_object.h>
 #include <rcsc/player/view_area.h>
+#include <rcsc/player/view_grid_map.h>
 
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_mode.h>
@@ -99,13 +100,15 @@ private:
 
     GameMode M_game_mode; //!< playmode and scores
 
-    bool M_valid;
+    GameTime M_training_time; //!< training start/end time for keepaway
+
+    bool M_valid; //!< if this world model is initialized, true.
 
     //////////////////////////////////////////////////
     // objects
     SelfObject M_self; //!< self object
     BallObject M_ball; //!< ball object
-    // instance of each group
+    // instance of player objects
     PlayerCont M_teammates; //!< unum known teammmates
     PlayerCont M_opponents; //!< unum known opponents
     PlayerCont M_unknown_players; //!< unknown players
@@ -130,31 +133,47 @@ private:
     //! uniform number of opponent goalie
     int M_opponent_goalie_unum;
 
-    AbstractPlayerCont M_all_players; //!< all players including self
-    AbstractPlayerCont M_all_teammates; //!< all teammates including self
-    AbstractPlayerCont M_all_opponents; //!< all opponents
+    AbstractPlayerCont M_all_players; //!< all players pointers includes self
+    AbstractPlayerCont M_all_teammates; //!< all teammates pointers includes self
+    AbstractPlayerCont M_all_opponents; //!< all opponents pointers includes unknown
 
     AbstractPlayerObject * M_known_teammates[12]; //!< unum known teammates (includes self)
     AbstractPlayerObject * M_known_opponents[12]; //!< unum known opponents (excludes unknown player)
 
     //////////////////////////////////////////////////
-    // strategic value
+    // analyzed result
 
-    // updated in  updateOffsideLine();
     double M_offside_line_x; //!< offside line x value
-    int M_offside_line_count; //!< accuracy count for the offside line
+    int M_offside_line_count; //!< accuracy count of the offside line
 
-    double M_defense_line_x; //!< our defense line x value(their offside line)
+    double M_our_offense_line_x; //!< our offense line x value(consider ball x)
+    double M_our_defense_line_x; //!< our defense line x value(consider ball x)(their offside line)
+    double M_their_offense_line_x; //!< their offense line x value(consider ball x)
+    double M_their_defense_line_x; //!< their defense line x value(consider ball x)
+    int M_their_defense_line_count; //!< accuracy count of their defense line x value
+
+    double M_our_offense_player_line_x; //!< our offense player line x value (not consider ball)
+    double M_our_defense_player_line_x; //!< our defense player line x value (not consider ball)
+    double M_their_offense_player_line_x; //!< their offense player line x value (not consider ball)
+    double M_their_defense_player_line_x; //!< their defense player line x value (not consider ball)
 
     bool M_exist_kickable_teammate; //!< true if exist kickable teammate
     bool M_exist_kickable_opponent; //!< true if exist kickable opponent
 
+    SideID M_last_kicker_side; //!< last ball kicked player's side
+
     //////////////////////////////////////////////////
     // player type management
 
     int M_teammate_types[11]; //!< teammate type reference
     int M_opponent_types[11]; //!< opponent type flag
 
+    //////////////////////////////////////////////////
+    // card information
+
+    Card M_teammate_card[11]; //!< teammates' yellow/red card status
+    Card M_opponent_card[11]; //!< teammates' yellow/red card status
+
     //////////////////////////////////////////////////
     // visual info
 
@@ -164,6 +183,9 @@ private:
     //! view area history
     ViewAreaCont M_view_area_cont;
 
+    //! accuracy count grid map
+    ViewGridMap M_view_grid_map;
+
     //////////////////////////////////////////////////
 
     //! not used
@@ -186,7 +208,7 @@ public:
       \brief get this world mode is valid or not
       \return true if this world model is valid
     */
-    bool valid() const;
+    bool isValid() const;
 
     /*!
       \brief set this world mode is valid or not
@@ -241,7 +263,7 @@ public:
     void setAudioMemory( boost::shared_ptr< AudioMemory > memory );
 
     /*!
-      \brief set teammate player type
+      \brief set teammate player type & reset card status
       \param unum uniform number of changed teammate
       \param id player type ID
     */
@@ -249,13 +271,31 @@ public:
                                 const int id );
 
     /*!
-      \brief set opponent player type
+      \brief set opponent player type & reset card status
       \param unum uniform number of changed opponent
       \param id player type ID
     */
     void setOpponentPlayerType( const int unum,
                                 const int id );
 
+    /*!
+      \brief set yellow card information
+      \param side punished player's side
+      \param unum punished player's unum
+      \param card card type
+     */
+    void setCard( const SideID side,
+                  const int unum,
+                  const Card card );
+
+    /*!
+      \brief set current penalty kick taker.
+      \param side kick taker's side id
+      \param unum kick taker's uniform number
+     */
+    void setPenaltyKickTaker( const SideID side,
+                              const int unum );
+
     // update stuff
 private:
     /*!
@@ -315,6 +355,16 @@ public:
     */
     void updateGameMode( const GameMode & game_mode,
                          const GameTime & current );
+
+    /*!
+      \brief set training start/end time
+      \param t game time
+     */
+    void setTrainingTime( const GameTime & t )
+      {
+          M_training_time = t;
+      }
+
     /*!
       \brief update self view move
       \param w new view width
@@ -465,10 +515,20 @@ private:
      */
     void updatePlayerType();
 
+    /*!
+      \brief update players' card state
+     */
+    void updatePlayerCard();
+
+    /*!
+      \brief estimate unknown players' uniform number
+     */
+    void estimateUnknownPlayerUnum();
+
     /*!
       \brief update player info. relation with ball and self.
      */
-    void updatePlayerMatrix();
+    void updatePlayerStateCache();
 
     /*!
       \brief update offside line
@@ -476,44 +536,58 @@ private:
     void updateOffsideLine();
 
     /*!
-      \brief update defense line
+      \brief update our offense line
     */
-    void updateDefenseLine();
+    void updateOurOffenseLine();
 
     /*!
-      \brief update back passer candidates.
+      \brief update our defense line
+    */
+    void updateOurDefenseLine();
+
+    /*!
+      \brief update their offense line
+    */
+    void updateTheirOffenseLine();
+
+    /*!
+      \brief update their defense line (offside line)
+    */
+    void updateTheirDefenseLine();
+
+    /*!
+      \brief update forward & defense player lines
      */
-    void updateBackPasser();
+    void updatePlayerLines();
+
+    /*!
+      \brief estimate last kicker player
+     */
+    void updateLastKicker();
+
+    /*!
+      \brief update intercept table
+     */
+    void updateInterceptTable();
 public:
 
     /*!
       \brief get our teamname
       \return const reference to the team name string
     */
-    const
-    std::string & teamName() const
-      {
-          return M_teamname;
-      }
+    const std::string & teamName() const { return M_teamname; }
 
     /*!
       \brief get our team side Id
       \return side Id
     */
-    SideID ourSide() const
-      {
-          return M_our_side;
-      }
+    SideID ourSide() const { return M_our_side; }
 
     /*!
       \brief get opponent teamname
       \return const reference to the team name string
     */
-    const
-    std::string & opponentTeamName() const
-      {
-          return M_opponent_teamname;
-      }
+    const std::string & opponentTeamName() const { return M_opponent_teamname; }
 
     /*!
       \brief get opponent team side Id
@@ -524,63 +598,29 @@ public:
           return M_our_side == LEFT ? RIGHT : LEFT;
       }
 
-    /*!
-      \brief check if our team is left or not
-      \return true if our team is left side
-    */
-    bool isOurLeft() const
-      {
-          return M_our_side == LEFT;
-      }
-
-    /*!
-      \brief check if our team is right or not
-      \return true if our team is right side
-    */
-    bool isOurRight() const
-      {
-          return M_our_side == RIGHT;
-      }
-
     /*!
       \brief get last updated time (== current game time)
       \return const reference to the game time object
     */
-    const
-    GameTime & time() const
-      {
-          return M_time;
-      }
+    const GameTime & time() const { return M_time; }
 
     /*!
       \brief get last time updated by sense_body
       \return const reference to the game time object
     */
-    const
-    GameTime & senseBodyTime() const
-      {
-          return M_sense_body_time;
-      }
+    const GameTime & senseBodyTime() const { return M_sense_body_time; }
 
     /*!
       \brief get last time updated by see
       \return const reference to the game time object
     */
-    const
-    GameTime & seeTime() const
-      {
-          return M_see_time;
-      }
+    const GameTime & seeTime() const { return M_see_time; }
 
     /*!
        \brief get last time updated by fullstate
        \return const reference to the game time object
      */
-    const
-    GameTime & fullstateTime() const
-      {
-          return M_fullstate_time;
-      }
+    const GameTime & fullstateTime() const { return M_fullstate_time; }
 
     /*!
       \brief get last time updated by fullstate
@@ -590,81 +630,55 @@ public:
       \brief get last setplay type playmode start time
       \return const reference to the game time object
     */
-    const
-    GameTime & lastSetPlayStartTime() const
-      {
-          return M_last_set_play_start_time;
-      }
+    const GameTime & lastSetPlayStartTime() const { return M_last_set_play_start_time; }
 
     /*!
       \brief get cycle count that setplay type playmode is keeped
       \return counted long integer
     */
-    const
-    long & setplayCount() const
-      {
-          return M_setplay_count;
-      }
+    const long & setplayCount() const { return M_setplay_count; }
 
     /*!
       \brief get current playmode info
       \return const reference to the GameMode object
     */
-    const
-    GameMode & gameMode() const
-      {
-          return M_game_mode;
-      }
+    const GameMode & gameMode() const { return M_game_mode; }
+
+    /*!
+      \brief get training start/end time
+      \return game time object
+     */
+    const GameTime & trainingTime() const { return M_training_time; }
 
     /*!
       \brief get self info
       \return const reference to the SelfObject
     */
-    const
-    SelfObject & self() const
-      {
-          return M_self;
-      }
+    const SelfObject & self() const { return M_self; }
 
     /*!
       \brief get ball info
       \return const reference to the BallObject
     */
-    const
-    BallObject & ball() const
-      {
-          return M_ball;
-      }
+    const BallObject & ball() const { return M_ball; }
 
     /*!
       \brief get teammate info
       \return const reference to the PlayerObject container
     */
-    const
-    PlayerCont & teammates() const
-      {
-          return M_teammates;
-      }
+    const PlayerCont & teammates() const { return M_teammates; }
 
     /*!
       \brief get opponent info (except unknown players)
       \return const reference to the PlayerObject container
     */
-    const
-    PlayerCont & opponents() const
-      {
-          return M_opponents;
-      }
+    const PlayerCont & opponents() const { return M_opponents; }
 
     /*!
       \brief get unknown player info
       \return const reference to the PlayerObject container
     */
-    const
-    PlayerCont & unknownPlayers() const
-      {
-          return M_unknown_players;
-      }
+    const PlayerCont & unknownPlayers() const { return M_unknown_players; }
 
     // reference to the sorted players
 
@@ -672,89 +686,55 @@ public:
       \brief get teammates sorted by distance from self
       \return const reference to the PlayerObject pointer container
     */
-    const
-    PlayerPtrCont & teammatesFromSelf() const
-      {
-          return M_teammates_from_self;
-      }
+    const PlayerPtrCont & teammatesFromSelf() const { return M_teammates_from_self; }
 
     /*!
       \brief get opponents sorted by distance from self (includes unknown players)
       \return const reference to the PlayerObject pointer container
     */
-    const
-    PlayerPtrCont & opponentsFromSelf() const
-      {
-          return M_opponents_from_self;
-      }
+    const PlayerPtrCont & opponentsFromSelf() const { return M_opponents_from_self; }
 
     /*!
       \brief get teammates sorted by distance from ball
       \return const reference to the PlayerObject pointer container
     */
-    const
-    PlayerPtrCont & teammatesFromBall() const
-      {
-          return M_teammates_from_ball;
-      }
+    const PlayerPtrCont & teammatesFromBall() const { return M_teammates_from_ball; }
 
     /*!
       \brief get opponents sorted by distance from ball (includes unknown players)
       \return const reference to the PlayerObject pointer container
     */
-    const
-    PlayerPtrCont & opponentsFromBall() const
-      {
-          return M_opponents_from_ball;
-      }
+    const PlayerPtrCont & opponentsFromBall() const { return M_opponents_from_ball; }
 
     /*!
       \brief get the uniform number of teammate goalie
       \return uniform number value or Unum_Unknown
      */
-    int teammateGoalieUnum() const
-      {
-          return M_teammate_goalie_unum;
-      }
+    int teammateGoalieUnum() const { return M_teammate_goalie_unum; }
 
     /*!
       \brief get the uniform number of opponent goalie
       \return uniform number value or Unum_Unknown
      */
-    int opponentGoalieUnum() const
-      {
-          return M_opponent_goalie_unum;
-      }
+    int opponentGoalieUnum() const { return M_opponent_goalie_unum; }
 
     /*!
       \brief get all players includes self.
-      \return const rerefence to the AbstractPlayerObject pointer container..
+      \return const rerefence to the AbstractPlayerObject pointer container.
      */
-    const
-    AbstractPlayerCont & allPlayers() const
-      {
-          return M_all_players;
-      }
+    const AbstractPlayerCont & allPlayers() const { return M_all_players; }
 
     /*!
       \brief get all teammate playerss includes self.
-      \return const rerefence to the AbstractPlayerObject pointer container..
+      \return const rerefence to the AbstractPlayerObject pointer container.
      */
-    const
-    AbstractPlayerCont & allTeammates() const
-      {
-          return M_all_teammates;
-      }
+    const AbstractPlayerCont & allTeammates() const { return M_all_teammates; }
 
     /*!
       \brief get all opponent players (includes unknown players)
-      \return const rerefence to the AbstractPlayerObject pointer container..
+      \return const rerefence to the AbstractPlayerObject pointer container.
      */
-    const
-    AbstractPlayerCont & allOpponents() const
-      {
-          return M_all_opponents;
-      }
+    const AbstractPlayerCont & allOpponents() const { return M_all_opponents; }
 
     //////////////////////////////////////////////////////////
 
@@ -763,8 +743,7 @@ public:
       \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
-    const
-    AbstractPlayerObject * teammate( const int unum ) const
+    const AbstractPlayerObject * teammate( const int unum ) const
       {
           if ( unum <= 0 || 11 < unum ) return M_known_teammates[0];
           return M_known_teammates[unum];
@@ -775,8 +754,7 @@ public:
       \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
-    const
-    AbstractPlayerObject * opponent( const int unum ) const
+    const AbstractPlayerObject * opponent( const int unum ) const
       {
           if ( unum <= 0 || 11 < unum ) return M_known_opponents[0];
           return M_known_opponents[unum];
@@ -791,28 +769,26 @@ public:
       \param with_goalie if this value is false, goalie is ignored.
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
-    const
-    PlayerObject * getFirstPlayer( PlayerPtrCont::const_iterator first,
-                                   PlayerPtrCont::const_iterator last,
-                                   const int count_thr,
-                                   const bool with_goalie ) const
+    const PlayerObject * getFirstPlayer( const PlayerPtrCont & players,
+                                         const int count_thr,
+                                         const bool with_goalie ) const
       {
-          while ( first != last )
+          const PlayerPtrCont::const_iterator end = players.end();
+          for ( PlayerPtrCont::const_iterator p = players.begin();
+                p != end;
+                ++p )
           {
               if ( ! with_goalie
-                   && (*first)->goalie() )
+                   && (*p)->goalie() )
               {
-                  ++first;
                   continue;
               }
 
-              if ( ! (*first)->isGhost()
-                   && (*first)->posCount() <= count_thr )
+              if ( ! (*p)->isGhost()
+                   && (*p)->posCount() <= count_thr )
               {
-                  return *first;
+                  return *p;
               }
-
-              ++first;
           }
           return static_cast< PlayerObject * >( 0 );
       }
@@ -823,12 +799,10 @@ public:
       \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
-    const
-    PlayerObject * getTeammateNearestToSelf( const int count_thr,
-                                             const bool with_goalie = true ) const
+    const PlayerObject * getTeammateNearestToSelf( const int count_thr,
+                                                   const bool with_goalie = true ) const
       {
-          return getFirstPlayer( teammatesFromSelf().begin(),
-                                 teammatesFromSelf().end(),
+          return getFirstPlayer( teammatesFromSelf(),
                                  count_thr,
                                  with_goalie );
       }
@@ -839,18 +813,16 @@ public:
       \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
-    const
-    PlayerObject * getOpponentNearestToSelf( const int count_thr,
-                                             const bool with_goalie = true ) const
+    const PlayerObject * getOpponentNearestToSelf( const int count_thr,
+                                                   const bool with_goalie = true ) const
       {
-          return getFirstPlayer( opponentsFromSelf().begin(),
-                                 opponentsFromSelf().end(),
+          return getFirstPlayer( opponentsFromSelf(),
                                  count_thr,
                                  with_goalie );
       }
 
     /*!
-      \brief get the distance to teammate nearest to self wtth accuracy count
+      \brief get the distance from teammate nearest to self wtth accuracy count
       \param count_thr accuracy count threshold
       \param with_goalie include goalie if true
       \return distance to the matched opponent. if not found, a big value is returned.
@@ -863,7 +835,7 @@ public:
       }
 
     /*!
-      \brief get the distance to opponent nearest to self wtth accuracy count
+      \brief get the distance from opponent nearest to self wtth accuracy count
       \param count_thr accuracy count threshold
       \param with_goalie include goalie if true
       \return distance to the matched opponent. if not found, a big value is returned.
@@ -881,12 +853,10 @@ public:
       \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
-    const
-    PlayerObject * getTeammateNearestToBall( const int count_thr,
-                                             const bool with_goalie = true ) const
+    const PlayerObject * getTeammateNearestToBall( const int count_thr,
+                                                   const bool with_goalie = true ) const
       {
-          return getFirstPlayer( teammatesFromBall().begin(),
-                                 teammatesFromBall().end(),
+          return getFirstPlayer( teammatesFromBall(),
                                  count_thr,
                                  with_goalie );
       }
@@ -897,12 +867,10 @@ public:
       \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
-    const
-    PlayerObject * getOpponentNearestToBall( const int count_thr,
-                                             const bool with_goalie = true ) const
+    const PlayerObject * getOpponentNearestToBall( const int count_thr,
+                                                   const bool with_goalie = true ) const
       {
-          return getFirstPlayer( opponentsFromBall().begin(),
-                                 opponentsFromBall().end(),
+          return getFirstPlayer( opponentsFromBall(),
                                  count_thr,
                                  with_goalie );
       }
@@ -933,61 +901,92 @@ public:
           return ( p ? p->distFromBall() : 65535.0 );
       }
 
-    // tactical info
-
     /*!
       \brief get estimated offside line x coordinate
-      \return real value
+      \return offside line
     */
-    const
-    double & offsideLineX() const
-      {
-          return M_offside_line_x;
-      }
+    const double & offsideLineX() const { return M_offside_line_x; }
 
     /*!
       \brief get the accuracy count for the offside line
       \return accuracy count
      */
-    int offsideLineCount() const
-      {
-          return M_offside_line_count;
-      }
+    int offsideLineCount() const { return M_offside_line_count; }
 
     /*!
-      \brief get estimated defside line(offside line for opponent) x coordinate
-      \return real value
+      \brief our offense line (consider ball pos)
+      \return our offense line x
     */
-    const
-    double & defenseLineX() const
-      {
-          return M_defense_line_x;
-      }
+    const double & ourOffenseLineX() const { return M_our_offense_line_x; }
+
+    /*!
+      \brief get our defense line (consider ball pos)(offside line for opponent)
+      \return our defense line x
+    */
+    const double & ourDefenseLineX() const { return M_our_defense_line_x; }
+
+    /*!
+      \brief get their offense line (consider ball pos)
+      \return their offense line x
+    */
+    const double & theirOffenseLineX() const { return M_their_offense_line_x; }
+
+    /*!
+      \brief get their defense line x value (consider ball pos)
+      \return their offense line x
+    */
+    const double & theirDefenseLineX() const { return M_their_defense_line_x; }
+
+    /*!
+      \brief get our offense player line (not consder ball pos)
+      \return our offense player line x
+    */
+    const double & ourOffensePlayerLineX() const { return M_our_offense_player_line_x; }
+
+    /*!
+      \brief get our defense player line (not consder ball pos)
+      \return our defense player line x
+    */
+    const double & ourDefensePlayerLineX() const { return M_our_defense_player_line_x; }
+
+    /*!
+      \brief get their offense player line (not consder ball pos)
+      \return our defense player line x
+    */
+    const double & theirOffensePlayerLineX() const { return M_their_offense_player_line_x; }
+
+    /*!
+      \brief get estimated their offense line x value
+      \return their offense line x
+    */
+    const double & theirDefensePlayerLineX() const { return M_their_defense_player_line_x; }
+
 
     /*!
       \brief check if exist kickable teammate
       \return true if agent estimated that kickable teammate exists
     */
-    bool existKickableTeammate() const
-      {
-          return M_exist_kickable_teammate;
-      }
+    bool existKickableTeammate() const { return M_exist_kickable_teammate; }
 
     /*!
       \brief check if exist kickable opponent
       \return true if agent estimated that kickable opponent exists
     */
-    bool existKickableOpponent() const
-      {
-          return M_exist_kickable_opponent;
-      }
+    bool existKickableOpponent() const { return M_exist_kickable_opponent; }
+
+    /*!
+      \brief get the estimated last kicker's side
+      \return side id
+     */
+    SideID lastKickerSide() const { return M_last_kicker_side; }
+
 
     /*!
       \brief get player type Id of teammate
       \param unum target teammate uniform number
       \return player type Id. if unum is illegal, Default Id is returned.
     */
-    int teammateHeteroID( const int unum ) const
+    int teammatePlayerTypeId( const int unum ) const
       {
           if ( unum < 1 || 11 < unum )
           {
@@ -1003,15 +1002,14 @@ public:
       \param unum target teammate uniform number
       \return const pointer to the player type object instance
      */
-    const
-    PlayerType * teammatePlayerType( const int unum ) const;
+    const PlayerType * teammatePlayerType( const int unum ) const;
 
     /*!
       \brief get player type Id of opponent
       \param unum target teammate uniform number
       \return player type Id. if unum is illegal, Unknown is returned.
     */
-    int opponentHeteroID( const int unum ) const
+    int opponentPlayerTypeId( const int unum ) const
       {
           if ( unum < 1 || 11 < unum )
           {
@@ -1027,8 +1025,48 @@ public:
       \param unum target opponent uniform number
       \return const pointer to the player type object instance
      */
-    const
-    PlayerType * opponentPlayerType( const int unum ) const;
+    const PlayerType * opponentPlayerType( const int unum ) const;
+
+    //
+    // card information
+    //
+
+    /*!
+      \brief get teammate's yellow card status
+      \param unum uniform number
+      \return yellow card status
+     */
+    Card teammateCard( const int unum ) const
+      {
+          if ( unum < 1 || 11 < unum )
+          {
+              std::cerr << "(WorldModel::teammateCard) Illegal unum "
+                        << unum << std::endl;
+              return NO_CARD;
+          }
+          return M_teammate_card[ unum - 1 ];
+      }
+
+    /*!
+      \brief get opponent's yellow card status
+      \param unum uniform number
+      \return yellow card status
+     */
+    Card opponentCard( const int unum ) const
+      {
+          if ( unum < 1 || 11 < unum )
+          {
+              std::cerr << "(WorldModel::opponentCard) Illegal unum "
+                        << unum << std::endl;
+              return NO_CARD;
+          }
+          return M_opponent_card[ unum - 1 ];
+      }
+
+    //
+    // analyzed results
+    //
+
 
     // visual memory info
 
@@ -1069,11 +1107,13 @@ public:
       \brief get view area history container
       \return const refrence to the view area container.
      */
-    const
-    ViewAreaCont & viewAreaCont() const
-      {
-          return M_view_area_cont;
-      }
+    const ViewAreaCont & viewAreaCont() const { return M_view_area_cont; }
+
+    /*!
+      \brief get field grid map that holds observation accuracy count
+      \return const reference to the ViewGridMap instance
+     */
+    const ViewGridMap & viewGridMap() const { return M_view_grid_map; }
 
     /*!
       \brief get the specific point accuracy count
@@ -1136,28 +1176,87 @@ public:
       \brief get teammate goalie pointer
       \return if found pointer to goalie object, otherwise NULL
     */
-    const
-    PlayerObject * getTeammateGoalie() const;
+    const AbstractPlayerObject * getTeammateGoalie() const;
 
     /*!
       \brief get opponent goalie pointer
       \return if found pointer to goalie object, otherwise NULL
     */
-    const
-    PlayerObject * getOpponentGoalie() const;
+    const PlayerObject * getOpponentGoalie() const;
+
+
+    /*!
+      \brief get player pointer nearest to point (excludes self)
+      \param point considered point
+      \param players target players
+      \param count_thr confidence count threshold
+      \param dist_to_point variable pointer to store the distance
+      from retuned player to point
+      \return if found, pointer to player object, othewise NULL
+     */
+    const PlayerObject * getPlayerNearestTo( const Vector2D & point,
+                                             const PlayerPtrCont & players,
+                                             const int count_thr,
+                                             double * dist_to_point ) const;
+
+    /*!
+      \brief get the distance from input point to the nearest player
+      \param players target players
+      \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistPlayerNearestTo( const Vector2D & point,
+                                   const PlayerPtrCont & players,
+                                   const int count_thr ) const
+      {
+          double d = 65535.0;
+          const PlayerObject * p = getPlayerNearestTo( point, players, count_thr, &d );
+          return ( p ? d : 65535.0 );
+      }
+
+    /*!
+      \brief get the distance from input point to the nearest teammate
+      \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistTeammateNearestTo( const Vector2D & point,
+                                     const int count_thr ) const
+      {
+          double d = 65535.0;
+          const PlayerObject * p = getPlayerNearestTo( point, teammatesFromSelf(), count_thr, &d );
+          return ( p ? d : 65535.0 );
+      }
+
+    /*!
+      \brief get the distance from input point to the nearest opponent
+      \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
+      \return distance to the matched opponent. if not found, a big value is returned.
+     */
+    double getDistOpponentNearestTo( const Vector2D & point,
+                                     const int count_thr ) const
+      {
+          double d = 65535.0;
+          const PlayerObject * p = getPlayerNearestTo( point, opponentsFromSelf(), count_thr, &d );
+          return ( p ? d : 65535.0 );
+      }
 
     /*!
       \brief get teammate pointer nearest to point
       \param point considered point
-      \param count_thr cinfidence count threshold
+      \param count_thr confidence count threshold
       \param dist_to_point variable pointer to store the distance
       from retuned player to point
       \return if found, pointer to player object, othewise NULL
     */
-    const
-    PlayerObject * getTeammateNearestTo( const Vector2D & point,
-                                         const int count_thr,
-                                         double * dist_to_point ) const;
+    const PlayerObject * getTeammateNearestTo( const Vector2D & point,
+                                               const int count_thr,
+                                               double * dist_to_point ) const
+      {
+          return getPlayerNearestTo( point, teammatesFromSelf(), count_thr, dist_to_point );
+      }
 
     /*!
       \brief get teammate pointer nearest to the specified player
@@ -1167,10 +1266,9 @@ public:
       from retuned player to point
       \return if found, pointer to player object, othewise NULL
      */
-    const
-    PlayerObject * getTeammateNearestTo( const PlayerObject * p,
-                                         const int count_thr,
-                                         double * dist_to_point ) const
+    const PlayerObject * getTeammateNearestTo( const PlayerObject * p,
+                                               const int count_thr,
+                                               double * dist_to_point ) const
       {
           if ( ! p ) return static_cast< const PlayerObject * >( 0 );
           return getTeammateNearestTo( p->pos(), count_thr, dist_to_point );
@@ -1184,11 +1282,12 @@ public:
       from retuned player to point
       \return if found pointer to player object, othewise NULL
     */
-    const
-    PlayerObject * getOpponentNearestTo( const Vector2D & point,
-                                         const int count_thr,
-                                         double * dist_to_point ) const;
-
+    const PlayerObject * getOpponentNearestTo( const Vector2D & point,
+                                               const int count_thr,
+                                               double * dist_to_point ) const
+      {
+          return getPlayerNearestTo( point, opponentsFromSelf(), count_thr, dist_to_point );
+      }
 
     /*!
       \brief get teammate pointer nearest to the specified player
@@ -1198,8 +1297,7 @@ public:
       from retuned player to point
       \return if found, pointer to player object, othewise NULL
      */
-    const
-    PlayerObject * getOpponentNearestTo( const PlayerObject * p,
+    const PlayerObject * getOpponentNearestTo( const PlayerObject * p,
                                          const int count_thr,
                                          double * dist_to_point ) const
       {
@@ -1208,18 +1306,20 @@ public:
       }
 
     /*!
-      \brief template utility. check if teammate exist in the specified region.
+      \brief template utility. check if player exist in the specified region.
       \param region template parameter. region to be checked
+      \param players target players
       \param count_thr confdence count threshold for players
       \param with_goalie if true, goalie player is cheked.
      */
     template < typename REGION >
-    bool existTeammateIn( const REGION & region,
-                          const int count_thr,
-                          const bool with_goalie ) const
+    bool existPlayerIn( const REGION & region,
+                        const PlayerPtrCont & players,
+                        const int count_thr,
+                        const bool with_goalie ) const
       {
-          const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
-          for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
+          const PlayerPtrCont::const_iterator end = players.end();
+          for ( PlayerPtrCont::const_iterator it = players.begin();
                 it != end;
                 ++it )
           {
@@ -1240,6 +1340,21 @@ public:
           return false;
       }
 
+
+    /*!
+      \brief template utility. check if teammate exist in the specified region.
+      \param region template parameter. region to be checked
+      \param count_thr confdence count threshold for players
+      \param with_goalie if true, goalie player is cheked.
+     */
+    template < typename REGION >
+    bool existTeammateIn( const REGION & region,
+                          const int count_thr,
+                          const bool with_goalie ) const
+      {
+          return existPlayerIn( region, teammatesFromSelf(), count_thr, with_goalie );
+      }
+
     /*!
       \brief template utility. check if opponent exist in the specified region.
       \param region template parameter. region to be checked
@@ -1252,26 +1367,43 @@ public:
                           const int count_thr,
                           const bool with_goalie ) const
       {
-          const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
-          for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
+          return existPlayerIn( region, opponentsFromSelf(), count_thr, with_goalie );
+      }
+
+
+    /*!
+      \brief count the number of teammate exist in the specified region
+      \param region template parameter. region to be checked
+      \param players target players
+      \param count_thr confdence count threshold for players
+      \param with_goalie if true, goalie player is cheked.
+      \return total count of teammate existed in the region
+     */
+    template < typename REGION >
+    int countPlayersIn( const REGION & region,
+                        const PlayerPtrCont & players,
+                        const int count_thr,
+                        const bool with_goalie ) const
+      {
+          int count = 0;
+          const PlayerPtrCont::const_iterator end = players.end();
+          for ( PlayerPtrCont::const_iterator it = players.begin();
                 it != end;
                 ++it )
           {
               if ( (*it)->posCount() > count_thr
-                   || (*it)->isGhost() )
-              {
-                  continue;
-              }
-              if ( (*it)->goalie() && ! with_goalie )
+                   || (*it)->isGhost()
+                   || ( (*it)->goalie() && ! with_goalie )
+                   )
               {
                   continue;
               }
               if ( region.contains( (*it)->pos() ) )
               {
-                  return true;
+                  ++count;
               }
           }
-          return false;
+          return count;
       }
 
     /*!
@@ -1286,25 +1418,7 @@ public:
                           const int count_thr,
                           const bool with_goalie ) const
       {
-          int count = 0;
-          const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
-          for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
-                it != end;
-                ++it )
-          {
-              if ( (*it)->posCount() > count_thr
-                   || (*it)->isGhost()
-                   || ( (*it)->goalie() && ! with_goalie )
-                   )
-              {
-                  continue;
-              }
-              if ( region.contains( (*it)->pos() ) )
-              {
-                  ++count;
-              }
-          }
-          return count;
+          return countPlayersIn( region, teammatesFromSelf(), count_thr, with_goalie );
       }
 
     /*!
@@ -1319,25 +1433,7 @@ public:
                           const int count_thr,
                           const bool with_goalie ) const
       {
-          int count = 0;
-          const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
-          for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
-                it != end;
-                ++it )
-          {
-              if ( (*it)->posCount() > count_thr
-                   || (*it)->isGhost()
-                   || ( (*it)->goalie() && ! with_goalie )
-                   )
-              {
-                  continue;
-              }
-              if ( region.contains( (*it)->pos() ) )
-              {
-                  ++count;
-              }
-          }
-          return count;
+          return countPlayersIn( region, opponentsFromSelf(), count_thr, with_goalie );
       }
 
 };
