@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -79,6 +79,16 @@ private:
     */
     bool parseData( std::istream & is,
                     Handler & handler ) const;
+
+    /*!
+      \brief parse MSG_MODE info(msg_info_t)
+      \param is reference to the input stream
+      \param handler reference to the data handler object
+      \retval true if successfully parsed.
+      \retval false if failed to parse.
+    */
+    bool parseMsgInfo( std::istream & is,
+                       Handler & handler ) const;
 };
 
 } // end of namespace
