@@ -107,6 +107,9 @@ enum PlayerStatus {
     BACK_PASS       = 0x00004000,
     FREE_KICK_FAULT = 0x00008000,
     POST_COLLIDE    = 0x00010000, // player collided with goal posts
+    FOUL_CHARGED     = 0x00020000, // player is frozen by intentional tackle foul
+    YELLOW_CARD     = 0x00040000,
+    RED_CARD        = 0x00080000,
 };
 
 
@@ -312,9 +315,13 @@ struct player_type_t {
     Int32 effort_max; //!< max(initial) effort value
     Int32 effort_min; //!< min effort value
 
-    Int32 sparelong1; //!< spare variable
-    Int32 sparelong2; //!< spare variable
-    Int32 sparelong3; //!< spare variable
+    Int32 kick_power_rate; //!< kick power rate
+    Int32 foul_detect_probability; //!< foul detect probability
+    Int32 catchable_area_l_stretch; //!< catch area length stretch factor
+
+    //Int32 sparelong1;  replaced by kick_power_rate
+    //Int32 sparelong2;  replaced by foul_detect_probability
+    //Int32 sparelong3;  replaced by catchable_area_l_stretch
     Int32 sparelong4; //!< spare variable
     Int32 sparelong5; //!< spare variable
     Int32 sparelong6; //!< spare variable
@@ -496,11 +503,18 @@ struct player_params_t {
     Int32 new_dash_power_rate_delta_max; //!< maximal value wighin delta range
     Int32 new_stamina_inc_max_delta_factor; //!< tradeoff parameter
 
-    Int32 sparelong5; //!< spare variable
-    Int32 sparelong6; //!< spare variable
-    Int32 sparelong7; //!< spare variable
-    Int32 sparelong8; //!< spare variable
-    Int32 sparelong9; //!< spare variable
+    Int32 kick_power_rate_delta_min;
+    Int32 kick_power_rate_delta_max;
+    Int32 foul_detect_probability_delta_factor;
+
+    Int32 catchable_area_l_stretch_min;
+    Int32 catchable_area_l_stretch_max;
+
+    //Int32 sparelong5;  replaced by kick_power_rate_delta_min
+    //Int32 sparelong6;  replaced by kick_power_rate_delta_max
+    //Int32 sparelong7;  replaced by foul_detect_probability_delta_factor
+    //Int32 sparelong8;  replaced by catchable_area_l_stretch_min
+    //Int32 sparelong9;  replaced by catchable_area_l_stretch_max
     Int32 sparelong10; //!< spare variable
 
     Int16 allow_mult_default_type; //!< whether multiple default type is allowed
