@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,6 +48,7 @@
 #include <rcsc/param/param_map.h>
 #include <rcsc/param/conf_file_parser.h>
 #include <rcsc/param/cmd_line_parser.h>
+#include <rcsc/version.h>
 
 #include <sstream>
 #include <cstring>
@@ -56,10 +57,13 @@ namespace rcsc {
 
 ///////////////////////////////////////////////////////////////////////
 /*!
-  \struct TrainerAgentImpl
+  \struct Impl
   \brief trainer agent implementation
 */
-struct TrainerAgentImpl {
+struct TrainerAgent::Impl {
+
+    //! reference to the agent instance
+    TrainerAgent & agent_;
 
     //! flag to check if (think) message was received or not.
     bool think_received_;
@@ -82,13 +86,115 @@ struct TrainerAgentImpl {
     /*!
       \brief initialize all members
     */
-    TrainerAgentImpl()
-        : think_received_( false )
-        , server_cycle_stopped_( true )
-        , last_decision_time_( -1, 0 )
-        , current_time_( 0, 0 )
+    Impl( TrainerAgent & agent )
+        : agent_( agent ),
+          think_received_( false ),
+          server_cycle_stopped_( true ),
+          last_decision_time_( -1, 0 ),
+          current_time_( 0, 0 )
       { }
 
+    /*!
+      \brief send init or reconnect command to server
+
+      init commad is sent in BasicClient's run() method
+      Do not call this method yourself!
+    */
+    void sendInitCommand();
+
+    /*!
+      \brief send disconnection command message to server
+      and set the server status to end.
+    */
+    void sendByeCommand();
+
+
+    /*!
+      \brief analyze init replay message
+      \param msg raw server message
+    */
+    void analyzeInit( const char * msg );
+
+    /*!
+      \brief analyze cycle info in server message
+      \param msg raw server message
+      \param by_see_global if message type is see_global, this value becomes true
+    */
+    bool analyzeCycle( const char * msg,
+                       const bool by_see_global );
+
+    /*!
+      \brief analyze see_global message
+      \param msg raw server message
+    */
+    void analyzeSeeGlobal( const char * msg );
+
+    /*!
+      \brief analyze hear message
+      \param msg raw server message
+    */
+    void analyzeHear( const char * msg );
+
+    /*!
+      \brief analyze referee message
+      \param msg raw server message
+    */
+    void analyzeHearReferee( const char * msg );
+
+    /*!
+      \brief analyze audio message from player
+      \param msg raw server message
+    */
+    void analyzeHearPlayer( const char * msg );
+
+    /*!
+      \brief analyze change_player_type message
+      \param msg raw server message
+    */
+    void analyzeChangePlayerType( const char * msg );
+
+    /*!
+      \brief analyze player_type parameter message
+      \param msg raw server message
+    */
+    void analyzePlayerType( const char * msg );
+
+    /*!
+      \brief analyze player_param parameter message
+      \param msg raw server message
+    */
+    void analyzePlayerParam( const char * msg );
+
+    /*!
+      \brief analyze server_param parameter message
+      \param msg raw server message
+    */
+    void analyzeServerParam( const char * msg );
+
+    /*!
+      \brief analyze ok message
+      \param msg raw server message
+    */
+    void analyzeOK( const char * msg );
+
+    /*!
+      \brief analyze ok teamnames message
+      \param msg raw server message
+     */
+    void analyzeTeamNames( const char * msg );
+
+    /*!
+      \brief analyze error message
+      \param msg raw server message
+    */
+    void analyzeError( const char * msg );
+
+    /*!
+      \brief analyze warning message
+      \param msg raw server message
+    */
+    void analyzeWarning( const char * msg );
+
     /*!
       \brief update current time using analyzed time value
       \param new_time analyzed time value
@@ -110,8 +216,8 @@ struct TrainerAgentImpl {
 
 */
 void
-TrainerAgentImpl::updateCurrentTime( const long & new_time,
-                                     const bool by_see_global )
+TrainerAgent::Impl::updateCurrentTime( const long & new_time,
+                                       const bool by_see_global )
 {
     if ( server_cycle_stopped_ )
     {
@@ -148,7 +254,7 @@ TrainerAgentImpl::updateCurrentTime( const long & new_time,
 
 */
 void
-TrainerAgentImpl::updateServerStatus()
+TrainerAgent::Impl::updateServerStatus()
 {
     // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     //  called just after referee.parse()
@@ -174,10 +280,10 @@ TrainerAgentImpl::updateServerStatus()
 */
 TrainerAgent::TrainerAgent()
     : SoccerAgent()
-    , M_impl( new TrainerAgentImpl )
+    , M_impl( new Impl( *this ) )
     , M_config()
 {
-    M_worldmodel.init( NEUTRAL );
+    M_worldmodel.init( NEUTRAL, 999 );
 
     boost::shared_ptr< AudioMemory > audio_memory( new AudioMemory );
 
@@ -226,9 +332,9 @@ TrainerAgent::initImpl( CmdLineParser & cmd_parser )
     cmd_parser.parse( system_param_map );
     if ( help )
     {
+        std::cout << copyright() << std::endl;
         system_param_map.printHelp( std::cout );
         trainer_param_map.printHelp( std::cout );
-        M_client->setServerAlive( false );
         return false;
     }
 
@@ -242,12 +348,14 @@ TrainerAgent::initImpl( CmdLineParser & cmd_parser )
     // analyze command line for trainer options.
     cmd_parser.parse( trainer_param_map );
 
-    //if ( ! cmd_parser.invalidOptions().empty() )
-    //{
-    //  std::cerr << "trainer: ***WARNING*** detected invalid options: ";
-    //    cmd_parser.printError( std::cerr );
-    //    std::cerr << std::endl;
-    //}
+    if ( config().version() < 1.0
+         || 15.0 <= config().version() )
+    {
+        std::cerr << "Unsupported client version: " << config().version()
+                  << std::endl;
+        return false;
+    }
+
     return true;
 }
 
@@ -282,7 +390,7 @@ TrainerAgent::handleStart()
         return false;
     }
 
-    sendInitCommand();
+    M_impl->sendInitCommand();
     return true;
 }
 
@@ -361,7 +469,7 @@ TrainerAgent::finalize()
 {
     if ( M_client->isServerAlive() )
     {
-        sendByeCommand();
+        M_impl->sendByeCommand();
     }
     std::cerr << "trainer: finished."<< std::endl;
 }
@@ -376,11 +484,11 @@ TrainerAgent::parse( const char * msg )
 
     if ( ! std::strncmp( msg, "(see_global ", strlen("(see_global ")) )
     {
-        analyzeSeeGlobal( msg );
+        M_impl->analyzeSeeGlobal( msg );
     }
     else if ( ! std::strncmp( msg, "(hear ", 6 ) )
     {
-        analyzeHear( msg );
+        M_impl->analyzeHear( msg );
     }
     else if ( ! std::strncmp( msg, "(think)", 7 ) )
     {
@@ -388,31 +496,31 @@ TrainerAgent::parse( const char * msg )
     }
     else if ( ! std::strncmp( msg, "(change_player_type ", 20 ) )
     {
-        analyzeChangePlayerType( msg );
+        M_impl->analyzeChangePlayerType( msg );
     }
     else if ( ! std::strncmp( msg, "(player_type ", 13 ) )  // hetero param
     {
-        analyzePlayerType( msg );
+        M_impl->analyzePlayerType( msg );
     }
     else if ( ! std::strncmp( msg, "(player_param ", 14 ) ) // tradeoff param
     {
-        analyzePlayerParam( msg );
+        M_impl->analyzePlayerParam( msg );
     }
     else if ( ! std::strncmp( msg, "(server_param ", 14 ) )
     {
-        analyzeServerParam( msg );
+        M_impl->analyzeServerParam( msg );
     }
     else if ( ! std::strncmp( msg, "(ok ", 4 ) )
     {
-        analyzeOK( msg );
+        M_impl->analyzeOK( msg );
     }
     else if ( ! std::strncmp( msg, "(error ", 7 ) )
     {
-        analyzeError( msg );
+        M_impl->analyzeError( msg );
     }
     else if ( ! std::strncmp( msg, "(warning ", 9 ) )
     {
-        analyzeWarning( msg );
+        M_impl->analyzeWarning( msg );
     }
     else if ( ! std::strncmp( msg, "(score ", 7 ) )
     {
@@ -423,7 +531,7 @@ TrainerAgent::parse( const char * msg )
     else if ( ! std::strncmp( msg, "(init ", 6 )
               || ! std::strncmp( msg, "(reconnect ", 11 ) )
     {
-        analyzeInit( msg );
+        M_impl->analyzeInit( msg );
     }
     else
     {
@@ -439,37 +547,37 @@ TrainerAgent::parse( const char * msg )
 
 */
 void
-TrainerAgent::analyzeInit( const char * msg )
+TrainerAgent::Impl::analyzeInit( const char * msg )
 {
     if ( std::strncmp( msg, "(init ok)", std::strlen( "(init ok)" ) ) )
     {
         std::cerr << "trainer: Failed to init trainer.. init reply message=["
                   << msg << ']'
                   << std::endl;
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
         return;
     }
 
     // initialization commands....
-    if ( config().useEye() )
+    if ( agent_.config().useEye() )
     {
-        doEye( true );
+        agent_.doEye( true );
     }
 
-    if ( config().useEar() )
+    if ( agent_.config().useEar() )
     {
-        doEar( true );
+        agent_.doEar( true );
     }
 
     // set compression level
-    if ( 0 < config().compression()
-         && config().compression() <= 9 )
+    if ( 0 < agent_.config().compression()
+         && agent_.config().compression() <= 9 )
     {
-        TrainerCompressionCommand com( config().compression() );
-        sendCommand( com );
+        TrainerCompressionCommand com( agent_.config().compression() );
+        agent_.sendCommand( com );
     }
 
-    M_client->setIntervalMSec( ServerParam::i().simulatorStep() );
+    agent_.M_client->setIntervalMSec( ServerParam::i().simulatorStep() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -477,8 +585,8 @@ TrainerAgent::analyzeInit( const char * msg )
 
 */
 bool
-TrainerAgent::analyzeCycle( const char * msg,
-                            const bool by_see_global )
+TrainerAgent::Impl::analyzeCycle( const char * msg,
+                                  const bool by_see_global )
 {
     char buf[128];
     long cycle = 0;
@@ -496,7 +604,7 @@ TrainerAgent::analyzeCycle( const char * msg,
         return false;
     }
 
-    M_impl->updateCurrentTime( cycle, by_see_global );
+    updateCurrentTime( cycle, by_see_global );
     return true;
 }
 
@@ -505,23 +613,23 @@ TrainerAgent::analyzeCycle( const char * msg,
 
 */
 void
-TrainerAgent::analyzeSeeGlobal( const char * msg )
+TrainerAgent::Impl::analyzeSeeGlobal( const char * msg )
 {
     if ( ! analyzeCycle( msg, true ) )
     {
         return;
     }
 
-    M_impl->visual_.parse( msg,
-                           config().version(),
-                           M_impl->current_time_ );
+    visual_.parse( msg,
+                   agent_.config().version(),
+                   current_time_ );
 
     // update world model
-    if ( M_impl->visual_.time() == M_impl->current_time_
-         && world().time() != M_impl->current_time_ )
+    if ( visual_.time() == current_time_
+         && agent_.world().time() != current_time_ )
     {
-        M_worldmodel.updateAfterSeeGlobal( M_impl->visual_,
-                                           M_impl->current_time_ );
+        agent_.M_worldmodel.updateAfterSeeGlobal( visual_,
+                                                  current_time_ );
     }
 }
 
@@ -530,7 +638,7 @@ TrainerAgent::analyzeSeeGlobal( const char * msg )
 
 */
 void
-TrainerAgent::analyzeHear( const char * msg )
+TrainerAgent::Impl::analyzeHear( const char * msg )
 {
     //std::cerr << "Trainer hear " << msg << std::endl;
     if ( ! analyzeCycle( msg, false ) )
@@ -551,7 +659,7 @@ TrainerAgent::analyzeHear( const char * msg )
                          sender, &cycle ) != 2 )
     {
         std::cerr << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " *** ERROR *** failed to parse hear sender. ["
                   << msg << std::endl;
         return;
@@ -573,7 +681,7 @@ TrainerAgent::analyzeHear( const char * msg )
 
 */
 void
-TrainerAgent::analyzeHearReferee( const char * msg )
+TrainerAgent::Impl::analyzeHearReferee( const char * msg )
 {
     long cycle;
     char mode[512];
@@ -582,31 +690,80 @@ TrainerAgent::analyzeHearReferee( const char * msg )
          && std::sscanf( msg, "(hear referee %ld %511[^)]", &cycle, mode ) != 2 )
     {
         std::cerr << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " playmode scan error. "<< msg
                   << std::endl;
         return;
     }
 
-    if ( ! M_impl->game_mode_.update( mode, M_impl->current_time_ ) )
+    if ( ! game_mode_.update( mode, current_time_ ) )
     {
-        //std::cerr << "trainer: "
-        //          << M_impl->current_time_
-        //          << " Unknown playmode string." << mode
-        //          << std::endl;
+        if ( ! std::strncmp( mode, "yellow_card", std::strlen( "yellow_card" ) ) )
+        {
+            char side = '?';
+            int unum = Unum_Unknown;
+            if ( std::sscanf( mode, "yellow_card_%c_%d", &side, &unum ) != 2 )
+            {
+                std::cerr << agent_.config().teamName()
+                          << " coach: "
+                          << agent_.world().time()
+                          << " could not parse the yellow card message [" << msg << ']'
+                          << std::endl;
+            }
+
+            agent_.M_worldmodel.setYellowCard( ( side == 'l'
+                                                 ? LEFT
+                                                 : side == 'r'
+                                                 ? RIGHT
+                                                 : NEUTRAL ),
+                                               unum );
+        }
+        else if ( ! std::strncmp( mode, "red_card", std::strlen( "red_card" ) ) )
+        {
+            char side = '?';
+            int unum = Unum_Unknown;
+            if ( std::sscanf( mode, "red_card_%c_%d", &side, &unum ) != 2 )
+            {
+                std::cerr << agent_.config().teamName()
+                          << " coach: "
+                          << agent_.world().time()
+                          << " could not parse the red card message [" << msg << ']'
+                          << std::endl;
+            }
+
+            agent_.M_worldmodel.setRedCard( ( side == 'l'
+                                              ? LEFT
+                                              : side == 'r'
+                                              ? RIGHT
+                                              : NEUTRAL ),
+                                            unum );
+        }
+        else if ( ! std::strncmp( mode, "training", std::strlen( "training" ) ) )
+        {
+            // end keepaway (or some training) episode
+            agent_.M_worldmodel.setTrainingTime( current_time_ );
+            return;
+        }
+        else
+        {
+            std::cerr << "trainer: "
+                      << agent_.world().time()
+                      << " Unknown playmode string." << mode
+                      << std::endl;
+        }
+
         return;
     }
 
+    updateServerStatus();
 
-    M_impl->updateServerStatus();
-
-    if ( M_impl->game_mode_.isGameEndMode() )
+    if ( game_mode_.isGameEndMode() )
     {
         sendByeCommand();
         return;
     }
 
-    M_worldmodel.updatePlayMode( M_impl->game_mode_, M_impl->current_time_ );
+    agent_.M_worldmodel.updateGameMode( game_mode_, current_time_ );
 }
 
 /*-------------------------------------------------------------------*/
@@ -614,10 +771,10 @@ TrainerAgent::analyzeHearReferee( const char * msg )
 
 */
 void
-TrainerAgent::analyzeHearPlayer( const char * )
+TrainerAgent::Impl::analyzeHearPlayer( const char * )
 {
     //std::cerr << "trainer: "
-    //          << M_impl->current_time_
+    //          << current_time_
     //          << " recv player message [" << msg << "]"
     //          << std::endl;
 }
@@ -627,7 +784,7 @@ TrainerAgent::analyzeHearPlayer( const char * )
 
 */
 void
-TrainerAgent::analyzeChangePlayerType( const char * msg )
+TrainerAgent::Impl::analyzeChangePlayerType( const char * msg )
 {
     // "(ok change_player_type <teamname> <unum> <type>)"
 
@@ -642,32 +799,32 @@ TrainerAgent::analyzeChangePlayerType( const char * msg )
          || type < 0 )
     {
         std::cerr << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " ***ERROR*** parse error. "
                   << msg
                   << std::endl;
         return;
     }
 
-    if ( world().teamNameLeft() == teamname )
+    if ( agent_.world().teamNameLeft() == teamname )
     {
-        M_worldmodel.setPlayerType( LEFT, unum, type );
+        agent_.M_worldmodel.setPlayerType( LEFT, unum, type );
     }
-    else if ( world().teamNameRight() == teamname )
+    else if ( agent_.world().teamNameRight() == teamname )
     {
-        M_worldmodel.setPlayerType( RIGHT, unum, type );
+        agent_.M_worldmodel.setPlayerType( RIGHT, unum, type );
     }
-    else if ( world().teamNameLeft().empty()
+    else if ( agent_.world().teamNameLeft().empty()
               && std::strlen( teamname ) != 0 )
     {
-        M_worldmodel.setTeamName( LEFT, teamname );
-        M_worldmodel.setPlayerType( LEFT, unum, type );
+        agent_.M_worldmodel.setTeamName( LEFT, teamname );
+        agent_.M_worldmodel.setPlayerType( LEFT, unum, type );
     }
-    else if ( world().teamNameRight().empty()
+    else if ( agent_.world().teamNameRight().empty()
               && std::strlen( teamname ) != 0 )
     {
-        M_worldmodel.setTeamName( RIGHT, teamname );
-        M_worldmodel.setPlayerType( RIGHT, unum, type );
+        agent_.M_worldmodel.setTeamName( RIGHT, teamname );
+        agent_.M_worldmodel.setPlayerType( RIGHT, unum, type );
     }
 }
 
@@ -676,12 +833,11 @@ TrainerAgent::analyzeChangePlayerType( const char * msg )
 
 */
 void
-TrainerAgent::analyzePlayerType( const char * msg )
+TrainerAgent::Impl::analyzePlayerType( const char * msg )
 {
     //std::cerr << "trainer: parsed player_type " << std::endl;
 
-    PlayerType player_type( ServerParam::i(),
-                            msg, config().version() );
+    PlayerType player_type( msg, agent_.config().version() );
     PlayerTypeSet::instance().insert( player_type );
 }
 
@@ -690,9 +846,9 @@ TrainerAgent::analyzePlayerType( const char * msg )
 
 */
 void
-TrainerAgent::analyzePlayerParam( const char * msg )
+TrainerAgent::Impl::analyzePlayerParam( const char * msg )
 {
-    PlayerParam::instance().parse( msg, config().version() );
+    PlayerParam::instance().parse( msg, agent_.config().version() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -700,10 +856,10 @@ TrainerAgent::analyzePlayerParam( const char * msg )
 
 */
 void
-TrainerAgent::analyzeServerParam( const char * msg )
+TrainerAgent::Impl::analyzeServerParam( const char * msg )
 {
-    ServerParam::instance().parse( msg, config().version() );
-    PlayerTypeSet::instance().resetDefaultType( ServerParam::i() );
+    ServerParam::instance().parse( msg, agent_.config().version() );
+    PlayerTypeSet::instance().resetDefaultType();
 
     // update alarm interval
     if ( ! ServerParam::i().synchMode()
@@ -711,7 +867,7 @@ TrainerAgent::analyzeServerParam( const char * msg )
     {
         long interval = ( ServerParam::i().simulatorStep()
                           * ServerParam::i().slowDownFactor() );
-        M_client->setIntervalMSec( interval );
+        agent_.M_client->setIntervalMSec( interval );
     }
 }
 
@@ -720,20 +876,20 @@ TrainerAgent::analyzeServerParam( const char * msg )
   (ok
 */
 void
-TrainerAgent::analyzeOK( const char * msg )
+TrainerAgent::Impl::analyzeOK( const char * msg )
 {
     if ( ! std::strncmp( msg, "(ok look ",
                          std::strlen( "(ok look " ) ) )
     {
         std::cout << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " recv (ok look ..." << std::endl;
     }
     else if ( ! std::strncmp( msg, "(ok check_ball ",
                               std::strlen( "(ok check_ball " ) ) )
     {
         std::cout << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " recv (ok check_ball ..." << std::endl;;
     }
     else if ( ! std::strncmp( msg, "(ok compression ",
@@ -743,35 +899,35 @@ TrainerAgent::analyzeOK( const char * msg )
         if ( std::sscanf( msg, "(ok compression %d)", &level ) == 1 )
         {
             std::cerr << "trainer: "
-                      << M_impl->current_time_
+                      << current_time_
                       << " set compression level " << level << std::endl;
-            M_client->setCompressionLevel( level );
+            agent_.M_client->setCompressionLevel( level );
         }
     }
     else if ( ! std::strncmp( msg, "(ok eye ", std::strlen( "(ok eye " ) ) )
     {
         std::cout << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " recv " << msg << std::endl;
     }
     else if ( ! std::strncmp( msg, "(ok ear ", std::strlen( "(ok ear " ) ) )
     {
         std::cout << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " recv " << msg << std::endl;
     }
     else if ( ! std::strncmp( msg, "(ok team_names ",
                               std::strlen( "(ok team_names " ) ) )
     {
         std::cout << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " recv " << msg << std::endl;
         analyzeTeamNames( msg );
     }
     else
     {
         std::cout << "trainer: "
-                  << M_impl->current_time_
+                  << current_time_
                   << " recv " << msg << std::endl;
     }
 }
@@ -781,7 +937,7 @@ TrainerAgent::analyzeOK( const char * msg )
 
 */
 void
-TrainerAgent::analyzeTeamNames( const char * msg )
+TrainerAgent::Impl::analyzeTeamNames( const char * msg )
 {
     // "(ok team_names (team l <name>)[ (team r <name>)])"
 
@@ -792,12 +948,12 @@ TrainerAgent::analyzeTeamNames( const char * msg )
                          left, right );
     if ( n == 2 )
     {
-        M_worldmodel.setTeamName( LEFT, left );
-        M_worldmodel.setTeamName( RIGHT, right );
+        agent_.M_worldmodel.setTeamName( LEFT, left );
+        agent_.M_worldmodel.setTeamName( RIGHT, right );
     }
     else if ( n == 1 )
     {
-        M_worldmodel.setTeamName( LEFT, left );
+        agent_.M_worldmodel.setTeamName( LEFT, left );
     }
 }
 
@@ -806,10 +962,10 @@ TrainerAgent::analyzeTeamNames( const char * msg )
 
 */
 void
-TrainerAgent::analyzeError( const char * msg )
+TrainerAgent::Impl::analyzeError( const char * msg )
 {
     std::cerr << "trainer: "
-              << M_impl->current_time_
+              << current_time_
               << " recv " << msg  << std::endl;
 }
 
@@ -818,10 +974,10 @@ TrainerAgent::analyzeError( const char * msg )
 
 */
 void
-TrainerAgent::analyzeWarning( const char * msg )
+TrainerAgent::Impl::analyzeWarning( const char * msg )
 {
     std::cerr << "trainer: "
-              << M_impl->current_time_
+              << current_time_
               << " recv " << msg << std::endl;
 }
 
@@ -859,20 +1015,20 @@ TrainerAgent::sendCommand( const TrainerCommand & com )
 
 */
 void
-TrainerAgent::sendInitCommand()
+TrainerAgent::Impl::sendInitCommand()
 {
-    if ( ! M_client->isServerAlive() )
+    if ( ! agent_.M_client->isServerAlive() )
     {
         std::cerr << "trainer: server is not alive" << std::endl;
         return;
     }
 
     // make command string
-    TrainerInitCommand com( config().version() );
-    if ( ! sendCommand( com ) )
+    TrainerInitCommand com( agent_.config().version() );
+    if ( ! agent_.sendCommand( com ) )
     {
         std::cerr << "trainer: Failed to init...\nExit." << std::endl;
-        M_client->setServerAlive( false );
+        agent_.M_client->setServerAlive( false );
         return;
     }
 
@@ -884,10 +1040,10 @@ TrainerAgent::sendInitCommand()
 
 */
 void
-TrainerAgent::sendByeCommand()
+TrainerAgent::Impl::sendByeCommand()
 {
     // trainer has no bye type command
-    M_client->setServerAlive( false );
+    agent_.M_client->setServerAlive( false );
 }
 
 /*-------------------------------------------------------------------*/
