@@ -44,7 +44,6 @@
 
 namespace rcsc {
 
-struct TrainerAgentImpl;
 class TrainerCommand;
 class GlobalVisualSensor;
 
@@ -56,8 +55,11 @@ class TrainerAgent
     : public SoccerAgent {
 private:
 
+    struct Impl; //!< pimpl idiom
+    friend struct Impl;
+
     //! internal implementation object
-    boost::scoped_ptr< TrainerAgentImpl > M_impl;
+    boost::scoped_ptr< Impl > M_impl;
 
 protected:
 
@@ -79,6 +81,11 @@ public:
     virtual
     ~TrainerAgent();
 
+    /*!
+      \brief finalize all things when the process exits
+    */
+    void finalize();
+
     /*!
       \brief get configuration set
       \return const reference to the configuration class object
@@ -106,187 +113,6 @@ public:
     const
     GlobalVisualSensor & visualSensor() const;
 
-protected:
-    /*!
-      \brief analyze command line options
-      \param cmd_parser command line parser object
-      \return only if "help" option is given, false is returned.
-
-      This method is called from SoccerAgent::init( argc, argv )
-      SoccerAgent::init(argc,argv) should be called in main().
-      Do NOT call this method by yourself!
-    */
-    virtual
-    bool initImpl( CmdLineParser & cmd_parser );
-
-    /*!
-      \brief handle start event
-      \return status of start procedure.
-
-      This method is called on the top of BasicClient::run() method.
-      The concrete agent must connect to the server and send init command.
-      Do NOT call this method by yourself!
-    */
-    virtual
-    bool handleStart();
-
-    /*!
-      \brief handle server message event
-
-      Do NOT call this method by yourself!
-    */
-    virtual
-    void handleMessage();
-
-    /*!
-      \brief handle timeout event
-      \param timeout_count count of timeout without sensory message.
-      \param waited_msec elapsed milli seconds sinc last sensory message.
-
-      This method is called when select() timeout occurs
-      Do NOT call this method by yourself!
-    */
-    virtual
-    void handleTimeout( const int timeout_count,
-                        const int waited_msec );
-
-    /*!
-      \brief handle exit event
-    */
-    virtual
-    void handleExit();
-
-public:
-    /*!
-      \brief finalize program process
-    */
-    void finalize();
-
-private:
-
-    /*!
-      \brief send init or reconnect command to server
-
-      init commad is sent in BasicClient's run() method
-      Do not call this method yourself!
-    */
-    void sendInitCommand();
-
-    /*!
-      \brief send disconnection command message to server
-      and set the server status to end.
-    */
-    void sendByeCommand();
-
-    ////////////////////////////////////
-
-    /*!
-      \brief launch each parsing method depending on the message type
-      \param msg raw server message
-    */
-    void parse( const char * msg );
-
-    /*!
-      \brief analyze init replay message
-      \param msg raw server message
-    */
-    void analyzeInit( const char * msg );
-
-    /*!
-      \brief analyze cycle info in server message
-      \param msg raw server message
-      \param by_see_global if message type is see_global, this value becomes true
-    */
-    bool analyzeCycle( const char * msg,
-                       const bool by_see_global );
-
-    /*!
-      \brief analyze see_global message
-      \param msg raw server message
-    */
-    void analyzeSeeGlobal( const char * msg );
-
-    /*!
-      \brief analyze hear message
-      \param msg raw server message
-    */
-    void analyzeHear( const char * msg );
-
-    /*!
-      \brief analyze referee message
-      \param msg raw server message
-    */
-    void analyzeHearReferee( const char * msg );
-
-    /*!
-      \brief analyze audio message from player
-      \param msg raw server message
-    */
-    void analyzeHearPlayer( const char * msg );
-
-    /*!
-      \brief analyze change_player_type message
-      \param msg raw server message
-    */
-    void analyzeChangePlayerType( const char * msg );
-
-    /*!
-      \brief analyze player_type parameter message
-      \param msg raw server message
-    */
-    void analyzePlayerType( const char * msg );
-
-    /*!
-      \brief analyze player_param parameter message
-      \param msg raw server message
-    */
-    void analyzePlayerParam( const char * msg );
-
-    /*!
-      \brief analyze server_param parameter message
-      \param msg raw server message
-    */
-    void analyzeServerParam( const char * msg );
-
-    /*!
-      \brief analyze ok message
-      \param msg raw server message
-    */
-    void analyzeOK( const char * msg );
-
-    /*!
-      \brief analyze ok teamnames message
-      \param msg raw server message
-     */
-    void analyzeTeamNames( const char * msg );
-
-    /*!
-      \brief analyze error message
-      \param msg raw server message
-    */
-    void analyzeError( const char * msg );
-
-    /*!
-      \brief analyze warning message
-      \param msg raw server message
-    */
-    void analyzeWarning( const char * msg );
-
-
-    /*!
-      \brief main action decision. this method is called from handleMessage()
-    */
-    void action();
-
-    /*!
-      \brief send command string to the rcssserver
-      \param com trainer command object
-      \return true if command is sent
-    */
-    bool sendCommand( const TrainerCommand & com );
-
-public:
-
     /*!
       \brief send check_ball command
       \return true if command is generated and sent
@@ -388,7 +214,76 @@ public:
     */
     bool doSay( const std::string & msg );
 
+private:
+
+    /*!
+      \brief launch each parsing method depending on the message type
+      \param msg raw server message
+    */
+    void parse( const char * msg );
+
+
+    /*!
+      \brief main action decision. this method is called from handleMessage()
+    */
+    void action();
+
+    /*!
+      \brief send command string to the rcssserver
+      \param com trainer command object
+      \return true if command is sent
+    */
+    bool sendCommand( const TrainerCommand & com );
+
 protected:
+    /*!
+      \brief analyze command line options
+      \param cmd_parser command line parser object
+      \return only if "help" option is given, false is returned.
+
+      This method is called from SoccerAgent::init( argc, argv )
+      SoccerAgent::init(argc,argv) should be called in main().
+      Do NOT call this method by yourself!
+    */
+    virtual
+    bool initImpl( CmdLineParser & cmd_parser );
+
+    /*!
+      \brief handle start event
+      \return status of start procedure.
+
+      This method is called on the top of BasicClient::run() method.
+      The concrete agent must connect to the server and send init command.
+      Do NOT call this method by yourself!
+    */
+    virtual
+    bool handleStart();
+
+    /*!
+      \brief handle server message event
+
+      Do NOT call this method by yourself!
+    */
+    virtual
+    void handleMessage();
+
+    /*!
+      \brief handle timeout event
+      \param timeout_count count of timeout without sensory message.
+      \param waited_msec elapsed milli seconds sinc last sensory message.
+
+      This method is called when select() timeout occurs
+      Do NOT call this method by yourself!
+    */
+    virtual
+    void handleTimeout( const int timeout_count,
+                        const int waited_msec );
+
+    /*!
+      \brief handle exit event
+    */
+    virtual
+    void handleExit();
 
     /*!
       \brief pure virtual method. register decision.
