@@ -192,7 +192,7 @@ TrainerMoveBallCommand::toStr( std::ostream & to ) const
 {
     to << "(move (ball) " << M_pos.x << " " << M_pos.y;
 
-    if ( ! M_vel.valid() )
+    if ( ! M_vel.isValid() )
     {
         to << ")";
     }
@@ -368,7 +368,7 @@ TrainerMovePlayerCommand::toStr( std::ostream & to ) const
 
     to << " " << M_angle;
 
-    if ( M_vel.valid() )
+    if ( M_vel.isValid() )
     {
         to << " " << M_vel.x << " " << M_vel.y;
     }
