@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -66,8 +66,8 @@ void
 TrainerConfig::setDefaultParam()
 {
     // basic setting
-    M_team_name = "HELIOS2007";
-    M_version = 9.4;
+    M_team_name = "HELIOS_base";
+    M_version = 14;
 
     M_server_wait_seconds = 5;
 
@@ -94,7 +94,7 @@ TrainerConfig::createParamMap( ParamMap & param_map )
         ( "server_wait_seconds", "", &M_server_wait_seconds )
 
         ( "host", "h", &M_rcssserver_host )
-        ( "port", "", &M_rcssserver_port )
+        ( "port", "p", &M_rcssserver_port )
 
         ( "compression", "", &M_compression )
 
