@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,6 +36,7 @@
 #include <rcsc/game_mode.h>
 
 #include <iostream>
+#include <cstdlib>
 #include <cstring>
 
 namespace {
@@ -58,6 +59,13 @@ private:
 
 public:
 
+    ~MapHolder()
+      {
+          //std::cerr << "delete MapHolder map" << std::endl;
+          M_playmode_map.clear();
+          //std::cerr << "delete MapHolder done map.clear" << std::endl;
+      }
+
     static
     const
     MapHolder & instance()
@@ -104,19 +112,19 @@ MapHolder::MapHolder()
     M_playmode_map["drop_ball"] = std::make_pair( GameMode::PlayOn, NEUTRAL );
     M_playmode_map["offside_l"] = std::make_pair( GameMode::OffSide_, LEFT );
     M_playmode_map["offside_r"] = std::make_pair( GameMode::OffSide_, RIGHT );
-    // M_playmode_map["penalty_kick_l"]  = std::make_pair( GameMode::PenaltyKick_, LEFT );
-    // M_playmode_map["penalty_kick_r"]  = std::make_pair( GameMode::PenaltyKick_, RIGHT );
-    // M_playmode_map["first_half_over"] = std::make_pair( GameMode::FirsthalfOver, NEUTRAL );
-    // M_playmode_map["pause"]           = std::make_pair( GameMode::Pause, NEUTRAL );
-    // M_playmode_map["human_judge"]     = std::make_pair( GameMode::Human, NEUTRAL );
-    // M_playmode_map["foul_charge_l"]   = std::make_pair( GameMode::Foul_Charge_, LEFT );
-    // M_playmode_map["foul_charge_r"]   = std::make_pair( GameMode::Foul_Charge_, RIGHT );
-    // M_playmode_map["foul_push_l"]     = std::make_pair( GameMode::FoulPush_,    LEFT );
-    // M_playmode_map["foul_push_r"]     = std::make_pair( GameMode::FoulPush_,    RIGHT );
-    // M_playmode_map["foul_multiple_attack_l"] = std::make_pair( GameMode::FoulMultipleAttacker_, LEFT );
-    // M_playmode_map["foul_multiple_attack_r"] = std::make_pair( GameMode::FoulMultipleAttacker_, RIGHT );
-    // M_playmode_map["foul_ballout_l"]  = std::make_pair( GameMode::FoulBallOur_, LEFT );
-    // M_playmode_map["foul_ballout_r"]  = std::make_pair( GameMode::FoulBallOur_, RIGHT );
+    M_playmode_map["penalty_kick_l"]  = std::make_pair( GameMode::PenaltyKick_, LEFT );
+    M_playmode_map["penalty_kick_r"]  = std::make_pair( GameMode::PenaltyKick_, RIGHT );
+    M_playmode_map["first_half_over"] = std::make_pair( GameMode::FirstHalfOver, NEUTRAL );
+    M_playmode_map["pause"]           = std::make_pair( GameMode::Pause, NEUTRAL );
+    M_playmode_map["human_judge"]     = std::make_pair( GameMode::Human, NEUTRAL );
+    M_playmode_map["foul_charge_l"]   = std::make_pair( GameMode::FoulCharge_, LEFT );
+    M_playmode_map["foul_charge_r"]   = std::make_pair( GameMode::FoulCharge_, RIGHT );
+    M_playmode_map["foul_push_l"]     = std::make_pair( GameMode::FoulPush_,    LEFT );
+    M_playmode_map["foul_push_r"]     = std::make_pair( GameMode::FoulPush_,    RIGHT );
+    M_playmode_map["foul_multiple_attack_l"] = std::make_pair( GameMode::FoulMultipleAttacker_, LEFT );
+    M_playmode_map["foul_multiple_attack_r"] = std::make_pair( GameMode::FoulMultipleAttacker_, RIGHT );
+    M_playmode_map["foul_ballout_l"]  = std::make_pair( GameMode::FoulBallOut_, LEFT );
+    M_playmode_map["foul_ballout_r"]  = std::make_pair( GameMode::FoulBallOut_, RIGHT );
     M_playmode_map["back_pass_l"] = std::make_pair( GameMode::BackPass_,  LEFT );
     M_playmode_map["back_pass_r"] = std::make_pair( GameMode::BackPass_, RIGHT );
     M_playmode_map["free_kick_fault_l"] = std::make_pair( GameMode::FreeKickFault_, LEFT );
@@ -237,18 +245,17 @@ GameMode::parse( const std::string & mode_str )
         return it->second;
     }
 
-    if ( ! std::strncmp( mode_str.c_str(), "goal_l", 6 ) )
+    if ( ! mode_str.compare( 0, 6, "goal_l") )
     {
         M_score_left = std::atoi( mode_str.substr( std::strlen( "goal_l_" ) ).c_str() );
         return std::make_pair( AfterGoal_, LEFT );
     }
-    else if ( ! std::strncmp( mode_str.c_str(), "goal_r", 6 ) )
+    else if ( ! mode_str.compare( 0, 6, "goal_r" ) )
     {
         M_score_right = std::atoi( mode_str.substr( std::strlen( "goal_r_" ) ).c_str() );
         return std::make_pair( AfterGoal_, RIGHT );
     }
 
-
     //std::cerr << __FILE__ << ": " << __LINE__
     //          << " Received unsupported playmode : [" << mode_str << "]"
     //          << std::endl;
@@ -267,6 +274,8 @@ GameMode::isServerCycleStoppedMode() const
     case BeforeKickOff:
     case AfterGoal_:
     case OffSide_:
+    case FoulCharge_:
+    case FoulPush_:
     case FreeKickFault_:
     case BackPass_:
     case CatchFault_:
@@ -314,7 +323,7 @@ GameMode::isPenaltyKickMode() const
 
 */
 bool
-GameMode::isOurSetPlay( const SideID ourside ) const
+GameMode::isTeamsSetPlay( const SideID team_side ) const
 {
     switch ( type() ) {
     case KickOff_:
@@ -324,16 +333,18 @@ GameMode::isOurSetPlay( const SideID ourside ) const
     case FreeKick_:
     case GoalieCatch_:
     case IndFreeKick_:
-        if ( ourside == side() )
+        if ( team_side == side() )
         {
             return true;
         }
         return false;
     case OffSide_:
+    case FoulCharge_:
+    case FoulPush_:
     case FreeKickFault_:
     case BackPass_:
     case CatchFault_:
-        if ( ourside != side() )
+        if ( team_side != side() )
         {
             return true;
         }
@@ -502,8 +513,12 @@ GameMode::print( std::ostream & os ) const
         break;
         //Pause,
         //Human,
-        //Foul_Charge_,
-        //Foul_Push_,
+    case FoulCharge_:
+        os << " foul_charge";
+        break;
+    case FoulPush_:
+        os << " foul_push";
+        break;
         //Foul_MultipleAttacker_,
         //Foul_BallOut_,
     case BackPass_:
